 
	program mc_execute
 
c this is a simple driver program so that the MC routine, written to
c be a subroutine for the INDUCT95 (c) fluid code, could be run
c as an independent code.
 
c currently this code allows the MC to be run in three modes:
c	1) 1-d, 3-v
c	2) 2-d, 3-v, cartesian
c       3) 2-d, 3-v, cylinder, azimuthal symmetry.
 
c this 'driver' simply does what the hybrid INDUCT95/MC code does: sets
c up data structures which the MC code then uses.
 
c the algorithm is trivial. first read in user data. second, construct
c data structures. third call the MC routine. finally exit.
 
	write(*,*)
	write(*,"('Simple Monte Carlo, v2.5 (c) G.J. Parker')")
	write(*,*)
 
	write(*,"('  calling MC setup routine...')")
	call input_setup
 
	write(*,*)
	write(*,"('  calling MC routine...')")
	write(*,*)
	call baccarat_mc
 
	write(*,*)
	write(*,"('SMC successful completion!')")
c	pause
	end
 
c told you it was simple...
 
 
 
 
 
c-------------------------------------
 
	subroutine input_setup
 
	include 'mc1.inc'
	include 'mc2.inc'
 
	logical ext_p, spec_read_p(inspec_mc), uniform_s_p, xsym_p
	logical read_e_p, read_ez_p, read_er_p
	character*80 flnm, st1, st2, maskfile, phifile, denfile,
     ^		elaunchfile, noblankupper_mc, sheathfile, ezfile,
     ^		erxfile
	character*80 bplrxfile,bplthyfile,bplzfile
	character*80 bh_rxfile,bh_thyfile,bh_zfile
	character*80 eh_rxfile,eh_thyfile,eh_zfile
	real*8 x0, y0, x1 !, y1
	real*8 e_n_0d
	real ezt(nrmax_mc,nzmax_mc),ert(nrmax_mc,nzmax_mc)
 
	namelist /structure/ nr_mc, nz_mc,
     ^		r_max_mc, z_max_mc, nspec_mc, specn_mc,
     ^		te0, maskfile, phifile, denfile, sheathfile,
     ^		elaunchfile, lcyl_mc, uniform_s_p,e_n_0d, xsym_p,
     ^		erxfile,ezfile,read_e_p,
     ^		bplrxfile,bplthyfile,bplzfile,
     ^		bh_rxfile,bh_thyfile,bh_zfile,
     ^		eh_rxfile,eh_thyfile,eh_zfile, rf_freq_mc
 
 
 
	nr_mc       = 1
	nz_mc       = 1
	r_max_mc    = 1.
	z_max_mc    = 1.
	nspec_mc    = 0
	te0         = 2.
	maskfile    = 'none.msk'
	phifile     = 'none.phi'
	denfile     = 'none.den'
	elaunchfile = 'none.elh'
	sheathfile  = 'none.shp'
	erxfile     = 'none.erx'
	ezfile      = 'none.ez'
	lcyl_mc     = .true.
	uniform_s_p = .true.
	ef0d        = 0.
	rf_freq_mc  = 13.56e6
	e_n_0d      = 0.
	xsym_p      = .true.
	read_e_p    = .false.
	bplrxfile   = 'none.bprx'
	bplthyfile  = 'none.bpty'
	bplzfile    = 'none.bpz'
	bh_rxfile   = 'none.bhrx'
	bh_thyfile  = 'none.bhty'
	bh_zfile    = 'none.bhz'
	eh_rxfile   = 'none.ehrx'
	eh_thyfile  = 'none.ehty'
	eh_zfile    = 'none.ehz'
 
c read in 'structure.in' data file...
	inquire(file='structure.in',exist=ext_p)
	if (.not.ext_p) then
	  write(*,"('    ERROR- file ''structure.in'' does ',
     ^			'not exist,')")
	  write(*,"('    run aborting...')")
c	  pause
	  stop
	endif
 
	write(*,"('    reading input files')")
	open(15,file='structure.in',status='old')
	 read(15,structure)
	close(15)
 
c do some simple consistancy checks...
        nr_mc = max(1, nr_mc)
	if (nr_mc.gt.nrmax_mc) then
	  write(*,"('    ERROR- nr_mc > nrmax_mc, run aborting...')")
c	  pause
	  stop
	endif
	nz_mc = max(1, nz_mc)
	if (nz_mc.gt.nzmax_mc) then
	  write(*,"('    ERROR- nz_mc > nzmax_mc, run aborting...')")
c	  pause
	  stop
	endif
	if (nspec_mc.gt.inspec_mc.or.nspec_mc.lt.1) then
	  write(*,"('    ERROR- nspec_mc > inspec_mc, ',
     ^			'run aborting...')")
c	  pause
	  stop
	endif
	if (z_max_mc.le.0.0) then
	  write(*,"('    ERROR- z_max_mc =< 0.0, run aborting...')")
c	  pause
	  stop
	endif
	if (r_max_mc.le.0.0) then
	  write(*,"('    ERROR- r_max_mc =< 0.0, run aborting...')")
c	  pause
	  stop
	endif
 
	do j = 0, nz_mc+1
	  do i = 0, nr_mc+1
	    ele_s_mc(i,j) = 0.
	    e_density_mc(1,max(1,min(nr_mc,i)),max(1,min(nz_mc,j))) = 0.
	  enddo
	enddo
 
c 0d?
	if (nr_mc .eq. 1 .and. nz_mc .eq. 1) then
	  if (e_n_0d .eq. 0.) then
	    write(*,"('    ERROR- 0d but E=0,  run aborting...')")
c	    pause
	    stop
	  endif
 
	  xsym_p = .false.
	  lcyl_mc = .false.
c mask...
	  do j = 0, 2
	    do i = 0, 2
	      mask_mc(i,j) = 1
	    enddo
	  enddo
	  mask_mc(1,1) = 0
 
c launch distribution...
	  ele_s_mc(1,1) = 1e10
 
	  go to 67
	endif
 
	if (nz_mc.eq.1 .and. nr_mc.ne.1) then
	  xsym_p = .false.
	  lcyl_mc = .true.
	endif
	if (nr_mc.eq.1 .and. nz_mc.ne.1) then
	  xsym_p = .true.
	  lcyl_mc = .false.
	endif
 
	if (lcyl_mc) xsym_p = .false.
	if (xsym_p)  lcyl_mc = .false.
	if (nspec_mc.lt.1) then
	  write(*,"('    ERROR- no non-electron species')")
	  write(*,"('    run aborting...')")
c	  pause
	  stop
	endif
 
c read in mask file...
	flnm = maskfile
	inquire(file=flnm,exist=ext_p)
	if (.not.ext_p) then
	  if ( flnm(1:8) .ne. 'none.msk' ) then
	    write(*,"('    WARNING- file ',a,' does not exist,',
     ^		' using a zeroed mask structure')")
     ^		flnm(1:index(flnm,' ')-1)
	    write(*,*)
	  endif
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      mask_mc(i,j) = 0
	    enddo
	  enddo
	else
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991)
     ^		((mask_mc(i,j),i=1,nr_mc),j=nz_mc,1,-1)
	  close(15)
	endif
	do i = 0, nr_mc+1
	  mask_mc(i,      0) = 9
	  mask_mc(i,nz_mc+1) = 8
	enddo
	do j = 1, nz_mc
	  mask_mc(      0,j) = 7
	  if ((xsym_p.or.lcyl_mc).and.mask_mc(1,j).eq.0)
     ^		mask_mc(0,j) = 0
	  mask_mc(nr_mc+1,j) = 6
	enddo
 
c read in phi file... (values at centers of cells....)
	if (e_n_0d.eq.0.0) then
	  flnm = phifile
	  inquire(file=flnm,exist=ext_p)
	  if (.not.ext_p) then
	    write(*,"('    ERROR- file ',a,' does not exist,')")
     ^		  flnm(1:index(flnm,' ')-1)
	    write(*,"('    run aborting...')")
c	    pause
	    stop
	  endif
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991) ((phi_pl_mc(i,j,1),
     ^			i=0,nr_mc+1),j=nz_mc+1,0,-1)
	  close(15)
	  if ( xsym_p.or.lcyl_mc ) then
	    do j = 0, nz_mc+1
	      phi_pl_mc(0,j,1) = phi_pl_mc(1,j,1)
	    enddo
          endif
	endif
 
c read in Ez file... (values at cell corners...)
	if (e_n_0d.eq.0.0) then
	  flnm = ezfile
	  inquire(file=flnm,exist=ext_p)
	  if (.not.ext_p.and.read_e_p) then
	    write(*,"('    ERROR- file ',a,' does not exist,')")
     ^		flnm(1:index(flnm,' ')-1)
	    write(*,"('    run aborting...')")
c	    pause
	    stop
	  endif
	  read_ez_p = .false.
	  if (read_e_p) then
	    open(15,file=flnm,status='old')
  	     read(15,"(a)",err=991,end=991) st2
	     do while (st2(1:4).ne.'>>>>')
  	       read(15,"(a)",err=991,end=991) st2
	     enddo
	     read(15,*,err=991,end=991) ((epl_z_mc(i,j,1),
     ^			i=0,nr_mc),j=nz_mc,0,-1)
	    close(15)
	    read_ez_p = .true.
	  endif
	endif
 
c read in harmonic Ez file... (values at cell corners...)
	flnm = eh_zfile
	inquire(file=flnm,exist=ext_p)
	if (ext_p) then
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991) ((e_z_mc(i,j),
     ^			i=0,nr_mc),j=nz_mc,0,-1)
	  close(15)
	endif
 
c read in harmonic Bz file... (values at cell corners...)
	flnm = bh_zfile
	inquire(file=flnm,exist=ext_p)
	if (ext_p) then
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991) ((b_z_mc(i,j),
     ^			i=0,nr_mc),j=nz_mc,0,-1)
	  close(15)
	endif
 
c read in Bz file... (values at cell corners...)
	flnm = bplzfile
	inquire(file=flnm,exist=ext_p)
	if (ext_p) then
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991) ((bpl_z_mc(i,j),
     ^			i=0,nr_mc),j=nz_mc,0,-1)
	  close(15)
	endif
 
c read in harmonic Bth/y file... (values at cell corners...)
	flnm = bh_thyfile
	inquire(file=flnm,exist=ext_p)
	if (ext_p) then
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991) ((b_th_mc(i,j),
     ^			i=0,nr_mc),j=nz_mc,0,-1)
	  close(15)
	endif
 
c read in harmonic Eth/y file... (values at cell corners...)
	flnm = eh_thyfile
	inquire(file=flnm,exist=ext_p)
	if (ext_p) then
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991) ((e_theta_mc(i,j),
     ^			i=0,nr_mc),j=nz_mc,0,-1)
	  close(15)
	endif
 
c read in Bth/y file... (values at cell corners...)
	flnm = bplthyfile
	inquire(file=flnm,exist=ext_p)
	if (ext_p) then
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991) ((bpl_th_mc(i,j),
     ^			i=0,nr_mc),j=nz_mc,0,-1)
	  close(15)
	endif
 
c read in Erx file... (values at cell corners...)
	if (e_n_0d.eq.0.0) then
	  flnm = erxfile
	  inquire(file=flnm,exist=ext_p)
	  if (.not.ext_p.and.read_e_p) then
	    write(*,"('    ERROR- file ',a,' does not exist,')")
     ^		flnm(1:index(flnm,' ')-1)
	    write(*,"('    run aborting...')")
c	    pause
	    stop
	  endif
	  read_er_p = .false.
	  if (read_e_p) then
	    open(15,file=flnm,status='old')
  	     read(15,"(a)",err=991,end=991) st2
	     do while (st2(1:4).ne.'>>>>')
  	       read(15,"(a)",err=991,end=991) st2
	     enddo
	     read(15,*,err=991,end=991) ((epl_r_mc(i,j,1),
     ^			i=0,nr_mc),j=nz_mc,0,-1)
	    close(15)
	    if ( xsym_p.or.lcyl_mc ) then
	      do j = 0, nz_mc
	        epl_r_mc(0,j,1) = 0.
	      enddo
	    endif
	    read_er_p = .true.
	  endif
	endif
 
c read in Brx file... (values at cell corners...)
	flnm = bplrxfile
	inquire(file=flnm,exist=ext_p)
	if (ext_p) then
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991) ((bpl_r_mc(i,j),
     ^			i=0,nr_mc),j=nz_mc,0,-1)
	  close(15)
	  if ( xsym_p.or.lcyl_mc ) then
	    do j = 0, nz_mc
	      bpl_r_mc(0,j) = 0.
	    enddo
	  endif
	endif
 
c read in harmonic Brx file... (values at cell corners...)
	flnm = bh_rxfile
	inquire(file=flnm,exist=ext_p)
	if (ext_p) then
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991) ((b_r_mc(i,j),
     ^			i=0,nr_mc),j=nz_mc,0,-1)
	  close(15)
	  if ( xsym_p.or.lcyl_mc ) then
	    do j = 0, nz_mc
	      b_r_mc(0,j) = (0.,0.)
	    enddo
	  endif
	endif
 
c read in harmonic Erx file... (values at cell corners...)
	flnm = eh_rxfile
	inquire(file=flnm,exist=ext_p)
	if (ext_p) then
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991) ((e_r_mc(i,j),
     ^			i=0,nr_mc),j=nz_mc,0,-1)
	  close(15)
	  if ( xsym_p.or.lcyl_mc ) then
	    do j = 0, nz_mc
	      e_r_mc(0,j) = (0.,0.)
	    enddo
	  endif
	endif
 
c read in sheath potential file...
	flnm = sheathfile
	inquire(file=flnm,exist=ext_p)
	if (.not.ext_p) then
	  do j = 0, nz_mc+1
	    do i = 0, nr_mc+1
	      sheath_potential_mc(i,j) = 0.
	    enddo
	  enddo
	else
	  k = 0
	  if ( xsym_p.or.lcyl_mc ) k = 1
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   read(15,*,err=991,end=991) ((sheath_potential_mc(i,j),
     ^		i=k,nr_mc+1),j=nz_mc+1,0,-1)
	  close(15)
	  if (k.eq.1) then
	    do j = 0, nz_mc+1
	      sheath_potential_mc(0,j) = sheath_potential_mc(1,j)
	    enddo
	  endif
	endif
 
c read in elaunch file...
c	if (.not. uniform_s_p) then
	  flnm = elaunchfile
	  inquire(file=flnm,exist=ext_p)
	  if (.not.ext_p) then
	    if ( flnm(1:8) .ne. 'none.elh' ) then
	      write(*,"('    WARNING- file ',a,' does not exist,',
     ^		' uniform_s_p will be set to true')")
     ^		flnm(1:index(flnm,' ')-1)
	      write(*,*)
	    endif
	    uniform_s_p = .true.
	  else
	    uniform_s_p = .false.
	  endif
c	endif
	if (.not. uniform_s_p) then
	  open(15,file=flnm,status='old')
  	   read(15,"(a)",err=991,end=991) st2
	   do while (st2(1:4).ne.'>>>>')
  	     read(15,"(a)",err=991,end=991) st2
	   enddo
	   k = 0
	   if ( xsym_p.or.lcyl_mc ) k = 1
	   read(15,*,err=991,end=991)
     ^		((ele_s_mc(i,j),i=k,nr_mc+1),j=nz_mc+1,0,-1)
	  close(15)
	endif
 
 
c read in den file...
 67	flnm = denfile
	inquire(file=flnm,exist=ext_p)
	if (.not.ext_p) then
	  write(*,"('    ERROR- file ',a,' does not exist,')")
     ^		flnm(1:index(flnm,' ')-1)
	  write(*,"('    run aborting...')")
c	  pause
	  stop
	endif
	do k = 1, nspec_mc
	  spec_read_p(k) = .false.
	enddo
	n = 0
	open(15,file=flnm,status='old')
 
 57 	continue !repeat
	    read(15,"(a)",err=991,end=991) st2
	    do while (st2(1:4).ne.'>>>>')
  	      read(15,"(a)",err=991,end=991) st2
	    enddo
	    read(15,"(a)",err=991,end=991) st2
	    st1 = noblankupper_mc(st2,80)
	    do k = 1, nspec_mc
	      st2 = noblankupper_mc(specn_mc(k),80)
	      if (st1.eq.st2) exit
	    enddo
	    if (k .le. nspec_mc) then
	      if (spec_read_p(k))  then
		write(*,"('    WARNING- ',a,
     ^			'species density read again')")
     ^			st1(1:index(st1,' ')-1)
	        write(*,*)
	      else
		n = n + 1
	        spec_read_p(k) = .true.
	      endif
	      read(15,*,err=991,end=991)
     ^		((n_density_mc(1,i,j,k),i=1,nr_mc),j=nz_mc,1,-1)
	    endif
 
	  if (n .ne. nspec_mc) go to 57
	close(15)
 
	write(*,"('    constructing data structures')")
 
	rf_freq_mc = 1.
 
	delta_z_mc = z_max_mc / nz_mc
	delta_r_mc = r_max_mc / nr_mc
 
	zt_mc(nz_mc+1) = z_max_mc
	do i = 1, nz_mc
	  zt_mc( i) = delta_z_mc * (i - 1.0)
	  zcm_mc(i) = delta_z_mc * (i - 0.5)
	  dz_mc( i) = delta_z_mc
	  dzi_mc(i) = 1.0 / delta_z_mc
	enddo
	dz_mc(       0) = delta_z_mc
	dzi_mc(      0) = 1.0 / delta_z_mc
	dz_mc( nz_mc+1) = delta_z_mc
	dzi_mc(nz_mc+1) = 1.0 / delta_z_mc
	z_min_mc = 0.
 
	rt_mc(nr_mc+1) = r_max_mc
	do i = 1, nr_mc
	  rt_mc( i) = delta_r_mc * (i - 1.0)
	  rcm_mc(i) = delta_r_mc * (i - 0.5)
	  dr_mc( i) = delta_r_mc
	  dri_mc(i) = 1.0 / delta_r_mc
	enddo
	dr_mc(       0) = delta_r_mc
	dri_mc(      0) = 1.0 / delta_r_mc
	dr_mc( nr_mc+1) = delta_r_mc
	dri_mc(nr_mc+1) = 1.0 / delta_r_mc
 
	it1_max_mc = 1
	inductp_mc = .false.
	do j = 0, nz_mc
	  do i = 0, nr_mc
	    bpl_r_mc(                 i ,j) = 0.
	    bpl_th_mc(                i ,j) = 0.
	    bpl_z_mc(                 i ,j) = 0.
	    b_r_mc(                   i ,j) = (0.,0.)
	    b_th_mc(                  i ,j) = (0.,0.)
	    b_z_mc(                   i ,j) = (0.,0.)
	    e_r_mc(                   i ,j) = (0.,0.)
	    e_theta_mc(               i ,j) = (0.,0.)
	    e_z_mc(                   i ,j) = (0.,0.)
	  enddo
	enddo

	if ((nr_mc .eq. 1 .and. nz_mc .eq. 1).or.
     ^    (e_n_0d.ne.0.0 .and. (nr_mc.eq.1.or.nz_mc.eq.1))) then

	  read_er_p = .true.
	  read_ez_p = .true.
 
	  x0 = 1.
          if (nr_mc .gt. nz_mc) x0 = 0.
          do j = 1, nz_mc
	    do i = 1, nr_mc
	      ef0d = 0.
	      do k = 1, nspec_mc
		ef0d = ef0d + n_density_mc(1,i,j,k)
	      enddo
	      ef0d = - e_n_0d * ef0d * 1d-17
              ert(i,j) = ef0d * (1.-x0)
	      ezt(i,j) = ef0d *     x0
	    enddo
	  enddo

	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      epl_r_mc(i,j,1) = 0.25*(
     ^		ert(max(    1,i  ),max(    1,j  )) +
     ^		ert(min(nr_mc,i+1),max(    1,j  )) +
     ^		ert(max(    1,i  ),min(nz_mc,j+1)) +
     ^          ert(min(nr_mc,i+1),min(nz_mc,j+1)) )
	      epl_z_mc(i,j,1) = 0.25*(
     ^		ezt(max(    1,i  ),max(    1,j  )) +
     ^		ezt(min(nr_mc,i+1),max(    1,j  )) +
     ^		ezt(max(    1,i  ),min(nz_mc,j+1)) +
     ^          ezt(min(nr_mc,i+1),min(nz_mc,j+1)) )
	    enddo
	  enddo

c phi (ie E)
	  if ( nz_mc .ge. nr_mc ) then
	    do i = 0, nr_mc+1
	      phi_pl_mc(i,0,1) = 0.
	    enddo
	    do j = 1, nz_mc+1
	      do i = 1, nr_mc
	        phi_pl_mc(i,j,1) = phi_pl_mc(i,j-1,1)-
     ^		  abs(0.5*(epl_z_mc(i,j,1)+epl_z_mc(i-1,j,1))
     ^		  *delta_z_mc)
	      enddo
	      phi_pl_mc(      0,j,1) = phi_pl_mc(    1,j,1)
	      phi_pl_mc(nr_mc+1,j,1) = phi_pl_mc(nr_mc,j,1)
	    enddo
	  else
	    do j = 0, nz_mc+1
	      phi_pl_mc(0,j,1) = 0.
	    enddo
	    do i = 1, nr_mc+1
	      do j = 1, nz_mc
	        phi_pl_mc(i,j,1) = phi_pl_mc(i-1,j,1)-
     ^		  abs(0.5*(epl_r_mc(i,j,1)+epl_r_mc(i,j-1,1))
     ^		  *delta_r_mc)
	      enddo
	      phi_pl_mc(i,      0,1) = phi_pl_mc(i,    1,1)
	      phi_pl_mc(i,nz_mc+1,1) = phi_pl_mc(i,nz_mc,1)
	    enddo
	  endif
 
	endif
 
c compute fields from potential....
c potential is at cell centers, fields are at cell corners!
	if (nr_mc .ne. 1 .or. nz_mc .ne. 1) then
 
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      if (.not.read_er_p)
     ^	        epl_r_mc(i,j,1) = 0.5 * (
     ^	         (phi_pl_mc(i  ,j  ,1)-phi_pl_mc(i+1,j  ,1)) +
     ^	         (phi_pl_mc(i  ,j+1,1)-phi_pl_mc(i+1,j+1,1)))
     ^				/ delta_r_mc
c              epl_r_mc(i,j,1) = 17.3611111111112 * rt_mc(i+1)
c	      epl_r_mc(i,j,1) = 0.5 * (
c     ^	         (phi_pl_mc(i  ,j  ,1)+phi_pl_mc(i  ,j+1,1)) -
c     ^	         (phi_pl_mc(i+1,j  ,1)+phi_pl_mc(i+1,j+1,1)))
c     ^				/ delta_r_mc
	      if (.not.read_ez_p)
     ^	        epl_z_mc(i,j,1) = 0.5 * (
     ^	         (phi_pl_mc(i  ,j  ,1)-phi_pl_mc(i  ,j+1,1)) +
     ^	         (phi_pl_mc(i+1,j  ,1)-phi_pl_mc(i+1,j+1,1)))
     ^				/ delta_z_mc
c	      epl_z_mc(i,j,1) = 0.5 * (
c     ^	         (phi_pl_mc(i  ,j  ,1)+phi_pl_mc(i+1,j  ,1)) -
c     ^	         (phi_pl_mc(i  ,j+1,1)+phi_pl_mc(i+1,j+1,1)))
c     ^				/ delta_z_mc
	    enddo
	  enddo
	endif
 
c	write(*,"(1p,5e13.5)") ((epl_r_mc(i,j,1),i=0,nr_mc),j=0,nz_mc)
c	write(*,*)
c	write(*,"(1p,5e13.5)") ((epl_z_mc(i,j,1),i=0,nr_mc),j=0,nz_mc)
 
	x1 = acos(-1.d0)
	y0 = 0.d0
	if ( lcyl_mc ) then
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      if ( mask_mc(i,j) .eq. 0 )
     ^	        y0 = y0 + (zt_mc(j+1)-zt_mc(j)) *
     ^			  (rt_mc(i+1)**2-rt_mc(i)**2)
	    enddo
	  enddo
	else
	  x1 = 1.d0
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      if ( mask_mc(i,j) .eq. 0 )
     ^	        y0 = y0 + (zt_mc(j+1)-zt_mc(j)) *
     ^			  (rt_mc(i+1)-rt_mc(i))
	    enddo
	  enddo
	endif
	y0 = 1.d0 / (x1 * y0)
 
	ele_s_max_mc = 0.d0
	nt_f_mc(0)   = 0.d0
	ext_p        = .false.
	k            = 0
	if ( xsym_p.or.lcyl_mc ) k = 1
	do j = 1, nz_mc
	  do i = 1, nr_mc
	    if (mask_mc(i,j).ne.0) then
	      t_electron_mc(1,i,j) = 0.
	    else
	      t_electron_mc(1,i,j) = te0
	    endif
	    if (uniform_s_p) then
	      if (mask_mc(i,j) .eq. 0) then
		ele_s_e_mc(i,j) = te0
		x0 = x1 * (zt_mc(j+1) - zt_mc(j))
		if (lcyl_mc) then
		  x0 = x0 * (rt_mc(i+1)**2 - rt_mc(i)**2)
		else
		  x0 = x0 * (rt_mc(i+1)    - rt_mc(i)   )
		endif
		ele_s_mc(i,j) = y0 * x0
	        ele_s_max_mc = max(ele_s_max_mc, ele_s_mc(i,j))
	        nt_f_mc(0)   = nt_f_mc(0) + ele_s_mc(i,j)
		e_density_mc(1,i,j) = ele_s_mc(i,j) / x0
	      else
		ele_s_mc(i,j) = 0.
	      endif
	    else
	      if (mask_mc(i,j) .eq. 0) then
	        ele_s_e_mc(i,j) = te0
	        ele_s_max_mc = max(ele_s_max_mc, ele_s_mc(i,j))
	        nt_f_mc(0)   = nt_f_mc(0) + ele_s_mc(i,j)
		x0 = x1 * (zt_mc(j+1) - zt_mc(j))
		if (lcyl_mc) then
		  x0 = x0 * (rt_mc(i+1)**2 - rt_mc(i)**2)
		else
		  x0 = x0 * (rt_mc(i+1)    - rt_mc(i)   )
		endif
		e_density_mc(1,i,j) = ele_s_mc(i,j) / x0
	      else
c		ele_s_mc(i,j) = 0.				!don't allow launches from solids...
	        ext_p = (ext_p .or. ele_s_mc(i,j).gt.0.0)
	      endif
	    endif
	  enddo
	enddo
	ntn_fm_mc(0) = ele_s_max_mc
 
	k            = 0
	if ( xsym_p.or.lcyl_mc ) k = 1
	do j = 0, nz_mc+1, nz_mc+1
	  do i = 1, nr_mc
	    ext_p = (ext_p .or. ele_s_mc(i,j).gt.0.0)
	  enddo
	enddo
	do j = 0, nz_mc+1
	  do i = k, nr_mc+1, nr_mc+1-k
	    ext_p = (ext_p .or. ele_s_mc(i,j).gt.0.0)
	  enddo
	enddo
 
	tot_ion_out_mc = 0.d0
 
	if (ext_p) then   !source from a surface....
	  nt_mc  = 1            !just have one surface group...
	  ntn_mc = 0
	  do j = 0, nz_mc+1
	    do i = 0, nr_mc+1
	      if (mask_mc(i,j).ne.0 .and. ele_s_mc(i,j).ne.0.0) then
		x0 = 0.d0
		k  = 0
		if (i.gt.0 .and. mask_mc(max(0,i-1),j) .eq. 0) then
		  k = k + 1
		  ntn_mc = ntn_mc + 1
		  ntd_mc(1,ntn_mc) = i
		  ntd_mc(2,ntn_mc) = -1
		  ntd_mc(3,ntn_mc) = j
		  ntd_mc(4,ntn_mc) = 0
		  ntd_mc(5,ntn_mc) = 0
		  ntd_mc(6,ntn_mc) = 1
		  if (lcyl_mc) then
		    y0 = 2.0*pi*rt_mc(i)*delta_z_mc
		  else
		    y0 = delta_z_mc
		  endif
		  ntd_mc(7,ntn_mc) = y0
		  ntn_f_mc(ntn_mc) = y0 * ele_s_mc(i,j)
		  ntn_fe_mc(ntn_mc) = te0 * 3. / 2.
		  x0 = x0 + y0
		endif
		if (i.le.nr_mc .and. 
     ^			mask_mc(min(nr_mc+1,i+1),j) .eq. 0) then
		  k = k + 1
		  ntn_mc = ntn_mc + 1
		  ntd_mc(1,ntn_mc) = i
		  ntd_mc(2,ntn_mc) = +1
		  ntd_mc(3,ntn_mc) = j
		  ntd_mc(4,ntn_mc) = 0
		  ntd_mc(5,ntn_mc) = 0
		  ntd_mc(6,ntn_mc) = 1
		  if (lcyl_mc) then
		    y0 = 2.0*pi*rt_mc(i+1)*delta_z_mc
		  else
		    y0 = delta_z_mc
		  endif
		  ntd_mc(7,ntn_mc) = y0
		  ntn_f_mc(ntn_mc) = y0 * ele_s_mc(i,j)
		  ntn_fe_mc(ntn_mc) = te0 * 3. / 2.
		  x0 = x0 + y0
		endif
		if (j.gt.0 .and. mask_mc(i,max(0,j-1)) .eq. 0) then
		  k = k + 1
		  ntn_mc = ntn_mc + 1
		  ntd_mc(1,ntn_mc) = i
		  ntd_mc(2,ntn_mc) = 0
		  ntd_mc(3,ntn_mc) = j
		  ntd_mc(4,ntn_mc) = -1
		  ntd_mc(5,ntn_mc) = 0
		  ntd_mc(6,ntn_mc) = 1
		  if (lcyl_mc) then
		    y0 = pi*(rt_mc(i+1)**2-rt_mc(i)**2)
		  else
		    y0 = delta_r_mc
		  endif
		  ntd_mc(7,ntn_mc) = y0
		  ntn_f_mc(ntn_mc) = y0 * ele_s_mc(i,j)
		  ntn_fe_mc(ntn_mc) = te0 * 3. / 2.
		  x0 = x0 + y0
		endif
		if (j.le.nz_mc .and. 
     ^		    mask_mc(i,min(nz_mc+1,j+1)) .eq. 0) then
		  k = k + 1
		  ntn_mc = ntn_mc + 1
		  ntd_mc(1,ntn_mc) = i
		  ntd_mc(2,ntn_mc) = 0
		  ntd_mc(3,ntn_mc) = j
		  ntd_mc(4,ntn_mc) = +1
		  ntd_mc(5,ntn_mc) = 0
		  ntd_mc(6,ntn_mc) = 1
		  if (lcyl_mc) then
		    y0 = pi*(rt_mc(i+1)**2-rt_mc(i)**2)
		  else
		    y0 = delta_r_mc
		  endif
		  ntd_mc(7,ntn_mc) = y0
		  ntn_f_mc(ntn_mc) = y0 * ele_s_mc(i,j)
		  ntn_fe_mc(ntn_mc) = te0 * 3. / 2.
		  x0 = x0 + y0
		endif
		if (k.gt.1) then !ugh
		  do n = ntn_mc-k+1, ntn_mc
		    ntn_f_mc(n) = ele_s_mc(i,j) * ntd_mc(6,n)/x0
		  enddo
		endif
	      endif
	    enddo
	  enddo
	  if (ntn_mc .eq. 0) then
	    nt_mc = 0
	  else
	    ntsn_mc(1,1) = 1
	    ntsn_mc(2,1) = ntn_mc
	  endif
	endif
 
	do i = 1, nt_mc
	  nt_f_mc(i) = 0.
	  ntn_fm_mc(i) = 0.
	  do j = ntsn_mc(1,i), ntsn_mc(2,i)
	    nt_f_mc(i) = nt_f_mc(i) + ntn_f_mc(j)
	    ntn_fm_mc(i) = max(ntn_fm_mc(i), ntn_f_mc(j))
	  enddo
	enddo
 
	x0 = 0.d0
	do i = 0, nt_mc
	  x0 = max( real(x0), nt_f_mc(i))
	enddo
	if (x0.le.0.d0) then
	  write(*,"('    ERROR- there isn''t any particles to ',
     ^			'launch!')")
	  write(*,"('    run aborting...')")
c	  pause
	  stop
	endif
 
	it1_max_mc = 1
	ph1_t_max_mc = 1.d0
	ph1_t_min_mc = 1.d0
	ph1_v_mc(0)  = 0.d0
	ph1_v_mc(1)  = 1.d0
 
	do j = 0, nz_mc
	  do i = 0, nr_mc
	    e_theta_mc(i, j) = (0.d0, 0.d0)
	    b_r_mc(    i, j) = (0.d0, 0.d0)
	    b_z_mc(    i, j) = (0.d0, 0.d0)
	  enddo
	enddo
 
	nqi_mc = nspec_mc
	do i = 1, nspec_mc
	  ic_mc(i) = i
	  raten_fl_mc(i) = 'mc-standalone gjp'
	enddo
 
	do j = 0, nz_mc+1
	  do i = 0, nr_mc+1
	    if (i.eq.0.or.mask_mc(i,j).ne.0) ele_s_mc(i,j) = 0.
	  enddo
	enddo
 
	return
 
 991	continue
	write(*,"('    ERROR occured while reading ',a)")
     ^		flnm(1:index(flnm,' ')-1)
	write(*,"('    run aborting...')")
	if (.true.) then
c	  pause
	  stop
	endif
 
	end
 
 
