 
c        Program BACCARAT_mc
	subroutine baccarat_mc
 
c       3-D Monte Carlo code for electron transport in weakly ionized
c       plasmas
 
c       June 1994
c       originally written by W.L. Morgan
c                             Kinema Research
c                             P.O. Box 1147
c                             Monument, CO  80132  USA
c                             Telephone: 719.488.3187
c                             Facsimile: 719.481.9140
c                             E-mail   : morgan@kinema.com
 
c       June 95 - Dec 95
c       expanded and revised and completely rewritten almost beyond recogniztion by
c                             G.J. Parker
c                             Lawerence Livermore National Lab
c                             P.O. Box 808, L-418
c                             Livermore, CA   94551
c                             510 422 9942
c                             510 422 2956 (FAX)
c                             parker9@llnl.gov
c
c       Dec 95 - Dec 97
c       expanded and revised and completely rewritten almost beyond recogniztion
c	several times by
c                             G.J. Parker
c                             parker9@cwws.net
c
c the following routines should be inlined if easily allowed for at
c compile/link time:
c
c     q_interp and RANF
 
c------------------------------------------------
 
c                                *** NOTES ***
 
c       most subroutines and functions have the suffix of
c       '_mc' to avoid collisions w/ fluid modules, eg ranf is ranf_mc
 
c       Units: cgs except that energies are in eV
c              and magnetic fields are in guass / 1e8 (we'll call them Morgans)
 
c       The coordinates are cartesian.
 
c       BACCARAT presently uses the include files:
 
c            mc1.inc & mc2.inc
 
c       These could probably be consolidated into fewer files but these
c       have been useful in developing the code.
 
c       Use is made of rejection technique (see Press, et al.)
c       for generating arbitrarily distributed random numbers; the
c       uniform random number generator used is taken from Press,
c       et al., "Numerical Recipes", 2nd ed.
 
c       BACCARAT currently deals with electron elastic, excitation,
c       ionization, and attachment collisions but does not include
c       electron-ion collisions or electron-electron collisions
c       (a tough problem).
c	if there are 'confined' electrons, approximation for energy
c	transfer via ee collisions is made if trapped_p = true
c
 
c       Input files:
c         unit  5 = input parameters; namelist file 'mc.in'
c         unit 15 = cross section data;
c                   1st it is the differential cross section file
c                   2nd it is the total cross section file;
c                   these file names are given by the user in the input
c                   parameter file; the total cross sections are in the
c                   format used by ELENDIF (sort of)
 
c       Output files:  the output files are named using an 8 character label,
c                      called 'Run_label', that is the 1st line of the input
c                      parameters in the input parameter file
c         unit  7 = 'run_label.7' ; this is the main output file of BACCARAT
c                                   and contains statistics on the run and
c                                   Te(r,z), where Te = 2 <energy> / 3
c         unit  8 = 'run_label.8' ; contains the log of the volume averaged
c                                   electron energy distribution function
 
c------------------------------------------------
 
      	implicit real*4 (a-h,o-z)
        real*8 num_e
 
c------------------------------------------------
 
c input run parameters, data from fluid (ie INDUCT) and cross sections.
      	call mc_input_mc
 
c ok, got all of that, now let's start running some electrons...
 
      	call experiment_mc
 
      	call statistics_mc(num_e)
 
      	call output_mc(num_e)
 
c      	stop
 
	return
      	end
 
 
 
c---------------------------------------------------------------------72
 
        subroutine experiment_mc
 
c called by BACCARAT.
c run individual electron trajectories until the total time
c followed is upto a user specified amount
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
        logical end_traj, good_traj !last is new
        logical anotherp
	real*8 t, ave_n, ave_t0, ave_n0
	real*4 max_ke
	logical firstp
	integer ip0
	character*8 outname
 
c	real*8 x,y,z,vx,vy,vz
 
	integer nstack_max
	integer instack,nstack
	parameter (instack=1000)
	real*4  v_stk(instack),z_stk(instack),t_stk(instack)
	real*4  r_stk(instack)
	logical stk_err
	common /stk_com/ v_stk,z_stk,t_stk,nstack,r_stk,stk_err
 
	firstp = .true.
	outname = 'electron'
	if (spec_t.ne.0) then
	  run_type_mc = 5
	  outname = 'particle'
	elseif (it1_max_mc.eq.1) then     !time slices? no...
	  if (lcyl_mc) then
	    run_type_mc = 2
	  else
	    run_type_mc = 1
	  endif
	else				  !time slices
	  if (lcyl_mc) then
	    run_type_mc = 4
	  else
	    run_type_mc = 3
	  endif
	  write(*,*) 'ERROR- time slices not implemented'
c	  pause
	  stop
	endif
 
	ave_unconf_t_mc   = 0.d0
	ave_n 		  = 0.d0
 
	nstack            = 0
	nstack_max        = 0
	stk_err           = .false.
 
        total_time        = 0.
        total_time_steps  = 0
        ave_time          = 0.
        ave_steps         = 0
        numgp             = 0
	max_ke            = 0.0
	i2                = 0
	i02               = 0
 
	do nreg0 = nt_mc, -1, -1
 
	  nreg = nreg0

	  ave_t0 = 0.d0
	  ave_n0 = 0.d0
 
          anotherp = (n_traj_region(nreg).gt.0)
          i        = 0
	  i0       = 0
 
	  if (anotherp
     ^		.and. nreg.eq.-1 .and. run_type_mc.ne.5) then
	    call trapped_dist_mc
	  endif
 
	  if (anotherp) then !only print out if we are doing region!
      	    open (unit=7, file=run_label(1:l_label)//'.7',	!HP
     ^			status='old',access='append')
c      	    open (unit=7, file=Run_label(1:L_label)//'.7',	!ABSOFT PC
c     ^			status='old',position='append')
c      	    open (unit=7, file=Run_label(1:L_label)//'.7')	!IBM
	    if ( nreg .gt. 0 ) then
              write(*,"(x,'Surface #',i2,' emission, running',i8,
     ^         ' ',a,'(s)... (w/ weight =',1p,e14.6,')')") nreg,
     ^		n_traj_region(nreg),outname,weight_reg(nreg)
              write(7,"(x,'Surface #',i2,' emission, running',i8,
     ^         ' ',a,'(s)... (w/ weight =',1p,e14.6,')')") nreg,
     ^		n_traj_region(nreg),outname,weight_reg(nreg)
	    elseif ( nreg .eq. 0 ) then
              write(*,"(x,'Volume produced, running',i8,
     ^         ' ',a,'(s)... (w/ weight =',1p,e14.6,')')")
     ^		n_traj_region(nreg),outname,weight_reg(nreg)
              write(7,"(x,'Volume produced, running',i8,
     ^         ' ',a,'(s)... (w/ weight =',1p,e14.6,')')")
     ^		n_traj_region(nreg),outname,weight_reg(nreg)
	    else
              write(*,"(x,'Confined, running',i8,
     ^         ' ',a,'(s)... (w/ weight =',1p,e14.6,')')")
     ^		n_traj_region(nreg),outname,weight_reg(nreg)
              write(7,"(x,'Confined, running',i8,
     ^         ' ',a,'(s)... (w/ weight =',1p,e14.6,')')")
     ^		n_traj_region(nreg),outname,weight_reg(nreg)
	    endif
            write(*,"(x,'[',a,' simulated',1p,e12.5,
     ^       ' (s) before recording; up to maximum of',e12.5,
     ^	     ' (s)]')") outname,t_min_region(nreg),t_max_region(nreg)
            write(7,"(x,'[',a,' simulated',1p,e12.5,
     ^       ' (s) before recording; up to maximum of',e12.5,
     ^	     ' (s)]')") outname,t_min_region(nreg),t_max_region(nreg)
	    close(7)
c	    ip0 = max(100, int(10**(int(
c     ^		log10(float(n_traj_region(nreg)))-0.5))))
	    ip0 = max(2, n_traj_region(nreg) / 20 )
	  endif
 
 
c do another electron?
          do while (anotherp)
 
c count the number of electrons (aka trajectories)
	    i=i+1
	    if (nstack.eq.0) then
	      i0 = i0 + 1
c	      write(*,*) n_traj_region(nreg),i0
	    endif
c	    write(*,*) i0,nstack
 
	    if (see_p_mc .and. nstack.gt.0) then
	      n_traj_region(nreg) = max( i0, n_traj_region(nreg)-1 )
	      if (n_traj_region(nreg).eq.i0+1) write(*,*) '    *'
	    endif
 
c printout once in awhile to let them know we are doing something...
            if (mod(i0,min(max(1,n_traj_region(nreg)-1),ip0))
     ^		    +nstack.eq.0)  then
      	      open (unit=7, file=run_label(1:l_label)//'.7',	!HP
     ^			status='old',access='append')
c      	      open (unit=7, file=Run_label(1:L_label)//'.7',	!ABSOFT PC
c     ^			status='old',position='append')
c      	      open (unit=7, file=Run_label(1:L_label)//'.7')	!IBM
              write(7,"('  ',a,' #',i7,', time of last:',1pe12.5,
     ^       ' (s), total time:',1pe12.5)") outname,i0,t,ave_time
	      close(7)
              write(*,"('  ',a,' #',i7,', time of last:',1pe12.5,
     ^       ' (s), total time:',1pe12.5)") outname,i0,t,ave_time
	    endif
 
c run this electron
 10         continue
	    if (run_type_mc.eq.1) then
	      call trajectory_cart_mc  (x, y, z, vx, vy, vz,
     ^	  	  t, nt, end_traj,good_traj,max_ke,nreg,firstp)
	    elseif (run_type_mc.eq.2) then
	      call trajectory_cyl_mc  (x, y, z, vx, vy, vz,
     ^	  	  t, nt, end_traj,good_traj,max_ke,nreg,firstp)
	    elseif (run_type_mc.eq.5) then
	      call trajectory_2d_i_cart_mc  (x, y, z, vx, vy, vz,
     ^	  	  t, nt, end_traj,good_traj,max_ke,nreg,firstp)
c	    elseif (run_type_mc.eq.3) then
c	      call TRAJECTORY_cart_t_mc  (x, y, z, vx, vy, vz,
c     ^	  	  t, nt, end_traj,good_traj,max_ke,nreg,firstp)
c	    elseif (run_type_mc.eq.4) then
c	      call TRAJECTORY_cyl_t_mc  (x, y, z, vx, vy, vz,
c     ^	  	  t, nt, end_traj,good_traj,max_ke,nreg,firstp)
	    else
	      write(*,*) '      ERROR- undefined run_type_mc:',
     ^		      run_type_mc
c	      pause
	      stop
	    endif
 
c if electron didn't go long enough, do it again...
	    if (.not.good_traj) then
	      numgp=numgp+1           !count the number of 'failed' attempts
	      go to 10
	    endif
 
c otherwise it did, update counters...
            ave_time  = ave_time + t
            ave_steps = ave_steps + nt
 
	    ave_t0 = ave_t0 + t
	    ave_n0 = ave_n0 + 1.d0
c another?
	    anotherp = (i0.lt.n_traj_region(nreg).or.nstack.ne.0)
	    nstack_max = max(nstack_max,nstack)
 
          enddo

	  if ( nreg .gt. -1 ) then
	    ave_unconf_t_mc = ave_unconf_t_mc + 
     ^			ave_t0 * weight_reg(nreg)
	    ave_n = ave_n + ave_n0 * weight_reg(nreg)
	  endif
 
c	  write(*,*) n_traj_region(nreg),i0
 
	  if (n_traj_region(nreg).ne.0) then !only print out if we are doing region!
            write(*,"(4x,'  done:',i9,' secondaries, sec/e:',f8.4,
     ^	      ', <t> =',1p,e12.5)") i-n_traj_region(nreg),
     ^	      real(i)/real(n_traj_region(nreg))-1.0,ave_t0/ave_n0
      	    open (unit=7, file=run_label(1:l_label)//'.7',	!HP
     ^			status='old',access='append')
c      	    open (unit=7, file=Run_label(1:L_label)//'.7',	!ABSOFT PC
c     ^			status='old',position='append')
c      	    open (unit=7, file=Run_label(1:L_label)//'.7')	!IBM
            write(7,"('  done:',i9,' secondaries, sec/e:',f8.4,
     ^	      ', <t> =',1p,e12.5)") i-n_traj_region(nreg),
     ^	      real(i)/real(n_traj_region(nreg))-1.0,ave_t0/ave_n0
	    close(7)
	  endif
 
	  i2  = i2  + i
	  i02 = i02 + i0
 
	enddo
 
c store number of electrons (trajectories)
c        N_traj = i
 
c tell them we are about done...
      	open (unit=7, file=run_label(1:l_label)//'.7',	!HP
     ^			status='old',access='append')
c      	open (unit=7, file=Run_label(1:L_label)//'.7',	!ABSOFT PC
c     ^			status='old',position='append')
c      	open (unit=7, file=Run_label(1:L_label)//'.7')	!IBM
 
	write(*,*)
	write(7,*)
 
        write(*,"(4x,'Number of invalid (',i8,') and valid (',
     ^	      i8,') trajectories.')") numgp,n_traj
        write(7,"('Number of invalid (',i8,') and valid (',
     ^	      i8,') trajectories.')") numgp,n_traj
 
	if ( ave_n .ne. 0.d0 )
     ^	      ave_unconf_t_mc = ave_unconf_t_mc / ave_n
 
	if (see_p_mc .or. secondary_p) then
	  write(*,"(4x,'total number of electrons: ',i9)") i2
	  write(*,"(4x,'maximum # on stack ',i3)") nstack_max
	  if (stk_err) write(*,*) ' ERROR- stack too small'
 
	  write(7,"('total number of electrons: ',i9)") i2
	  write(7,"('maximum # on stack ',i3)") nstack_max
	  if (stk_err) write(7,*) ' ERROR- stack too small'
	endif
 
	write(*,"(4x,'maximum kinetic energy ',f7.2,' (eV)')") max_ke
	write(7,"('maximum kinetic energy ',f7.2,' (eV)')") max_ke

	if (n_traj_region(-1) .eq. 0) then
	  write(*,"(4x,'average life-time ',1p,
     ^			e12.5,' (s)')") ave_unconf_t_mc
	  write(7,"(   'average life-time ',1p,
     ^			e12.5,' (s)')") ave_unconf_t_mc
	else
	  write(*,"(4x,'unconfined average life-time ',1p,
     ^			e12.5,' (s)')") ave_unconf_t_mc
	  write(7,"(   'unconfined average life-time ',1p,
     ^			e12.5,' (s)')") ave_unconf_t_mc
	endif
 
	if (see_p_mc) then
	  t = 0.d0
	  do j = 0, nz_mc+1
	    do i = 0, nr_mc+1
	      t = t + see_surfq_mc(i,j)
	    enddo
	  enddo
	  write(*,"(4x,'surface: out-in =',1p,e13.5)") t
	  write(7,"('surface: out-in =',1p,e13.5)") t
	endif
	write(7,*)
 
	close(7)
 
        return
        end
 
c---------------------------------------------------------------------72
 
        subroutine statistics_mc(num_e)
 
c       called by output_mc
c       performs normalizations of sampled quantities
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
	real*8 y,x
	logical zerop,allp
	integer itm
	real*4 vt(-it1_mc:2*it1_mc),vt_max
	real*4 nudt(it1_mc)
	real*8 wt,swt,aswt
	real*8 tot_tn(it1_mc,-1:ntmax_mc)
	real*4 tmp(nrmax_mc,nzmax_mc)
        real*8 num_e
 
 
	do itm=1,it1_max_mc-1
	  nudt(itm) = 0.5d0 * (ph1_v_mc(itm+1) - ph1_v_mc(itm-1))
	enddo
	nudt(it1_max_mc) =  0.5d0 * (ph1_v_mc(1) +
     ^		    ph1_v_mc(it1_max_mc) - ph1_v_mc(it1_max_mc-1))
 
c first find the 'global' EEDF
        y=0.d0
        do i=1, n_e
          e_energy(i) = delta_e * (i - 0.5)  !construct energy mesh
	enddo
 
	call int_rate_mc(num_e)
 
	y = 0.d0
	do i = n_e, 1, -1
	  y = y + f_of_e(i)
	enddo
 
c normalize so that integral of f sqrt(E) dE = 1
        do i=1,n_e
	  f_of_e(i) = f_of_e(i) / y / (sqrt(e_energy(i)) * delta_e)
        enddo
 
c find the global average electron energy
        e_bar       = e_bar / total_time
 
c find the globlal elastic and inelastic collision frequencies
        f_elastic   = n_elastic   / total_time
        f_inelastic = n_inelastic / total_time
 
c and find the local electron temperature... (~ (2/3) average energy)
	do nreg = -1, nt_mc
	  do itm = 1, it1_max_mc
	    swt = 0.d0
	    do j = 1, nz_mc
	      wt = 0.d0
	      do i = 1, nr_mc
		if (mask_mc(i,j).eq.0) wt = wt + nsteps_rz(itm,i,j,nreg)
	      enddo
	      swt = swt + wt
	    enddo
	    tot_tn(itm, nreg) = swt
	  enddo
	enddo
 
        do j=1, nz_mc
          do i=1, nr_mc
	    if (mask_mc(i,j).eq.0) then
	      x = 0.d0
	      zerop = .false.
	      vt_max = -1.e22
	      do itm = 1, it1_max_mc
		
		aswt = 0.d0
		x    = 0.d0
 
		do nreg = -1, nt_mc
		  e_bar_rz(itm,i,j,nreg) = e_bar_rz(itm,i,j,nreg) /
     ^		            (nsteps_rz(itm,i,j,nreg) + 1.d-302)
		  x = x + w_d(nreg) * n_traj_region(nreg) *
     ^		   nsteps_rz(itm,i,j,nreg)/(tot_tn(itm,nreg)+1.d-300)
		  aswt = aswt + e_bar_rz(itm,i,j,nreg) *
     ^		   w_d(nreg) * n_traj_region(nreg) *
     ^		   nsteps_rz(itm,i,j,nreg)/(tot_tn(itm,nreg)+1.d-300)
		enddo
                y = x
		if (x.ne.0.d0) y = 1.d0 / x
                if (aswt.gt. 0.d0) then
		  t_electron_mc(itm,i,j) = 0.666666667 *
     ^		      aswt*y
		else
		  t_electron_mc(itm,i,j) = 0.
		  zerop = .true.
	        endif
		vt(itm) = t_electron_mc(itm,i,j)
		vt_max  = max(vt_max,vt(itm))
	      enddo
	      if (zerop) then   !there is `a' time slice which is 0
		if (vt_max.eq.0.0) then !all are zero!
		  do itm = 1, it1_max_mc
c		    t_electron_mc(itm,i,j) = 0.6666667 * e_bar
		    t_electron_mc(itm,i,j) = 0.
		  enddo
		else
		  do itm = 0, - it1_max_mc+2, -1
		    vt(itm) = vt(it1_max_mc + itm)
		  enddo
		  do itm = it1_max_mc+1, 2*it1_max_mc - 1
		    vt(itm) = vt(itm - it1_max_mc)
		  enddo
		  do itm = 1, it1_max_mc
		    if (vt(itm) .eq. 0.0) then
		      do i1 = itm+1, itm+it1_max_mc-1
			if (vt(i1).ne.0.0) exit
		      enddo
		      do i2 = itm-1, itm-it1_max_mc+1, -1
			if (vt(i2).ne.0.0) exit
		      enddo
		      if (i1.eq.i2) then
			t_electron_mc(itm,i,j) = vt(i1)
		      else
			t_electron_mc(itm,i,j) = vt(i2) +
     ^			     (vt(i1)-vt(i2))/(i1-i2)*(itm-i2)
		      endif
		    endif
		  enddo
		endif
	      endif
	    else
	      do itm = 1, it1_max_mc
	        t_electron_mc(itm,i,j) = 0.
	      enddo
	    endif
          enddo
        enddo

c now fil in any spatial locations where all time slices are zero
c by searching for the closest non-zero cell...
	do itm = 1, it1_max_mc
	  zerop = .false.
	  allp  = .true.
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      if ( mask_mc(i,j) .eq. 0 ) then
	        zerop = (zerop .or. t_electron_mc(itm,i,j).eq.0.)
		allp  = (allp .and. t_electron_mc(itm,i,j).eq.0.)
	      endif
	    enddo
	  enddo

c every spatial location has Te=0! unlikely, but if so, set equal to average Te
	  if (allp) then
	    zerop = .false.
	    do j = 1, nz_mc
	      do i = 1, nr_mc
		if ( mask_mc(i,j) .eq. 0 ) 
     ^		  t_electron_mc(itm,i,j) = 0.6666667 * e_bar
	      enddo
	    enddo
	  endif

c keep looping until all plasma cells get something for Te
	  do while (zerop)
	    zerop = .false.
	    do j = 1, nz_mc
	      do i = 1, nr_mc
		tmp(i,j) = 0.
		if (mask_mc(i,j) .eq. 0 .and. 
     ^				t_electron_mc(itm,i,j) .eq. 0.) then
		  x=0.
		  if(t_electron_mc(itm,min(nr_mc,i+1),j).gt.0.)x=x+1.
		  if(t_electron_mc(itm,max(    1,i-1),j).gt.0.)x=x+1.
		  if(t_electron_mc(itm,i,min(nz_mc,j+1)).gt.0.)x=x+1.
		  if(t_electron_mc(itm,i,max(    1,j-1)).gt.0.)x=x+1.
		  if ( x .gt. 0.5 ) then
		    tmp(i,j) = 1. / x * (
     ^		         t_electron_mc(itm,min(nr_mc,i+1),j) +
     ^			 t_electron_mc(itm,max(    1,i-1),j) +
     ^			 t_electron_mc(itm,i,min(nz_mc,j+1)) +
     ^			 t_electron_mc(itm,i,max(    1,j-1)))
		  else
		    zerop = .true.
		  endif
		endif
	      enddo
	    enddo
	    do j = 1, nz_mc
	      do i = 1, nr_mc
		t_electron_mc(itm,i,j) = t_electron_mc(itm,i,j) +
     ^				tmp(i,j)
	      enddo
	    enddo
	  enddo
	enddo
 
        return
        end
 
c---------------------------------------------------------------------72
 
        subroutine output_mc(num_e)
 
c	called by BACCARAT
c       main output routine; the other is RATE_COEFFS
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
	real*4 xave(nrmax_mc)
	real*8 x,nudt(it1_mc),y,z,num_e
 
 
      	open (unit=7, file=run_label(1:l_label)//'.7',	!HP
     ^		status='old',access='append')
c      	open (unit=7, file=Run_label(1:L_label)//'.7',	!ABSOFT PC
c     ^		status='old',position='append')
c      	open (unit=7, file=Run_label(1:L_label)//'.7')	!IBM
 
        z = 0.d0
	write(7,"(' Region      # e',14x,'wall',4x,
     ^		'att+left',8x,'cold')")
	do j = -1, nt_mc
          write(7,"(3x,i2,5x,1p,e12.5,i12,i12,i12)") j,
     ^		w_d(j)*n_traj_region(j)*w_d_sc,
     ^		no_loss(1,j),no_loss(2,j),n_cold(j)
          z = z + w_d(j)*n_traj_region(j)*w_d_sc
	enddo
        write(7,"(' Total  ',1p,e12.5)") z
        write(7,"(' Fluid  ',1p,e12.5)") num_e
	write(7,*)
 
        write(7,"('<E> =',f7.3,' (eV), Te =',f7.3,' (eV)')")
     ^		  e_bar, 2.*e_bar/3.
        write(7,"('Number of elastic & inelastic collisions / traj = ',
     1            2f11.2)")
     2            dfloat(n_elastic)/n_traj, dfloat(n_inelastic)/n_traj
        write(7,*) 'elastic, inelastic & ionize coll freq : (1/s)'
        write(7,'(1p3e14.3)') f_elastic, f_inelastic, n_ion/total_time
        write(7,"('Max collision probability = ',1pe10.3)")
     ^	          1.d0-exp(-coll_prob_max)
        write(7,"('mfp / time step = ', 1pe10.3,' (cm)')")
     x            (mfp_per_step)/float(total_time_steps)
        write(7,"('average dt = ', 1pe10.3,' (s)')")
     x            (total_time)/float(total_time_steps)
        write(7,"('Number of collisions with r=R_max / traj = ',
     x           f8.3)") dfloat(n_coll_r)/n_traj
        write(7,"('Number of collisions with z=Z_min / traj = ',
     x           f8.3)") dfloat(n_coll_zmin)/n_traj
        write(7,"('Number of collisions with z=Z_max / traj = ',
     x           f8.3)") dfloat(n_coll_zmax)/n_traj
        write(7,"('# electrons lost over sheath = ', i6)") n_e_lost
        write(7,"('Ave length of trajectory:')")
        write(7,"(i9,' steps',1pe14.3,' sec')")
     x      ave_steps / n_traj, ave_time / n_traj
 
        write(7,*) 'Total number of collisions:'
        write(7,"('     elastic', i8)") n_elastic
        write(7,"('   inelastic', i8)") n_inelastic
        write(7,"('  attachment', i8)") n_attach
        write(7,"('  ionization', i8)") n_ion
 
        call rate_coeffs_mc
 
	write(7,*)
        write(7,"('Number of ionizations / electron:')") 
	write(7,"(' Region      ionization/electron ')")
	do nreg = -1, nt_mc
	  if (n_traj_region(nreg) .gt. 0) then
            x = 0.
            y = 0.
	    do i1 = 1, nq
	      if (iq_flag(i1).eq.2) then
                y = y + n_collisions_region(i1,nreg)
	        do j = 1, nz_mc
	          do i = 1, nr_mc
		    if (mask_mc(i,j) .eq. 0) then
		      do itm = 1, it1_max_mc
		        x = x + gps_mc(itm,i,j,i1,nreg)
		      enddo
                    endif
		  enddo
		enddo
	      endif
	    enddo
            write(7,"(3x,i2,9x,1p,e12.6,' (',e12.6,')')") nreg,
     ^		y / float(n_traj_region(nreg)), x
	  endif
	enddo

	do itm=1,it1_max_mc-1
	  nudt(itm) = 0.5d0 * (ph1_v_mc(itm+1) - ph1_v_mc(itm-1))
	enddo
	nudt(it1_max_mc) =  0.5d0 * (ph1_v_mc(1) +
     ^		    ph1_v_mc(it1_max_mc) - ph1_v_mc(it1_max_mc-1))
 
        write(7,*)
        write(7,*) 'Spatial Distribution <E>(r,z): (eV)'
	if (nr_mc .ne. 1) then
          do j = nz_mc, 1, -1
	    write(7,"('iz =',i3,' z ~',1p,e10.3,' (cm)')") j,zcm_mc(j)
            write(7,10) (t_electron_mc(1,i,j)*1.5,i=1,nr_mc)
          enddo
	else
	  do j = nz_mc, 1, -1
	    write(7,"('iz =',i3,' z ~',1p,e10.3,'  ',e12.5)")
     ^		j, zcm_mc(j), t_electron_mc(1,1,j)*1.5
	  enddo
	endif
   10   format ((1p10e10.3))
 
c       the following is sometimes useful as a diagnostic:
        write(7,*)
        write(7,*) 'Spatial Distribution N(r,z): (arb/cc)'
	y = 0.d0
	do j = 1, nz_mc
	  do i = 1, nr_mc
	    if (mask_mc(i,j) .eq. 0) then
	      x = gpne_mc(1,i,j) * nudt(1)
	      do itm = 2, it1_max_mc
		x = x + gpne_mc(itm,i,j) * nudt(itm)
	      enddo
	      y = max(y, x / ph1_t_max_mc / vol_cell(i,j))
	    endif
	  enddo
	enddo
        z = y
	if (y.ne.0.d0) z = 1.d0 / y
	if (nr_mc .ne. 1) then
          do j = nz_mc, 1, -1
	    write(7,"('iz =',i3,' z ~',1p,e10.3,' (cm)')") j,zcm_mc(j)
	    do i = 1, nr_mc
	      x = gpne_mc(it1_max_mc,i,j) * nudt(it1_max_mc)
	      do itm = 1, it1_max_mc-1
	        x = x + gpne_mc(itm,i,j) * nudt(itm)
	      enddo
	      xave(i) = x / ph1_t_max_mc
	    enddo
 
            write(7,10) (z*xave(i)/vol_cell(i,j),i=1,nr_mc)
          enddo
	else
	  do j = nz_mc, 1, -1
	    x = gpne_mc(it1_max_mc,1,j) * nudt(it1_max_mc)
	    do itm = 1, it1_max_mc-1
	      x = x + gpne_mc(itm,1,j) * nudt(itm)
	    enddo
	    x = z * x / ph1_t_max_mc / vol_cell(1,j)
	    write(7,"('iz =',i3,' z ~',1p,e10.3,'  ',e12.5)")
     ^		j, zcm_mc(j), x
	  enddo
	endif
   20   format ((1p5e12.3))
 
        write(7,*)
c        write(7,*) 'Ave Distribution Function: (1/eV)'
c        write(7,30) (e_energy(i),
c     ^	    f_of_e(i)*sqrt(e_energy(i)), i=1, N_e)
        write(7,*) 'Ave Distribution Function: (1/eV**3/2)'
        write(7,30) (e_energy(i), f_of_e(i), i=1, n_e)
   30   format (2(1p2e14.3))
 
c        do i=1, N_e
c          write(7,30) e_energy(i), log(f_of_e(i)+1.e-37)
c        enddo
 
      	close(7)
 
	if (spec_t.ne.0) then
c	  pause
	  stop
 	endif
 	
c	if (.false.) then
	do nreg = 0, nt_mc
	  if (n_traj_region(nreg) .gt. 0) then
	    do j = 1, nz_mc
	      do i = 1, nr_mc
		if (mask_mc(i,j) .eq. 0) then
		  do i1 = 1, nq
		    do itm = 1, it1_max_mc
		      gps_mc(    itm,i,j,i1,nreg) =
     ^			  gps_mc(itm,i,j,i1,nreg)/
     ^			  n_density_mc(itm,i,j,ic0(i1))
		    enddo
		  enddo
		endif
	      enddo
	    enddo
	  endif
	enddo
c	endif
 
	if (.false.) then
c	if (r_max_mc / z_max_mc .gt. 1.e3 ) then
	  do nreg = -2, nt_mc
	    if (n_traj_region(nreg) .gt. 0) then
	      do j = 1, nz_mc
		do i1 = 1, nq
		  do itm = 1, it1_max_mc
		    x = 0.d0
		    y = 0.d0
		    do i = 1, nr_mc
		      if (mask_mc(i,j) .eq. 0 .and.
     ^                   gps_mc(itm,i,j,i1,nreg) .ne. 0.) then
			x = x + gps_mc(itm,i,j,i1,nreg)
			y = y + 1.d0
		      endif
		    enddo
		    if ( y.ne.0.d0 ) y = x / y
		    do i = 1, nr_mc
		      if (mask_mc(i,j) .eq. 0)
     ^                	gps_mc(itm,i,j,i1,nreg) = y
		    enddo
		  enddo
		enddo
	      enddo
	    endif
	  enddo
	  do j = 1, nz_mc
	    do itm = 1, it1_max_mc
		x = 0.d0
		y = 0.d0
		do i = 1, nr_mc
		  if (mask_mc(i,j) .eq. 0 .and.
     ^                t_electron_mc(itm,i,j) .ne. 0.) then
		    x = x + t_electron_mc(itm,i,j)
		    y = y + 1.d0
		  endif
		enddo
		if ( y.ne.0.d0 ) y = x / y
		do i = 1, nr_mc
		  if (mask_mc(i,j) .eq. 0)
     ^               t_electron_mc(itm,i,j) = y
		enddo
	    enddo
	  enddo
	endif
 
        return
        end
 
c---------------------------------------------------------------------72
 
        subroutine rate_coeffs_mc
 
c called by output_mc
c       calculate rate coefficients (cm^3/s) for various electron
c       impact processes
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
        real*4 ne0(nrmax_mc,nzmax_mc)
        dimension power_loss(nq_max_mc)
	real*4 nsteps(nrmax_mc,nzmax_mc)
	real*8 x,y,x0,y0
	real*8 nudt(it1_mc),xxt(-1:ntmax_mc)
	real*4 xx(nrmax_mc,nzmax_mc)
	real*4 rate_gp(nq_max_mc),power_logp(nq_max_mc),v_gp(3)
	character*1 dim2
 
 
	do itm=1,it1_max_mc-1
	  nudt(itm) = 0.5d0 * (ph1_v_mc(itm+1) - ph1_v_mc(itm-1))
	enddo
	nudt(it1_max_mc) =  0.5d0 * (ph1_v_mc(1) +
     ^		    ph1_v_mc(it1_max_mc) - ph1_v_mc(it1_max_mc-1))
 
c rates via integrating eedf...
	do i1 = 1, nq
	  x = 0.d0
	  y = 0.d0
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      if (mask_mc(i,j) .eq. 0) then
c		x0 = gpne_mc(1,i,j) * nudt(1)
c		y0 = gps_mc(1,i,j,i1,-2) * nudt(1)
c		do itm = 2, it1_max_mc
c		  x0 = x0 + gpne_mc(itm,i,j) * nudt(itm)
c		  y0 = y0 + gps_mc(itm,i,j,i1,-2) * nudt(itm)
c		enddo
c		x0 = x0 / ph1_t_max_mc
c		y0 = y0 / ph1_t_max_mc
		
		y0 = gpne_mc(1,i,j) * gps_mc(1,i,j,i1,-2) * nudt(1)
		x0 = gpne_mc(1,i,j) * nudt(1)
		do itm = 2, it1_max_mc
		  y0 = y0 + gpne_mc(itm,i,j) *
     ^			gps_mc(itm,i,j,i1,-2) * nudt(itm)
		  x0 = x0 + gpne_mc(itm,i,j) * nudt(itm)
		enddo
		x0 = x0 / ph1_t_max_mc   ! ~ average electron number
		if (x0 .ne. 0.d0) y0 = y0 / ph1_t_max_mc / x0  !average <v sig>
 
		x  = x + x0
		y  = y + y0*x0
	      endif
	    enddo
	  enddo
          rate_gp(i1) = 0.
	  if (x.ne.0.d0) rate_gp(i1) = y / x
	enddo
 
	if (nr_mc.eq.nz_mc .and. nr_mc.eq.1) then
	  x = 0.d0
	  do i = 1, nspec_mc
	    x = x + n_density_mc(1,1,1,i)
	  enddo
	  write(7,"(' E/N   =',1p,e13.5,' (Td)')") abs(ef0d/x*1d17)
	endif
 
c average components of V
	do i1 = 1, 3
	  x = 0.d0
	  y = 0.d0
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      if (mask_mc(i,j) .eq. 0) then
		x0 = gpne_mc(1,i,j) * nudt(1)
		do itm = 2, it1_max_mc
		  x0 = x0 + gpne_mc(itm,i,j) * nudt(itm)
		enddo
		x0 = x0 / ph1_t_max_mc
		x  = x + x0
		y = y + v_ave(i1,i,j,-1)*x0
	      endif
	    enddo
	  enddo
	  v_gp(i1) = 0.
	  if (x.ne.0.d0) v_gp(i1) = y / x
	enddo
 
	if (lcyl_mc) then
	  write(7,"(' <Vr>  =',1p,e13.5,' (cm/s)')") v_gp(1)
	  write(7,"(' <Vth> =',1p,e13.5,' (cm/s)')") v_gp(2)
	else
	  write(7,"(' <Vx>  =',1p,e13.5,' (cm/s)')") v_gp(1)
	  write(7,"(' <Vy>  =',1p,e13.5,' (cm/s)')") v_gp(2)
	endif
	write(7,"(' <Vz>  =',1p,e13.5,' (cm/s)')") v_gp(3)
 
	do j = 1, nz_mc
	  do i = 1, nr_mc
	    x = ne(it1_max_mc,i,j) * nudt(it1_max_mc)
	    y = nsteps_rz(it1_max_mc,i,j,0) +
     ^		nsteps_rz(it1_max_mc,i,j,1) +
     ^		nsteps_rz(it1_max_mc,i,j,2)
	    do itm = 1, it1_max_mc-1
	      x = x +  ne(itm,i,j) * nudt(itm)
	      y = y + nsteps_rz(itm,i,j,0) +
     ^		      nsteps_rz(itm,i,j,1) +
     ^		      nsteps_rz(itm,i,j,2)
	    enddo
	    ne0(i,j) = x / ph1_t_max_mc
	    nsteps(i,j) = y
	  enddo
	enddo
 
        write(7,"(' power (fields) =         ',
     x           1p,e12.5,' [',e12.5,'] (W)')") pow_tt,pow_tr
        write(7,"(' power (fields) / Ne =    ',
     x           1p,e12.5,' (W)')") pow_tt/pow_tn
        write(7,"(' power (collision) =      ',
     x           1p,e12.5,' [',e12.5,'] (W)')") pow_ct,pow_cr
        write(7,"(' power (collision) / Ne = ',
     x           1p,e12.5,' (W)')") pow_ct/pow_tn
        write(7,"(' power (inductive) =      ',
     x           1p,e12.5,' [',e12.5,'] (W)')") pow_it,pow_ir
        write(7,"(' power (inductive) / Ne = ',
     x           1p,e12.5,' (W)')") pow_it/pow_tn
        write(7,"(' power (external) =       ',
     x           1p,e12.5,' [',e12.5,'] (W)')") pow_xt,pow_xr
        write(7,"(' power (external) / Fe =  ',
     x           1p,e12.5,' (W)')") pow_xt/pow_xn
 
 
c       calculate the power loss for each process and the total:
c       units are eV/sec
 
	  do nreg = nt_mc, -1, -1
	    xxt(nreg) = 0.d0
	    if ( n_traj_region(nreg) .gt. 0 ) then
	      do j = 1, nz_mc
	        do i = 1, nr_mc
		  if ( mask_mc(i,j) .eq. 0 ) then
		    x = nsteps_rz(1,i,j,nreg)
		    do itm = 2, it1_max_mc
		      x = x + nsteps_rz(itm,i,j,nreg)
		    enddo
		    xxt(nreg) = xxt(nreg) + x
	  	  endif
	        enddo
	      enddo
	    endif
	  enddo
	  x=0.d0
	  do nreg = -1, nt_mc
	    if ( n_traj_region(nreg) .gt. 0 ) x = x + weight_reg(nreg)
	  enddo
          rk = x
	  if (x.ne.0.d0) rk = 1.d0 / x
	  do nreg = -1, nt_mc
	    if ( xxt(nreg) .gt. 0.d0) xxt(nreg) = rk / xxt(nreg)
	  enddo
 
        tot_power_logp = 0.
        tot_power_loss = 0.
        do i=1, nq
          rk = n_collisions(i)/(nave(ic0(i))*total_time)
	  rk = 0.d0
	  do nreg = -1, nt_mc
	    rk = rk + weight_reg(nreg) *
     ^		n_collisions_region(i,nreg)*xxt(nreg)
	  enddo
	  rk = rk / nave(ic0(i))
          if (iq_flag(i) .eq. 0) then                                   ! elastic
            rmass           = 2. / (1. + 1822.35 * a_mass(i))
            power_loss(i)   = e_bar * rmass * rk
     x                     * nave(ic0(i))
            power_logp(i)   = e_bar * rmass * rate_gp(i)
     x                     * nave(ic0(i))
          else                                                 ! inelastic
            power_loss(i)   = energy_loss(i) * rk
     x                    *  nave(ic0(i))
            power_logp(i)   = energy_loss(i) * rate_gp(i)
     x                    *  nave(ic0(i))
            if (iq_flag(i) .eq. -1) then                       ! attachment
              power_loss(i) = e_bar*rk*nave(ic0(i))
              power_logp(i) = e_bar*nave(ic0(i))*rate_gp(i)
            endif
          endif
          tot_power_loss    = tot_power_loss + power_loss(i)
          tot_power_logp    = tot_power_logp + power_logp(i)
        enddo
 
c        write(7,"('Collision loss =',e12.5,' (W)')")
c     ^	  tot_power_loss * x / N_traj * 1.6021892d-19
 
        do i=1, nq
	  nc = ic0(i)
          write(7,*)
          write(7,'(i3,5x,a)') i, coll_p_name(i)
c         write(7,"('nc = ', i3, '  l = ', i3)") nc, l
          if (iq_flag(i) .eq. 0) then
            write(7,*) 'ELASTIC COLLISION'
            rmass = 2. / (1. + 1822.35 * a_mass(i))
            write(7,"('Energy loss per collision = ',1pe10.3,' (eV)')")
     x          rmass * e_bar
          else
            write(7,*) raten_in(i)
            write(7,"('Energy loss per collision = ',1pe10.3,' (eV)')")
     x          energy_loss(i)
          endif
          write(7,"('# collisions = ', i8)") n_collisions(i)
 
          rk = n_collisions(i)/(nave(ic0(i))*total_time)
	  rk = 0.d0
	  do nreg = -1, nt_mc
	    rk = rk + weight_reg(nreg) *
     ^		n_collisions_region(i,nreg)*xxt(nreg)
	  enddo
	  rk = rk / nave(ic0(i))
          write(7,"('ave rate coeff = ', 1p,e12.3,
     ^	    ' <',e11.5,'> (cc/s)')") rk,rate_gp(i)
          if (int(n_collisions(i)) .gt. 0.) then
            call error_bars_mc (int(n_collisions(i)), c_min, c_max)
            write(7,*) 'Error bars: k_min & k_max = (cc/s)'
            write(7,'(1pe14.3,1pe14.3)') rk * c_min / n_collisions(i),
     x                                  rk * c_max / n_collisions(i)
          endif
          write(7,"('Energy loss rate (eV/sec) & % of total =')")
          write(7,"(' ',1p2e14.3,' ')") power_loss(i),
     x                         100. * power_loss(i) / tot_power_loss
          write(7,"('<',1p2e14.3,'>')") power_logp(i),
     x                         100. * power_logp(i) / tot_power_logp
 
        enddo
 
c output data
 
	if (dumpfile_p) then
	  dim2 = 'x'
	  if (lcyl_mc) dim2='r'
	  
c reaction rate (cm^3/s)
c	  open(25,file='v_sigma.k',form='unformatted',status='unknown')
c	  write(25) ((((real(gps_mc(itm,j,k,i,-2)),itm=1,
c     ^		it1_max_mc),j=1,nr_mc),k=1,nz_mc),i=1,nq)
	  open(25,file='v_sigma.k',status='unknown')
	  do i = 1, nq
	    do k = nz_mc, 1, -1
	      write(25,"(1p,300e10.3)") ((real(gps_mc(itm,j,k,i,-2)),
     ^		itm=1,it1_max_mc),j=1,nr_mc)
	    enddo
	  enddo
	  write(25,*)
	  do i = 1, nq
	    write(25,"(' collision #',i3,': ',a)") i,raten_in(i)
	  enddo
	  write(25,*)
	  write(25,"(' <v sigma_k (t,',a,',z) >: t,',a,
     ^		',z then k [cm^3/s]')") dim2,dim2
	  write(25,"(2x,'do k = 1 to ',i3)") nq
	  write(25,"(4x,'do z = ',i3,' to 1')") nz_mc
	  write(25,"(6x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
	  write(25,"(8x,'do t = 1 to ',i3)") it1_max_mc
	  close(25)
 
c densities (1/cm^3)
c	  open(25,file='den.k',form='unformatted',status='unknown')
c	  write(25) ((((real(n_density_mc(itm,j,k,i)),itm=1,
c     ^		it1_max_mc),j=1,nr_mc),k=1,nz_mc),i=1,nspec_mc)
	  open(25,file='den.k',status='unknown')
	  do i = 1, nspec_mc
	    do k = nz_mc, 1, -1
	      write(25,"(1p,300e10.3)") ((
     ^	        real(n_density_mc(itm,j,k,i)),
     ^		itm=1,it1_max_mc),j=1,nr_mc)
	    enddo
	  enddo
	  write(25,*)
	  do i = 1, nspec_mc
	    write(25,"(' species #',i3,': ',a)") i,specn_mc(i)
	  enddo
	  write(25,*)
	  write(25,"(' density (t,',a,',z, spec): t,',a,
     ^		',z then k [cm^3/s]')") dim2,dim2
	  write(25,"(2x,'do k = 1 to ',i3)") nspec_mc
	  write(25,"(4x,'do z = ',i3,' to 1')") nz_mc
	  write(25,"(6x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
	  write(25,"(8x,'do t = 1 to ',i3)") it1_max_mc
	  close(25)
 
c total time in each cell ~ number in each cell
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      xx(i,j) = gpne_mc(1,i,j)
	      do itm = 2, it1_max_mc
		xx(i,j) = xx(i,j) + gpne_mc(itm,i,j)
	      enddo
	    enddo
	  enddo
c	  open(25,file='gpt.k',form='unformatted',status='unknown')
c	  write(25) ((xx(i,j),i=1,nr_mc),j=1,nz_mc)
	  open(25,file='gpt.k',status='unknown')
	  do j = nz_mc, 1, -1
	   write(25,"(1p,300e10.3)") (xx(i,j)/vol_cell(i,j),i=1,nr_mc)
	  enddo
	  write(25,*)
	  write(25,"(' < n_e(',a,',z) >: ',a,' then z [#]')")
     ^		dim2,dim2
	  write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
	  write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
	  close(25)
 
c total time in each cell ~ number in each cell
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      xx(i,j) = 0.
	    enddo
	  enddo
	  do nreg = nt_mc, -1, -1
	    do j = 1, nz_mc
	      do i = 1, nr_mc
		x = nsteps_rz(1,i,j,nreg)
		do itm = 2, it1_max_mc
		  x = x + nsteps_rz(itm,i,j,nreg)
		enddo
		xx(i,j) = xx(i,j) + x * w_d(nreg) !*weight_reg(nreg)
	      enddo
	    enddo
	  enddo
c	  open(25,file='tt.k',form='unformatted',status='unknown')
c	  write(25) ((xx(i,j),i=1,nr_mc),j=1,nz_mc)
	  open(25,file='tt.k',status='unknown')
	  do j = nz_mc, 1, -1
	    write(25,"(1p,300e10.3)") (xx(i,j),i=1,nr_mc)
	  enddo
	  write(25,*)
	  write(25,"(' < time(',a,',z) >: ',a,' then z [s]')")
     ^		dim2,dim2
	  write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
	  write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
	  close(25)
 
c field power
c	  open(25,file='pow_f.k',form='unformatted',status='unknown')
c	  write(25) ((real(pow_t(i,j,-2)),i=1,nr_mc),j=1,nz_mc)
c	  open(25,file='pow_f.k',status='unknown')
c	  do j = nz_mc, 1, -1
c	    write(25,"(1p,300e11.3)") (pow_t(i,j,-2)/pow_tn,i=1,nr_mc)
c	  enddo
c	  write(25,*)
c	  write(25,"(' < power (fields) / N_e(',a,
c     ^		',z) >: ',a,' then z [s]')") dim2,dim2
c	  write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
c	  write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
c	  close(25)
 
c inductive power
c	  open(25,file='pow_i.k',form='unformatted',status='unknown')
c	  write(25) ((real(pow_i(i,j,-2)),i=1,nr_mc),j=1,nz_mc)
c	  open(25,file='pow_i.k',status='unknown')
c	  do j = nz_mc, 1, -1
c	    write(25,"(1p,300e11.3)") (pow_i(i,j,-2)/pow_tn,i=1,nr_mc)
c	  enddo
c	  write(25,*)
c	  write(25,"(' < power (inductive) / N_e(',a,
c     ^		',z) >: ',a,' then z [s]')") dim2,dim2
c	  write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
c	  write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
c	  close(25)
 
c external power
c	  open(25,file='pow_x.k',form='unformatted',status='unknown')
c	  write(25) ((real(pow_x(i,j,-2)),i=1,nr_mc),j=1,nz_mc)
c	  open(25,file='pow_x.k',status='unknown')
c	  do j = nz_mc, 1, -1
c	    write(25,"(1p,300e11.3)") (pow_x(i,j,-2)/pow_xn,i=1,nr_mc)
c	  enddo
c	  write(25,*)
c	  write(25,"(' < power (external) / F_e (',a,
c     ^		',z) >: ',a,' then z [s]')") dim2,dim2
c	  write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
c	  write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
c	  close(25)
 
c collision power
c	  open(25,file='pow_c.k',form='unformatted',status='unknown')
c	  write(25) ((real(pow_c(i,j,-2)),i=1,nr_mc),j=1,nz_mc)
c	  open(25,file='pow_c.k',status='unknown')
c	  do j = nz_mc, 1, -1
c	    write(25,"(1p,300e11.3)") (pow_c(i,j,-2)/pow_tn,i=1,nr_mc)
c	  enddo
c	  write(25,*)
c	  write(25,"(' < power (collisions) / N_e(',a,
c     ^		',z) >: ',a,' then z [s]')") dim2,dim2
c	  write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
c	  write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
c	  close(25)
 
c average Vs...
	  if (run_type_mc.eq.3 .or. run_type_mc.eq.2) then
c	    open(25,file='vr.k',form='unformatted',status='unknown')
c	    write(25) ((v_ave(1,i,j,-1),i=1,nr_mc),j=1,nz_mc)
	    open(25,file='vr.k',status='unknown')
	    do j = nz_mc,1,-1
	      write(25,"(1p,300e11.3)")
     ^		(v_ave(1,i,j,-1),i=1,nr_mc)
	    enddo
	    write(25,*)
	    write(25,"(' < vr(r,z) >: r then z [cm/s]')")
	    write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
	    write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
	    close(25)
c	    open(25,file='vth.k',form='unformatted',status='unknown')
c	    write(25) ((v_ave(2,i,j,-1),i=1,nr_mc),j=1,nz_mc)
	    open(25,file='vth.k',status='unknown')
	    do j = nz_mc, 1, -1
	      write(25,"(1p,300e11.3)")
     ^		(v_ave(2,i,j,-1),i=1,nr_mc)
	    enddo
	    write(25,*)
	    write(25,"(' < vth(r,z) >: r then z [cm/s]')")
	    write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
	    write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
	    close(25)
	  else
c	    open(25,file='vx.k',form='unformatted',status='unknown')
c	    write(25) ((v_ave(1,i,j,-1),i=1,nr_mc),j=1,nz_mc)
	    open(25,file='vx.k',status='unknown')
	    do j = nz_mc,1,-1
	      write(25,"(1p,300e11.3)")
     ^		(v_ave(1,i,j,-1),i=1,nr_mc)
	    enddo
	    write(25,*)
	    write(25,"(' < vx(x,z) >: x then z [cm/s]')")
	    write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
	    write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
	    close(25)
c	    open(25,file='vy.k',form='unformatted',status='unknown')
c	    write(25) ((v_ave(2,i,j,-1),i=1,nr_mc),j=1,nz_mc)
	    open(25,file='vy.k',status='unknown')
	    do j = nz_mc, 1, -1
	      write(25,"(1p,300e11.3)")
     ^		(v_ave(2,i,j,-1),i=1,nr_mc)
	    enddo
	    write(25,*)
	    write(25,"(' < vy(x,z) >: x then z [cm/s]')")
	    write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
	    write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
	    close(25)
	  endif
 
c	  open(25,file='vz.k',form='unformatted',status='unknown')
c	  write(25) ((v_ave(3,i,j,-1),i=1,nr_mc),j=1,nz_mc)
	  open(25,file='vz.k',status='unknown')
	  do j = nz_mc, 1, -1
	    write(25,"(1p,300e11.3)")
     ^		(v_ave(3,i,j,-1),i=1,nr_mc)
	  enddo
	  write(25,*)
	  write(25,"(' < vz(',a,',z) >: ',a,' then z [cm/s]')")
     ^		dim2,dim2
	  write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
	  write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
	  close(25)
 
	  open(25,file='ingold.k',status='unknown')
	  do k = 1,8+nq
	    if (    k .eq. 1) then
	      write(25,"(7x,'D =',1p,e19.12,' [cm^2/s]')")
     ^			ing_coeff(k)
	    elseif (k .eq. 2) then
	      write(25,"(6x,'mu =',1p,e19.12,' [cm^2/V/s]')")
     ^			ing_coeff(k)
	    elseif (k .eq. 7) then
	      dim2 = dim2
c	      write(25,"(5x,'mu* =',1p,e19.12,' [cm^2/V/s]')")
c     ^			ing_coeff(k)
	    elseif (k .eq. 3) then
	      write(25,"(7x,'G =',1p,e19.12,' [cm^2 V/s]')")
     ^			ing_coeff(k)
	    elseif (k .eq. 4) then
	      write(25,"(7x,'B =',1p,e19.12,' [cm^2/s]')")
     ^			ing_coeff(k)
	    elseif (k .eq. 8) then
	      dim2 = dim2
c	      write(25,"(6x,'B* =',1p,e19.12,' [cm^2/s]')")
c     ^			ing_coeff(k)
	    elseif (k .eq. 5) then
	      write(25,"(5x,'<e> =',1p,e19.12,' [eV]')")
     ^			ing_coeff(k)
	    elseif (k .eq. 6) then
	      write(25,"('<e> nu_e =',1p,e19.12,' [eV/s]')")
     ^			ing_coeff(k)
	    else
	      write(25,"('coll #',i2,' =',1p,e19.12,' [1/s]')")
     ^			k-8,ing_coeff(k)
	    endif
	  enddo
	  close(25)
 
c sheath potential
c	  open(25,file='shp.k',form='unformatted',status='unknown')
c	  write(25) ((real(sheath_potential_mc(i,j)),
c     ^		i=0,nr_mc+1),j=0,nz_mc+1)
	  open(25,file='shp.k',status='unknown')
	  do j = nz_mc+1, 0, -1
	    write(25,"(1p,300e11.3)")(real(sheath_potential_mc(i,j)),
     ^		i=0,nr_mc+1)
	  enddo
	  write(25,*)
	  write(25,"(' sheath potential(',a,',z): ',a,' then z [V]')")
     ^		dim2,dim2
	  write(25,"(2x,'do z = ',i3,' to 0')") nz_mc+1
	  write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc+1
	  close(25)
 
c SEE distribution
	  if (see_p_mc) then
	    open(25,file='see.k',status='unknown')
	    do j = nz_mc+1, 0, -1
	      write(25,"(1p,300e11.3)")(real(see_surfq_mc(i,j)),
     ^		i=0,nr_mc+1)
	    enddo
	    write(25,*)
	    write(25,"(' SEE/initial (',a,',z): ',a,' then z [#]')")
     ^		dim2,dim2
	    write(25,"(2x,'do z = ',i3,' to 0')") nz_mc+1
	    write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc+1
	    close(25)
	  endif
 
c loss distribution
c	  open(25,file='loss.k',form='unformatted',status='unknown')
c	  write(25) ((real(n_loss(i,j)),i=0,nr_mc+1),j=0,nz_mc+1)
	  open(25,file='loss.k',status='unknown')
	  do j = nz_mc+1, 0, -1
	    write(25,"(1p,300e10.3)")
     ^		(real(n_loss(i,j)),i=0,nr_mc+1)
	  enddo
	  write(25,*)
	  write(25,"(' loss (',a,',z): ',a,' then z [#]')")
     ^		dim2,dim2
	  write(25,"(2x,'do z = ',i3,' to 0')") nz_mc+1
	  write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc+1
	  close(25)
 
c gain distribution
c	  open(25,file='gain.k',form='unformatted',status='unknown')
c	  write(25) ((real(n_gain(i,j)),i=0,nr_mc+1),j=0,nz_mc+1)
	  open(25,file='gain.k',status='unknown')
	  do j = nz_mc+1, 0, -1
	    write(25,"(1p,300e11.3)")
     ^		(real(n_gain(i,j)),i=0,nr_mc+1)
	  enddo
	  write(25,*)
	  write(25,"(' net gain/loss (',a,',z): ',a,' then z [#]')")
     ^		dim2,dim2
	  write(25,"(2x,'do z = ',i3,' to 0')") nz_mc+1
	  write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc+1
	  close(25)
 
c mask array
c	  open(25,file='mask.k',form='unformatted',status='unknown')
c	  write(25) ((real(mask_mc(i,j)),i=0,nr_mc+1),j=0,nz_mc+1)
	  open(25,file='mask.k',status='unknown')
	  do j = nz_mc+1, 0, -1
	    write(25,"(1p,300e10.3)")
     ^		(real(mask_mc(i,j)),i=0,nr_mc+1)
	  enddo
	  write(25,*)
	  write(25,"(' mask (',a,',z): ',a,' then z [#]')")
     ^		dim2,dim2
	  write(25,"(2x,'do z = ',i3,' to 0')") nz_mc+1
	  write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc+1
	  close(25)
 
c average electron energy
	  open(25,file='energy.k',status='unknown')
	  do j = nz_mc,1,-1
	    write(25,"(1p,300e11.3)")
     ^		((1.5*real(t_electron_mc(itm,i,j)),
     ^		itm=1,it1_max_mc),i=1,nr_mc)
	  enddo
	  write(25,*)
	  write(25,"(' energy (t,',a,',z) : t, ',a,' then z [eV]')")
     ^		dim2,dim2
	  write(25,"(2x,'do z = ',i3,' to 1')") nz_mc
	  write(25,"(4x,'do ',a,' = 1 to ',i3)") dim2,nr_mc
	  write(25,"(6x,'do t = 1 to ',i3)") it1_max_mc
	  close(25)
 
c 'harmonic' Br/Bx
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      x4 = max( x4, abs(b_r_mc(i,j)) )
	    enddo
	    if ( x4 .gt. 0. ) exit
	  enddo
	  if ( x4 .gt. 0. .and. b_fieldsp .and. harmonic_p) then
	   if (run_type_mc.eq.3 .or. run_type_mc.eq.2) then
c	    open(25,file='br_h.k',form='unformatted',status='unknown')
	    open(25,file='br_h.k',status='unknown')
	   else
c	    open(25,file='bx_h.k',form='unformatted',status='unknown')
	    open(25,file='bx_h.k',status='unknown')
	   endif
c	   write(25) ((b_r_mc(i,j),i=0,nr_mc),j=0,nz_mc)
	   do j = nz_mc,0,-1
	      write(25,"(1p,300('(',e11.3,',',e11.3,') '))")
     ^		(real(b_r_mc(i,j)),real((0.,-1.)*b_r_mc(i,j)),
     ^		i=0,nr_mc)
	   enddo
	   write(25,*)
	   write(25,"(' b',a,' harmonic(',a,',z) : ',a,
     ^		' then z [gauss]')") dim2,dim2,dim2
	   write(25,"(2x,'do z = ',i3,' to 0')") nz_mc
	   write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc
	   close(25)
	  endif
 
c 'harmonic' Er/Ex
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      x4 = max( x4, abs(e_r_mc(i,j)) )
	    enddo
	    if ( x4 .gt. 0. ) exit
	  enddo
	  if ( x4 .gt. 0. .and. harmonic_p) then
	   if (run_type_mc.eq.3 .or. run_type_mc.eq.2) then
c	    open(25,file='er_h.k',form='unformatted',status='unknown')
	    open(25,file='er_h.k',status='unknown')
	   else
c	    open(25,file='ex_h.k',form='unformatted',status='unknown')
	    open(25,file='ex_h.k',status='unknown')
	   endif
c	   write(25) ((e_r_mc(i,j),i=0,nr_mc),j=0,nz_mc)
	   do j = nz_mc,0,-1
	      write(25,"(1p,300('(',e11.3,',',e11.3,') '))")
     ^		(real(e_r_mc(i,j)),real((0.,-1.)*e_r_mc(i,j)),
     ^		i=0,nr_mc)
	   enddo
	   write(25,*)
	   write(25,"(' E',a,' harmonic(',a,',z) : ',a,
     ^		' then z [V/cm]')") dim2,dim2,dim2
	   write(25,"(2x,'do z = ',i3,' to 0')") nz_mc
	   write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc
	   close(25)
	  endif
 
c 'harmonic' Bth/By
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      x4 = max( x4, abs(b_th_mc(i,j)) )
	    enddo
	    if ( x4 .gt. 0. ) exit
	  enddo
	  if ( x4 .gt. 0. .and. b_fieldsp .and. harmonic_p) then
	   if (run_type_mc.eq.3 .or. run_type_mc.eq.2) then
c	    open(25,file='bth_h.k',form='unformatted',status='unknown')
	    open(25,file='bth_h.k',status='unknown')
	   else
c	    open(25,file='by_h.k',form='unformatted',status='unknown')
	    open(25,file='by_h.k',status='unknown')
	   endif
c	   write(25) ((b_th_mc(i,j),i=0,nr_mc),j=0,nz_mc)
	   do j = nz_mc,0,-1
	      write(25,"(1p,300('(',e11.3,',',e11.3,') '))")
     ^		(real(b_th_mc(i,j)),real((0.,-1.)*b_th_mc(i,j)),
     ^		i=0,nr_mc)
	   enddo
	   write(25,*)
	   write(25,"(' bth/y harmonic(',a,',z) : ',a,
     ^		' then z [gauss]')") dim2,dim2
	   write(25,"(2x,'do z = ',i3,' to 0')") nz_mc
	   write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc
	   close(25)
	  endif
 
c 'harmonic' Eth/Ey
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      x4 = max( x4, abs(e_theta_mc(i,j)) )
	    enddo
	    if ( x4 .gt. 0. ) exit
	  enddo
	  if ( x4 .gt. 0. .and. harmonic_p) then
	   if (run_type_mc.eq.3 .or. run_type_mc.eq.2) then
c	    open(25,file='eth_h.k',form='unformatted',status='unknown')
	    open(25,file='eth_h.k',status='unknown')
	   else
c	    open(25,file='ey_h.k',form='unformatted',status='unknown')
	    open(25,file='ey_h.k',status='unknown')
	   endif
c	   write(25) ((e_theta_mc(i,j),i=0,nr_mc),j=0,nz_mc)
	   do j = nz_mc,0,-1
	      write(25,"(1p,300('(',e11.3,',',e11.3,') '))")
     ^		(real(e_theta_mc(i,j)),real((0.,-1.)*e_theta_mc(i,j)),
     ^		i=0,nr_mc)
	   enddo
	   write(25,*)
	   write(25,"(' Eth/y harmonic(',a,',z) : ',a,
     ^		' then z [V/cm]')") dim2,dim2
	   write(25,"(2x,'do z = ',i3,' to 0')") nz_mc
	   write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc
	   close(25)
	  endif
 
c 'harmonic' Bz
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      x4 = max( x4, abs(b_z_mc(i,j)) )
	    enddo
	    if ( x4 .gt. 0. ) exit
	  enddo
	  if ( x4 .gt. 0. .and. b_fieldsp .and. harmonic_p) then
c	    open(25,file='bz_h.k',form='unformatted',status='unknown')
c	    write(25) ((b_z_mc(i,j),
c     ^		i=0,nr_mc),j=0,nz_mc)
	    open(25,file='bz_h.k',status='unknown')
	    do j = nz_mc,0,-1
	      write(25,"(1p,300('(',e11.3,',',e11.3,') '))")
     ^		(real(b_z_mc(i,j)),real((0.,-1.)*b_z_mc(i,j)),
     ^		i=0,nr_mc)
	    enddo
	    write(25,*)
	    write(25,"(' bz harmonic(',a,',z) : ',a,' then z [gauss]')")
     ^		dim2,dim2
	    write(25,"(2x,'do z = ',i3,' to 0')") nz_mc
	    write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc
	    close(25)
	  endif
 
c 'harmonic' Ez
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      x4 = max( x4, abs(e_z_mc(i,j)) )
	    enddo
	    if ( x4 .gt. 0. ) exit
	  enddo
	  if ( x4 .gt. 0. .and. harmonic_p) then
c	    open(25,file='ez_h.k',form='unformatted',status='unknown')
c	    write(25) ((e_z_mc(i,j),
c     ^		i=0,nr_mc),j=0,nz_mc)
	    open(25,file='ez_h.k',status='unknown')
	    do j = nz_mc,0,-1
	      write(25,"(1p,300('(',e11.3,',',e11.3,') '))")
     ^		(real(e_z_mc(i,j)),real((0.,-1.)*e_z_mc(i,j)),
     ^		i=0,nr_mc)
	    enddo
	    write(25,*)
	    write(25,"(' Ez harmonic(',a,',z) : ',a,' then z [V/cm]')")
     ^		dim2,dim2
	    write(25,"(2x,'do z = ',i3,' to 0')") nz_mc
	    write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc
	    close(25)
	  endif
 
c 'static' Bz
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      x4 = max( x4, abs(bpl_z_mc(i,j)) )
	    enddo
	    if ( x4 .gt. 0. .and. b_fieldsp) exit
	  enddo
	  if ( x4 .gt. 0. ) then
c	    open(25,file='bz.k',form='unformatted',status='unknown')
c	    write(25) ((real(bpl_z_mc(i,j),
c     ^		i=0,nr_mc),j=0,nz_mc)
	    open(25,file='bz.k',status='unknown')
	    do j = nz_mc,0,-1
	      write(25,"(1p,300e11.3)")
     ^		(real(bpl_z_mc(i,j)),i=0,nr_mc)
	    enddo
	    write(25,*)
	    write(25,"(' bz (',a,',z) : ',a,' then z [gauss]')")
     ^		dim2,dim2
	    write(25,"(2x,'do z = ',i3,' to 0')") nz_mc
	    write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc
	    close(25)
	  endif
 
c 'static' Bth/y
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      x4 = max( x4, abs(bpl_th_mc(i,j)) )
	    enddo
	    if ( x4 .gt. 0. ) exit
	  enddo
	  if ( x4 .gt. 0. .and. b_fieldsp) then
c	    open(25,file='bthy.k',form='unformatted',status='unknown')
c	    write(25) ((real(bpl_th_mc(i,j),
c     ^		i=0,nr_mc),j=0,nz_mc)
	    open(25,file='bthy.k',status='unknown')
	    do j = nz_mc,0,-1
	      write(25,"(1p,300e11.3)")
     ^		(real(bpl_th_mc(i,j)),i=0,nr_mc)
	    enddo
	    write(25,*)
	    write(25,"(' bth/y (',a,',z) : ',a,' then z [gauss]')")
     ^		dim2,dim2
	    write(25,"(2x,'do z = ',i3,' to 0')") nz_mc
	    write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc
	    close(25)
	  endif
 
c 'static' br/x
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      x4 = max( x4, abs(bpl_r_mc(i,j)) )
	    enddo
	    if ( x4 .gt. 0. ) exit
	  enddo
	  if ( x4 .gt. 0. .and. b_fieldsp) then
	   if (run_type_mc.eq.3 .or. run_type_mc.eq.2) then
c	    open(25,file='br.k',form='unformatted',status='unknown')
	    open(25,file='br.k',status='unknown')
	   else
c	    open(25,file='bx.k',form='unformatted',status='unknown')
	    open(25,file='bx.k',status='unknown')
	   endif
c	    write(25) (((real(bpl_r_mc(i,j)),
c     ^		i=0,nr_mc),j=0,nz_mc)
	    do j = nz_mc,0,-1
	      write(25,"(1p,300e11.3)")
     ^		(real(bpl_r_mc(i,j)),i=0,nr_mc)
	    enddo
	    write(25,*)
	    write(25,"(' B',a,' (',a,',z) : ',a,
     ^	       ' then z [gauss]')") dim2,dim2,dim2
	    write(25,"(2x,'do z = ',i3,' to 0')") nz_mc
	    write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc
	    close(25)
	  endif
 
c 'static' Er/x
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      do itm = 1, it1_max_mc
	        x4 = max( x4, abs(epl_r_mc(i,j,itm)) )
	      enddo
	    enddo
	    if ( x4 .gt. 0. ) exit
	  enddo
	  if ( x4 .gt. 0. ) then
	   if (run_type_mc.eq.3 .or. run_type_mc.eq.2) then
c	    open(25,file='er.k',form='unformatted',status='unknown')
	    open(25,file='er.k',status='unknown')
	   else
c	    open(25,file='ex.k',form='unformatted',status='unknown')
	    open(25,file='ex.k',status='unknown')
	   endif
c	    write(25) (((real(epl_r_mc(i,j,itm)),itm=1,it1_max_mc),
c     ^		i=0,nr_mc),j=0,nz_mc)
	    do j = nz_mc,0,-1
	      write(25,"(1p,300e11.3)")
     ^		((real(epl_r_mc(i,j,itm)),itm=1,it1_max_mc),
     ^		i=0,nr_mc)
	    enddo
	    write(25,*)
	    write(25,"(' E',a,' (t,',a,',z) : t, ',a,
     ^	       ' then z [V/cm]')") dim2,dim2,dim2
	    write(25,"(2x,'do z = ',i3,' to 0')") nz_mc
	    write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc
	    write(25,"(6x,'do t = 1 to ',i3)") it1_max_mc
	    close(25)
	  endif
 
c 'static' Ez
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      do itm = 1, it1_max_mc
	        x4 = max( x4, abs(epl_z_mc(i,j,itm)) )
	      enddo
	    enddo
	    if ( x4 .gt. 0. ) exit
	  enddo
	  if ( x4 .gt. 0. ) then
c	    open(25,file='ez.k',form='unformatted',status='unknown')
c	    write(25) (((real(epl_z_mc(i,j,itm)),itm=1,it1_max_mc),
c     ^		i=0,nr_mc),j=0,nz_mc)
	    open(25,file='ez.k',status='unknown')
	    do j = nz_mc,0,-1
	      write(25,"(1p,300e11.3)")
     ^		((real(epl_z_mc(i,j,itm)),itm=1,it1_max_mc),
     ^		i=0,nr_mc)
	    enddo
	    write(25,*)
	    write(25,"(' Ez (t,',a,',z) : t, ',a,' then z [V/cm]')")
     ^		dim2,dim2
	    write(25,"(2x,'do z = ',i3,' to 0')") nz_mc
	    write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc
	    write(25,"(6x,'do t = 1 to ',i3)") it1_max_mc
	    close(25)
	  endif
 
c electrostatic potential
	  x4 = 0.
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      do itm = 1, it1_max_mc
	        x4 = max( x4, abs(phi_pl_mc(i,j,itm)) )
	      enddo
	    enddo
	    if ( x4 .gt. 0. ) exit
	  enddo
	  if ( x4 .gt. 0. ) then
c	    open(25,file='phi.k',form='unformatted',status='unknown')
	    open(25,file='phi.k',status='unknown')
c	    write(25) (((real(phi_pl_mc(i,j,itm)),itm=1,it1_max_mc),
c     ^		i=0,nr_mc),j=0,nz_mc)
	    do j = nz_mc+1,0,-1
	      write(25,"(1p,300e11.3)")
     ^		((real(phi_pl_mc(i,j,itm)),itm=1,it1_max_mc),
     ^		i=0,nr_mc+1)
	    enddo
	    write(25,*)
	    write(25,"(' Phi(t,',a,',z) : t, ',a,
     ^		' then z [V]')") dim2,dim2
	    write(25,"(2x,'do z = ',i3,' to 0')") nz_mc+1
	    write(25,"(4x,'do ',a,' = 0 to ',i3)") dim2,nr_mc+1
	    write(25,"(6x,'do t = 1 to ',i3)") it1_max_mc
	    close(25)
	  endif
 
	endif
 
        return
        end
 
c---------------------------------------------------------------------72
 
        subroutine int_rate_mc(num_e)
 
c called by rate_coeffs
c calculates the collision rates via integrating over EEDF
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
	real*8 v,x,y,x0,y0,vv,v2,v1,x1,y1,vt,z2
	real*8 sum_tn(it1_mc,0:nrmax_mc+1,0:nzmax_mc+1,-1:ntmax_mc)
	real*8 tot_tn(it1_mc,-1:ntmax_mc)
	real*4 v0  !,u0
	real*8 tot_coll(nq_max_mc)
 
	logical zerop,file_p,allp
	real*8 nudt(it1_mc)
c	real*8 warm_cold
	real*8 feave(n_e,0:nrmax_mc+1,0:nzmax_mc+1)
	real*8 fesum(n_e,0:20),fetot(0:20)
	real*8 traped(it1_mc,nrmax_mc,nzmax_mc,nq_max_mc)
	real*8 fee(n_e)
 
	real*8 de_co(nq_max_mc)
 
c	real*8 feave_old(n_e,0:nrmax_mc+1,0:nzmax_mc+1)
c	real*8 de_dt_c, de_dt_ee, de_dt_cn
	real*8 n_h, n_c, t_c, t_h, num_e
	real*8 n_h_real, n_h_trap, n_h_etrp, n_c_real, n_h_wall
        real*8 num_trap
c	real*8 n_c0, n_c1, n_c2, n_c3, n_c4, n_c5
	real*4 tmp(nrmax_mc,nzmax_mc),tmp1(nrmax_mc,nzmax_mc)
 
	real gpf
	parameter(gpf = 2.84283897e-16) !0.5 * 9.109e-28 / 1.602e-12)
 
        data electron_mass / 9.10953179447e-28 /
        data erg_ev        / 1.6021892e-12 /
 
c	save feave_old
 
	if (spec_t.ne.0) then
	  x0 = 0.d0
	  open(unit=8,file=run_label(1:l_label)//'.9',status='unknown')
	    do j = 0, nz_mc+1
	      do i = 0, nr_mc+1
		x = 0.d0
		i1 = indx_e(i,j,-1)
		if (mask_mc(i,j).ne.0.and.i1.ne.0) then
		  x = 0.d0
		  if (.not. full_eedf_p) then
		    do k = 1, n_e
c		      x = x + tn_of_e_rz(1,k,i,j,-1)
		      x = x + tn_of_e_rz(1,k,i1)
                    enddo
		    if (x.ne.0.d0) then
	              write(8,"('r,z indices:',i4,',',i4,
     ^		        ' (',1p,e12.5,')')") i,j,x
		      do k = 1, n_e
c		        if (tn_of_e_rz(1,k,i,j,-1).ne.0.)
c     ^		          write(8,30) e_energy(k),
c     ^			    tn_of_e_rz(1,k,i,j,-1)
c		        tn_of_e_rz(1,k,i,j,-1) = 0.
		        if (tn_of_e_rz(1,k,i1).ne.0.)
     ^		          write(8,30) e_energy(k),
     ^			    tn_of_e_rz(1,k,i1)
		        tn_of_e_rz(1,k,i1) = 0.
		      enddo
		    endif
		  else
		    write(*,"(' Error- full eedf not implemented')")
c		    pause
		    stop
		  endif
		  x0 = x0 + x
		endif
	      enddo
	    enddo
	    write(8,*) x0
	  close(8)
	endif
 
c are we going to save distribution functions?
	if (fe_last(1,1,1).eq.-1 .and.
     ^		(.not.fe_last_p)) fe_last_p=.true.
 
c if we are doing time slices, set those up...
	do itm=1,it1_max_mc-1
	  nudt(itm) = 0.5d0 * (ph1_v_mc(itm+1) - ph1_v_mc(itm-1))
	enddo
	nudt(it1_max_mc) =  0.5d0 * (ph1_v_mc(1) +
     ^		    ph1_v_mc(it1_max_mc) - ph1_v_mc(it1_max_mc-1))
 
c clear out the average F...
	do j = 0, nz_mc+1
	  do i = 0, nr_mc+1
	    do k = 1, n_e
	      feave(k,i,j) = 0.d0
	    enddo
	  enddo
	enddo
 
	do k = 1, nq
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      do itm = 1, it1_max_mc
		traped(itm,i,j,k) = 0.d0
	      enddo
	    enddo
	  enddo
	enddo
 
	do j = 1, nz_mc
	  do i = 1, nr_mc
	    do itm = 1, it1_max_mc
	      gpne_mc(itm,i,j) = 0.
	    enddo
	  enddo
	enddo
 
	if (see_p_mc) then
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      do itm = 1, it1_max_mc
	        e_density_mc(itm,i,j) = 0.
	      enddo
	    enddo
	  enddo
	  x0 = 0.d0
	  do nreg = 0, nt_mc
	    x0 = x0 + n_traj_region(nreg) * see_wgt_mc(nreg)
	  enddo
          x = x0
	  if (x0 .ne. 0.d0) x = 1.d0 / x0
	  do j = 0, nz_mc+1
	    do i = 0, nr_mc+1
	      do k = 1, see_nmask_mc
		if (mask_mc(i,j) .eq. see_mask_mc(k)) exit
	      enddo
	      if (k.gt.see_nmask_mc) then
		see_surfq_mc(i,j) = 0.
	      else
		see_surfq_mc(i,j) = see_surfq_mc(i,j) * x
	      endif
	    enddo
	  enddo
	else
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      see_surfq_mc(i,j) = 0.
	    enddo
	  enddo
	endif
 
c clear out the collision rates/frequencies...
	do nreg = nt_mc, -2, -1			!for each electron group
	  do k=1,nq				!for each collision
	    do j=1,nz_mc			!for each z cell
	      do i=1,nr_mc			!for each r/x cell
	        do itm = 1, it1_max_mc		!for each time slice
	          gps_mc(itm,i,j,k,nreg)=0.0
	        enddo
	      enddo
	    enddo
	  enddo
	enddo
 
	do nreg = -1, nt_mc
	  do j = 0, nz_mc + 1
	    do i = 0, nr_mc + 1
	      do itm = 1, it1_max_mc
		sum_tn(itm,i,j,nreg) = 0.
	      enddo
	    enddo
	  enddo
	  do itm = 1, it1_max_mc
	    tot_tn(itm,nreg) = 0.d0
	  enddo
	enddo
 
c find total # of electrons that fluid says...
	num_e = 0.d0
	do j = 1, nz_mc
	  x1 = 0.d0
	  do i = 1, nr_mc
	    if (mask_mc(i,j) .eq. 0) then
	      y1 = 0.d0
	      do itm = 1, it1_max_mc
		y1 = y1 + e_density_mc(itm,i,j) *
     ^		  vol_cell(i,j) * nudt(itm)
	      enddo
	      x1 = x1 + y1
	    endif
	  enddo
	  y0 = y0 + x1
	enddo
	num_e = y0 / ph1_t_max_mc  !total NUMBER of electrons via fluid
 
c find F for each time slice, region, and space.
c store total and normalize
	do nreg = -1, nt_mc
	  if (n_traj_region(nreg).gt.0) then
	    do j = 0, nz_mc + 1
	      do i = 0, nr_mc + 1
		i1 = indx_e(i,j,nreg)
 
		if (.not. full_eedf_p) then
		  do itm = 1, it1_max_mc
		    x = 0.d0
		    do k = n_e, 1, -1
c		      x = x + tN_of_e_rz(itm,k,i,j,nreg)
		      x = x + tn_of_e_rz(itm,k,i1)
		    enddo
		    sum_tn(itm,i,j,nreg) = x
	          enddo
		elseif (i1 .ne. 0) then
		  do itm = 1, it1_max_mc
		    x = 0.d0
		    do i0 = 1, nmu_mc * nphi_mc
		      y = 0.d0
		      do k = n_e, 1, -1
			y = y +
     ^			  tn_of_e_rz(itm,k,i0+(i1-1)*nmu_mc*nphi_mc)
		      enddo
		      x = x + y
		    enddo
		    sum_tn(itm,i,j,nreg) = x
		  enddo
		elseif (min(i,j).gt.0.and.
     ^			i.le.nr_mc.and.j.le.nz_mc) then
		  do itm = 1, it1_max_mc
		    sum_tn(itm,i,j,nreg) = nsteps_rz(itm,i,j,nreg)
		  enddo
		endif
	      enddo
	    enddo
	    do itm = 1, it1_max_mc
	      x0 = 0.d0
	      do j = 1, nz_mc
	        x1 = 0.d0
	        do i = 1, nr_mc
	          if (mask_mc(i,j).eq.0) then
		    x1 = x1 + sum_tn(itm,i,j,nreg)
		    if (see_p_mc.and.nreg.ge.0)
     ^		     e_density_mc(itm,i,j) =
     ^		       e_density_mc(itm,i,j) + nt_f_mc(nreg) *
     ^		       sum_tn(itm,i,j,nreg)
     ^		       / (vol_cell(i,j) * n_traj_region(nreg))
		  endif
	        enddo
	        x0 = x0 + x1
	      enddo
	      tot_tn(itm,nreg) = x0
	    enddo
	  endif
	enddo
 
c ok, find the 'real' number of hot ones in the discharge...
	n_h_real = 0.d0
	n_h_trap = 0.d0
	n_h_wall = 0.d0
        num_trap = 0.d0
	do nreg = nt_mc, 0, -1
	  x0 = 0.d0
	  do itm = 1, it1_max_mc
	    x0 = x0 + tot_tn(itm,nreg)      ! total time group was followed
	  enddo
	  x0   = x0 / max(1, n_traj_region(nreg))   ! average time to exit this group
	  x1   = x0 * nt_f_mc(nreg)         ! <#> = (#/s) * <time>
	  y1   = n_cold(nreg) * nt_f_mc(nreg) /
     ^			max(1, n_traj_region(nreg))
	  w_d(nreg) = x1
	  n_h_real  = n_h_real + x1	   ! sum over groups of <#> in this group
	  n_h_trap  = n_h_trap + y1        !<#/s> which are trapped.
	  y1   = no_loss(1,nreg) * nt_f_mc(nreg) /
     ^			max(1, n_traj_region(nreg))
	  n_h_wall  = n_h_wall + y1

          z2 = no_loss(1,nreg)+no_loss(2,nreg)+n_cold(nreg)  !find another way for # trapped
          num_trap = num_trap + n_cold(nreg) * x1 / max(1.d0, z2)  !# trapped = # cold * <#> / number leaving
	enddo
	n_h_etrp = n_h_trap * ne_loss_av  !<J/s> which is transferred to cold...
 
c from the total number of electrons from the fluid, and the just calculated
c number of hot, we can find cold...
	if (n_traj_region(-1) .ne. 0) then
          z2 = max(0.d0, num_e - n_h_real)
          if (z2.ne.0.d0.and.num_trap.ne.0.d0) then  !both ways have number...
            w_d(-1) = 0.5d0 * (num_trap+z2)
          elseif (z2 .ne. 0.d0) then
            w_d(-1) = z2
          else
            w_d(-1) = num_trap
          endif
c         write(*,*) num_trap
c         write(*,*) w_d
c         write(*,*) (n_cold(j),j=0,nt_mc)
c         write(*,*) (no_loss(1,j),j=0,nt_mc)
c         write(*,*) (no_loss(2,j),j=0,nt_mc)
c	  w_d(-1) = max(0.d0, num_e - n_h_real)
	  n_c_real = w_d(-1)
	else
	  w_d(-1) = 0.d0
	  n_c_real = 0.d0
	endif
 
	x0 = 0.d0
	do nreg = 0, nt_mc
	  x0 = max(x0, w_d(nreg) /
     ^	          max(1, n_traj_region(nreg)))
	enddo
	w_d_sc = x0
        y1     = x0
	if (x0 .ne. 0.d0) y1 = 1.d0 / x0
	do nreg = -1, nt_mc
	  w_d(nreg) = y1 * w_d(nreg) /
     ^	          max(1, n_traj_region(nreg))
	enddo
c <#> in group = w_d(nreg) * w_d_sc * n_traj_region(nreg)
 
c from the total number of electrons from the fluid, and the just calculated
 
 
	inquire(file='mc.rates',exist=file_p)
	if (file_p)
     ^	  open(21,file='mc.rates',status='unknown')
 
c find 'hot' and 'cold' number and average energy...
	  n_h = 0.d0
	  t_h = 0.d0
	  n_c = 0.d0
	  t_c = 0.d0
	  vv  = 0.d0                !volume accessable to trapped...
	  vt  = 0.d0
c	  do nreg = nt_mc,  0, -1
	  do nreg = nt_mc, -1, -1
	    x0 = 0.d0
	    x1 = 0.d0
	    do j = 1, nz_mc
	      y0 = 0.d0
	      y1 = 0.d0
	      do i = 1, nr_mc
		if (mask_mc(i,j).eq.0) then
		  do itm = 1, it1_max_mc
		    y0 = y0 + nsteps_rz(itm,i,j,nreg)
		    y1 = y1 + e_bar_rz(itm,i,j,nreg)
		  enddo
		  if (nreg.eq.-1) then
		    if(-phi_pl_mc(i,j,1).lt.min_energy)
     ^			vv = vv + vol_cell(i,j)
		    vt = vt +vol_cell(i,j)
		  endif
		endif
	      enddo
	      x0 = x0 + y0
	      x1 = x1 + y1
	    enddo
	    if (nreg .ne. -1) then
	      n_h = n_h + x0*w_d(nreg) !weight_reg(nreg)
	      t_h = t_h + x1*w_d(nreg) !weight_reg(nreg)
	    else
	      n_c = x0
	      t_c = x1
	    endif
	  enddo
	  t_h = t_h / (n_h + 1.d-300) * 0.66666
	  t_c = t_c / (n_c + 1.d-300) * 0.66666
 
c dE/dt per cold electron from coulomb collisions...
	  fe_last_ee = n_h_real / vt *
     ^		5.964e-6 * 10.0 * (t_h-t_c)/(t_h+t_c)**(1.5)
	  if (.not.fe_last_p) fe_last_ee = 0.
 
	do nreg = nt_mc, -1, -1
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      do itm = 1, it1_max_mc
	  	gpne_mc(itm,i,j) = gpne_mc(itm,i,j) +
     ^		  sum_tn(itm,i,j,nreg) / (tot_tn(itm,nreg)+1.d-300) *
     ^		  w_d(nreg) * n_traj_region(nreg) * w_d_sc
	      enddo
	    enddo
	  enddo
	  do j = 0, nz_mc+1
	    do i = 0, nr_mc+1
	      i1 = indx_e(i,j,nreg)
	      do itm = 1, it1_max_mc
		if (.not. full_eedf_p) then
		  do k = n_e, 1, -1
		    feave(k,i,j) = feave(k,i,j) + n_traj_region(nreg)*
c     ^		      tn_of_e_rz(itm,k,i,j,nreg) * w_d(nreg) /
     ^		      tn_of_e_rz(itm,k,i1) * w_d(nreg) /
     ^		      (tot_tn(itm,nreg) + 1.d-300) * w_d_sc
		  enddo
		elseif (i1.ne.0) then
		  do k = n_e, 1, -1
		    x = 0.d0
		    do i0 = 1, nphi_mc * nmu_mc
		      x = x +
     ^			tn_of_e_rz(itm,k,i0+(i1-1)*nphi_mc*nmu_mc)
		    enddo
		    feave(k,i,j) = feave(k,i,j) + n_traj_region(nreg)
     ^		      * x * w_d(nreg) / (tot_tn(itm,nreg) + 1.e-300)
     ^		      * w_d_sc
		  enddo
		endif
	      enddo
	    enddo
	  enddo
	enddo
	do j = 1, nz_mc
	  do i = 1, nr_mc
	    if ( mask_mc(i,j) .ne. 0 ) then
	      do itm = 1, it1_max_mc
		gpne_mc(itm,i,j) = 0.
	      enddo
	    endif
	  enddo
	enddo
 
	if (n_traj_region(-1) .ne. 0 .and. fe_last_p) then
	  if (full_eedf_p) then
	    write(*,"(' ERROR- full eedf not valid w/ fe_last_p')")
	    write(*,"('  run aborting...')")
c	    pause
	    stop
	  endif
	  fe_lastmx = -1.d22
	  y1 = 0.d0
	  v1 = 0.d0
	  do j = 1, nz_mc
	    x1 = 0.d0
	    v2 = 0.d0
	    do i = 1, nr_mc
	      if(mask_mc(i,j).eq.0.and.n_c_real.gt.0.d0) then
		i1 = indx_e(i,j,-1)
		v2 = v2 + n_loss(i,j)
	        y0 = 0.d0
	        do k = n_e, 1, -1
c	          x0 = tn_of_e_rz(1,k,i,j,-1) * nudt(1)
	          x0 = tn_of_e_rz(1,k,i1) * nudt(1)
	          do itm = 2, it1_max_mc
c		    x0 = x0 + tn_of_e_rz(itm,k,i,j,-1) * nudt(itm)
		    x0 = x0 + tn_of_e_rz(itm,k,i1) * nudt(itm)
	          enddo
	          y0 = y0 + x0
	          fe_last(k,i,j) = x0 / ph1_t_max_mc
	        enddo
	        y0 = y0 / ph1_t_max_mc
	        fe_lastt(i,j) = y0
	        fe_lastmx = max(fe_lastmx, fe_lastt(i,j))
	        x1 = x1 + y0
	      else
		do k = 1, n_e
		  fe_last(k,i,j) = 0.
		enddo
		fe_lastt(i,j) = 0.
		fe_lastmx = max( fe_lastmx, 0.)
	      endif
	    enddo
	    y1 = y1 + x1
	    v1 = v1 + v2
	  enddo
 
c now add on the newly trapped ones too...
 
c find real number into trapped...
c	  x0 = 0.d0
c	  do nreg = nt_mc, 0, -1
c	    x0 = x0 + w_d(nreg)*w_d_sc*n_traj_region(nreg)*
c     ^		n_cold(nreg)/max(1,n_cold(nreg)+no_loss(1,nreg))
c	  enddo
c	  x0 = n_h_trap * t_max_region(-1)
          x0 = num_trap
c some coming in? then get the scaling right...
	  if (v1 .gt. 0.d0 .and. x0.gt.0.d0) then
	    if (n_c_real-x0 .le. 0.d0) then
	      y1 = 1.d0 / v1
	    else
	      y1 = y1 / v1 * min( 1.d0, x0 / (n_c_real-x0) )
	    endif
	    do j = 1, nz_mc
	      do i = 1, nr_mc
	        if (mask_mc(i,j).eq.0 .and.
     ^				n_loss(i,j).gt.0.0) then
		  x0 = max(delta_e, min(n_e*delta_e,
     ^		      ne_loss(i,j) + 0.5*delta_e))
		  k = max(1, min(n_e, int(x0*delta_ei)))
		  x0 = 1. - mod(real(x0), delta_e)*delta_ei
		  fe_last(k,i,j) = fe_last(k,i,j) +
     ^			x0*n_loss(i,j)*y1
		  k = min(n_e, k+1)
		  fe_last(k,i,j) = fe_last(k,i,j) +
     ^			(1.-x0)*n_loss(i,j)*y1
	        endif
	      enddo
	    enddo
	    fe_lastmx = -1.e22
	    do j = 1, nz_mc
	      do i = 1, nr_mc
	        y0 = 0.d0
		if (mask_mc(i,j).eq.0) then
	          do k = n_e, 1, -1
		    y0 = y0 + fe_last(k,i,j)
	          enddo
		  fe_lastt(i,j) = y0
		  fe_lastmx = max(fe_lastmx, fe_lastt(i,j))
		else
	          do k = n_e, 1, -1
		    fe_last(k,i,j) = 0.
	          enddo
		endif
	      enddo
	    enddo
	    if (fe_lastmx .le. 0.0) then
	      fe_lastmx = -1.e22
	    else
	      x1 = 1.d0 / fe_lastmx
	      fe_lastmx = -1.e22
	      do j = 1, nz_mc
		do i = 1, nr_mc
		  if (mask_mc(i,j) .eq. 0) then
		    y0 = 0.d0
		    do k = n_e, 1, -1
		      fe_last(k,i,j) = fe_last(k,i,j)*x1
		      y0 = y0 + fe_last(k,i,j)
		    enddo
		    fe_lastt(i,j) = y0
		    fe_lastmx = max(fe_lastmx, fe_lastt(i,j))
		  endif
		enddo
	      enddo
	    endif
	  endif
	else
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      do k = 1, n_e
		fe_last(k,i,j) = 0.
	      enddo
	      fe_lastt(i,j) = 0.
	    enddo
	  enddo
	  fe_lastmx = -1.
	endif
 
	inquire(file='fe_last.dat',exist=zerop)
	if (fe_last_p.and.zerop) then
	  open(67,file='fe_last.dat',form='unformatted',
     ^				status='unknown')
	  write(67) nr_mc, nz_mc, n_e, fe_last_ee, ee_max
	  write(67) (((fe_last(k,i,j),k=1,n_e),i=1,nr_mc),j=1,nz_mc)
	  close(67)
	endif
 
c now, for each distribution, find the average collision frequency (<N v sigma>)...
c no, find the average collision rate (<v sigma>(t,r,z,nq,nreg)) (not normalized!)
 
	i2 = -1
	do k = n_e, 1, -1
 
c get the set of cross sections at this energy...
	  v = sqrt (2.d0 * e_energy(k) * erg_ev / electron_mass )
	  v0 = real(e_energy(k))
          call t_cross_sections_mc (v0)
 
c find individual collision coefficients (cm^3/s)
          do i=1, nq
            coll_freq(i)  =  v * sigma(i)
          enddo
 
c now for each collision...
	  do i1=1 , nq
	    v = coll_freq(i1)
	    do nreg = i2, nt_mc
	      if (n_traj_region(nreg).gt.0) then
	        do j = 1, nz_mc
	          do i = 1, nr_mc
		    i0 = indx_e(i,j,nreg)
		    if (.not. full_eedf_p) then
		      do itm = 1, it1_max_mc
		        gps_mc(itm,i,j,i1,nreg)=
     ^			  gps_mc(itm,i,j,i1,nreg) +
c     ^		          tN_of_e_rz(itm,k,i,j,NREG) * v
     ^		          tn_of_e_rz(itm,k,i0) * v
		      enddo
		    elseif (i0.ne.0) then
		      do itm = 1, it1_max_mc
			x = 0.d0
			do j0 = 1, nphi_mc * nmu_mc
			  x = x +
     ^			    tn_of_e_rz(itm,k,j0+(i0-1)*nphi_mc*nmu_mc)
			enddo
		        gps_mc(itm,i,j,i1,nreg)=
     ^			  gps_mc(itm,i,j,i1,nreg) + x * v
		      enddo
		    endif
		  enddo
	        enddo
	      endif
	    enddo
	  enddo
	enddo
 
 
c now for each distribution, compute the # of collisions
c in each cell (space and time) given one initial electron.
 
	do nreg = -1, nt_mc
	  if (n_traj_region(nreg) .gt. 0) then
 
	    do i1 = 1, nq
	      tot_coll(i1) = 0.d0
	    enddo
 
	    do j = 1, nz_mc
	      do i = 1, nr_mc
		if (mask_mc(i,j) .ne. 0) then
		  do i1 = 1, nq
		    do itm = 1, it1_max_mc
		      gps_mc(itm,i,j,i1,nreg) = 0.
		    enddo
		  enddo
		else
		  if (nreg .ne. -1) then
		    do i1 = 1, nq
		      do itm = 1, it1_max_mc
		        gps_mc(itm,i,j,i1,-2) = gps_mc(itm,i,j,i1,-2)+
     ^			  gps_mc(itm,i,j,i1,nreg) * w_d(nreg)  !weight_reg(nreg)
     ^			  / (tot_tn(itm,nreg) + 1.d-302)
c     ^			  / (sum_tn(itm,i,j,nreg) + 1.d-32)
     ^			  * n_traj_region(nreg)
		        gps_mc(itm,i,j,i1,nreg) =
     ^			  gps_mc(itm,i,j,i1,nreg)*
     ^			  n_density_mc(itm,i,j,ic0(i1))
		        tot_coll(i1) = tot_coll(i1) +
     ^			  gps_mc(itm,i,j,i1,nreg)
		      enddo
		    enddo
		  else
		    do i1 = 1, nq
		      do itm = 1, it1_max_mc
		        gps_mc(itm,i,j,i1,-2) = gps_mc(itm,i,j,i1,-2)+
     ^			  gps_mc(itm,i,j,i1,nreg)*w_d(nreg)  !weight_reg(nreg)
     ^			   / (tot_tn(itm,nreg) + 1.d-302)
c     ^			   / (sum_tn(itm,i,j,nreg) + 1.d-32)
     ^			   * n_traj_region(nreg)
			traped(itm,i,j,i1) = gps_mc(itm,i,j,i1,nreg)
     ^			   * n_density_mc(itm,i,j,ic0(i1))
			tot_coll(i1) = tot_coll(i1) +
     ^			  gps_mc(itm,i,j,i1,nreg) *
     ^			  n_density_mc(itm,i,j,ic0(i1))
			gps_mc(itm,i,j,i1,nreg) =
     ^			   gps_mc(itm,i,j,i1,nreg) /
     ^			   (sum_tn(itm,i,j,nreg) + 1.d-32)
		      enddo
		    enddo
		  endif
		endif
	      enddo
	    enddo
 
	    v2 = 0.d0
	    v1 = 0.d0
	    do i1 = nq, 1, -1
	      v2 = v2 + dble(n_collisions_region(i1,nreg))
	      v1 = v1 + tot_coll(i1)
	    enddo
	    v2 = v2 / dble(n_traj_region(nreg))  !# of MC collisions / electron
 
	    x0 = 1.d0
c	    x0 = 0.d0
c	    do itm = 1, it1_max_mc
c	      x0 = x0 + tot_tn(itm,nreg)
c	    enddo
c	    v2 = v2 / (x0 + 1.e-32)		!MC collision rate (# / s)
 
c	    v2 = v2 / (v1 + 1.e-32)    ! # of MC collisions / # computed
	    v2 = 1.d0 / dble( n_traj_region(nreg) )  !# of computed collisions / electron
 
	    if (file_p) then
	      write(*,*) nreg,v2,w_d(nreg),weight_reg(nreg)
	      write(21,*) nreg,v2,w_d(nreg),weight_reg(nreg)
	      do i1 = 1, nq
		write(21,*) i1, dble(n_collisions_region(i1,nreg))
     ^		  / (1.e-32+tot_coll(i1) * n_traj_region(nreg)),
     ^		  dble(n_collisions_region(i1,nreg))/n_traj_region(nreg)
     ^			/ x0,
     ^		  v2 * tot_coll(i1),iq_flag(i1)
		write(*,*) i1, dble(n_collisions_region(i1,nreg))
     ^		  / (1.e-32+tot_coll(i1) * n_traj_region(nreg)),
     ^		  dble(n_collisions_region(i1,nreg))/n_traj_region(nreg)
     ^			/ x0,
     ^		  v2 * tot_coll(i1),iq_flag(i1)
	      enddo
	    endif
 
 
	    if (nreg .ne. -1) then
	      do i1 = 1, nq
		do j = 1, nz_mc
		  do i = 1, nr_mc
		    do itm = 1, it1_max_mc
		      gps_mc(itm,i,j,i1,nreg) =
     ^			gps_mc(itm,i,j,i1,nreg) * v2
     ^			+ traped(itm,i,j,i1) * n_cold(nreg) /
     ^			max(1, n_traj_region(nreg))
		    enddo
		  enddo
		enddo
	      enddo
	    else
	      do i1 = 1, nq
		do j = 1, nz_mc
		  do i = 1, nr_mc
		    do itm = 1, it1_max_mc
		      traped(itm,i,j,i1) =
     ^			traped(itm,i,j,i1) * v2
		    enddo
		  enddo
		enddo
	      enddo
	    endif
 
	  endif
	enddo
 
	do j = 1, nz_mc
	  do i = 1, nr_mc
	    if (mask_mc(i,j).eq.0) then
	      do itm = 1, it1_max_mc
		v1 = 0.d0
		do nreg = nt_mc, -1, -1
		  if (tot_tn(itm,nreg) .ne. 0.d0)
     ^		    v1 = v1 + w_d(nreg)*n_traj_region(nreg)
     ^		        *sum_tn(itm,i,j,nreg) / tot_tn(itm,nreg)
c		  if (sum_tn(itm,i,j,nreg) .ne. 0.d0)
c     ^		    v1 = v1 + w_d(nreg)*n_traj_region(nreg)
cc		  if (sum_tn(itm,i,j,nreg) .ne. 0.d0)
cc     ^		    v1 = v1 + weight_reg(nreg)*n_traj_region(nreg)
ccc	          v1 = v1 + weight_reg(nreg)*n_traj_region(nreg)*
ccc     ^			sum_tn(itm,i,j,nreg)
		enddo
		if (v1.ne.0.d0) then
		  do i1 = 1, nq
		    gps_mc(itm,i,j,i1,-2) = gps_mc(itm,i,j,i1,-2)/v1
		  enddo
		else
		  do i1 = 1, nq
		    gps_mc(itm,i,j,i1,-2) = 0.
		  enddo
		endif
	      enddo
	    else
	      do i1 = 1, nq
		do itm = 1, it1_max_mc
	          gps_mc(itm,i,j,i1,-2) = 0.
		enddo
	      enddo
	    endif
	  enddo
	enddo
 
	do j = 1, nz_mc
	  do i = 1, nr_mc
	    if (mask_mc(i,j) .eq. 0) then
	      do nreg = nt_mc, -1, -1
	        y = 0.d0
	        do itm = 1, it1_max_mc
		  y = y + nsteps_rz(itm,i,j,nreg)
	        enddo
                x = y
	        if (y.ne.0.d0) x = 1.d0 / y
	        do i1 = 1, 3
		  v_ave(i1,i,j,nreg) = v_ave(i1,i,j,nreg) * x
	        enddo
	      enddo
	      y = 0.d0
	      x = 0.d0
	      do i1 = 1, 3
	        y = 0.d0
	        x = 0.d0
	        do nreg = nt_mc, -1, -1
		  x = x + w_d(nreg) * n_traj_region(nreg)
		  y = y + w_d(nreg) * n_traj_region(nreg) *
     ^			v_ave(i1,i,j,nreg)
	        enddo
	        v_ave(i1,i,j,-1) = 0.
	        if (x.ne.0.d0) v_ave(i1,i,j,-1) = y / x
	      enddo
	    else
	      do nreg = -1, nt_mc
		do i1 = 1, 3
		  v_ave(i1,i,j,nreg) = 0.
		enddo
	      enddo
	    endif
	  enddo
	enddo

c fill in gaps in elastic collision frequencies
	if (.false.) then
	do i1 = 1, nq
	  if (iq_flag(i1) .eq. 0) then	!elastic collisions
	    do itm = 1, it1_max_mc
	      zerop = .false.
	      allp  = .true.
	      do j = 1, nz_mc
	        do i = 1, nr_mc
	          if ( mask_mc(i,j) .eq. 0 ) then
	            zerop = (zerop .or. gps_mc(itm,i,j,i1,-2).eq.0.)
		    allp  = (allp .and. gps_mc(itm,i,j,i1,-2).eq.0.)
	          endif
	        enddo
	      enddo

c every spatial location has nu_el=0! unlikely, but if so, punt!
	      if (allp) zerop = .false.

c keep looping until all plasma cells get something for Te
	      do while (zerop)
	        zerop = .false.
	        do j = 1, nz_mc
	          do i = 1, nr_mc
		    tmp(i,j) = 0.
		    if (mask_mc(i,j) .eq. 0 .and. 
     ^				gps_mc(itm,i,j,i1,-2) .eq. 0.) then
		      x=0.
		      if(gps_mc(itm,min(nr_mc,i+1),j,i1,-2)
     ^				.gt.0.)x=x+1.
		      if(gps_mc(itm,max(    1,i-1),j,i1,-2)
     ^				.gt.0.)x=x+1.
		      if(gps_mc(itm,i,min(nz_mc,j+1),i1,-2)
     ^				.gt.0.)x=x+1.
		      if(gps_mc(itm,i,max(    1,j-1),i1,-2)
     ^				.gt.0.)x=x+1.
		      if ( x .gt. 0.5 ) then
		        tmp(i,j) = 1. / x * (
     ^		             gps_mc(itm,min(nr_mc,i+1),j,i1,-2) +
     ^			     gps_mc(itm,max(    1,i-1),j,i1,-2) +
     ^			     gps_mc(itm,i,min(nz_mc,j+1),i1,-2) +
     ^			     gps_mc(itm,i,max(    1,j-1),i1,-2))
		      else
		        zerop = .true.
		      endif
		    endif
	          enddo
	        enddo
	        do j = 1, nz_mc
	          do i = 1, nr_mc
		    gps_mc(itm,i,j,i1,-2) = gps_mc(itm,i,j,i1,-2) +
     ^				tmp(i,j)
	          enddo
	        enddo
	      enddo
	    enddo
	  endif
	enddo
	endif
 
c use actual collisions to do the distribution....
c	do k = 1, Nq
c	  x0 = 0.d0
c	  if (n_collisions(k).ne.0)
c     ^	        x0 = 1.d0/dble(n_traj)
c	    do j = 1, nz_mc
c	      do i = 1, nr_mc
c		gps_mc(1,i,j,k) = dble(n_coll_rz(k,i,j))*x0
c	      enddo
c	    enddo
c	enddo
 
 
        open (unit=8,file=run_label(1:l_label)//'.0',status='unknown')  ! f(e)
	 write(8,"('eedf [(1/ev)**1.5] in user ',
     ^                'specified spatial cells')")
	do k0 = 1, nfe
	  j = izfe(k0)
	  i = irfe(k0)
	  write(8,"('r,z indices:',i4,',',i4)") i,j
	  write(8,"(6x,'energy',10x,'eedf',8x,'energy',10x,'eedf')")
	  x0 = 0.d0
	  do k = 1, n_e-1
	    x0 = x0 + feave(k,i,j)
	  enddo
          y0 = x0
	  if (x0.ne.0.d0) y0 = 1.d0 / x0
	  i1 = (n_e - 1) / 2
	  do k = 1, 2*i1, 2
              write(8,30) e_energy(k), feave(k,i,j) * y0 /
     ^		 (delta_e*sqrt(e_energy(k))+1.e-37)
     ^                   ,e_energy(k+1), feave(k+1,i,j) * y0 /
     ^		 (delta_e*sqrt(e_energy(k+1))+1.e-37)
          enddo
	  if (2*i1.ne.n_e-1)
     ^        write(8,30) e_energy(n_e-1), feave(n_e-1,i,j) * y0 /
     ^                 (delta_e*sqrt(e_energy(n_e-1))+1.e-37)
	enddo
	close(8)
 
	do k = 1, n_e
	  fee(k) = 0.d0
	enddo
	do j = 1, nz_mc
	  do i = 1, nr_mc
	    if (mask_mc(i,j) .eq. 0) then
	      do k = 1, n_e-1
	        fee(k) = fee(k) + feave(k,i,j)
	      enddo
	    endif
	  enddo
	enddo
	do k = 1, n_e
	  f_of_e(k) = fee(k)
	enddo
c	do j = 0, nz_mc+1
c	  do i = 0, nr_mc+1
c	    do k = 1, n_e
c	      feave_old(k,i,j) = feave(k,i,j)
c	    enddo
c	  enddo
c	enddo
 
c only for 1-d! and one time slice....
c        if (it1_max_mc .eq. 1 .and.nr_mc.eq.1) then
c          do j = 1, nz_mc
c            do i = 1, nr_mc
c              if (mask_mc(i,j).eq.0) then
c                x0 = 0.d0
c                y0 = 0.d0
c                do k = n_e-1, 1, -1
c                  x0 = x0 + feave(k,i,j)
c                  y0 = y0 + feave(k,i,j) * e_energy(k)
c                enddo
c                if (x0.ne.0.d0.and.y0.ne.0.d0) then
c                  t_electron_mc(1,i,j) = 2.d0 * y0 / (3.d0 * x0)
c                endif
c              endif
c            enddo
c          enddo
c        endif
c end only for 1-d!
 
	if (dumpfile_p .or. df_mb_mc) then
c calculate Ingold's coefficients and mobility/diffusion coefs. feave(E,r,z) is f(e) sqrt(e) de
	  do j = 0, 8 + nq
	    ing_coeff(j) = 0.d0
	  enddo
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	    
	      gpmb_mc(i,j) = 0.d0
	      gpdf_mc(i,j) = 0.d0
 
	      if ( mask_mc(i,j) .eq. 0 ) then
	        feave(n_e,i,j) = 0.d0
 
	        do i1 = 1, nq
	          de_co(i1) = 0.d0
	        enddo
	        x  = 0.d0
	        y  = 0.d0
	        x0 = 0.d0
	        x1 = 0.d0
	        y0 = 0.d0
	        y1 = 0.d0
	        vt = 0.d0
	        do k = n_e-1, 1, -1
 
	          vv = feave(k,i,j)  ! # in de = f_0(e) sqrt(e) de
	          vt = vt + vv	 ! total number
 
	          v = sqrt(2.d0*e_energy(k)*erg_ev/electron_mass)
	          v0 = real(e_energy(k))
                  call t_cross_sections_mc (v0)
 
	          v2 = 0.d0
	          v1 = 0.d0
                  do i1 = 1, nq
		    de_co(i1) = de_co(i1) + vv * v * sigma(i1) *
     ^			    n_density_mc(1,i,j,ic0(i1))
		    if (iq_flag(i1) .eq. 0) then
		      v1 = v1 + v * sigma(i1) *
     ^			    n_density_mc(1,i,j,ic0(i1))
c		      v2 = v2 + n_density_mc(1,i,j,ic0(i1))
		    endif
                  enddo
c	          v1 = v1 / (v2 + 1.d-22)	! <v sigma>
 
	          v2 = e_energy(k)		! e
 
	          x  = x + v1 * v2 * vv	! <N v sigma> e^(3/2) f_0(e) de
 
	          v2 = v2 / v1		! e / <N v sigma>
 
c calculate df_0/de, actually store sqrt(e) (df_0/de) de
	          if ( k .ne. 1 ) then
		    v1 = .5d0 * (feave(k+1,i,j) / sqrt(e_energy(k+1)) -
     ^			         feave(k-1,i,j) / sqrt(e_energy(k-1)) )
     ^			         * delta_ei * sqrt(e_energy(k))
	          else
		    v1 = (feave(2,i,j) / sqrt(e_energy(2)) -
     ^		          feave(1,i,j) / sqrt(e_energy(1)) )
     ^		          * delta_ei * sqrt(e_energy(1))
	          endif
 
	          x0 = x0 + v2 * vv
	          y0 = y0 + v2 * v1
 
	          x1 = x1 + e_energy(k) * v2 * vv
 	          y1 = y1 + e_energy(k) * v2 * v1
	          y  = y  + e_energy(k) * vv
 
	        enddo

		if ( x0 .ne. 0. ) 
     ^	        	gpdf_mc(i,j) = abs( 1.5 * vt / x0)	!1/mD (1/s/eV)
		if ( y0 .ne. 0. )
     ^	        	gpmb_mc(i,j) = abs(-1.5 * vt / y0)	!e / m mob (1/s)
 
	        ing_coeff(0) =  vt + ing_coeff(0)
	        ing_coeff(1) =  x0 * 1.1725368d15 + ing_coeff(1)
	        ing_coeff(2) = -y0 * 1.1725368d15 + ing_coeff(2)
	        ing_coeff(3) =  x1 * 1.1725368d15 + ing_coeff(3)
	        ing_coeff(4) = -y1 * 1.1725368d15 + ing_coeff(4)
	        ing_coeff(5) =  y  + ing_coeff(5)
	        ing_coeff(6) =  x  + ing_coeff(6)
	        do i1 = 1, nq
	          ing_coeff(i1+8) = de_co(i1) + ing_coeff(i1+8)
	        enddo
	      endif
 
	    enddo
	  enddo
	  y = ing_coeff(0)
	  if ( y .ne. 0.d0) y = 1.d0 / y
	  do j = 1, 8 + nq
	    ing_coeff(j) = ing_coeff(j) * y
	  enddo

c now fil in any spatial locations where 'collision frequencies' are zero
c by searching for the closest non-zero cell...
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      tmp1(i,j) = gpdf_mc(i,j)
	    enddo
	  enddo
	  itm = 0

 1967	  zerop = .false.
	  allp  = .true.
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      if ( mask_mc(i,j) .eq. 0 ) then
	        zerop = (zerop .or. tmp1(i,j).eq.0.)
		allp  = (allp .and. tmp1(i,j).eq.0.)
	      endif
	    enddo
	  enddo

c every spatial location has nu=0! unlikely, but if so, punt!
	  if (allp) then
	    zerop = .false.
	    do j = 1, nz_mc
	      do i = 1, nr_mc
		if ( mask_mc(i,j) .eq. 0 ) 
     ^		  tmp1(i,j) = 0.
	      enddo
	    enddo
	  endif

c keep looping until all plasma cells get something for nu
	  do while (zerop)
	    zerop = .false.
	    do j = 1, nz_mc
	      do i = 1, nr_mc
		tmp(i,j) = 0.
		if (mask_mc(i,j) .eq. 0 .and. 
     ^				tmp1(i,j) .eq. 0.) then
		  x=0.
		  if(tmp1(min(nr_mc,i+1),j).gt.0.)x=x+1.
		  if(tmp1(max(    1,i-1),j).gt.0.)x=x+1.
		  if(tmp1(i,min(nz_mc,j+1)).gt.0.)x=x+1.
		  if(tmp1(i,max(    1,j-1)).gt.0.)x=x+1.
		  if ( x .gt. 0.5 ) then
		    tmp(i,j) = 1. / x * (
     ^		         tmp1(min(nr_mc,i+1),j) +
     ^			 tmp1(max(    1,i-1),j) +
     ^			 tmp1(i,min(nz_mc,j+1)) +
     ^			 tmp1(i,max(    1,j-1)))
		  else
		    zerop = .true.
		  endif
		endif
	      enddo
	    enddo
	    do j = 1, nz_mc
	      do i = 1, nr_mc
		tmp1(i,j) = tmp1(i,j) + tmp(i,j)
	      enddo
	    enddo
	  enddo

	  if (itm .eq. 0) then
	    do j = 1, nz_mc
	      do i = 1, nr_mc
		gpdf_mc(i,j) = tmp1(   i,j)
		tmp1(   i,j) = gpmb_mc(i,j)
	      enddo
	    enddo
	    itm = 1
	    go to 1967
	  else
	    do j = 1, nz_mc
	      do i = 1, nr_mc
		gpmb_mc(i,j) = tmp1(   i,j)
	      enddo
	    enddo
	  endif
 
	endif
 
 
	do i0 = 0, 20
	  fetot(i0) = 0.d0
	  do k = 1, n_e -1
	    fesum(k,i0) = 0.d0
	  enddo
	enddo
	do j = 0, nz_mc+1
	  do i = 0, nr_mc+1
	    i0 = max(0,min(20, mask_mc(i,j)))
	    do k = 1, n_e-1
	      fesum(k,i0) = fesum(k,i0) + feave(k,i,j)
	      fetot(i0)   = fetot(i0)   + feave(k,i,j)
	    enddo
	  enddo
	enddo
        open (unit=8, file=run_label(1:l_label)//'.1',status='unknown')  ! f(e)
	 write(8,"('energy distribution for each mask')")
	 do i0 = 0, 20
	   if (fetot(i0).ne.0.d0) then
	     write(8,"('mask =',i2,', total # =',1p,e12.5)")
     ^			i0,fetot(i0)
	     write(8,"(7x,'energy',5x,'probability',6x
     ^                ,'energy',5x,'probability')")
	     i = (n_e - 1) /2
	     do k = 1, 2*i, 2
               write(8,30) e_energy(k), fesum(k,i0)/fetot(i0),
     ^                   e_energy(k+1), fesum(k+1,i0)/fetot(i0)
             enddo
	     if (2*i.ne.n_e-1)
     ^         write(8,30) e_energy(n_e-1), fesum(n_e-1,i0)/fetot(i0)
	   endif
	 enddo
	 write(8,"('mask array:')")
	 write(8,"(3x,'z/r',99i3)") (i, i = 0, nr_mc+1)
	 write(8,"(3x,' *-',99(a3))") ('---',i = 0, nr_mc+1)
	 do j = nz_mc+1, 0, -1
	   write(8,"(i3,' | ',99i3)") j,(mask_mc(i,j), i= 0, nr_mc+1)
	 enddo
	close(8)
 
	if (file_p)
     ^	    close(21)
 
 
	inquire(file='time.out',exist=zerop)
	if ( zerop ) then
	  open(57,file='time.out', status='old',	!HP
     ^	         access='append')
c	  open(57,file='time.out', status='old',	!ABSOFT PC
c     ^	         position='append')
c	  open(57,file='time.out', status='old')	!IBM
 
	  write(57,"(1p,30e13.5)") t_mc,(qvol_mc(i),i=1,nspec_mc+1),
     ^	    num_e,n_c_real/num_e,
     ^	    n_h_real/num_e,t_c,t_h,tot_ion_out_mc,n_h_trap+n_h_wall,
     ^	    n_h_trap/(n_h_trap+n_h_wall),
     ^	    n_h_wall/(n_h_trap+n_h_wall), ne_loss_av,
     ^	    no_loss(1,-1)/max(1,n_traj_region(-1)),fe_last_ee,
     ^	    (nmax(i),i=1,nspec_mc),min_energy
	  close(57)
	endif
	
	if (full_eedf_p) then
          open(unit=8,file=run_label(1:l_label)//'.2',status='unknown')  ! f(e)
	  write(8,"('eedf(r,E,mu,phi) [(1/ev)**1.5/dSigma] in user ',
     ^                'specified spatial cells')")
	  write(8,"(' dmu = 2 /',i3,' dphi = 2 pi /',i3)") nmu_mc,nphi_mc
	  do k0 = 1, nfe
	    j = izfe(k0)
	    i = irfe(k0)
 
	    nreg = -1
	    do while (indx_e(i,j,nreg).eq.0)
	      nreg = nreg + 1
	    enddo
	    i0 = indx_e(i,j,nreg)
 
	    x0 = 0.d0
	    do k = 1, n_e-1
	      do j1 = 1, nmu_mc
		do j0 = 1, nphi_mc
		  do nreg = -1, nt_mc
		    i1 = indx_e(i,j,nreg)
		    i1 = max(0 ,j0 + nphi_mc*((j1-1)+nmu_mc*(i1-1)))
		    if (indx_e(i,j,nreg).ne.0) then
		      do itm = 2, it1_max_mc
		        tn_of_e_rz(1,k,i1) = tn_of_e_rz(1,k,i1) +
     ^				tn_of_e_rz(itm,k,i1)
		      enddo
		    else
		      tn_of_e_rz(1,k,i1) = 0.
		    endif
		    if (indx_e(i,j,nreg).ne.i0) then
		      ii = j0 + nphi_mc*((j1-1)+nmu_mc*(i0-1))
		      tn_of_e_rz(1,k,ii) = tn_of_e_rz(1,k,ii) +
     ^				tn_of_e_rz(1,k,i1)
		    endif
		  enddo
		  ii = j0 + nphi_mc*((j1-1)+nmu_mc*(i0-1))
		  x0 = x0 + tn_of_e_rz(1,k,ii)
		enddo
	      enddo
	    enddo
 
	    write(8,"('r,z indices:',i4,',',i4,', norm =',
     ^			1p,e12.5)") i,j,x0
 
            y0 = x0
	    if (x0.ne.0.d0) y0 = 1.d0 / x0
	    do k = 1, n_e-1
	      do j1 = 1, nmu_mc
		write(8,"('energy(',i3,'/',i3,') =',1p,e12.5,
     ^		  ' eV   mu(',i3,'/',i3,') '
     ^		  ,300e10.3
     ^	          )")
     ^		  k,n_e-1,e_energy(k),j1,nmu_mc,
     ^		  (y0*tn_of_e_rz(1,k,j0+nphi_mc*((j1-1)+
     ^		  nmu_mc*(i0-1)))/(delta_e*sqrt(e_energy(k))+1.e-37),
     ^		  j0=1,nphi_mc)
c		write(8,"('energy(',i3,'/',i3,') =',1p,e12.5,
c     ^		  ' eV   mu(',i3,'/',i3,') '
c     ^	          )")
c     ^		  k,n_e-1,e_energy(k),j1,nmu_mc
c		write(8,"(1p,300e10.3)")
c     ^		  (y0*tn_of_e_rz(1,k,j0+nphi_mc*((j1-1)+
c     ^		  nmu_mc*(i0-1)))/(delta_e*sqrt(e_energy(k))+1.e-37),
c     ^		  j0=1,nphi_mc)
	      enddo
	    enddo
 
	  enddo
	 close(8)
	endif
 
	pow_tn = 0.d0
	pow_tr = 0.d0
	pow_cr = 0.d0
	pow_ir = 0.d0
	pow_xr = 0.d0
	pow_tt = 0.d0
	pow_ct = 0.d0
	pow_it = 0.d0
	pow_xt = 0.d0
	do j = 0, nz_mc+1
	  do i = 0, nr_mc+1
	    pow_t(i,j,-2) = 0.d0
	    pow_c(i,j,-2) = 0.d0
	    pow_i(i,j,-2) = 0.d0
	    pow_x(i,j,-2) = 0.d0
	    if ( mask_mc(i,j) .eq. 0 ) then
	      do nreg = nt_mc, -1, -1
	        x0 = 0.d0
		y0 = 1.d-300
	        do itm = 1, it1_max_mc
		  x0 = x0 +
     ^		    sum_tn(itm,i,j,nreg) / (tot_tn(itm,nreg)+1.d-300)*
     ^		    w_d(nreg) * n_traj_region(nreg) * w_d_sc
		  y0 = y0 + sum_tn(itm,i,j,nreg)
	        enddo
		pow_tr = pow_tr + pow_t(i,j,nreg) * weight_reg(nreg)
		pow_cr = pow_cr + pow_c(i,j,nreg) * weight_reg(nreg)
		pow_ir = pow_ir + pow_i(i,j,nreg) * weight_reg(nreg)
		pow_xr = pow_xr + pow_x(i,j,nreg) * weight_reg(nreg)
		pow_t(i,j,nreg) = pow_t(i,j,nreg) * x0 / y0
		pow_t(i,j,-2)   = pow_t(i,j,-2) + pow_t(i,j,nreg)
		pow_c(i,j,nreg) = pow_c(i,j,nreg) * x0 / y0
		pow_c(i,j,-2)   = pow_c(i,j,-2) + pow_c(i,j,nreg)
		pow_i(i,j,nreg) = pow_i(i,j,nreg) * x0 / y0
     ^					* 1.660565e-31 * spec_tm
		pow_i(i,j,-2)   = pow_i(i,j,-2) + pow_i(i,j,nreg)
		if ( nreg .ne. -1 ) then
		  pow_x(i,j,nreg) = pow_x(i,j,nreg) * weight_reg(nreg)
		else
		  pow_x(i,j,nreg) = pow_x(i,j,nreg) * n_h_trap /
     ^			max( 1, n_traj_region(nreg) )
		endif
		pow_x(i,j,-2)   = pow_x(i,j,-2) + pow_x(i,j,nreg)
		pow_tn = pow_tn + x0
	      enddo
	      pow_tt = pow_tt + pow_t(i,j,-2)
	      pow_ct = pow_ct + pow_c(i,j,-2)
	      pow_it = pow_it + pow_i(i,j,-2)
	      pow_xt = pow_xt + pow_x(i,j,-2)
	    else
	      do nreg = nt_mc, -1, -1
	        pow_xr = pow_xr + pow_x(i,j,nreg) * weight_reg(nreg)
		if ( nreg .ne. -1 ) then
		  pow_x(i,j,nreg) = pow_x(i,j,nreg) * weight_reg(nreg)
		else
		  pow_x(i,j,nreg) = pow_x(i,j,nreg) * n_h_trap /
     ^			max( 1, n_traj_region(nreg) )
		endif
		pow_x(i,j,-2)   = pow_x(i,j,-2) + pow_x(i,j,nreg)
	      enddo
	      pow_xt = pow_xt + pow_x(i,j,-2)
	    endif
	  enddo
	enddo
	pow_tr = pow_tr * weight_mx
	pow_cr = pow_cr * weight_mx
	pow_ir = pow_ir * weight_mx
	pow_xr = pow_xr * weight_mx
 
	pow_xn = n_h_trap + 1.d-300
	do nreg = 0, nt_mc
	  pow_xn = pow_xn + nt_f_mc(nreg)
	enddo
 
 30     format (2(1p2e14.3))
 
	return
	end
 
c---------------------------------------------------------------------72
 
        subroutine error_bars_mc (n, nmin, nmax)
 
c called by new_coeffs
c       95% confidence intervals for the Poisson distribution
 
 
        implicit real*4 (a-h,o-z)
 
        real*4 nmin, nmax
 
        dimension p95_min(0:20), p95_max(0:20)
        dimension p2550_min(6) , p2550_max(6)
 
        data p95_min /0.,
     1		    .0513,.355,.818,1.37,1.97,2.61,3.29,3.98,4.7,
     2              5.43,6.17,6.92,7.69,8.46,9.25,10.04,10.83,11.63,
     3              12.44,13.25/
 
        data p95_max /3.,
     1		    4.74,6.3,7.75,9.15,10.51,11.84,13.15,14.43,
     2              15.71,16.96,18.21,19.44,20.57,21.89,23.1,24.3,
     3              25.5,26.69,27.88,29.06/
 
        data p2550_min /17.38,21.59,25.87,30.2,34.56,38.96/
 
        data p2550_max / 34.92,40.69,46.4,52.07,57.69,63.29/
 
 
c for counts above 50:
        p_min(ncounts) = ncounts + 1.92 - 1.96 * sqrt(ncounts + 1.0)
        p_max(ncounts) = ncounts + 1.92 + 1.96 * sqrt(ncounts + 1.0)
 
        if (n .le. 20) then
          nmin = p95_min(n)
          nmax = p95_max(n)
          return
        endif
 
        if (n .le. 50) then
          i    = (n - 25) / 5 + 1    ! the lower entry in the p2550 vectors
	  i    = max(1, min(5, i))
          nmin = p2550_min(i) + dfloat(n) / 5.
     x                      * (p2550_min(i+1) - p2550_min(i))
          nmax = p2550_max(i) + dfloat(n) / 5.
     x                      * (p2550_max(i+1) - p2550_max(i))
          return
        endif
 
        nmin = p_min(n)
        nmax = p_max(n)
 
        return
        end
 
 
c---------------------------------------------------------------------72
 
	subroutine accel_cyl_h(x,y,z,vx,vy,vz,xp,yp,zp,vxp,vyp,vzp,
     ^		t1,dt,ir0,iz0,ir,iz,acx,acz,r,c_th,s_th,acxm,aczm,
     ^		abr,abt,abz,nreg)
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
	real*4 acz(0:nrmax_mc,0:nzmax_mc)
	real*4 acx(0:nrmax_mc,0:nzmax_mc),rr
	real*8 t1
	real*4 abt(0:nrmax_mc,0:nzmax_mc)
	real*4 abr(0:nrmax_mc,0:nzmax_mc)
	real*4 abz(0:nrmax_mc,0:nzmax_mc),bvz,bvt,bvr
 
c	real*8 x,y,z,vx,vy,vz,xp,yp,zp,vxp,vyp,vzp
 
	integer find_iz, find_ir
 
c	real*8 kx(4),kvx(4),kz(4),kvz(4),ky(4),kvy(4)
	real   kx(4),kvx(4),kz(4),kvz(4),ky(4),kvy(4)
	complex*8 expiwt
 
 
	ir0 = ir
	iz0 = iz
	if (nz_mc.ne.1) then
 
c find initial acceleration...
	  fz=min(1.,max(0.,(z-zt_mc(iz))*dzi_mc(iz)))
	  fx=min(1.,max(0.,(r-rt_mc(ir))*dri_mc(ir)))
	  expiwt = (-1.75880521211e15,0.)*exp((0.,-1.)*omega0_mc*t1)
 
	  bvz =((1.-fx)*abz(ir-1,iz  )+fx*abz(ir,iz  ))*fz
     ^	     + ((1.-fx)*abz(ir-1,iz-1)+fx*abz(ir,iz-1))*(1.-fz)
     ^	     + 1.e-8 * real((((1.-fx)*b_z_mc(ir-1,iz  )
     ^	     +	      fx *b_z_mc(ir  ,iz  ))*fz
     ^	     +       ((1.-fx)*b_z_mc(ir-1,iz-1)
     ^	     +	      fx *b_z_mc(ir,iz-1))*(1.-fz)) * expiwt)
	  bvr =((1.-fx)*abr(ir-1,iz  )+fx*abr(ir,iz  ))*fz
     ^	     + ((1.-fx)*abr(ir-1,iz-1)+fx*abr(ir,iz-1))*(1.-fz)
     ^	     + 1.e-8 * real((((1.-fx)*b_r_mc(ir-1,iz  )
     ^	     +	      fx *b_r_mc(ir  ,iz  ))*fz
     ^	     +       ((1.-fx)*b_r_mc(ir-1,iz-1)
     ^	     +	      fx *b_r_mc(ir,iz-1))*(1.-fz)) * expiwt)
	  bvt =((1.-fx)*abt(ir-1,iz  )+fx*abt(ir,iz  ))*fz
     ^	     + ((1.-fx)*abt(ir-1,iz-1)+fx*abt(ir,iz-1))*(1.-fz)
     ^	     + 1.e-8 * real((((1.-fx)*b_th_mc(ir-1,iz  )
     ^	     +	      fx *b_th_mc(ir  ,iz  ))*fz
     ^	     +       ((1.-fx)*b_th_mc(ir-1,iz-1)
     ^	     +	      fx *b_th_mc(ir,iz-1))*(1.-fz)) * expiwt)
	  bvx = c_th * bvr - s_th * bvt
	  bvy = c_th * bvt + s_th * bvr
	
	  aczm =     ((1.-fx)*acz(ir-1,iz) +
     ^		          fx *acz(ir  ,iz))*fz
     ^	        +    ((1.-fx)*acz(ir-1,iz-1) +
     ^		          fx *acz(ir  ,iz-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_z_mc(ir-1,iz  )
     ^		 +	      fx *e_z_mc(ir  ,iz  ))*fz
     ^		 +       ((1.-fx)*e_z_mc(ir-1,iz-1)
     ^		 +	      fx *e_z_mc(ir,iz-1))*(1.-fz)) * expiwt)
	  acxm =     ((1.-fx)*acx(ir-1,iz) +
     ^	    	          fx *acx(ir  ,iz))*fz
     ^	        +    ((1.-fx)*acx(ir-1,iz-1) +
     ^		          fx *acx(ir  ,iz-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_r_mc(ir-1,iz  )
     ^		 +	      fx *e_r_mc(ir  ,iz  ))*fz
     ^		 +       ((1.-fx)*e_r_mc(ir-1,iz-1)
     ^		 +	      fx *e_r_mc(ir,iz-1))*(1.-fz)) * expiwt)
	  actm0 =
     ^	           real((((1.-fx)*e_theta_mc(ir-1,iz  )
     ^		 +	      fx *e_theta_mc(ir  ,iz  ))*fz
     ^		 +       ((1.-fx)*e_theta_mc(ir-1,iz-1)
     ^		 +	      fx *e_theta_mc(ir,iz-1))*(1.-fz)) 
     ^		 * expiwt)
	
	  acym = acxm * s_th + actm0 * c_th
	  acxm = acxm * c_th - actm0 * s_th
 
c first push
	  kx(1)  = dt * vx
	  kvx(1) = dt * (acxm + vy * bvz - vz * bvy)
	  ky(1)  = dt * vy
	  kvy(1) = dt * (acym + vz * bvx - vx * bvz)
	  kz(1)  = dt * vz
	  kvz(1) = dt * (aczm + vx * bvy - vy * bvx)
 
c find accel
	  zk = z + 0.5 * kz(1)
	  xk = x + 0.5 * kx(1)
	  yk = y + 0.5 * ky(1)
 
c	  izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	  izp = min(nz_mc, max(1, find_iz(zk,zk-z,iz)))
	  fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	  rk = sqrt(xk*xk + yk*yk)
	  if (rk.eq.0.) then
		rk = rt_mc(2) * 1.e-3
		xk = rk * sign(1., .5 - ranf_mc(2))
	  endif
c	  irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	  irp = min(nr_mc, max(1, find_ir( rk, rk - r, ir)))
	  fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	  expiwt = (-1.75880521211e15,0.) *
     ^			exp((0.,-1.)*omega0_mc*(t1+.5*dt))
 
	  bvz =((1.-fx)*abz(irp-1,izp  )+fx*abz(irp,izp  ))*fz
     ^		 + ((1.-fx)*abz(irp-1,izp-1)+fx*abz(irp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_z_mc(irp-1,izp  )
     ^		 +	      fx *b_z_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*b_z_mc(irp-1,izp-1)
     ^		 +	      fx *b_z_mc(irp,izp-1))*(1.-fz)) * expiwt)
	  bvr =((1.-fx)*abr(irp-1,izp  )+fx*abr(irp,izp  ))*fz
     ^		 + ((1.-fx)*abr(irp-1,izp-1)+fx*abr(irp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_r_mc(irp-1,izp  )
     ^		 +	      fx *b_r_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*b_r_mc(irp-1,izp-1)
     ^		 +	      fx *b_r_mc(irp,izp-1))*(1.-fz)) * expiwt)
	  bvt =((1.-fx)*abt(irp-1,izp  )+fx*abt(irp,izp  ))*fz
     ^		 + ((1.-fx)*abt(irp-1,izp-1)+fx*abt(irp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_th_mc(irp-1,izp  )
     ^		 +	      fx *b_th_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*b_th_mc(irp-1,izp-1)
     ^		 +	     fx *b_th_mc(irp,izp-1))*(1.-fz)) * expiwt)
	  bvx = (xk * bvr - yk * bvt) / rk
	  bvy = (xk * bvt + yk * bvr) / rk
	
	  aczm = ((1.-fx)*acz(irp-1,izp) +
     ^		          fx *acz(irp  ,izp))*fz
     ^	        +    ((1.-fx)*acz(irp-1,izp-1) +
     ^		          fx *acz(irp  ,izp-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_z_mc(irp-1,izp  )
     ^		 +	      fx *e_z_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_z_mc(irp-1,izp-1)
     ^		 +	     fx *e_z_mc(irp  ,izp-1))*(1.-fz))* expiwt)
	  acxm = ((1.-fx)*acx(irp-1,izp) +
     ^	    	          fx *acx(irp  ,izp))*fz
     ^	        +    ((1.-fx)*acx(irp-1,izp-1) +
     ^		          fx *acx(irp  ,izp-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_r_mc(irp-1,izp  )
     ^		 +	      fx *e_r_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_r_mc(irp-1,izp-1)
     ^		 +	    fx *e_r_mc(irp  ,izp-1))*(1.-fz)) * expiwt)
	  actm =
     ^	           real((((1.-fx)*e_theta_mc(irp-1,izp  )
     ^		 +	      fx *e_theta_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_theta_mc(irp-1,izp-1)
     ^		 +	  fx *e_theta_mc(irp,izp-1))*(1.-fz)) * expiwt)
	
	  acym = (acxm * yk + actm * xk) / rk
	  acxm = (acxm * xk - actm * yk) / rk
 
	  vzk = vz + .5 * kvz(1)
	  vxk = vx + .5 * kvx(1)
	  vyk = vy + .5 * kvy(1)
	  if ( push_order .eq. 2 ) then
	    xp = x + dt * vxk
	    yp = y + dt * vyk
	    zp = z + dt * vzk
	    vxp = vx + dt * (acxm + vzk*bvx - vxk*bvz)
	    vyp = vy + dt * (acym + vzk*bvx - vxk*bvz)
	    vzp = vz + dt * (aczm + vxk*bvy - vyk*bvx)
	  else
c second push
	      kx(2)  = dt * vxk
	      kvx(2) = dt * (acxm + vyk*bvz - vzk*bvy)
	      ky(2)  = dt * vyk
	      kvy(2) = dt * (acym + vzk*bvx - vxk*bvz)
	      kz(2)  = dt * vzk
	      kvz(2) = dt * (aczm + vxk*bvy - vyk*bvx)
 
c find accel
	      zk = z + 0.5 * kz(2)
	      xk = x + 0.5 * kx(2)
	      yk = y + 0.5 * ky(2)
 
c	      izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	      izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	      fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	      rk = sqrt(xk*xk + yk*yk)
	      if (rk.eq.0.) then
		rk = rt_mc(2) * 1.e-3
		xk = rk * sign(1., .5 - ranf_mc(2))
	      endif
c	      irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	      irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir)))
	      fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	      bvz =((1.-fx)*abz(irp-1,izp  )+fx*abz(irp,izp  ))*fz
     ^		 + ((1.-fx)*abz(irp-1,izp-1)+fx*abz(irp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_z_mc(irp-1,izp  )
     ^		 +	      fx *b_z_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*b_z_mc(irp-1,izp-1)
     ^		 +	      fx *b_z_mc(irp,izp-1))*(1.-fz)) * expiwt)
	      bvr =((1.-fx)*abr(irp-1,izp  )+fx*abr(irp,izp  ))*fz
     ^		 + ((1.-fx)*abr(irp-1,izp-1)+fx*abr(irp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_r_mc(irp-1,izp  )
     ^		 +	      fx *b_r_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*b_r_mc(irp-1,izp-1)
     ^		 +	      fx *b_r_mc(irp,izp-1))*(1.-fz)) * expiwt)
	      bvt =((1.-fx)*abt(irp-1,izp  )+fx*abt(irp,izp  ))*fz
     ^		 + ((1.-fx)*abt(irp-1,izp-1)+fx*abt(irp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_th_mc(irp-1,izp  )
     ^		 +	     fx *b_th_mc(irp  ,izp  ))*fz
     ^		 +      ((1.-fx)*b_th_mc(irp-1,izp-1)
     ^		 +	     fx *b_th_mc(irp,izp-1))*(1.-fz)) * expiwt)
	      bvx = (xk * bvr - yk * bvt) / rk
	      bvy = (xk * bvt + yk * bvr) / rk
	      aczm = ((1.-fx)*acz(irp-1,izp) +
     ^		          fx *acz(irp  ,izp))*fz
     ^	        +    ((1.-fx)*acz(irp-1,izp-1) +
     ^		          fx *acz(irp  ,izp-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_z_mc(irp-1,izp  )
     ^		 +	      fx *e_z_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_z_mc(irp-1,izp-1)
     ^		 +	    fx *e_z_mc(irp  ,izp-1))*(1.-fz)) * expiwt)
	      acxm = ((1.-fx)*acx(irp-1,izp) +
     ^	    	          fx *acx(irp  ,izp))*fz
     ^	        +    ((1.-fx)*acx(irp-1,izp-1) +
     ^		          fx *acx(irp  ,izp-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_r_mc(irp-1,izp  )
     ^		 +	      fx *e_r_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_r_mc(irp-1,izp-1)
     ^		 +	    fx *e_r_mc(irp  ,izp-1))*(1.-fz)) * expiwt)
	      actm =
     ^	           real((((1.-fx)*e_theta_mc(irp-1,izp  )
     ^		 +	      fx *e_theta_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_theta_mc(irp-1,izp-1)
     ^		 +	  fx *e_theta_mc(irp,izp-1))*(1.-fz)) * expiwt)
	
	      acym = (acxm * yk + actm * xk) / rk
	      acxm = (acxm * xk - actm * yk) / rk
 
	      vzk = vz + .5 * kvz(2)
	      vxk = vx + .5 * kvx(2)
	      vyk = vy + .5 * kvy(2)
c third push
	      kx(3)  = dt * vxk
	      kvx(3) = dt * (acxm + vyk*bvz - vzk*bvy)
	      ky(3)  = dt * vyk
	      kvy(3) = dt * (acym + vzk*bvx - vxk*bvz)
	      kz(3)  = dt * vzk
	      kvz(3) = dt * (aczm + vxk*bvy - vyk*bvx)
 
c find accel
	      zk = z + kz(3)
	      xk = x + kx(3)
	      yk = y + ky(3)
 
c	      izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	      izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	      fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	      rk = sqrt(xk*xk + yk*yk)
	      if (rk.eq.0.) then
		rk = rt_mc(2) * 1.e-3
		xk = rk * sign(1., .5 - ranf_mc(2))
	      endif
c	      irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	      irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir) ))
	      fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	      expiwt = (-1.75880521211e15,0.) *
     ^			exp((0.,-1.)*omega0_mc*(t1+dt))
	      bvz =((1.-fx)*abz(irp-1,izp  )+fx*abz(irp,izp  ))*fz
     ^		 + ((1.-fx)*abz(irp-1,izp-1)+fx*abz(irp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_z_mc(irp-1,izp  )
     ^		 +	     fx *b_z_mc(irp  ,izp  ))*fz
     ^		 +      ((1.-fx)*b_z_mc(irp-1,izp-1)
     ^		 +	     fx *b_z_mc(irp,izp-1))*(1.-fz)) * expiwt)
	      bvr =((1.-fx)*abr(irp-1,izp  )+fx*abr(irp,izp  ))*fz
     ^		 + ((1.-fx)*abr(irp-1,izp-1)+fx*abr(irp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_r_mc(irp-1,izp  )
     ^		 +	     fx *b_r_mc(irp  ,izp  ))*fz
     ^		 +      ((1.-fx)*b_r_mc(irp-1,izp-1)
     ^		 +	     fx *b_r_mc(irp,izp-1))*(1.-fz)) * expiwt)
	      bvt =((1.-fx)*abt(irp-1,izp  )+fx*abt(irp,izp  ))*fz
     ^		 + ((1.-fx)*abt(irp-1,izp-1)+fx*abt(irp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_th_mc(irp-1,izp  )
     ^		 +	     fx *b_th_mc(irp  ,izp  ))*fz
     ^		 +      ((1.-fx)*b_th_mc(irp-1,izp-1)
     ^		 +	     fx *b_th_mc(irp,izp-1))*(1.-fz)) * expiwt)
	      bvx = (xk * bvr - yk * bvt) / rk
	      bvy = (xk * bvt + yk * bvr) / rk
	      aczm = ((1.-fx)*acz(irp-1,izp) +
     ^		          fx *acz(irp  ,izp))*fz
     ^	        +    ((1.-fx)*acz(irp-1,izp-1) +
     ^		          fx *acz(irp  ,izp-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_z_mc(irp-1,izp  )
     ^		 +	      fx *e_z_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_z_mc(irp-1,izp-1)
     ^		 +	    fx *e_z_mc(irp  ,izp-1))*(1.-fz)) * expiwt)
	      acxm = ((1.-fx)*acx(irp-1,izp) +
     ^	    	          fx *acx(irp  ,izp))*fz
     ^	        +    ((1.-fx)*acx(irp-1,izp-1) +
     ^		          fx *acx(irp  ,izp-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_r_mc(irp-1,izp  )
     ^		 +	      fx *e_r_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_r_mc(irp-1,izp-1)
     ^		 +	    fx *e_r_mc(irp  ,izp-1))*(1.-fz)) * expiwt)
	      actm =
     ^	           real((((1.-fx)*e_theta_mc(irp-1,izp  )
     ^		 +	      fx *e_theta_mc(irp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_theta_mc(irp-1,izp-1)
     ^		 +	  fx *e_theta_mc(irp,izp-1))*(1.-fz)) * expiwt)
	
	      acym = (acxm * yk + actm * xk) / rk
	      acxm = (acxm * xk - actm * yk) / rk
 
	      vzk = vz + kvz(3)
	      vxk = vx + kvx(3)
	      vyk = vy + kvy(3)
c fourth push
	      kx(4)  = dt * vxk
	      kvx(4) = dt * (acxm + vyk*bvz - vzk*bvy)
	      ky(4)  = dt * vyk
	      kvy(4) = dt * (acym + vzk*bvx - vxk*bvz)
	      kz(4)  = dt * vzk
	      kvz(4) = dt * (aczm + vxk*bvy - vyk*bvx)
 
	      xp = x + (kx(1) + kx(4) + 2.*(kx(2)+kx(3))) / 6.0
	      yp = y + (ky(1) + ky(4) + 2.*(ky(2)+ky(3))) / 6.0
	      zp = z + (kz(1) + kz(4) + 2.*(kz(2)+kz(3))) / 6.0
	      vxp = vx + (kvx(1) + kvx(4) + 2.*(kvx(2)+kvx(3))) / 6.0
	      vyp = vy + (kvy(1) + kvy(4) + 2.*(kvy(2)+kvy(3))) / 6.0
	      vzp = vz + (kvz(1) + kvz(4) + 2.*(kvz(2)+kvz(3))) / 6.0
	  endif
	
	  rr = r
	  r = sqrt(xp*xp + yp*yp)
	  if (r.eq.0.) then
		r  = rt_mc(2) * 1.e-3
		xp = r * sign(1., .5 - ranf_mc(2))
	  endif
 
c         iz = min(nz_mc+1,
c     ^  	abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 ))
c	  if (zp.lt.z_min_mc) iz=0
	  iz = find_iz(zp, zp-z, iz)
 
c	  ir = min(nr_mc+1, int( r * delta_ri_mc ) + 1 )
	  ir = find_ir(r,r-rr,ir)
	  c_th = xp / r
	  s_th = yp / r
 
	  pow_i(ir,iz,nreg) = pow_i(ir,iz,nreg) +
     ^	    .25 * (actm + actm0) * (c_th*(vy+vyp) - s_th*(vx+vxp))*dt
 
	else !  1d-r
 
c find initial acceleration...
	    fx=min(1.,max(0.,(r-rt_mc(ir))*dri_mc(ir)))
	    expiwt = (-1.75880521211e15,0.)*exp((0.,-1.)*omega0_mc*t1)
 
	    bvz =(1.-fx)*abz(ir-1,1)+fx*abz(ir,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_z_mc(ir-1,1)+fx*b_z_mc(ir,1))*expiwt)
	    bvr =(1.-fx)*abr(ir-1,1)+fx*abr(ir,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_r_mc(ir-1,1)+fx*b_r_mc(ir,1))*expiwt)
	    bvt =(1.-fx)*abt(ir-1,1)+fx*abt(ir,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_th_mc(ir-1,1)+fx*b_th_mc(ir,1))*expiwt)
	    bvx = c_th * bvr - s_th * bvt
	    bvy = c_th * bvt + s_th * bvr
	    acxm = (1.-fx)*acx(ir-1,1)+fx*acx(ir,1) +real(expiwt*(
     ^		(1.-fx)*e_r_mc(ir-1,1)+fx*e_r_mc(ir,1)))
	    aczm = (1.-fx)*acz(ir-1,1)+fx*acz(ir,1) +real(expiwt*(
     ^		(1.-fx)*e_z_mc(ir-1,1)+fx*e_z_mc(ir,1)))
	    actm0 = real( expiwt * (
     ^		(1.-fx)*e_theta_mc(ir-1,1)+fx*e_theta_mc(ir,1)))
	    acym = acxm * s_th + actm0 * c_th
	    acxm = acxm * c_th - actm0 * s_th
 
c first push
	    kx(1)  = dt * vx
	    kvx(1) = dt * (acxm + vy*bvz - vz*bvy)
	    ky(1)  = dt * vy
	    kvy(1) = dt * (acym + vz*bvx - vx*bvz)
	    kz(1)  = dt * vz
	    kvz(1) = dt * (aczm + vx*bvy - vy*bvx)
 
c find accel
	    xk = x + 0.5 * kx(1)
	    yk = y + 0.5 * ky(1)
 
	    rk = sqrt(xk*xk + yk*yk)
	    if (rk.eq.0.) then
		rk = rt_mc(2) * 1.e-3
		xk = rk * sign(1., .5 - ranf_mc(2))
	    endif
c	    irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	    irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir) ))
	    fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	    expiwt = (-1.75880521211e15,0.)*exp((0.,-1.)*
     ^			omega0_mc*(t1+.5*dt))
 
	    bvz =(1.-fx)*abz(irp-1,1)+fx*abz(irp,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_z_mc(irp-1,1)+fx*b_z_mc(irp,1))*expiwt)
	    bvr =(1.-fx)*abr(irp-1,1)+fx*abr(irp,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_r_mc(irp-1,1)+fx*b_r_mc(irp,1))*expiwt)
	    bvt =(1.-fx)*abt(irp-1,1)+fx*abt(irp,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_th_mc(irp-1,1)+fx*b_th_mc(irp,1))*expiwt)
	    bvx = (xk * bvr - yk * bvt) / rk
	    bvy = (xk * bvt + yk * bvr) / rk
 
	    acxm = (1.-fx)*acx(irp-1,1)+fx*acx(irp,1) +real(expiwt*(
     ^		(1.-fx)*e_r_mc(irp-1,1)+fx*e_r_mc(irp,1)))
	    aczm = (1.-fx)*acz(irp-1,1)+fx*acz(irp,1) +real(expiwt*(
     ^		(1.-fx)*e_z_mc(irp-1,1)+fx*e_z_mc(irp,1)))
	    actm = real( expiwt * (
     ^		(1.-fx)*e_theta_mc(irp-1,1)+fx*e_theta_mc(irp,1)))
	    acym = (acxm * yk + actm * xk) / rk
	    acxm = (acxm * xk - actm * yk) / rk
 
	    vxk = vx + .5 * kvx(1)
	    vyk = vy + .5 * kvy(1)
	    vzk = vy + .5 * kvz(1)
 
	    if ( push_order .eq. 2 ) then
	      xp = x + dt * vxk
	      yp = y + dt * vyk
	      zp = z + dt * vzk
	      vxp = vx + dt * (acxm + vzk*bvx - vxk*bvz)
	      vyp = vy + dt * (acym + vzk*bvx - vxk*bvz)
	      vzp = vz + dt * (aczm + vxk*bvy - vyk*bvx)
	    else
c second push
	      kx(2)  = dt * vxk
	      kvx(2) = dt * (acxm + vyk*bvz - vzk*bvy)
	      ky(2)  = dt * vyk
	      kvy(2) = dt * (acym + vzk*bvx - vxk*bvz)
	      kz(2)  = dt * vzk
	      kvz(2) = dt * (aczm + vxk*bvy - vyk*bvx)
	
c find accel
	      xk = x + 0.5 * kx(2)
	      yk = y + 0.5 * ky(2)
 
	      rk = sqrt(xk*xk + yk*yk)
	      if (rk.eq.0.) then
		rk = rt_mc(2) * 1.e-3
		xk = rk * sign(1., .5 - ranf_mc(2))
	      endif
c	      irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	      irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir) ))
	      fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	      bvz =(1.-fx)*abz(irp-1,1)+fx*abz(irp,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_z_mc(irp-1,1)+fx*b_z_mc(irp,1))*expiwt)
	      bvr =(1.-fx)*abr(irp-1,1)+fx*abr(irp,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_r_mc(irp-1,1)+fx*b_r_mc(irp,1))*expiwt)
	      bvt =(1.-fx)*abt(irp-1,1)+fx*abt(irp,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_th_mc(irp-1,1)+fx*b_th_mc(irp,1))*expiwt)
	      bvx = (xk * bvr - yk * bvt) / rk
	      bvy = (xk * bvt + yk * bvr) / rk
 
	      acxm = (1.-fx)*acx(irp-1,1)+fx*acx(irp,1) +real(expiwt*(
     ^		(1.-fx)*e_r_mc(irp-1,1)+fx*e_r_mc(irp,1)))
	      aczm = (1.-fx)*acz(irp-1,1)+fx*acz(irp,1) +real(expiwt*(
     ^		(1.-fx)*e_z_mc(irp-1,1)+fx*e_z_mc(irp,1)))
	      actm = real( expiwt * (
     ^		(1.-fx)*e_theta_mc(irp-1,1)+fx*e_theta_mc(irp,1)))
	      acym = (acxm * yk + actm * xk) / rk
	      acxm = (acxm * xk - actm * yk) / rk
 
	      vxk = vx + .5 * kvx(2)
	      vyk = vy + .5 * kvy(2)
	      vzk = vy + .5 * kvz(2)
c third push
	      kx(3)  = dt * vxk
	      kvx(3) = dt * (acxm + vyk*bvz - vzk*bvy)
	      ky(3)  = dt * vyk
	      kvy(3) = dt * (acym + vzk*bvx - vxk*bvz)
	      kz(3)  = dt * vzk
	      kvz(3) = dt * (aczm + vxk*bvy - vyk*bvx)
 
c find accel
	      xk = x + kx(3)
	      yk = y + ky(3)
 
	      rk = sqrt(xk*xk + yk*yk)
	      if (rk.eq.0.) then
		rk = rt_mc(2) * 1.e-3
		xk = rk * sign(1., .5 - ranf_mc(2))
	      endif
c	      irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	      irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir) ))
	      fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	      expiwt = (-1.75880521211e15,0.)*exp((0.,-1.)*
     ^			omega0_mc*(t1+dt))
 
	      bvz =(1.-fx)*abz(irp-1,1)+fx*abz(irp,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_z_mc(irp-1,1)+fx*b_z_mc(irp,1))*expiwt)
	      bvr =(1.-fx)*abr(irp-1,1)+fx*abr(irp,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_r_mc(irp-1,1)+fx*b_r_mc(irp,1))*expiwt)
	      bvt =(1.-fx)*abt(irp-1,1)+fx*abt(irp,1) + 1.e-8*real(
     ^	       ((1.-fx)*b_th_mc(irp-1,1)+fx*b_th_mc(irp,1))*expiwt)
	      bvx = (xk * bvr - yk * bvt) / rk
	      bvy = (xk * bvt + yk * bvr) / rk
 
	      acxm = (1.-fx)*acx(irp-1,1)+fx*acx(irp,1) +real(expiwt*(
     ^		(1.-fx)*e_r_mc(irp-1,1)+fx*e_r_mc(irp,1)))
	      aczm = (1.-fx)*acz(irp-1,1)+fx*acz(irp,1) +real(expiwt*(
     ^		(1.-fx)*e_z_mc(irp-1,1)+fx*e_z_mc(irp,1)))
	      actm = real( expiwt * (
     ^		(1.-fx)*e_theta_mc(irp-1,1)+fx*e_theta_mc(irp,1)))
	      acym = (acxm * yk + actm * xk) / rk
	      acxm = (acxm * xk - actm * yk) / rk
 
	      vxk = vx + kvx(3)
	      vyk = vy + kvy(3)
	      vzk = vy + kvz(3)
c fourth push
	      kx(4)  = dt * vxk
	      kvx(4) = dt * (acxm + vyk*bvz - vzk*bvy)
	      ky(4)  = dt * vyk
	      kvy(4) = dt * (acym + vzk*bvx - vxk*bvz)
	      kz(4)  = dt * vzk
	      kvz(4) = dt * (aczm + vxk*bvy - vyk*bvx)
 
	      xp = x + (kx(1) + kx(4) + 2.*(kx(2)+kx(3))) / 6.0
	      yp = y + (ky(1) + ky(4) + 2.*(ky(2)+ky(3))) / 6.0
	      zp = z + (kz(1) + kz(4) + 2.*(kz(2)+kz(3))) / 6.0
	      vxp = vx + (kvx(1) + kvx(4) + 2.*(kvx(2)+kvx(3))) / 6.0
	      vyp = vy + (kvy(1) + kvy(4) + 2.*(kvy(2)+kvy(3))) / 6.0
	      vzp = vz + (kvz(1) + kvz(4) + 2.*(kvz(2)+kvz(3))) / 6.0
	    endif
	
	    rr = r
	    r = sqrt(xp*xp + yp*yp)
	    if (r.eq.0.) then
		r  = rt_mc(2) * 1.e-3
		xp = r * sign(1., .5 - ranf_mc(2))
	    endif
 
c	    ir = min(nr_mc+1, int( r * delta_ri_mc ) + 1 )
	    ir = find_ir(r, r-rr, ir)
	    c_th = xp / r
	    s_th = yp / r
 
	    pow_i(ir,iz,nreg) = pow_i(ir,iz,nreg) +
     ^	      .25 * (actm + actm0) * (c_th*(vy+vyp) - s_th*(vx+vxp))*dt
 
	endif
 
	return
	end
 
c---------------------------------------------------------------------72
 
	subroutine accel_cyl(x,y,z,vx,vy,vz,xp,yp,zp,vxp,vyp,vzp,
     ^		t1,dt,ir0,iz0,ir,iz,acx,acz,r,c_th,s_th,acxm,aczm,
     ^		abr,abt,abz)
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
	real*4 acz(0:nrmax_mc,0:nzmax_mc)
	real*4 acx(0:nrmax_mc,0:nzmax_mc),rr
	real*8 t1
	real*4 abt(0:nrmax_mc,0:nzmax_mc)
	real*4 abr(0:nrmax_mc,0:nzmax_mc)
	real*4 abz(0:nrmax_mc,0:nzmax_mc),bvz,bvt,bvr
 
c	real*8 x,y,z,vx,vy,vz,xp,yp,zp,vxp,vyp,vzp
 
	integer find_iz, find_ir
 
c	real*8 kx(4),kvx(4),kz(4),kvz(4),ky(4),kvy(4)
	real   kx(4),kvx(4),kz(4),kvz(4),ky(4),kvy(4)
 
 
	ir0 = ir
	iz0 = iz
	    if (nz_mc.ne.1) then
 
c find initial acceleration...
	      fz=min(1.,max(0.,(z-zt_mc(iz))*dzi_mc(iz)))
	      fx=min(1.,max(0.,(r-rt_mc(ir))*dri_mc(ir)))
 
	      bvz =((1.-fx)*abz(ir-1,iz  )+fx*abz(ir,iz  ))*fz
     ^		 + ((1.-fx)*abz(ir-1,iz-1)+fx*abz(ir,iz-1))*(1.-fz)
	      bvr =((1.-fx)*abr(ir-1,iz  )+fx*abr(ir,iz  ))*fz
     ^		 + ((1.-fx)*abr(ir-1,iz-1)+fx*abr(ir,iz-1))*(1.-fz)
	      bvt =((1.-fx)*abt(ir-1,iz  )+fx*abt(ir,iz  ))*fz
     ^		 + ((1.-fx)*abt(ir-1,iz-1)+fx*abt(ir,iz-1))*(1.-fz)
	      bvx = c_th * bvr - s_th * bvt
	      bvy = c_th * bvt + s_th * bvr
	
	      aczm = ((1.-fx)*acz(ir-1,iz) +
     ^		          fx *acz(ir  ,iz))*fz
     ^	        +    ((1.-fx)*acz(ir-1,iz-1) +
     ^		          fx *acz(ir  ,iz-1))*(1.-fz)
	      acxm = ((1.-fx)*acx(ir-1,iz) +
     ^	    	          fx *acx(ir  ,iz))*fz
     ^	        +    ((1.-fx)*acx(ir-1,iz-1) +
     ^		          fx *acx(ir  ,iz-1))*(1.-fz)
 
c first push
	      kx(1)  = dt * vx
	      kvx(1) = dt * (acxm * c_th + vy * bvz - vz * bvy)
	      ky(1)  = dt * vy
	      kvy(1) = dt * (acxm * s_th + vz * bvx - vx * bvz)
	      kz(1)  = dt * vz
	      kvz(1) = dt * (aczm + vx * bvy - vy * bvx)
 
c find accel
	      zk = z + 0.5 * kz(1)
	      xk = x + 0.5 * kx(1)
	      yk = y + 0.5 * ky(1)
 
c	      izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	      izp = min(nz_mc, max(1, find_iz(zk,zk-z,iz)))
	      fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	      rk = sqrt(xk*xk + yk*yk)
	      if (rk.eq.0.) then
		rk = rt_mc(2) * 1.e-3
		xk = rk * sign(1., .5 - ranf_mc(2))
	      endif
c	      irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	      irp = min(nr_mc, max(1, find_ir( rk, rk - r, ir)))
	      fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	      bvz =((1.-fx)*abz(irp-1,izp  )+fx*abz(irp,izp  ))*fz
     ^		 + ((1.-fx)*abz(irp-1,izp-1)+fx*abz(irp,izp-1))*(1.-fz)
	      bvr =((1.-fx)*abr(irp-1,izp  )+fx*abr(irp,izp  ))*fz
     ^		 + ((1.-fx)*abr(irp-1,izp-1)+fx*abr(irp,izp-1))*(1.-fz)
	      bvt =((1.-fx)*abt(irp-1,izp  )+fx*abt(irp,izp  ))*fz
     ^		 + ((1.-fx)*abt(irp-1,izp-1)+fx*abt(irp,izp-1))*(1.-fz)
	      bvx = (xk * bvr - yk * bvt) / rk
	      bvy = (xk * bvt + yk * bvr) / rk
	
	      aczm = ((1.-fx)*acz(irp-1,izp) +
     ^		          fx *acz(irp  ,izp))*fz
     ^	        +    ((1.-fx)*acz(irp-1,izp-1) +
     ^		          fx *acz(irp  ,izp-1))*(1.-fz)
	      acxm = ((1.-fx)*acx(irp-1,izp) +
     ^	    	          fx *acx(irp  ,izp))*fz
     ^	        +    ((1.-fx)*acx(irp-1,izp-1) +
     ^		          fx *acx(irp  ,izp-1))*(1.-fz)
 
	      vzk = vz + .5 * kvz(1)
	      vxk = vx + .5 * kvx(1)
	      vyk = vy + .5 * kvy(1)
 
	      if ( push_order .eq. 2 ) then
	        xp = x + dt * vxk
	        yp = y + dt * vyk
	        zp = z + dt * vzk
	        vxp = vx + dt * (acxm * yk / rk + vzk*bvx - vxk*bvz)
	        vyp = vy + dt * (acxm * xk / rk + vzk*bvx - vxk*bvz)
	        vzp = vz + dt * (aczm + vxk*bvy - vyk*bvx)
	      else
c second push
	        kx(2)  = dt * vxk
	        kvx(2) = dt * (acxm * xk / rk + vyk*bvz - vzk*bvy)
	        ky(2)  = dt * vyk
	        kvy(2) = dt * (acxm * yk / rk + vzk*bvx - vxk*bvz)
	        kz(2)  = dt * vzk
	        kvz(2) = dt * (aczm + vxk*bvy - vyk*bvx)
 
c find accel
	        zk = z + 0.5 * kz(2)
	        xk = x + 0.5 * kx(2)
	        yk = y + 0.5 * ky(2)
 
c	        izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	        izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	        fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	        rk = sqrt(xk*xk + yk*yk)
	        if (rk.eq.0.) then
		  rk = rt_mc(2) * 1.e-3
		  xk = rk * sign(1., .5 - ranf_mc(2))
	        endif
c	        irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	        irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir)))
	        fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	        bvz =((1.-fx)*abz(irp-1,izp  )+fx*abz(irp,izp  ))*fz
     ^		  +((1.-fx)*abz(irp-1,izp-1)+fx*abz(irp,izp-1))*(1.-fz)
	        bvr =((1.-fx)*abr(irp-1,izp  )+fx*abr(irp,izp  ))*fz
     ^		  +((1.-fx)*abr(irp-1,izp-1)+fx*abr(irp,izp-1))*(1.-fz)
	        bvt =((1.-fx)*abt(irp-1,izp  )+fx*abt(irp,izp  ))*fz
     ^		  +((1.-fx)*abt(irp-1,izp-1)+fx*abt(irp,izp-1))*(1.-fz)
 
	        bvx = (xk * bvr - yk * bvt) / rk
	        bvy = (xk * bvt + yk * bvr) / rk
	        aczm = ((1.-fx)*acz(irp-1,izp) +
     ^		            fx *acz(irp  ,izp))*fz
     ^	        +      ((1.-fx)*acz(irp-1,izp-1) +
     ^		            fx *acz(irp  ,izp-1))*(1.-fz)
	        acxm = ((1.-fx)*acx(irp-1,izp) +
     ^	    	            fx *acx(irp  ,izp))*fz
     ^	        +      ((1.-fx)*acx(irp-1,izp-1) +
     ^		            fx *acx(irp  ,izp-1))*(1.-fz)
 
	        vzk = vz + .5 * kvz(2)
	        vxk = vx + .5 * kvx(2)
	        vyk = vy + .5 * kvy(2)
c third push
	        kx(3)  = dt * vxk
	        kvx(3) = dt * (acxm * xk / rk + vyk*bvz - vzk*bvy)
	        ky(3)  = dt * vyk
	        kvy(3) = dt * (acxm * yk / rk + vzk*bvx - vxk*bvz)
	        kz(3)  = dt * vzk
	        kvz(3) = dt * (aczm + vxk*bvy - vyk*bvx)
 
c find accel
	        zk = z + kz(3)
	        xk = x + kx(3)
	        yk = y + ky(3)
 
c	        izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	        izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	        fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	        rk = sqrt(xk*xk + yk*yk)
	        if (rk.eq.0.) then
		  rk = rt_mc(2) * 1.e-3
		  xk = rk * sign(1., .5 - ranf_mc(2))
	        endif
c	        irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	        irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir) ))
	        fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	        bvz =((1.-fx)*abz(irp-1,izp  )+fx*abz(irp,izp  ))*fz
     ^		 + ((1.-fx)*abz(irp-1,izp-1)+fx*abz(irp,izp-1))*(1.-fz)
	        bvr =((1.-fx)*abr(irp-1,izp  )+fx*abr(irp,izp  ))*fz
     ^		 + ((1.-fx)*abr(irp-1,izp-1)+fx*abr(irp,izp-1))*(1.-fz)
	        bvt =((1.-fx)*abt(irp-1,izp  )+fx*abt(irp,izp  ))*fz
     ^		 + ((1.-fx)*abt(irp-1,izp-1)+fx*abt(irp,izp-1))*(1.-fz)
	        bvx = (xk * bvr - yk * bvt) / rk
	        bvy = (xk * bvt + yk * bvr) / rk
	        aczm = ((1.-fx)*acz(irp-1,izp) +
     ^		            fx *acz(irp  ,izp))*fz
     ^	        +      ((1.-fx)*acz(irp-1,izp-1) +
     ^		            fx *acz(irp  ,izp-1))*(1.-fz)
	        acxm = ((1.-fx)*acx(irp-1,izp) +
     ^	    	            fx *acx(irp  ,izp))*fz
     ^	        +      ((1.-fx)*acx(irp-1,izp-1) +
     ^		            fx *acx(irp  ,izp-1))*(1.-fz)
 
	        vzk = vz + kvz(3)
	        vxk = vx + kvx(3)
	        vyk = vy + kvy(3)
c fourth push
	        kx(4)  = dt * vxk
	        kvx(4) = dt * (acxm * xk / rk + vyk*bvz - vzk*bvy)
	        ky(4)  = dt * vyk
	        kvy(4) = dt * (acxm * yk / rk + vzk*bvx - vxk*bvz)
	        kz(4)  = dt * vzk
	        kvz(4) = dt * (aczm + vxk*bvy - vyk*bvx)
 
	        xp = x + (kx(1) + kx(4) + 2.*(kx(2)+kx(3))) / 6.0
	        yp = y + (ky(1) + ky(4) + 2.*(ky(2)+ky(3))) / 6.0
	        zp = z + (kz(1) + kz(4) + 2.*(kz(2)+kz(3))) / 6.0
	        vxp = vx + (kvx(1) + kvx(4) + 2.*(kvx(2)+kvx(3))) / 6.0
	        vyp = vy + (kvy(1) + kvy(4) + 2.*(kvy(2)+kvy(3))) / 6.0
	        vzp = vz + (kvz(1) + kvz(4) + 2.*(kvz(2)+kvz(3))) / 6.0
	      endif !4th order
	
	      rr = r
	      r = sqrt(xp*xp + yp*yp)
	      if (r.eq.0.) then
		r  = rt_mc(2) * 1.e-3
		xp = r * sign(1., .5 - ranf_mc(2))
	      endif
 
c              iz = min(nz_mc+1,
c     ^  	abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 ))
c	      if (zp.lt.z_min_mc) iz=0
	      iz = find_iz(zp, zp-z, iz)
 
c	      ir = min(nr_mc+1, int( r * delta_ri_mc ) + 1 )
	      ir = find_ir(r,r-rr,ir)
	      c_th = xp / r
	      s_th = yp / r
 
	    else !  1d-r
c	      aczm = acz(1,1)
c	      zp   = z + dt * (vz + 0.5 * aczm * dt)
c	      vzp  = vz + dt * acz(1,1)
 
c find initial acceleration...
	      fx=min(1.,max(0.,(r-rt_mc(ir))*dri_mc(ir)))
	      bvz =(1.-fx)*abz(ir-1,iz  )+fx*abz(ir,iz  )
	      bvr =(1.-fx)*abr(ir-1,iz  )+fx*abr(ir,iz  )
	      bvt =(1.-fx)*abt(ir-1,iz  )+fx*abt(ir,iz  )
	      bvx = c_th * bvr - s_th * bvt
	      bvy = c_th * bvt + s_th * bvr
	      acxm = (1.-fx)*acx(ir-1,1) + fx*acx(ir,1)
	      aczm = (1.-fx)*acz(ir-1,1) + fx*acz(ir,1)
 
c first push
	      kx(1)  = dt * vx
	      kvx(1) = dt * (acxm * c_th + vy*bvz - vz*bvy)
	      ky(1)  = dt * vy
	      kvy(1) = dt * (acxm * s_th + vz*bvx - vx*bvz)
	      kz(1)  = dt * vz
	      kvz(1) = dt * (aczm + vx*bvy - vy*bvx)
 
c find accel
	      xk = x + 0.5 * kx(1)
	      yk = y + 0.5 * ky(1)
 
	      rk = sqrt(xk*xk + yk*yk)
	      if (rk.eq.0.) then
		rk = rt_mc(2) * 1.e-3
		xk = rk * sign(1., .5 - ranf_mc(2))
	      endif
c	      irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	      irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir) ))
	      fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	      bvz =(1.-fx)*abz(irp-1,1)+fx*abz(irp,1)
	      bvr =(1.-fx)*abr(irp-1,1)+fx*abr(irp,1)
	      bvt =(1.-fx)*abt(irp-1,1)+fx*abt(irp,1)
	      bvx = (xk * bvr - yk * bvt) / rk
	      bvy = (xk * bvt + yk * bvr) / rk
	      acxm = (1.-fx)*acx(irp-1,1) + fx*acx(irp,1)
	      aczm = (1.-fx)*acz(irp-1,1) + fx*acz(irp,1)
 
	      vxk = vx + .5 * kvx(1)
	      vyk = vy + .5 * kvy(1)
	      vzk = vy + .5 * kvz(1)
 
	      if ( push_order .eq. 2 ) then
	        xp = x + dt * vxk
	        yp = y + dt * vyk
	        zp = z + dt * vzk
	        vxp = vx + dt * (acxm * yk / rk + vzk*bvx - vxk*bvz)
	        vyp = vy + dt * (acxm * xk / rk + vzk*bvx - vxk*bvz)
	        vzp = vz + dt * (aczm + vxk*bvy - vyk*bvx)
	      else
c second push
	        kx(2)  = dt * vxk
	        kvx(2) = dt * (acxm * xk / rk + vyk*bvz - vzk*bvy)
	        ky(2)  = dt * vyk
	        kvy(2) = dt * (acxm * yk / rk + vzk*bvx - vxk*bvz)
	        kz(2)  = dt * vzk
	        kvz(2) = dt * (aczm + vxk*bvy - vyk*bvx)
	
c find accel
	        xk = x + 0.5 * kx(2)
	        yk = y + 0.5 * ky(2)
 
	        rk = sqrt(xk*xk + yk*yk)
	        if (rk.eq.0.) then
		  rk = rt_mc(2) * 1.e-3
		  xk = rk * sign(1., .5 - ranf_mc(2))
	        endif
c	        irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	        irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir) ))
	        fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	        bvz =(1.-fx)*abz(irp-1,1)+fx*abz(irp,1)
	        bvr =(1.-fx)*abr(irp-1,1)+fx*abr(irp,1)
	        bvt =(1.-fx)*abt(irp-1,1)+fx*abt(irp,1)
	        bvx = (xk * bvr - yk * bvt) / rk
	        bvy = (xk * bvt + yk * bvr) / rk
	        acxm = (1.-fx)*acx(irp-1,1) + fx*acx(irp,1)
	        aczm = (1.-fx)*acz(irp-1,1) + fx*acz(irp,1)
 
	        vxk = vx + .5 * kvx(2)
	        vyk = vy + .5 * kvy(2)
	        vzk = vy + .5 * kvz(2)
c third push
	        kx(3)  = dt * vxk
	        kvx(3) = dt * (acxm * xk / rk + vyk*bvz - vzk*bvy)
	        ky(3)  = dt * vyk
	        kvy(3) = dt * (acxm * yk / rk + vzk*bvx - vxk*bvz)
	        kz(3)  = dt * vzk
	        kvz(3) = dt * (aczm + vxk*bvy - vyk*bvx)
 
c find accel
	        xk = x + kx(3)
	        yk = y + ky(3)
 
	        rk = sqrt(xk*xk + yk*yk)
	        if (rk.eq.0.) then
		  rk = rt_mc(2) * 1.e-3
		  xk = rk * sign(1., .5 - ranf_mc(2))
	        endif
c	        irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	        irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir) ))
	        fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
	        bvz =(1.-fx)*abz(irp-1,1)+fx*abz(irp,1)
	        bvr =(1.-fx)*abr(irp-1,1)+fx*abr(irp,1)
	        bvt =(1.-fx)*abt(irp-1,1)+fx*abt(irp,1)
	        bvx = (xk * bvr - yk * bvt) / rk
	        bvy = (xk * bvt + yk * bvr) / rk
	        acxm = (1.-fx)*acx(irp-1,1) + fx*acx(irp,1)
	        aczm = (1.-fx)*acz(irp-1,1) + fx*acz(irp,1)
 
	        vxk = vx + kvx(3)
	        vyk = vy + kvy(3)
	        vzk = vy + kvz(3)
c fourth push
	        kx(4)  = dt * vxk
	        kvx(4) = dt * (acxm * xk / rk + vyk*bvz - vzk*bvy)
	        ky(4)  = dt * vyk
	        kvy(4) = dt * (acxm * yk / rk + vzk*bvx - vxk*bvz)
	        kz(4)  = dt * vzk
	        kvz(4) = dt * (aczm + vxk*bvy - vyk*bvx)
 
	        xp = x + (kx(1) + kx(4) + 2.*(kx(2)+kx(3))) / 6.0
	        yp = y + (ky(1) + ky(4) + 2.*(ky(2)+ky(3))) / 6.0
	        zp = z + (kz(1) + kz(4) + 2.*(kz(2)+kz(3))) / 6.0
	        vxp = vx + (kvx(1) + kvx(4) + 2.*(kvx(2)+kvx(3))) / 6.0
	        vyp = vy + (kvy(1) + kvy(4) + 2.*(kvy(2)+kvy(3))) / 6.0
	        vzp = vz + (kvz(1) + kvz(4) + 2.*(kvz(2)+kvz(3))) / 6.0
	      endif
	
	      rr = r
	      r = sqrt(xp*xp + yp*yp)
	      if (r.eq.0.) then
		r  = rt_mc(2) * 1.e-3
		xp = r * sign(1., .5 - ranf_mc(2))
	      endif
 
c	      ir = min(nr_mc+1, int( r * delta_ri_mc ) + 1 )
	      ir = find_ir(r, r-rr, ir)
	      c_th = xp / r
	      s_th = yp / r
 
	    endif
 
	return
	end
 
c---------------------------------------------------------------------72
 
	subroutine accel_cart_h(x,y,z,vx,vy,vz,xp,yp,zp,vxp,vyp,vzp,
     ^	      t1,dt,ir0,iz0,ir,iz,acx,acz,acxm,aczm,abx,aby,abz,nreg)
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
	real*4 acz(0:nrmax_mc,0:nzmax_mc)
	real*4 acx(0:nrmax_mc,0:nzmax_mc)
	real*4 aby(0:nrmax_mc,0:nzmax_mc)
	real*4 abx(0:nrmax_mc,0:nzmax_mc)
	real*4 abz(0:nrmax_mc,0:nzmax_mc),bvz,bvy,bvx
	real*8 t1
 
	integer find_iz, find_ir
 
	real kx(4),kvx(4),kz(4),kvz(4),ky(4),kvy(4)
 
	complex*8 expiwt
 
	ir0 = ir
	iz0 = iz
	if (nr_mc.ne.1) then
 
c initial acceleration...
	  fz=min(1.,max(0.,(z-zt_mc(iz))*dzi_mc(iz)))
	  fx=min(1.,max(0.,(x-rt_mc(ir))*dri_mc(ir)))
	  expiwt = (-1.75880521211e15,0.)*exp((0.,-1.)*omega0_mc*t1)
 
	  bvz =    ((1.-fx)*abz(ir-1,iz  )+fx*abz(ir,iz  ))*fz
     ^		 + ((1.-fx)*abz(ir-1,iz-1)+fx*abz(ir,iz-1))*(1.-fz)
     ^	         + 1.e-8 * real((
     ^		 +     ((1.-fx)*b_z_mc(ir-1,iz  )
     ^		 +	    fx *b_z_mc(ir  ,iz  ))*fz
     ^		 +     ((1.-fx)*b_z_mc(ir-1,iz-1)
     ^		 +          fx *b_z_mc(ir  ,iz-1))*(1.-fz)) * expiwt)
	  bvx =    ((1.-fx)*abx(ir-1,iz  )+fx*abx(ir,iz  ))*fz
     ^		 + ((1.-fx)*abx(ir-1,iz-1)+fx*abx(ir,iz-1))*(1.-fz)
     ^	         + 1.e-8 * real((
     ^		 +     ((1.-fx)*b_r_mc(ir-1,iz  )
     ^		 +	    fx *b_r_mc(ir  ,iz  ))*fz
     ^		 +     ((1.-fx)*b_r_mc(ir-1,iz-1)
     ^		 +	    fx *b_r_mc(ir  ,iz-1))*(1.-fz)) * expiwt)
	  bvy =    ((1.-fx)*aby(ir-1,iz  )+fx*aby(ir,iz  ))*fz
     ^		 + ((1.-fx)*aby(ir-1,iz-1)+fx*aby(ir,iz-1))*(1.-fz)
     ^	         + 1.e-8 * real((
     ^		 +     ((1.-fx)*b_th_mc(ir-1,iz  )
     ^		 +	    fx *b_th_mc(ir  ,iz  ))*fz
     ^		 +     ((1.-fx)*b_th_mc(ir-1,iz-1)
     ^		 +	    fx *b_th_mc(ir  ,iz-1))*(1.-fz)) * expiwt)
	
	  aczm =         ((1.-fx)*acz(   ir-1,iz) +
     ^		              fx *acz(   ir  ,iz))*fz
     ^	        +        ((1.-fx)*acz(   ir-1,iz-1) +
     ^		              fx *acz(   ir  ,iz-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_z_mc(ir-1,iz  )
     ^		 +	      fx *e_z_mc(ir  ,iz  ))*fz
     ^		 +       ((1.-fx)*e_z_mc(ir-1,iz-1)
     ^		 +	      fx *e_z_mc(ir  ,iz-1))*(1.-fz)) * expiwt)
	  acxm =            ((1.-fx)*acx(ir-1,iz) +
     ^	    	                 fx *acx(ir  ,iz))*fz
     ^	        +           ((1.-fx)*acx(ir-1,iz-1) +
     ^		                 fx *acx(ir  ,iz-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_r_mc(ir-1,iz  )
     ^		 +	      fx *e_r_mc(ir  ,iz  ))*fz
     ^		 +       ((1.-fx)*e_r_mc(ir-1,iz-1)
     ^		 +	      fx *e_r_mc(ir  ,iz-1))*(1.-fz)) * expiwt)
	  acym0 =
     ^	           real((((1.-fx)*e_theta_mc(ir-1,iz  )
     ^		 +	      fx *e_theta_mc(ir  ,iz  ))*fz
     ^		 +       ((1.-fx)*e_theta_mc(ir-1,iz-1)
     ^		 +	      fx *e_theta_mc(ir  ,iz-1))*(1.-fz)) 
     ^		 * expiwt)
	
c 1st push
	  kx( 1) = dt * vx
	  kvx(1) = dt * (acxm + vy * bvz - vz * bvy)
	  kz( 1) = dt * vz
	  kvz(1) = dt * (aczm + vx * bvy - vy * bvx)
	  ky( 1) = dt * vy
	  kvy(1) = dt * (acym0 + vz * bvx - vx * bvz)
 
c find accel
	  vzk = vz + .5 * kvz(1)
	  vxk = vx + .5 * kvx(1)
	  vyk = vy + .5 * kvy(1)
	  zk = z + 0.5 * kz(1)
	  xk = x + 0.5 * kx(1)
c	  izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	  izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	  fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	  if (xk.lt.0.0.and.mask_mc(0,izp).eq.0) xk=abs(xk)
	  xk = max( 0., xk )
c	  ixp = min(Nr_mc, max(1,
c     ^            iabs( int(  xk * delta_ri_mc ) + 1 )))
	  ixp = min(nr_mc, max(1, find_ir(xk, xk-x, ir) ))
	  fx=min(1.,max(0.,(xk-rt_mc(ixp))*dri_mc(ixp)))
	  expiwt = (-1.75880521211e15,0.) *
     ^			exp((0.,-1.)*omega0_mc*(t1+.5*dt))
 
	  bvz =    ((1.-fx)*abz(ixp-1,izp  )+fx*abz(ixp,izp  ))*fz
     ^		 + ((1.-fx)*abz(ixp-1,izp-1)+fx*abz(ixp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((
     ^		 +     ((1.-fx)*b_z_mc(ixp-1,izp  )
     ^		 +	    fx *b_z_mc(ixp  ,izp  ))*fz
     ^		 +     ((1.-fx)*b_z_mc(ixp-1,izp-1)
     ^		 +	    fx *b_z_mc(ixp  ,izp-1))*(1.-fz)) * expiwt)
	  bvx =    ((1.-fx)*abx(ixp-1,izp  )+fx*abx(ixp,izp  ))*fz
     ^		 + ((1.-fx)*abx(ixp-1,izp-1)+fx*abx(ixp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((
     ^		 +     ((1.-fx)*b_r_mc(ixp-1,izp  )
     ^		 +	    fx *b_r_mc(ixp  ,izp  ))*fz
     ^		 +     ((1.-fx)*b_r_mc(ixp-1,izp-1)
     ^		 +	    fx *b_r_mc(ixp  ,izp-1))*(1.-fz)) * expiwt)
	  bvy =    ((1.-fx)*aby(ixp-1,izp  )+fx*aby(ixp,izp  ))*fz
     ^		 + ((1.-fx)*aby(ixp-1,izp-1)+fx*aby(ixp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((
     ^		 +     ((1.-fx)*b_th_mc(ixp-1,izp  )
     ^		 +	    fx *b_th_mc(ixp  ,izp  ))*fz
     ^		 +     ((1.-fx)*b_th_mc(ixp-1,izp-1)
     ^		 +	    fx *b_th_mc(ixp  ,izp-1))*(1.-fz)) * expiwt)
	
	  aczm =           ((1.-fx)*acz(ixp-1,izp) +
     ^		                fx *acz(ixp  ,izp))*fz
     ^	        +          ((1.-fx)*acz(ixp-1,izp-1) +
     ^		                fx *acz(ixp  ,izp-1))*(1.-fz)
     ^	         +real((((1.-fx)*e_z_mc(ixp-1,izp  )
     ^		 +	     fx *e_z_mc(ixp  ,izp  ))*fz
     ^		 +      ((1.-fx)*e_z_mc(ixp-1,izp-1)
     ^		 +	     fx *e_z_mc(ixp  ,izp-1))*(1.-fz))* expiwt)
	  acxm =           ((1.-fx)*acx(ixp-1,izp) +
     ^	    	                fx *acx(ixp  ,izp))*fz
     ^	        +          ((1.-fx)*acx(ixp-1,izp-1) +
     ^		                fx *acx(ixp  ,izp-1))*(1.-fz)
     ^	         +real((((1.-fx)*e_r_mc(ixp-1,izp  )
     ^		 +	     fx *e_r_mc(ixp  ,izp  ))*fz
     ^		 +      ((1.-fx)*e_r_mc(ixp-1,izp-1)
     ^		 +	    fx *e_r_mc(ixp  ,izp-1))*(1.-fz)) * expiwt)
	  acym =
     ^	          real((((1.-fx)*e_theta_mc(ixp-1,izp  )
     ^		 +	     fx *e_theta_mc(ixp  ,izp  ))*fz
     ^		 +      ((1.-fx)*e_theta_mc(ixp-1,izp-1)
     ^		 +	  fx *e_theta_mc(ixp,izp-1))*(1.-fz)) * expiwt)
 
	  if ( push_order .eq. 2 ) then
	    xp  = x + dt * vxk
	    yp  = y + dt * vyk
	    zp  = z + dt * vzk
	    vxp = vx + dt * (acxm * sign(1.,xk) + vyk * bvz - vzk * bvy)
	    vyp = vy + dt * (acym + vzk * bvx - vxk * bvz)
	    vzp = vz + dt * (aczm + vxk * bvy - vyk * bvx)
	  else
c 2nd push
	    kx( 2) = dt * vxk
	    kvx(2) = dt * (acxm * sign(1.,xk) + vyk * bvz - vzk * bvy)
	    kz( 2) = dt * vzk
	    kvz(2) = dt * (aczm + vxk * bvy - vyk * bvx)
	    ky( 2) = dt * vyk
	    kvy(2) = dt * (acym + vzk * bvx - vxk * bvz)
 
c find accel
	    vzk = vz + .5 * kvz(2)
	    vxk = vx + .5 * kvx(2)
	    vyk = vy + .5 * kvy(2)
	    zk = z + 0.5 * kz(2)
	    xk = x + 0.5 * kx(2)
c	    izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	    izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	    fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	    if (xk.lt.0.0.and.mask_mc(0,izp).eq.0) xk=abs(xk)
	    xk = max( 0., xk )
c	    ixp = min(Nr_mc, max(1,
c     ^            iabs( int(  xk * delta_ri_mc ) + 1 )))
	    ixp = min(nr_mc, max(1, find_ir(xk, xk-x, ir) ))
	    fx=min(1.,max(0.,(xk-rt_mc(ixp))*dri_mc(ixp)))
 
	    bvz =  ((1.-fx)*abz(ixp-1,izp  )+fx*abz(ixp,izp  ))*fz
     ^		 + ((1.-fx)*abz(ixp-1,izp-1)+fx*abz(ixp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_z_mc(ixp-1,izp  )
     ^		 +	    fx *b_z_mc(ixp  ,izp  ))*fz
     ^		 +     ((1.-fx)*b_z_mc(ixp-1,izp-1)
     ^		 +	    fx *b_z_mc(ixp  ,izp-1))*(1.-fz)) * expiwt)
	    bvx =  ((1.-fx)*abx(ixp-1,izp  )+fx*abx(ixp,izp  ))*fz
     ^		 + ((1.-fx)*abx(ixp-1,izp-1)+fx*abx(ixp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_r_mc(ixp-1,izp  )
     ^		 +	      fx *b_r_mc(ixp  ,izp  ))*fz
     ^		 +       ((1.-fx)*b_r_mc(ixp-1,izp-1)
     ^		 +	      fx *b_r_mc(ixp,izp-1))*(1.-fz)) * expiwt)
	    bvy =  ((1.-fx)*aby(ixp-1,izp  )+fx*aby(ixp,izp  ))*fz
     ^		 + ((1.-fx)*aby(ixp-1,izp-1)+fx*aby(ixp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_th_mc(ixp-1,izp  )
     ^		 +	      fx *b_th_mc(ixp  ,izp  ))*fz
     ^		 +       ((1.-fx)*b_th_mc(ixp-1,izp-1)
     ^		 +	     fx *b_th_mc(ixp,izp-1))*(1.-fz)) * expiwt)
	
	    aczm =   ((1.-fx)*acz(ixp-1,izp) +
     ^		          fx *acz(ixp  ,izp))*fz
     ^	        +    ((1.-fx)*acz(ixp-1,izp-1) +
     ^		          fx *acz(ixp  ,izp-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_z_mc(ixp-1,izp  )
     ^		 +	      fx *e_z_mc(ixp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_z_mc(ixp-1,izp-1)
     ^		 +	     fx *e_z_mc(ixp  ,izp-1))*(1.-fz))* expiwt)
	    acxm =   ((1.-fx)*acx(ixp-1,izp) +
     ^	    	          fx *acx(ixp  ,izp))*fz
     ^	        +    ((1.-fx)*acx(ixp-1,izp-1) +
     ^		          fx *acx(ixp  ,izp-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_r_mc(ixp-1,izp  )
     ^		 +	      fx *e_r_mc(ixp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_r_mc(ixp-1,izp-1)
     ^		 +	    fx *e_r_mc(ixp  ,izp-1))*(1.-fz)) * expiwt)
	    acym =
     ^	           real((((1.-fx)*e_theta_mc(ixp-1,izp  )
     ^		 +	      fx *e_theta_mc(ixp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_theta_mc(ixp-1,izp-1)
     ^		 +	  fx *e_theta_mc(ixp,izp-1))*(1.-fz)) * expiwt)
 
c 3rd push
	    kx( 3) = dt * vxk
	    kvx(3) = dt * (acxm * sign(1.,xk) + vyk * bvz - vzk * bvy)
	    kz( 3) = dt * vzk
	    kvz(3) = dt * (aczm + vxk * bvy - vyk * bvx)
	    ky( 3) = dt * vyk
	    kvy(3) = dt * (acym + vzk * bvx - vxk * bvz)
 
c find accel
	    vzk = vz + kvz(3)
	    vxk = vx + kvx(3)
	    vyk = vy + kvy(3)
	    zk = z + kz(3)
	    xk = x + kx(3)
c	    izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	    izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	    fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	    if (xk.lt.0.0.and.mask_mc(0,izp).eq.0) xk=abs(xk)
	    xk = max( 0., xk )
c	    ixp = min(Nr_mc, max(1,
c     ^            iabs( int(  xk * delta_ri_mc ) + 1 )))
	    ixp = min(nr_mc, max(1, find_ir(xk, xk-x, ir) ))
	    fx=min(1.,max(0.,(xk-rt_mc(ixp))*dri_mc(ixp)))
	    expiwt = (-1.75880521211e15,0.) *
     ^			exp((0.,-1.)*omega0_mc*(t1+dt))
 
	    bvz =  ((1.-fx)*abz(ixp-1,izp  )+fx*abz(ixp,izp  ))*fz
     ^		 + ((1.-fx)*abz(ixp-1,izp-1)+fx*abz(ixp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_z_mc(ixp-1,izp  )
     ^		 +	      fx *b_z_mc(ixp  ,izp  ))*fz
     ^		 +       ((1.-fx)*b_z_mc(ixp-1,izp-1)
     ^		 +	      fx *b_z_mc(ixp,izp-1))*(1.-fz)) * expiwt)
	    bvx =  ((1.-fx)*abx(ixp-1,izp  )+fx*abx(ixp,izp  ))*fz
     ^		 + ((1.-fx)*abx(ixp-1,izp-1)+fx*abx(ixp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_r_mc(ixp-1,izp  )
     ^		 +	      fx *b_r_mc(ixp  ,izp  ))*fz
     ^		 +       ((1.-fx)*b_r_mc(ixp-1,izp-1)
     ^		 +	      fx *b_r_mc(ixp,izp-1))*(1.-fz)) * expiwt)
	    bvy =  ((1.-fx)*aby(ixp-1,izp  )+fx*aby(ixp,izp  ))*fz
     ^		 + ((1.-fx)*aby(ixp-1,izp-1)+fx*aby(ixp,izp-1))*(1.-fz)
     ^	         + 1.e-8 * real((((1.-fx)*b_th_mc(ixp-1,izp  )
     ^		 +	      fx *b_th_mc(ixp  ,izp  ))*fz
     ^		 +       ((1.-fx)*b_th_mc(ixp-1,izp-1)
     ^		 +	     fx *b_th_mc(ixp,izp-1))*(1.-fz)) * expiwt)
	
	    aczm =   ((1.-fx)*acz(ixp-1,izp) +
     ^		          fx *acz(ixp  ,izp))*fz
     ^	        +    ((1.-fx)*acz(ixp-1,izp-1) +
     ^		          fx *acz(ixp  ,izp-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_z_mc(ixp-1,izp  )
     ^		 +	      fx *e_z_mc(ixp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_z_mc(ixp-1,izp-1)
     ^		 +	     fx *e_z_mc(ixp  ,izp-1))*(1.-fz))* expiwt)
	    acxm =   ((1.-fx)*acx(ixp-1,izp) +
     ^	    	          fx *acx(ixp  ,izp))*fz
     ^	        +    ((1.-fx)*acx(ixp-1,izp-1) +
     ^		          fx *acx(ixp  ,izp-1))*(1.-fz)
     ^	         + real((((1.-fx)*e_r_mc(ixp-1,izp  )
     ^		 +	      fx *e_r_mc(ixp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_r_mc(ixp-1,izp-1)
     ^		 +	    fx *e_r_mc(ixp  ,izp-1))*(1.-fz)) * expiwt)
	    acym =
     ^	           real((((1.-fx)*e_theta_mc(ixp-1,izp  )
     ^		 +	      fx *e_theta_mc(ixp  ,izp  ))*fz
     ^		 +       ((1.-fx)*e_theta_mc(ixp-1,izp-1)
     ^		 +	  fx *e_theta_mc(ixp,izp-1))*(1.-fz)) * expiwt)
 
c 4th push
	    kx( 4) = dt * vxk
	    kvx(4) = dt * (acxm * sign(1.,xk) + vyk * bvz - vzk * bvy)
	    kz( 4) = dt * vzk
	    kvz(4) = dt * (aczm + vxk * bvy - vyk * bvx)
	    ky( 4) = dt * vyk
	    kvy(4) = dt * (acym + vzk * bvx - vxk * bvz)
 
	    xp  =  x + (kx( 1) + kx( 4) + 2.*(kx( 2)+kx( 3))) / 6.0
	    zp  =  z + (kz( 1) + kz( 4) + 2.*(kz( 2)+kz( 3))) / 6.0
	    yp  =  y + (ky( 1) + ky( 4) + 2.*(ky( 2)+ky( 3))) / 6.0
	    vxp = vx + (kvx(1) + kvx(4) + 2.*(kvx(2)+kvx(3))) / 6.0
	    vzp = vz + (kvz(1) + kvz(4) + 2.*(kvz(2)+kvz(3))) / 6.0
	    vyp = vy + (kvy(1) + kvy(4) + 2.*(kvy(2)+kvy(3))) / 6.0
 
	  endif	!4th order
c         iz = min(nz_mc+1,
c     ^  	abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 ))
	  iz = find_iz(zp, zp-z, iz)
	  if (zp.lt.z_min_mc) iz=0
 
	  if (xp.lt.0.0.and.mask_mc(0,iz).eq.0) then
	      xp = abs(xp)
	      vxp = abs(vxp)
	  endif
 
c	  ir = min(nr_mc+1, int( xp * delta_ri_mc ) + 1 )
	  ir = find_ir(xp, xp-x, ir)
	  if (xp.lt.0.) ir = 0
 
	  pow_i(ir,iz,nreg) = pow_i(ir,iz,nreg) +
     ^	  		.25 * (acym0 + acym) * (vy + vyp) * dt
 
	else !  1d-z
 
c initial acceleration...
	  fz=min(1.,max(0.,(z-zt_mc(iz))*dzi_mc(iz)))
	  expiwt = (-1.75880521211e15,0.) *
     ^			exp((0.,-1.)*omega0_mc*t1)
	  bvz  = abz(1,iz) * fz +    abz(1,iz-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_z_mc(1,iz) * fz + b_z_mc(1,iz-1) * (1.-fz)))
	  bvy  =  aby(1,iz) * fz +     aby(1,iz-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_th_mc(1,iz) * fz + b_th_mc(1,iz-1) * (1.-fz)))
	  bvx  = abx(1,iz) * fz +    abx(1,iz-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_r_mc(1,iz) * fz + b_r_mc(1,iz-1) * (1.-fz)))
	  acxm = acx(1,iz) * fz + acx(1,iz-1) * (1.-fz)
     ^	      + real( expiwt * (
     ^	        e_r_mc(1,iz) * fz + e_r_mc(1,iz-1) * (1.-fz)))
	  aczm = acz(1,iz) * fz + acz(1,iz-1) * (1.-fz)
     ^	      + real( expiwt * (
     ^	        e_z_mc(1,iz) * fz + e_z_mc(1,iz-1) * (1.-fz)))
	  acym0 = real( expiwt * (
     ^	        e_theta_mc(1,iz) * fz + e_theta_mc(1,iz-1) * (1.-fz)))
 
c 1st push
	  kz( 1) = dt * vz
	  kvz(1) = dt * (aczm + vx * bvy - vy * bvx)
	  kx( 1) = dt * vx
	  kvx(1) = dt * (acxm + vy * bvz - vz * bvy)
	  ky( 1) = dt * vy
	  kvy(1) = dt * (acym0 + vz * bvx - vx * bvz)
 
c find accel
	  vxk = vx + .5 * kvx(1)
	  vyk = vy + .5 * kvy(1)
	  vzk = vz + .5 * kvz(1)
	  zk = z + 0.5 * kz(1)
c	  izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	  izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	  fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	  expiwt = (-1.75880521211e15,0.) *
     ^			exp((0.,-1.)*omega0_mc*(t1+.5*dt))
	  bvz  = abz(1,izp) * fz +    abz(1,izp-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_z_mc(1,izp) * fz + b_z_mc(1,izp-1) * (1.-fz)))
	  bvy  =  aby(1,izp) * fz +     aby(1,izp-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_th_mc(1,izp) * fz + b_th_mc(1,izp-1) * (1.-fz)))
	  bvx  = abx(1,izp) * fz +    abx(1,izp-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_r_mc(1,izp) * fz + b_r_mc(1,izp-1) * (1.-fz)))
	  acxm = acx(1,izp) * fz + acx(1,izp-1) * (1.-fz)
     ^	      + real( expiwt * (
     ^	        e_r_mc(1,izp) * fz + e_r_mc(1,izp-1) * (1.-fz)))
	  aczm = acz(1,izp) * fz + acz(1,izp-1) * (1.-fz)
     ^	      + real( expiwt * (
     ^	        e_z_mc(1,izp) * fz + e_z_mc(1,izp-1) * (1.-fz)))
	  acym = real( expiwt * (
     ^	        e_theta_mc(1,izp) * fz + e_theta_mc(1,izp-1)*(1.-fz)))
 
	  if ( push_order .eq. 2 ) then
	    xp  = x + dt * vxk
	    yp  = y + dt * vyk
	    zp  = z + dt * vzk
	    vxp = vx + dt * (acxm + vyk * bvz - vzk * bvy)
	    vyp = vy + dt * (acym + vzk * bvx - vxk * bvz)
	    vzp = vz + dt * (aczm + vxk * bvy - vyk * bvx)
	  else
 
c 2nd push
	    kz( 2) = dt * vzk
	    kvz(2) = dt * (aczm + vxk * bvy - vyk * bvx)
	    kx( 2) = dt * vxk
	    kvx(2) = dt * (acxm + vyk * bvz - vzk * bvy)
	    ky( 2) = dt * vyk
	    kvy(2) = dt * (acym + vzk * bvx - vxk * bvz)
 
c find accel
	    vxk = vx + .5 * kvx(2)
	    vyk = vy + .5 * kvy(2)
	    vzk = vz + .5 * kvz(2)
	    zk = z + 0.5 * kz(2)
c	    izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	    izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	    fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	    bvz  = abz(1,izp) * fz +    abz(1,izp-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_z_mc(1,izp) * fz + b_z_mc(1,izp-1) * (1.-fz)))
	    bvy  =  aby(1,izp) * fz +     aby(1,izp-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_th_mc(1,izp) * fz + b_th_mc(1,izp-1) * (1.-fz)))
	    bvx  = abx(1,izp) * fz +    abx(1,izp-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_r_mc(1,izp) * fz + b_r_mc(1,izp-1) * (1.-fz)))
	    acxm = acx(1,izp) * fz + acx(1,izp-1) * (1.-fz)
     ^	      + real( expiwt * (
     ^	        e_r_mc(1,izp) * fz + e_r_mc(1,izp-1) * (1.-fz)))
	    aczm = acz(1,izp) * fz + acz(1,izp-1) * (1.-fz)
     ^	      + real( expiwt * (
     ^	        e_z_mc(1,izp) * fz + e_z_mc(1,izp-1) * (1.-fz)))
	    acym = real( expiwt * (
     ^	        e_theta_mc(1,izp) * fz + e_theta_mc(1,izp-1)*(1.-fz)))
 
c 3rd push
	    kz( 3) = dt * vzk
	    kvz(3) = dt * (aczm + vxk * bvy - vyk * bvx)
	    kx( 3) = dt * vxk
	    kvx(3) = dt * (acxm + vyk * bvz - vzk * bvy)
	    ky( 3) = dt * vyk
	    kvy(3) = dt * (acym + vzk * bvx - vxk * bvz)
 
c find accel
	    vxk = vx + kvx(3)
	    vyk = vy + kvy(3)
	    vzk = vz + kvz(3)
	    zk = z + kz(3)
c	    izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	    izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	    fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	    expiwt = (-1.75880521211e15,0.) *
     ^			exp((0.,-1.)*omega0_mc*(t1+dt))
	    bvz  = abz(1,izp) * fz +    abz(1,izp-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_z_mc(1,izp) * fz + b_z_mc(1,izp-1) * (1.-fz)))
	    bvy  =  aby(1,izp) * fz +     aby(1,izp-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_th_mc(1,izp) * fz + b_th_mc(1,izp-1) * (1.-fz)))
	    bvx  = abx(1,izp) * fz +    abx(1,izp-1) * (1.-fz)
     ^	      + 1.e-8 * real( expiwt * (
     ^	        b_r_mc(1,izp) * fz + b_r_mc(1,izp-1) * (1.-fz)))
	    acxm = acx(1,izp) * fz + acx(1,izp-1) * (1.-fz)
     ^	      + real( expiwt * (
     ^	        e_r_mc(1,izp) * fz + e_r_mc(1,izp-1) * (1.-fz)))
	    aczm = acz(1,izp) * fz + acz(1,izp-1) * (1.-fz)
     ^	      + real( expiwt * (
     ^	        e_z_mc(1,izp) * fz + e_z_mc(1,izp-1) * (1.-fz)))
	    acym = real( expiwt * (
     ^	        e_theta_mc(1,izp) * fz + e_theta_mc(1,izp-1)*(1.-fz)))
 
c 4th push
	    kz( 4) = dt * vzk
	    kvz(4) = dt * (aczm + vxk * bvy - vyk * bvx)
	    kx( 4) = dt * vxk
	    kvx(4) = dt * (acxm + vyk * bvz - vzk * bvy)
	    ky( 4) = dt * vyk
	    kvy(4) = dt * (acym + vzk * bvx - vxk * bvz)
 
c finally...
	    zp  =  z + (kz( 1) + kz( 4) + 2.*(kz( 2)+kz( 3))) / 6.0
	    xp  =  x + (kx( 1) + kx( 4) + 2.*(kx( 2)+kx( 3))) / 6.0
	    yp  =  y + (ky( 1) + ky( 4) + 2.*(ky( 2)+ky( 3))) / 6.0
	    vzp = vz + (kvz(1) + kvz(4) + 2.*(kvz(2)+kvz(3))) / 6.0
	    vxp = vx + (kvx(1) + kvx(4) + 2.*(kvx(2)+kvx(3))) / 6.0
	    vyp = vy + (kvy(1) + kvy(4) + 2.*(kvy(2)+kvy(3))) / 6.0
	  endif
 
c         iz = min(nz_mc+1,
c     ^  	abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 ))
	  iz = find_iz(zp, zp-z, iz)
	  if (zp.lt.z_min_mc) iz=0
 
	  pow_i(ir,iz,nreg) = pow_i(ir,iz,nreg) +
     ^	  		.25 * (acym0 + acym) * (vy + vyp) * dt
 
	endif
 
	return
	end
 
c---------------------------------------------------------------------72
 
	subroutine accel_cart(x,y,z,vx,vy,vz,xp,yp,zp,vxp,vyp,vzp,
     ^		t1,dt,ir0,iz0,ir,iz,acx,acz,acxm,aczm,abx,aby,abz)
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
	real*4 acz(0:nrmax_mc,0:nzmax_mc)
	real*4 acx(0:nrmax_mc,0:nzmax_mc)
	real*4 aby(0:nrmax_mc,0:nzmax_mc)
	real*4 abx(0:nrmax_mc,0:nzmax_mc)
	real*4 abz(0:nrmax_mc,0:nzmax_mc),bvz,bvy,bvx
	real*8 t1
 
	integer find_iz, find_ir
 
	real kx(4),kvx(4),kz(4),kvz(4),ky(4),kvy(4)
 
	ir0 = ir
	iz0 = iz
	if (nr_mc.ne.1) then
 
c initial acceleration...
	  fz=min(1.,max(0.,(z-zt_mc(iz))*dzi_mc(iz)))
	  fx=min(1.,max(0.,(x-rt_mc(ir))*dri_mc(ir)))
	  bvz =((1.-fx)*abz(ir-1,iz  )+fx*abz(ir,iz  ))*fz
     ^	     + ((1.-fx)*abz(ir-1,iz-1)+fx*abz(ir,iz-1))*(1.-fz)
	  bvy =((1.-fx)*aby(ir-1,iz  )+fx*aby(ir,iz  ))*fz
     ^	     + ((1.-fx)*aby(ir-1,iz-1)+fx*aby(ir,iz-1))*(1.-fz)
	  bvx =((1.-fx)*abx(ir-1,iz  )+fx*abx(ir,iz  ))*fz
     ^	     + ((1.-fx)*abx(ir-1,iz-1)+fx*abx(ir,iz-1))*(1.-fz)
 
	  aczm =   ((1.-fx)*acz(ir-1,iz) +
     ^		        fx *acz(ir  ,iz))*fz
     ^	      +    ((1.-fx)*acz(ir-1,iz-1) +
     ^		        fx *acz(ir  ,iz-1))*(1.-fz)
     ^		+ vx * bvy - vy * bvx
	  acxm =   ((1.-fx)*acx(ir-1,iz) +
     ^	    	        fx *acx(ir  ,iz))*fz
     ^	      +    ((1.-fx)*acx(ir-1,iz-1) +
     ^		        fx *acx(ir  ,iz-1))*(1.-fz)
     ^		+ vy * bvz - vz * bvy
 
c 1st push
	  kx( 1) = dt * vx
	  kvx(1) = dt * acxm
	  kz( 1) = dt * vz
	  kvz(1) = dt * aczm
	  ky( 1) = dt * vy
	  kvy(1) = dt * (vz * bvx - vx * bvz)
 
c find accel
	  vzk = vz + .5 * kvz(1)
	  vxk = vx + .5 * kvx(1)
	  vyk = vy + .5 * kvy(1)
	  zk = z + 0.5 * kz(1)
	  xk = x + 0.5 * kx(1)
c	  izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	  izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	  fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	  if (xk.lt.0.0.and.mask_mc(0,izp).eq.0) xk=abs(xk)
	  xk = max( 0., xk )
c	  ixp = min(Nr_mc, max(1,
c     ^            iabs( int(  xk * delta_ri_mc ) + 1 )))
	  ixp = min(nr_mc, max(1, find_ir(xk, xk-x, ir) ))
	  fx=min(1.,max(0.,(xk-rt_mc(ixp))*dri_mc(ixp)))
	  bvz =((1.-fx)*abz(ixp-1,izp  )+fx*abz(ixp,izp  ))*fz
     ^	     + ((1.-fx)*abz(ixp-1,izp-1)+fx*abz(ixp,izp-1))*(1.-fz)
	  bvy =((1.-fx)*aby(ixp-1,izp  )+fx*aby(ixp,izp  ))*fz
     ^	     + ((1.-fx)*aby(ixp-1,izp-1)+fx*aby(ixp,izp-1))*(1.-fz)
	  bvx =((1.-fx)*abx(ixp-1,izp  )+fx*abx(ixp,izp  ))*fz
     ^	     + ((1.-fx)*abx(ixp-1,izp-1)+fx*abx(ixp,izp-1))*(1.-fz)
	  aczm =   ((1.-fx)*acz(ixp-1,izp) +
     ^		        fx *acz(ixp  ,izp))*fz
     ^	      +    ((1.-fx)*acz(ixp-1,izp-1) +
     ^		        fx *acz(ixp  ,izp-1))*(1.-fz)
     ^		+ vxk * bvy - vyk * bvx
	  acxm =  (((1.-fx)*acx(ixp-1,izp) +
     ^	    	        fx *acx(ixp  ,izp))*fz
     ^	      +    ((1.-fx)*acx(ixp-1,izp-1) +
     ^		        fx *acx(ixp  ,izp-1))*(1.-fz)
     ^		+ vyk * bvz - vzk * bvy)
     ^		  * sign(1.,xk)
 
	  if ( push_order .eq. 2 ) then
	    xp  = x + dt * vxk
	    yp  = y + dt * vyk
	    zp  = z + dt * vzk
	    vzp = vz + dt * aczm
	    vxp = vx + dt * acxm
	    vyp = vy + dt * (vzk * bvx - vxk * bvz)
	  else
c 2nd push
	    kx( 2) = dt * vxk
	    kvx(2) = dt * acxm
	    kz( 2) = dt * vzk
	    kvz(2) = dt * aczm
	    ky( 2) = dt * vyk
	    kvy(2) = dt * (vzk * bvx - vxk * bvz)
 
c find accel
	    vzk = vz + .5 * kvz(2)
	    vxk = vx + .5 * kvx(2)
	    vyk = vy + .5 * kvy(2)
	    zk = z + 0.5 * kz(2)
	    xk = x + 0.5 * kx(2)
c	    izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	    izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	    fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	    if (xk.lt.0.0.and.mask_mc(0,izp).eq.0) xk=abs(xk)
	    xk = max( 0., xk )
c	    ixp = min(Nr_mc, max(1,
c     ^            iabs( int(  xk * delta_ri_mc ) + 1 )))
	    ixp = min(nr_mc, max(1, find_ir(xk, xk-x, ir) ))
	    fx=min(1.,max(0.,(xk-rt_mc(ixp))*dri_mc(ixp)))
	    bvz =((1.-fx)*abz(ixp-1,izp  )+fx*abz(ixp,izp  ))*fz
     ^	       + ((1.-fx)*abz(ixp-1,izp-1)+fx*abz(ixp,izp-1))*(1.-fz)
	    bvy =((1.-fx)*aby(ixp-1,izp  )+fx*aby(ixp,izp  ))*fz
     ^	       + ((1.-fx)*aby(ixp-1,izp-1)+fx*aby(ixp,izp-1))*(1.-fz)
	    bvx =((1.-fx)*abx(ixp-1,izp  )+fx*abx(ixp,izp  ))*fz
     ^	       + ((1.-fx)*abx(ixp-1,izp-1)+fx*abx(ixp,izp-1))*(1.-fz)
	    aczm = ((1.-fx)*acz(ixp-1,izp) +
     ^		        fx *acz(ixp  ,izp))*fz
     ^	      +    ((1.-fx)*acz(ixp-1,izp-1) +
     ^		        fx *acz(ixp  ,izp-1))*(1.-fz)
     ^		+ vxk * bvy - vyk * bvx
	    acxm =(((1.-fx)*acx(ixp-1,izp) +
     ^	    	        fx *acx(ixp  ,izp))*fz
     ^	      +    ((1.-fx)*acx(ixp-1,izp-1) +
     ^		        fx *acx(ixp  ,izp-1))*(1.-fz)
     ^		+ vyk * bvz - vzk * bvy)
     ^		  * sign(1.,xk)
 
c 3rd push
	    kx( 3) = dt * vxk
	    kvx(3) = dt * acxm
	    kz( 3) = dt * vzk
	    kvz(3) = dt * aczm
	    ky( 3) = dt * vyk
	    kvy(3) = dt * (vzk * bvx - vxk * bvz)
 
c find accel
	    vzk = vz + kvz(3)
	    vxk = vx + kvx(3)
	    vyk = vy + kvy(3)
	    zk = z + kz(3)
	    xk = x + kx(3)
c	    izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	    izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	    fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	    if (xk.lt.0.0.and.mask_mc(0,izp).eq.0) xk=abs(xk)
	    xk = max( 0., xk )
c	    ixp = min(Nr_mc, max(1,
c     ^            iabs( int(  xk * delta_ri_mc ) + 1 )))
	    ixp = min(nr_mc, max(1, find_ir(xk, xk-x, ir) ))
	    fx=min(1.,max(0.,(xk-rt_mc(ixp))*dri_mc(ixp)))
	    bvz =((1.-fx)*abz(ixp-1,izp  )+fx*abz(ixp,izp  ))*fz
     ^	       + ((1.-fx)*abz(ixp-1,izp-1)+fx*abz(ixp,izp-1))*(1.-fz)
	    bvy =((1.-fx)*aby(ixp-1,izp  )+fx*aby(ixp,izp  ))*fz
     ^	       + ((1.-fx)*aby(ixp-1,izp-1)+fx*aby(ixp,izp-1))*(1.-fz)
	    bvx =((1.-fx)*abx(ixp-1,izp  )+fx*abx(ixp,izp  ))*fz
     ^	       + ((1.-fx)*abx(ixp-1,izp-1)+fx*abx(ixp,izp-1))*(1.-fz)
	    aczm = ((1.-fx)*acz(ixp-1,izp) +
     ^		        fx *acz(ixp  ,izp))*fz
     ^	      +    ((1.-fx)*acz(ixp-1,izp-1) +
     ^		        fx *acz(ixp  ,izp-1))*(1.-fz)
     ^		+ vxk * bvy - vyk * bvx
	    acxm =(((1.-fx)*acx(ixp-1,izp) +
     ^	    	        fx *acx(ixp  ,izp))*fz
     ^	      +    ((1.-fx)*acx(ixp-1,izp-1) +
     ^		        fx *acx(ixp  ,izp-1))*(1.-fz)
     ^		+ vyk * bvz - vzk * bvy)
     ^		  * sign(1.,xk)
 
c 4th push
	    kx( 4) = dt * vxk
	    kvx(4) = dt * acxm
	    kz( 4) = dt * vzk
	    kvz(4) = dt * aczm
	    ky( 4) = dt * vyk
	    kvy(4) = dt * (vzk * bvx - vxk * bvz)
 
	    xp  =  x + (kx( 1) + kx( 4) + 2.*(kx( 2)+kx( 3))) / 6.0
	    zp  =  z + (kz( 1) + kz( 4) + 2.*(kz( 2)+kz( 3))) / 6.0
	    yp  =  y + (ky( 1) + ky( 4) + 2.*(ky( 2)+ky( 3))) / 6.0
	    vxp = vx + (kvx(1) + kvx(4) + 2.*(kvx(2)+kvx(3))) / 6.0
	    vzp = vz + (kvz(1) + kvz(4) + 2.*(kvz(2)+kvz(3))) / 6.0
	    vyp = vy + (kvy(1) + kvy(4) + 2.*(kvy(2)+kvy(3))) / 6.0
	  endif
 
c         iz = min(nz_mc+1,
c     ^  	abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 ))
	  iz = find_iz(zp, zp-z, iz)
	  if (zp.lt.z_min_mc) iz=0
 
	  if (xp.lt.0.0.and.mask_mc(0,iz).eq.0) then
	      xp = abs(xp)
	      vxp = abs(vxp)
	  endif
 
c	  ir = min(nr_mc+1, int( xp * delta_ri_mc ) + 1 )
	  ir = find_ir(xp, xp-x, ir)
	  if (xp.lt.0.) ir = 0
 
	else !  1d-z
 
c initial acceleration...
	  fz=min(1.,max(0.,(z-zt_mc(iz))*dzi_mc(iz)))
	  bvz  = abz(1,iz) * fz + abz(1,iz-1) * (1.-fz)
	  bvy  = aby(1,iz) * fz + aby(1,iz-1) * (1.-fz)
	  bvx  = abx(1,iz) * fz + abx(1,iz-1) * (1.-fz)
	  acxm = acx(1,iz) * fz + acx(1,iz-1) * (1.-fz)
     ^		 + vy * bvz - vz * bvy
	  aczm = acz(1,iz) * fz + acz(1,iz-1) * (1.-fz)
     ^		 + vx * bvy - vy * bvx
 
c 1st push
	  kz( 1) = dt * vz
	  kvz(1) = dt * aczm
	  kx( 1) = dt * vx
	  kvx(1) = dt * acxm
	  ky( 1) = dt * vy
	  kvy(1) = dt * (vz * bvx - vx * bvz)
 
c find accel
	  vxk = vx + .5 * kvx(1)
	  vyk = vy + .5 * kvy(1)
	  vzk = vz + .5 * kvz(1)
	  zk = z + 0.5 * kz(1)
c	  izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	  izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	  fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	  bvz  = abz(1,izp) * fz + abz(1,izp-1) * (1.-fz)
	  bvy  = aby(1,izp) * fz + aby(1,izp-1) * (1.-fz)
	  bvx  = abx(1,izp) * fz + abx(1,izp-1) * (1.-fz)
	  acxm = acx(1,izp) * fz + acx(1,izp-1) * (1.-fz)
     ^		 + vyk * bvz - vzk * bvy
	  aczm = acz(1,izp) * fz + acz(1,izp-1) * (1.-fz)
     ^		 + vxk * bvy - vyk * bvx
 
	  if ( push_order .eq. 2 ) then
	    xp  = x + dt * vxk
	    yp  = y + dt * vyk
	    zp  = z + dt * vzk
	    vzp = vz + dt * aczm
	    vxp = vx + dt * acxm
	    vyp = vy + dt * (vzk * bvx - vxk * bvz)
	  else
c 2nd push
	    kz( 2) = dt * vzk
	    kvz(2) = dt * aczm
	    kx( 2) = dt * vxk
	    kvx(2) = dt * acxm
	    ky( 2) = dt * vyk
	    kvy(2) = dt * (vzk * bvx - vxk * bvz)
 
c find accel
	    vxk = vx + .5 * kvx(2)
	    vyk = vy + .5 * kvy(2)
	    vzk = vz + .5 * kvz(2)
	    zk = z + 0.5 * kz(2)
c	    izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	    izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	    fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	    bvz  = abz(1,izp) * fz + abz(1,izp-1) * (1.-fz)
	    bvy  = aby(1,izp) * fz + aby(1,izp-1) * (1.-fz)
	    bvx  = abx(1,izp) * fz + abx(1,izp-1) * (1.-fz)
	    acxm = acx(1,izp) * fz + acx(1,izp-1) * (1.-fz)
     ^		 + vyk * bvz - vzk * bvy
	    aczm = acz(1,izp) * fz + acz(1,izp-1) * (1.-fz)
     ^		 + vxk * bvy - vyk * bvx
 
c 3rd push
	    kz( 3) = dt * vzk
	    kvz(3) = dt * aczm
	    kx( 3) = dt * vxk
	    kvx(3) = dt * acxm
	    ky( 3) = dt * vyk
	    kvy(3) = dt * (vzk * bvx - vxk * bvz)
 
c find accel
	    vxk = vx + kvx(3)
	    vyk = vy + kvy(3)
	    vzk = vz + kvz(3)
	    zk = z + kz(3)
c	    izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	    izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	    fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	    bvz  = abz(1,izp) * fz + abz(1,izp-1) * (1.-fz)
	    bvy  = aby(1,izp) * fz + aby(1,izp-1) * (1.-fz)
	    bvx  = abx(1,izp) * fz + abx(1,izp-1) * (1.-fz)
	    aczm = acz(1,izp) * fz + acz(1,izp-1) * (1.-fz)
     ^		 + vxk * bvy - vyk * bvx
	    acxm = acx(1,izp) * fz + acx(1,izp-1) * (1.-fz)
     ^		 + vyk * bvz - vzk * bvy
 
c 4th push
	    kz( 4) = dt * vzk
	    kvz(4) = dt * aczm
	    kx( 4) = dt * vxk
	    kvx(4) = dt * acxm
	    ky( 4) = dt * vyk
	    kvy(4) = dt * (vzk * bvx - vxk * bvz)
 
c finally...
	    zp  =  z + (kz( 1) + kz( 4) + 2.*(kz( 2)+kz( 3))) / 6.0
	    xp  =  x + (kx( 1) + kx( 4) + 2.*(kx( 2)+kx( 3))) / 6.0
	    yp  =  y + (ky( 1) + ky( 4) + 2.*(ky( 2)+ky( 3))) / 6.0
	    vzp = vz + (kvz(1) + kvz(4) + 2.*(kvz(2)+kvz(3))) / 6.0
	    vxp = vx + (kvx(1) + kvx(4) + 2.*(kvx(2)+kvx(3))) / 6.0
	    vyp = vy + (kvy(1) + kvy(4) + 2.*(kvy(2)+kvy(3))) / 6.0
	  endif
 
c         iz = min(nz_mc+1,
c     ^  	abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 ))
	  iz = find_iz(zp, zp-z, iz)
	  if (zp.lt.z_min_mc) iz=0
 
	endif
 
	return
	end
 
c---------------------------------------------------------------------72
 
 
 
        subroutine trajectory_cart_mc
     ^	 (x,y,z,vx,vy,vz,t,itgp,end_traj,good_traj,max_ke,nreg,firstp)
 
c this subroutine pushes electrons in a 2-d (or 1-d) electrostatic
c potential; cartesian
c
c  the 'r' variables are really just 'x'.
c
c called by experiment
c       integrates each electron trajectory and calls collision
c       routine during course of trajectory
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
        logical coll_flag, end_traj, hit_bdry,good_traj
        real*4 gptime
c       real*4 Ecar(3,2),Bcar(3,2)
        real*4 nextcoll
	real*4 max_ke
        integer itgp,nreg
c       real*4 rpp(2),zpp(2),c_thpp(2),s_thpp(2),t0pp(2),t1pp(2)
c       integer irpp(2),izpp(2),it0pp(2),it1pp(2)
        parameter(gpe=-1.6021892e-19)
        parameter(electron_mass = 9.10953179447e-28)
        parameter(e_chg_cgs     =-1.6021892e-12)
        parameter(e_by_m        = 2.84283897135e-16) !0.5e0 * e_chg_cgs / electron_mass)
        parameter(gpf = 2.84283897135e-16) !0.5 * 9.109e-28 / 1.602e-12)
	real*8 t,t1
	real energy_evl
 
	integer find_ir, find_iz
 
	integer instack,nstack
	parameter (instack=1000)
	real*4  v_stk(instack),z_stk(instack),t_stk(instack)
	real*4  r_stk(instack)
	logical stk_err
	common /stk_com/ v_stk,z_stk,t_stk,nstack,r_stk,stk_err
 
	real*4 acz(0:nrmax_mc,0:nzmax_mc)
	real*4 aczmx(0:nrmax_mc,nzmax_mc)
	real*4 acx(0:nrmax_mc,0:nzmax_mc)
	real*4 acxmx(nrmax_mc,0:nzmax_mc)
	real*4 abz(0:nrmax_mc,0:nzmax_mc)
	real*4 abx(0:nrmax_mc,0:nzmax_mc)
	real*4 aby(0:nrmax_mc,0:nzmax_mc)
	real*4 bfrq(0:nrmax_mc,0:nzmax_mc)
	logical firstp, d1_p, simf_p, d0_p, d1s_p
	logical escape_mc
 
c	data firstp /.true./
 
c	save firstp,acz,aczmx,acx,acxmx
	save acz,aczmx,acx,acxmx,d1_p,d0_p,simf_p, d1s_p
	save abz,abx,aby,bfrq
 
c------------------------------------------------
 
	if (firstp) then
	  firstp = .false.
	  d1_p   = .false.
	  d0_p   = .false.
	  d1s_p  = .false.
	  simf_p = (it1_max_mc.eq.1.and.(.not.b_fieldsp)
     ^		    .and.(.not.harmonic_p).and.push_order.eq.2)
c compute -eE/m
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      acx(i,j) = -epl_r_mc( i,j,1) /(2.*gpf) !* 1.75880521211e15
	      acz(i,j) = -epl_z_mc( i,j,1) /(2.*gpf) !* 1.75880521211e15
	      abz(i,j) = -bpl_z_mc( i,j  ) /(2.*gpf) * 1.e-8
	      abx(i,j) = -bpl_r_mc( i,j  ) /(2.*gpf) * 1.e-8
	      aby(i,j) = -bpl_th_mc(i,j  ) /(2.*gpf) * 1.e-8
	    enddo
	    if (mask_mc(0,j).eq.0) acx(0,j) = 0.0
	  enddo
	  if (nr_mc.eq.1) then
	    d1_p  = .true.
	    d1s_p = simf_p
	    do j = 0, nz_mc
c	      acx(0,j) = 0.
c	      acx(1,j) = 0.
	      acx(0,j) = acx(1,j)
	      acz(0,j) = acz(1,j)
	    enddo
	    if (nz_mc.eq.1) then
	      d1_p = .true.
	      d0_p = .true.
	      acz(0,0) = -ef0d /(2.*gpf) !* 1.75880521211e15
	      acz(1,0) = -ef0d /(2.*gpf) !* 1.75880521211e15
	      acz(0,1) = -ef0d /(2.*gpf) !* 1.75880521211e15
	      acz(1,1) = -ef0d /(2.*gpf) !* 1.75880521211e15
	      acx(0,0) = 0.0
	      acx(1,0) = 0.0
	      acx(0,1) = 0.0
	      acx(1,1) = 0.0
	    endif
	  endif
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      aczmx(i,j) = max( abs(acz(i-1,j-1)),
     ^		                abs(acz(i  ,j-1)),
     ^                          abs(acz(i-1,j  )),
     ^		                abs(acz(i  ,j  )) )
	      acxmx(i,j) = max( abs(acx(i-1,j-1)),
     ^		                abs(acx(i  ,j-1)),
     ^                          abs(acx(i-1,j  )),
     ^		                abs(acx(i  ,j  )) )
	      aczmx(i,j) = max(
     ^		abs(   acz(i-1,j-1)), abs(   acz(i  ,j-1)),
     ^          abs(   acz(i-1,j  )), abs(   acz(i  ,j  )) )
     ^		+  1. / (2. * gpf) * max(
     ^		abs(e_z_mc(i-1,j-1)), abs(e_z_mc(i  ,j-1)),
     ^          abs(e_z_mc(i-1,j  )), abs(e_z_mc(i  ,j  )) )
	      acxmx(i,j) = max(
     ^		abs(   acx(i-1,j-1)), abs(   acx(i  ,j-1)),
     ^          abs(   acx(i-1,j  )), abs(   acx(i  ,j  )) )
     ^		+  1. / (2. * gpf) * max(
     ^		abs(e_r_mc(i-1,j-1)), abs(e_r_mc(i  ,j-1)),
     ^          abs(e_r_mc(i-1,j  )), abs(e_r_mc(i  ,j  )) )
	      bfrq(i,j)  = sqrt(
     ^	      ( max( abs(bpl_z_mc( i-1,j-1)), abs(bpl_z_mc( i  ,j-1)),
     ^               abs(bpl_z_mc( i-1,j  )), abs(bpl_z_mc( i  ,j  )))
     ^	      + max( abs(b_z_mc(   i-1,j-1)), abs(b_z_mc(   i  ,j-1)),
     ^		     abs(b_z_mc(   i-1,j  )), abs(b_z_mc(   i,  j  )))
     ^						)**2 +
     ^	      ( max( abs(bpl_r_mc( i-1,j-1)), abs(bpl_r_mc( i  ,j-1)),
     ^               abs(bpl_r_mc( i-1,j  )), abs(bpl_r_mc( i  ,j  )))
     ^	      + max( abs(b_r_mc(   i-1,j-1)), abs(b_r_mc(   i  ,j-1)),
     ^		     abs(b_r_mc(   i-1,j  )), abs(b_r_mc(   i,  j  )))
     ^						)**2 +
     ^	      ( max( abs(bpl_th_mc(i-1,j-1)), abs(bpl_th_mc(i  ,j-1)),
     ^               abs(bpl_th_mc(i-1,j  )), abs(bpl_th_mc(i  ,j  )))
     ^	      + max( abs(b_th_mc(  i-1,j-1)), abs(b_th_mc(  i  ,j-1)),
     ^		     abs(b_th_mc(  i-1,j  )), abs(b_th_mc(  i,  j  )))
     ^						)**2 )
	      if ( bfrq(i,j) .ne. 0. ) then
	        bfrq(i,j) =  3.57241681109e-7 / bfrq(i,j)
	      else
	        bfrq(i,j) = 5.e10
	      endif
	    enddo
	  enddo
 
	  if (it1_max_mc.ne.1) then
	    write(*,*) '   ERROR- 1d: it1_max_mc .ne. 1',it1_max_mc
c	    pause
	    stop
	  endif
	endif
 
 
c       misc stuff used in this routine:
 
	dt        = 1.0
 
        end_traj  = .false.
        kcflag    = 0
        coll_flag = .false.
        hit_bdry  = .false.
        gptime    = 0.
 
        nextcoll = delta_t*abs(log(ranf_mc(1)))!time interval to next collision
 
c get initial position and velocity...
        i = 1
	if (nstack.ne.0) then
	  x = r_stk(nstack)
	  y = 0.
	  z = z_stk(nstack)
	  ir0 = find_ir(x,x-rt_mc(1),1)
	  iz0 = find_iz(z,z-zt_mc(1),1)
	  if (t_stk(nstack) .ge. 0.) then
	    vx = v_stk(nstack)
	    vy = 2.0 * ranf_mc(18) - 1.0
	    vz = vx * vy
	    vy = vx * sqrt(1.0 - vy*vy)
	    t1 = 2. * pi * ranf_mc(17)
	    vx = vy * cos(t1)
	    vy = vy * sin(t1)
	    t1 = t_stk(nstack)
c	    t1 = 0.  !t_stk(nstack)
c	    if (nreg.lt.0) t1 = t_stk(nstack)
	  else
	    call see_v_mc(vx,vy,vz,abs(t_stk(nstack)))
	    vz = vz * sign( 1., v_stk(nstack) )
	    if (abs(v_stk(nstack)) .gt. 1.5) then
	      t1 = vz
	      vz = vx
	      vx = t1
c	      if (v_stk(nstack).lt.0.) ir0 = ir0 - 1
c	    elseif (v_stk(nstack).lt.0.) then
c	      iz0 = iz0 - 1
	    endif
	    t1 = 0.
	  endif
	  nstack = nstack - 1
	  good_traj = .true.
	else
	  vx = 0.
	  vy = 0.
	  vz = 0.
	  y  = 0.
	  t1 = 0.d0
	  good_traj = (t1.ge.t_min_region(nreg))
 
	  if (nreg.gt.0) then            !coming off electrodes
	    do ir0 = ntsn_mc(1, nreg), ntsn_mc(2,nreg)
	      if (ntd_mc(7,ir0).ne.0) exit
	    enddo
	    if (ir0 .gt. ntsn_mc(2,nreg)) then
	      write(*,*) '    error- not enough electrons',
     ^			nreg,n_traj_region(nreg)
c	      pause
	      stop
	    endif
	    ntd_mc(7,ir0) = ntd_mc(7,ir0) - 1
	    ir00 = ir0
 
	    call x_v_initial_mc(vx, vy, vz, x, z,
     ^                   1, 1, ntn_fe_mc(ir0)*2./3.)
	    vy = sqrt(vy*vy + vx*vx + vz*vz)
 
	    vx = ntd_mc(2,ir0)
	    vz = ntd_mc(4,ir0)
c	    vy = sqrt( te0 / gpf )  !speed coming off surface
c	    vy = sqrt( ntn_fe_mc(ir0) / gpf )
	    r1 = 2. * pi * ranf_mc(17)   !phi- reference line is arbitary....
	    th1 = sqrt( ranf_mc(18) )    !cos(theta_normal)
	    if (abs(vx).gt.0.5) then  !normal is +- x
	      i = ntd_mc(3,ir0)
	      z = (zt_mc(i+1)-zt_mc(i))*ranf_mc(4) + zt_mc(i)
	      x = rt_mc(ntd_mc(1,ir0)+(1+vx)/2.0)
	      vx = vx * vy * th1
	      th1 = vy * sqrt( 1. - th1*th1 )
	      vz = th1 * sin(r1)
	      vy = th1 * cos(r1)
	      ir0 = find_ir(x,x-rt_mc(1),1)
	      iz0 = i
	    else			!normal is +- z
	      z = zt_mc(ntd_mc(3,ir0)+(1+vz)/2.0)
	      i = ntd_mc(1,ir0)
	      x = (rt_mc(i+1)-rt_mc(i))*ranf_mc(3) + rt_mc(i)
	      vz = vz * vy * th1
	      th1 = vy * sqrt( 1. - th1*th1 )
	      vx = th1 * sin(r1)
	      vy = th1 * cos(r1)
	      ir0 = i
	      iz0 = find_iz(z,z-zt_mc(1),1)
	    endif
 
	  elseif (nreg.eq.0) then	! `external' source of electrons
	    do iz0 = 1, nz_mc
	      do ir0 = 1, nr_mc
		if (ele_nl_mc(ir0,iz0).ne.0) exit
	      enddo
	      if (ir0.le. nr_mc) exit
	    enddo
	    if (ir0.gt.nr_mc.or.iz0.gt.nz_mc) then
	      write(*,*) '    error- not enough electrons',
     ^			nreg,n_traj_region(0)
c	      pause
	      stop
	    endif
	    ele_nl_mc(ir0,iz0) = ele_nl_mc(ir0,iz0) - 1
	    ir00 = ir0
	    iz00 = iz0
 
	    if (ext_max_p) then
	      call x_v_initial_mc(vx, vy, vz, x, z,
     ^               ir0, iz0, real(ele_s_e_mc(ir0,iz0)))
	    else
	      z  = zcm_mc(iz0)
	      x  = rcm_mc(ir0)
	      vx = sqrt(ele_s_e_mc(ir0,iz0) / gpf)
	      vy = 2.0 * ranf_mc(18) - 1.0
	      vz = vx * vy
	      r  = vx * sqrt(1.0 - vy*vy)
	      vy =  2.0 * pi * ranf_mc(17)
	      vx = r * cos(vy)
	      vy = r * sin(vy)
	    endif
 
	  else                           !'trapped' electrons in bulk...
 
	    do iz0 = 1, nz_mc
	      do ir0 = 1, nr_mc
		if (ele_trapped(ir0,iz0).ne.0) exit
	      enddo
	      if (ir0.le.nr_mc) exit
	    enddo
	    if (ir0.gt.nr_mc.or.iz0.gt.nz_mc) then
	      write(*,*) '    error- not enough trapped electrons',
     ^			nreg,n_traj_region(-1)
c	      pause
	      stop
	    endif
	    ele_trapped(ir0,iz0) = ele_trapped(ir0,iz0) - 1
	    ir00 = ir0
	    iz00 = iz0
 
	    z  = zcm_mc(iz0)
	    x  = rcm_mc(ir0)
	    if (fe_last_p.and.fe_lastmx.gt.0.0) then
		call fe_launch_mc(vx,vy,vz,ir0,iz0)
	    else
	      vx = sqrt(max(0.,ne_loss(ir0,iz0))/gpf)
	      vy = 2.0 * ranf_mc(18) - 1.0
	      vz = vx * vy
	      r  = vx * sqrt(1.0 - vy*vy)
	      vy = 2.0 * pi * ranf_mc(17)
	      vx = r * cos(vy)
	      vy = r * sin(vy)
	    endif
	  endif
	endif
	r = x
	if( good_traj )
     ^	  pow_x(ir0,iz0,nreg) = pow_x(ir0,iz0,nreg) +
     ^		gpf*(vx*vx+vy*vy+vz*vz)
 
c assume acceleration and vz in same direction, then
c  dt1 is the time to cross 0.5 of a cell width...
	dt1 = (abs(vz)+sqrt(vz*vz+
     ^	      (2.0 * del_xt * dz_mc(iz0))*aczmx(ir0,iz0)))
	dt1 = (2.0 * del_xt * dz_mc(iz0)) / (dt1+1.e-20)
	if (.not.d1_p) then
	  dt  = (abs(vx)+sqrt(vx*vx+
     ^	        (2.0 * del_xt * dr_mc(ir0))*acxmx(ir0,iz0)))
	  dt  = (2.0 * del_xt * dr_mc(ir0)) / (dt+1.e-20)
	  dt1 = min(dt,dt1)
	endif
	if (d0_p) dt1 = 1.
	dt1 = min( dt1, del_bf * bfrq(ir0,iz0) )
	
c or do we have a collision first...
        if (nextcoll.gt.dt1) then
	  nextcoll = nextcoll - dt1
        else
	  dt1      = nextcoll
	  nextcoll = -1.0
        endif
 
c set some variable that will not change in this 1d calc...
	ir=ir0
	iz=iz0
 
	yp  = y
	vyp = vy

c------- Beginning of Time Loop -----------------------\
 
        t    = 0.d0     !time of recorded trajectory
	t    = t1
        it   = 0      !total number of time steps
        itgp = 0      !number of recorded time steps
 
	del_e_2 = delta_e * 0.5
	del_em  = n_e * delta_e

c do a length of time
        do while (t.lt.t_max_region(nreg))
  	  it=it+1
  	
  	  energy_evl = gpf*(vx*vx+vy*vy+vz*vz)
 
	  if (d0_p) then
 
	      ir0 = ir
	      iz0 = iz
 
	      acxm = 0.
	      aczm = acz(1,1)
 
	      zp  = z + dt1 * (vz + 0.5 * aczm * dt1)
	      xp  = x + dt1 *  vx
	      yp  = y + dt1 *  vy
 
	      vzp = vz + dt1 * acz(1,1)
	      vxp = vx
	      vyp = vy
 
	  elseif (d1s_p) then			!1d- z
 
	      ir0 = ir
	      iz0 = iz
 
	      acxm = 0.
 
	      yp  = y + vy*dt1
	      vyp = vy
c	      xp  = x + vx*dt1
c	      vxp = vx
 
	      fz=min(1.,max(0.,(z-zt_mc(iz))*dzi_mc(iz)))
	
	      xp = x + dt1 * ( vx + 0.5 * dt1 *
     ^		    (acx(1,iz)*fz + acx(1,iz-1)*(1.-fz)) )
	      zp = z + dt1 * ( vz + 0.5 * dt1 *
     ^		    (acz(1,iz)*fz + acz(1,iz-1)*(1.-fz)) )
	
	      zk = z + dt1 * vz * 0.5
c	      izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	      izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	      fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
	      aczm = fz*acz(1,izp) + (1.-fz)*acz(1,izp-1)
	      acxm = fz*acx(1,izp) + (1.-fz)*acx(1,izp-1)
 
	      vzp = vz + dt1 * aczm
	      vxp = vx + dt1 * acxm
 
c              iz = min(nz_mc+1,
c     ^  	abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 ))
c	      if (zp.lt.z_min_mc) iz=0
	      iz = find_iz(zp, zp-z, iz)
 
	  elseif (simf_p) then
 
	      ir0 = ir
	      iz0 = iz
 
	      yp  = y + vy*dt1
	      vyp = vy
 
	      fz=min(1.,max(0.,(z-zt_mc(iz))*dzi_mc(iz)))
	      fx=min(1.,max(0.,(x-rt_mc(ir))*dri_mc(ir)))
	
	      aczm = ((1.-fx)*acz(ir-1,iz) +
     ^		          fx *acz(ir  ,iz))*fz
     ^	        +    ((1.-fx)*acz(ir-1,iz-1) +
     ^		          fx *acz(ir  ,iz-1))*(1.-fz)
	      acxm = ((1.-fx)*acx(ir-1,iz) +
     ^	    	          fx *acx(ir  ,iz))*fz
     ^	        +    ((1.-fx)*acx(ir-1,iz-1) +
     ^		          fx *acx(ir  ,iz-1))*(1.-fz)
 
	      zp = z + dt1 * ( vz + 0.5 * dt1 * aczm )
	      xp = x + dt1 * ( vx + 0.5 * dt1 * acxm )
 
	      zk = z + dt1 * vz * 0.5
c	      izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	      izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	      fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
 
	      xk = x + dt1 * vx * 0.5
	      if (xk.lt.0.0.and.mask_mc(0,izp).eq.0) xk=abs(xk)
	      xk = max( 0., xk )
c	      ixp = min(Nr_mc, max(1,
c     ^            iabs( int(  xk * delta_ri_mc ) + 1 )))
	      ixp = min(nr_mc, max(1, find_ir(xk, xk-x, ir) ))
	      fx=min(1.,max(0.,(xk-rt_mc(ixp))*dri_mc(ixp)))
 
	      aczm = ((1.-fx)*acz(ixp-1,izp) +
     ^		          fx *acz(ixp  ,izp))*fz
     ^	        +    ((1.-fx)*acz(ixp-1,izp-1) +
     ^		          fx *acz(ixp  ,izp-1))*(1.-fz)
	      acxm =(((1.-fx)*acx(ixp-1,izp) +
     ^	    	          fx *acx(ixp  ,izp))*fz
     ^	        +    ((1.-fx)*acx(ixp-1,izp-1) +
     ^		          fx *acx(ixp  ,izp-1))*(1.-fz))
     ^		    * sign(1.,xk)
 
	      vzp = vz + dt1 * aczm
	      vxp = vx + dt1 * acxm
 
c              iz = min(nz_mc+1,
c     ^  	abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 ))
c	      if (zp.lt.z_min_mc) iz=0
	      iz = find_iz(zp, zp-z, iz)
 
	      if (xp.lt.0.0.and.mask_mc(0,iz).eq.0) then
	        xp = abs(xp)
	        vxp = abs(vxp)
	      endif
 
c	      ir = min(nr_mc+1, int( xp * delta_ri_mc ) + 1 )
c	      if (xp.lt.0.) ir = 0
	      ir = find_ir(xp, xp-x, ir)
 
	  elseif ( harmonic_p ) then
	    call accel_cart_h(x,y,z,vx,vy,vz,xp,yp,zp,vxp,vyp,vzp,
     ^	     t1,dt1,ir0,iz0,ir,iz,acx,acz,acxm,aczm,abx,aby,abz,nreg)
	  else
	    call accel_cart(x,y,z,vx,vy,vz,xp,yp,zp,vxp,vyp,vzp,
     ^		t1,dt1,ir0,iz0,ir,iz,acx,acz,acxm,aczm,abx,aby,abz)
	  endif
 
c update time..
	  t1=t1+dt1
	  gptime=gptime+dt1
 
c distance travelled
	  gpmfp = sqrt( (xp-x)*(xp-x) + (yp-y)*(yp-y) +
     ^			(zp-z)*(zp-z))
	  yp = y
	  if (d1_p) then
	    xp = x
	    if (d0_p) zp = z
	  endif
 
c out of bounds?
   	  if (mask_mc(ir,iz).ne.0) then
	   if (escape_mc(xp,yp,zp,vxp,vyp,vzp,ir ,iz ,
     ^                         x ,y ,z ,vx ,vy ,vz ,ir0,iz0)) then
	    if (see_p_mc) call see_mc(xp,yp,zp,vxp,vyp,vzp,ir ,iz ,
     ^                    x ,y ,z ,vx ,vy ,vz ,ir0,iz0,nreg)
	    z  = zp
	    vz = vzp
	    x  = xp
	    vx = vxp
	    y  = yp
	    vy = vyp
            if (good_traj) then
	      no_loss(1,nreg) = no_loss(1,nreg) + 1
	      n_e_lost = n_e_lost + 1
	      n_loss(ir,iz) = n_loss(ir,iz)+weight_reg(nreg)
	      n_gain(ir,iz) = n_gain(ir,iz)-weight_reg(nreg)
	      energy_ev0 = (vxp*vxp+vyp*vyp+vzp*vzp)*gpf
	      pow_x(ir,iz,nreg) = pow_x(ir,iz,nreg) + energy_ev0
	      frac_e = max(delta_e,min(del_em,
     ^	        energy_ev0+del_e_2))
	      i1 = indx_e(ir,iz,nreg)
	      i  = max(1,min(n_e,int( frac_e * delta_ei )))
	      i0 = min(n_e,i+1)
	      frac_e = 1. - mod(frac_e,delta_e)*delta_ei
	      if (.not. full_eedf_p) then
c	        tN_of_e_rz(1,i,ir,iz,Nreg) =
c     ^	  	  tN_of_e_rz(1,i,ir,iz,Nreg) + frac_e
c	        tN_of_e_rz(1,i0,ir,iz,Nreg) =
c     ^		  tN_of_e_rz(1,i0,ir,iz,Nreg) + (1.0 - frac_e)
	        tn_of_e_rz(1,i,i1) =
     ^	  	  tn_of_e_rz(1,i,i1) + frac_e
	        tn_of_e_rz(1,i0,i1) =
     ^		  tn_of_e_rz(1,i0,i1) + (1.0 - frac_e)
	      elseif (i1.ne.0) then
	        frac_e = 1./sqrt(energy_ev0 / gpf)
	        j1 = max(1, min(nmu_mc,
     ^			int(1+ (1. + vzp * frac_e  )*dimu_mc )))
   	        j0 = max(1, min(nphi_mc,
     ^			int(1+ (pi + atan2(vyp,vxp))*diphi_mc)))
	        i1 = j0 + nphi_mc*((j1-1) + nmu_mc*(i1-1))
	        tn_of_e_rz(1,i,i1) =
     ^		  tn_of_e_rz(1,i,i1) + dt1 * frac_e
	        tn_of_e_rz(1,i0,i1) =
     ^		  tn_of_e_rz(1,i0,i1) + dt1 * (1.0 - frac_e)
	      endif
 
	    elseif (nreg.gt.0) then
	      ntd_mc(7,ir00) = ntd_mc(7,ir00) + 1
	    elseif (nreg.eq.0) then
	      ele_nl_mc(ir00,iz00) = ele_nl_mc(ir00,iz00) + 1
	    else
	      ele_trapped(ir00,iz00) = ele_trapped(ir00,iz00) + 1
	    endif
            return           ! sheath potential; terminate the trajectory
	   endif
          endif
 
c go at least t_min_region(...) seconds before we record data...
	  if (good_traj.or.t1.gt.t_min_region(nreg)) then
	    itgp              = itgp+1
	    t                 = t + dt1
            total_time        = total_time + dt1! delta_t
            total_time_steps  = total_time_steps + 1
            mfp_per_step      = mfp_per_step + gpmfp
 
	    energy_ev0 = (vxp*vxp+vyp*vyp+vzp*vzp)*gpf
	    if( .not. good_traj )  then
	      pow_x(ir,iz,nreg) = pow_x(ir,iz,nreg) + energy_ev0
	      good_traj         = .true.
	    endif
 
	    pow_t(ir,iz,nreg) = pow_t(ir,iz,nreg) +
     ^		energy_ev0 - energy_evl
c energy, eV
	    energy_ev0 = (energy_ev0+energy_evl)*.5
	    max_ke              = max(max_ke,energy_ev0)
	    energy_ev           = energy_ev0 * dt1
	    e_bar               = e_bar + energy_ev
 
	    e_bar_rz(1,ir,iz,nreg) =
     ^		e_bar_rz(1,ir,iz,nreg) + energy_ev
 
	    frac_e = max(delta_e,min(del_em,
     ^	      energy_ev0+del_e_2))
	    i1 = indx_e(ir,iz,nreg)
	    i = max(1,min(n_e,int( frac_e * delta_ei )))
	    i0 = min(n_e,i+1)
	    frac_e = 1. - mod(frac_e,delta_e)*delta_ei
	    if (.not. full_eedf_p) then
c	      tN_of_e_rz(1,i,ir,iz,Nreg) =
c     ^		tN_of_e_rz(1,i,ir,iz,Nreg) + dt1 * frac_e
c	      tN_of_e_rz(1,i0,ir,iz,Nreg) =
c     ^		tN_of_e_rz(1,i0,ir,iz,Nreg) + dt1 * (1.0 - frac_e)
	      tn_of_e_rz(1,i,i1) =
     ^		tn_of_e_rz(1,i,i1) + dt1 * frac_e
	      tn_of_e_rz(1,i0,i1) =
     ^		tn_of_e_rz(1,i0,i1) + dt1 * (1.0 - frac_e)
	    elseif (i1.ne.0) then
	      frac_e = 1./sqrt(energy_ev0 / gpf)
	      j1 = max(1, min(nmu_mc,
     ^			int(1+ (1. + vzp * frac_e  )*dimu_mc )))
   	      j0 = max(1, min(nphi_mc,
     ^			int(1+ (pi + atan2(vyp,vxp))*diphi_mc)))
	      i1 = j0 + nphi_mc*((j1-1) + nmu_mc*(i1-1))
	      tn_of_e_rz(1,i,i1) =
     ^		tn_of_e_rz(1,i,i1) + dt1 * frac_e
	      tn_of_e_rz(1,i0,i1) =
     ^		tn_of_e_rz(1,i0,i1) + dt1 * (1.0 - frac_e)
	    endif
 
            nsteps_rz(1,ir,iz,nreg) =
     ^		nsteps_rz(1,ir,iz,nreg) + dt1
 
	    v_ave(1,ir,iz,nreg) = v_ave(1,ir,iz,nreg) + dt1*vxp
	    v_ave(2,ir,iz,nreg) = v_ave(2,ir,iz,nreg) + dt1*vyp
	    v_ave(3,ir,iz,nreg) = v_ave(3,ir,iz,nreg) + dt1*vzp
 
	    if (nreg.ne.-1.and.
     ^		energy_ev0-phi_pl_mc(ir,iz,1).lt.min_energy) then
	      n_loss(ir,iz) = n_loss(ir,iz) + weight_reg(nreg) !1
	      energy_ev0 = gpf*(vxp*vxp+vyp*vyp+vzp*vzp)
	      ne_loss(ir,iz) = ne_loss(ir,iz) +
     ^                  energy_ev0 * weight_reg(nreg)
	      end_traj = .true.
	      n_cold(nreg) = n_cold(nreg) + 1
	      if( good_traj )
     ^	        pow_x(ir,iz,nreg) = pow_x(ir,iz,nreg) - energy_ev0
	      return
	    endif
          endif
 
c time to do a collision (real or null)?
          if (nextcoll.le.0.) then
c            call COLLISIONS_mc (vxp, vyp, vzp, dt1, coll_prob,
c     x           kcflag, ir, iz, coll_flag,good_traj,
c     ^           1,1,1.,0.,zp,xp,t1,Nreg)
 
	    rngp = max_coll_freq * ranf_mc(11)
	    v    = vxp*vxp + vyp*vyp + vzp*vzp
	    energy_ev0 = gpf * v
	    v    = sqrt( v )
 
	    tot_coll_freq = 0.
            i = -1
            nc = 0
            do while (nc.lt.nq.and.i.lt.0)
              nc=nc+1
	      call q_interp_mc( energy_ev0, qx, nc)
	      tot_coll_freq = tot_coll_freq + v * qx *
     ^	         n_density_mc(1,ir,iz,ic0(nc))
	      if ( tot_coll_freq .ge. rngp .and. qx.gt.0.) i=nc
	    enddo
	    coll_prob_max = max(coll_prob_max,tot_coll_freq*dt)
	    coll_flag = .false.
	    if ( i .gt. 0 )
     ^        call collisions_mc (vxp, vyp, vzp, dt1, coll_prob,
     x           kcflag, ir, iz, coll_flag,good_traj,
     ^           1,1,1.,0.,zp,xp,t1,nreg,energy_ev0,v,i)
            nextcoll=delta_t*abs(log(ranf_mc(1)))!time interval to NEXT collision
            if (kcflag .eq. -1) return     ! attachment occurs, end trajectory
	  endif
 
c new is now old...
          z   = zp
          vz  = vzp
          x   = xp
          vx  = vxp
	  y   = yp
	  vy  = vyp
 
c coulomb collision of trapped guys?
	  if (fe_last_ee.ne.0.0.and.nreg.eq.-1) then
	    vzp = (vz*vz + vy*vy + vx*vx)*gpf
	    vzp = sqrt((vzp + fe_last_ee*dt1)/vzp)
	    vz  = vz * vzp
	    vy  = vy * vzp
	    vx  = vx * vzp
          endif
 
c new time step...
	  dt1 = (abs(vz)+sqrt(vz*vz+
     ^	        (2.0 * del_xt * dz_mc(iz))*aczmx(ir,iz)))
	  dt1 = (2.0 * del_xt * dz_mc(iz)) / (dt1+1.e-20)
	  if (.not.d1_p) then
	    dt = (abs(vx)+sqrt(vx*vx+
     ^	         (2.0 * del_xt * dr_mc(ir))*acxmx(ir,iz)))
	    dt = (2.0 * del_xt * dr_mc(ir)) / (dt+1.e-20)
	    dt1 = min(dt,dt1)
	  endif
	  if (d0_p) dt1 = 1.
	  dt1 = min( dt1, del_bf * bfrq(ir,iz) )
 
 
c or do we have a collision first?
	  if (nextcoll.gt.dt1) then
	    nextcoll = nextcoll - dt1     !shrink time to next collision....
	  else
	    dt1      = nextcoll
	    nextcoll = -1.
	  endif
 
        enddo
 
	if( good_traj ) then
                no_loss(2,nreg) = no_loss(2,nreg) + 1
      	        pow_x(ir,iz,nreg) = pow_x(ir,iz,nreg) -
     ^		  gpf*(vx*vx+vy*vy+vz*vz)
        endif
 
        return
        end
 
 
c---------------------------------------------------------------------72
 
 
        subroutine trajectory_cyl_mc
     ^	 (x,y,z,vx,vy,vz,t,itgp,end_traj,good_traj,max_ke,nreg,firstp)
 
c this subroutine pushes electrons in a 2-d (or 1-d) electrostatic
c potential in cylindrical geometry...
c
c  the 'x' variables are really just 'r'.
c
c called by experiment
c       integrates each electron trajectory and calls collision
c       routine during course of trajectory
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
c	real*8 x,y,z,vx,vy,vz,xp,yp,zp,vxp,vyp,vzp
 
        logical coll_flag, end_traj, hit_bdry,good_traj
        real*4 gptime
        real*4 c_th,s_th
c       real*4 Ecar(3,2),Bcar(3,2)
        real*4 nextcoll
	real*4 max_ke
        integer itgp,nreg
c       real*4 rpp(2),zpp(2),c_thpp(2),s_thpp(2),t0pp(2),t1pp(2)
c       integer irpp(2),izpp(2),it0pp(2),it1pp(2)
        parameter(gpe=-1.6021892e-19)
        parameter(electron_mass = 9.10953179447e-28)
        parameter(e_chg_cgs     =-1.6021892e-12)
        parameter(e_by_m        = 2.84283897135e-16) !0.5e0 * e_chg_cgs / electron_mass)
        parameter(gpf = 2.84283897135e-16) !0.5 * 9.109e-28 / 1.602e-12)
	real*8 t,t1
	real energy_evl
 
	integer find_ir, find_iz
 
	integer instack,nstack
	parameter (instack=1000)
	real*4  v_stk(instack),z_stk(instack),t_stk(instack)
	real*4  r_stk(instack)
	logical stk_err
	common /stk_com/ v_stk,z_stk,t_stk,nstack,r_stk,stk_err
 
	real*4 acz(0:nrmax_mc,0:nzmax_mc)
	real*4 aczmx(0:nrmax_mc,nzmax_mc)
	real*4 acx(0:nrmax_mc,0:nzmax_mc)
	real*4 acxmx(nrmax_mc,0:nzmax_mc)
	real*4 abz(0:nrmax_mc,0:nzmax_mc)
	real*4 abx(0:nrmax_mc,0:nzmax_mc)
	real*4 aby(0:nrmax_mc,0:nzmax_mc)
	real*4 bfrq(0:nrmax_mc,0:nzmax_mc)
	logical firstp,d1_p,simf_p,d1s_p
 
c	real*8 delz
 
c	data firstp /.true./
 
c	save firstp,acz,aczmx,acx,acxmx
	save acz,aczmx,acx,acxmx,d1_p,simf_p,d1s_p
	save abz,abx,aby,bfrq
 
c------------------------------------------------
 
 
c	delz = 0.d0
 
	if (firstp) then
	  firstp = .false.
	  d1_p   = .false.
	  d1s_p  = .false.
	  simf_p = (it1_max_mc.eq.1.and.(.not.b_fieldsp)
     ^		    .and.(.not.harmonic_p).and.push_order.eq.2)
c compute -eE/m
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      acx(i,j) = -epl_r_mc( i,j,1) / (2.*gpf) !* 1.75880521211e15
	      acz(i,j) = -epl_z_mc( i,j,1) / (2.*gpf) !* 1.75880521211e15
	      abz(i,j) = -bpl_z_mc( i,j  ) /(2.*gpf) * 1.e-8
	      abx(i,j) = -bpl_r_mc( i,j  ) /(2.*gpf) * 1.e-8
	      aby(i,j) = -bpl_th_mc(i,j  ) /(2.*gpf) * 1.e-8
	    enddo
	    acx(0,j) = 0.0
	  enddo
	  if (nz_mc.eq.1) then    !only radial do we care about...
	    d1_p  = .true.
	    d1s_p = simf_p
	    do i = 0, nr_mc
	      acx(i,0) = acx(i,1)
	      acz(i,0) = acz(i,1)
	    enddo
	  endif
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      aczmx(i,j) = max(
     ^		abs(   acz(i-1,j-1)), abs(   acz(i  ,j-1)),
     ^          abs(   acz(i-1,j  )), abs(   acz(i  ,j  )) )
     ^		+  1. / (2. * gpf) * max(
     ^		abs(e_z_mc(i-1,j-1)), abs(e_z_mc(i  ,j-1)),
     ^          abs(e_z_mc(i-1,j  )), abs(e_z_mc(i  ,j  )) )
	      acxmx(i,j) = sqrt ( (max(
     ^		abs(   acx(i-1,j-1)), abs(   acx(i  ,j-1)),
     ^          abs(   acx(i-1,j  )), abs(   acx(i  ,j  )) )
     ^		+  1. / (2. * gpf) * max(
     ^		abs(e_r_mc(i-1,j-1)), abs(e_r_mc(i  ,j-1)),
     ^          abs(e_r_mc(i-1,j  )), abs(e_r_mc(i  ,j  )) ))**2
     ^		+ (1. / (2. * gpf) * max(
     ^		abs(e_theta_mc(i-1,j-1)), abs(e_theta_mc(i  ,j-1)),
     ^          abs(e_theta_mc(i-1,j  )), abs(e_theta_mc(i  ,j  ))
     ^			))**2 )
	      bfrq(i,j)  = sqrt(
     ^	      ( max( abs(bpl_z_mc( i-1,j-1)), abs(bpl_z_mc( i  ,j-1)),
     ^               abs(bpl_z_mc( i-1,j  )), abs(bpl_z_mc( i  ,j  )))
     ^	      + max( abs(b_z_mc(   i-1,j-1)), abs(b_z_mc(   i  ,j-1)),
     ^		     abs(b_z_mc(   i-1,j  )), abs(b_z_mc(   i,  j  )))
     ^						)**2 +
     ^	      ( max( abs(bpl_r_mc( i-1,j-1)), abs(bpl_r_mc( i  ,j-1)),
     ^               abs(bpl_r_mc( i-1,j  )), abs(bpl_r_mc( i  ,j  )))
     ^	      + max( abs(b_r_mc(   i-1,j-1)), abs(b_r_mc(   i  ,j-1)),
     ^		     abs(b_r_mc(   i-1,j  )), abs(b_r_mc(   i,  j  )))
     ^						)**2 +
     ^	      ( max( abs(bpl_th_mc(i-1,j-1)), abs(bpl_th_mc(i  ,j-1)),
     ^               abs(bpl_th_mc(i-1,j  )), abs(bpl_th_mc(i  ,j  )))
     ^	      + max( abs(b_th_mc(  i-1,j-1)), abs(b_th_mc(  i  ,j-1)),
     ^		     abs(b_th_mc(  i-1,j  )), abs(b_th_mc(  i,  j  )))
     ^						)**2 )
	      if ( bfrq(i,j) .ne. 0. ) then
	        bfrq(i,j) =  3.57241681109e-7 / bfrq(i,j)
	      else
	        bfrq(i,j) = 5.e10
	      endif
	    enddo
	  enddo
 
	  if (it1_max_mc.ne.1) then
	    write(*,*) '   ERROR- 1d: it1_max_mc .ne. 1',it1_max_mc
c	    pause
	    stop
	  endif
	endif
 
 
c       misc stuff used in this routine:
 
	dt        = 1.0
 
        end_traj  = .false.
        kcflag    = 0
        coll_flag = .false.
        hit_bdry  = .false.
        gptime    = 0.
 
        nextcoll = delta_t*abs(log(ranf_mc(1)))!time interval to next collision
 
c get initial position and velocity...
        i = 1
	if (nstack.ne.0) then
	  x = r_stk(nstack)
	  y = 0.
	  z = z_stk(nstack)
	  ir0 = find_ir(x,x-rt_mc(1),1)
	  iz0 = find_iz(z,z-zt_mc(1),1)
	  vx = v_stk(nstack)
	  vy = 2.0 * ranf_mc(18) - 1.0
	  vz = vx * vy
	  vy = vx * sqrt(1.0 - vy*vy)
	  t1 = 2. * pi * ranf_mc(17)
	  vx = vy * cos(t1)
	  vy = vy * sin(t1)
	  t1 = t_stk(nstack)
c	  t1 = 0.  !t_stk(nstack)
c	  if (nreg.lt.0) t1 = t_stk(nstack)
	  nstack = nstack - 1
	  good_traj = .true.
	else
	  vx = 0.
	  vy = 0.
	  vz = 0.
	  y  = 0.
	  t1 = 0.d0
	  good_traj = (t1.ge.t_min_region(nreg))
 
	  if (nreg.gt.0) then            !coming off electrodes
	    do ir0 = ntsn_mc(1, nreg), ntsn_mc(2,nreg)
	      if (ntd_mc(7,ir0).ne.0) exit
	    enddo
	    if (ir0 .gt. ntsn_mc(2,nreg)) then
	      write(*,*) '    error- not enough electrons',
     ^			nreg,n_traj_region(nreg)
c	      pause
	      stop
	    endif
	    ntd_mc(7,ir0) = ntd_mc(7,ir0) - 1
	    ir00 = ir0
 
	    call x_v_initial_mc(vx, vy, vz, x, z,
     ^                   1, 1, ntn_fe_mc(ir0)*2./3.)
	    vy = sqrt(vy*vy + vx*vx + vz*vz)
 
	    vx = ntd_mc(2,ir0)
	    vz = ntd_mc(4,ir0)
c	    vy = sqrt( te0 / gpf )  !speed coming off surface
c	    vy = sqrt( ntn_fe_mc(ir0) / gpf )
	    r1 = 2. * pi * ranf_mc(17)   !phi- reference line is arbitary....
	    th1 = sqrt( ranf_mc(18) )    !cos(theta_normal)
	    if (abs(vx).gt.0.5) then  !normal is +- x
	      i = ntd_mc(3,ir0)
	      z = (zt_mc(i+1)-zt_mc(i))*ranf_mc(4) + zt_mc(i)
	      x = rt_mc(ntd_mc(1,ir0)+(1+vx)/2.0)
	      vx = vx * vy * th1
	      th1 = vy * sqrt( 1. - th1*th1 )
	      vz = th1 * sin(r1)
	      vy = th1 * cos(r1)
	    else			!normal is +- z
	      z = zt_mc(ntd_mc(3,ir0)+(1+vz)/2.0)
	      i = ntd_mc(1,ir0)
	      x = (rt_mc(i+1)-rt_mc(i))*ranf_mc(3) + rt_mc(i)
	      vz = vz * vy * th1
	      th1 = vy * sqrt( 1. - th1*th1 )
	      vx = th1 * sin(r1)
	      vy = th1 * cos(r1)
	    endif
	    ir0 = find_ir(x,x-rt_mc(1),1)
	    iz0 = find_iz(z,z-zt_mc(1),1)
 
	  elseif (nreg.eq.0) then	! `external' source of electrons
	    do iz0 = 1, nz_mc
	      do ir0 = 1, nr_mc
		if (ele_nl_mc(ir0,iz0).ne.0) exit
	      enddo
	      if (ir0.le. nr_mc) exit
	    enddo
	    if (ir0.gt.nr_mc.or.iz0.gt.nz_mc) then
	      write(*,*) '    error- not enough electrons',
     ^			nreg,n_traj_region(0)
c	      pause
	      stop
	    endif
	    ele_nl_mc(ir0,iz0) = ele_nl_mc(ir0,iz0) - 1
	    ir00 = ir0
	    iz00 = iz0
 
	    if (ext_max_p) then
	      call x_v_initial_mc(vx, vy, vz, x, z,
     ^                   ir0, iz0, real(ele_s_e_mc(ir0,iz0)))
	    else
	      z  = zcm_mc(iz0)
	      x  = rcm_mc(ir0)
	      vx = sqrt(ele_s_e_mc(ir0,iz0) / gpf)
	      vy = 2.0 * ranf_mc(18) - 1.0
	      vz = vx * vy
	      r  = vx * sqrt(1.0 - vy*vy)
	      vy =  2.0 * pi * ranf_mc(17)
	      vx = r * cos(vy)
	      vy = r * sin(vy)
	    endif
 
	  else                           !'trapped' electrons in bulk...
 
	    do iz0 = 1, nz_mc
	      do ir0 = 1, nr_mc
		if (ele_trapped(ir0,iz0).ne.0) exit
	      enddo
	      if (ir0.le.nr_mc) exit
	    enddo
	    if (ir0.gt.nr_mc.or.iz0.gt.nz_mc) then
	      write(*,*) '    error- not enough trapped electrons',
     ^			nreg,n_traj_region(-1)
c	      pause
	      stop
	    endif
	    ele_trapped(ir0,iz0) = ele_trapped(ir0,iz0) - 1
	    ir00 = ir0
	    iz00 = iz0
 
	    z  = zcm_mc(iz0)
	    x  = rcm_mc(ir0)
	    if (fe_last_p.and.fe_lastmx.gt.0.0) then
		call fe_launch_mc(vx,vy,vz,ir0,iz0)
	    else
	      vx = sqrt(max(0.,ne_loss(ir0,iz0))/gpf)
	      vy = 2.0 * ranf_mc(18) - 1.0
	      vz = vx * vy
	      r  = vx * sqrt(1.0 - vy*vy)
	      vy = 2.0 * pi * ranf_mc(17)
	      vx = r * cos(vy)
	      vy = r * sin(vy)
	    endif
	  endif
	endif
	r = x
 
	if( good_traj )
     ^	        pow_x(ir0,iz0,nreg) = pow_x(ir0,iz0,nreg) +
     ^		  gpf*(vx*vx+vy*vy+vz*vz)
 
c assume acceleration and vz in same direction, then
c  dt1 is the time to cross 0.5 of a cell width...
	vp  = sqrt(vx*vx + vy*vy)
	dt1 = (abs(vp)+sqrt(vp*vp+
     ^	      (2.0 * del_xt * dr_mc(ir0))*acxmx(ir0,iz0)))
	dt1 = (2.0 * del_xt * dr_mc(ir0)) / (dt1+1.e-20)
	if (.not.d1_p) then
	  dt  = (abs(vz)+sqrt(vz*vz+
     ^	      (2.0 * del_xt * dz_mc(iz0))*aczmx(ir0,iz0)))
	  dt  = (2.0 * del_xt * dz_mc(iz0)) / (dt+1.e-20)
	  dt1 = min(dt,dt1)
	endif
	dt1 = min( dt1, del_bf * bfrq(ir0,iz0) )
	
c or do we have a collision first...
        if (nextcoll.gt.dt1) then
	  nextcoll = nextcoll - dt1
        else
	  dt1      = nextcoll
	  nextcoll = -1.0
        endif
 
c set some variable that will not change in this 1d calc...
	ir=ir0
	iz=iz0
 
 
	if (r.eq.0.) then
	  r = 1.e-3 * rt_mc(2)
	  x = r * sign(1.0, 0.5 - ranf_mc(2))
	endif
	c_th = x/r
	s_th = y/r
 
c------- Beginning of Time Loop -----------------------\
 
        t    = 0.d0     !time of recorded trajectory
	t    = t1
        it   = 0      !total number of time steps
        itgp = 0      !number of recorded time steps
 
	del_e_2 = delta_e * 0.5
	del_em  = n_e * delta_e
 
c do a length of time
        do while (t.lt.t_max_region(nreg))
  	  it=it+1
 
	  energy_evl = gpf*(vx*vx+vy*vy+vz*vz)
 
c do a simplified pushed in 1-d...
 
	  if (d1s_p) then
 
	      ir0 = ir
	      iz0 = iz
 
c	      aczm = acz(1,1)
 
c	      zp  = z + dt1 * ( vz + 0.5 * dt1 * aczm)
c	      vzp = vz + dt1 * acz(1,1)
 
	      fx=min(1.,max(0.,(r-rt_mc(ir))*dri_mc(ir)))
	
	      acxm = (1.-fx)*acx(ir-1,1) + fx *acx(ir  ,1)
	      aczm = (1.-fx)*acz(ir-1,1) + fx *acz(ir  ,1)
 
	      xp = x + dt1 * ( vx + 0.5 * dt1 * acxm * c_th)
	      yp = y + dt1 * ( vy + 0.5 * dt1 * acxm * s_th)
	      zp = z + dt1 * ( vz + 0.5 * dt1 * aczm )
 
	      xk = x + dt1 * vx * 0.5
	      yk = y + dt1 * vy * 0.5
	      rk = sqrt(xk*xk + yk*yk)
	      if (rk.eq.0.) then
		rk = rt_mc(2) * 1.e-3
		xk = rk * sign(1., .5 - ranf_mc(2))
	      endif
c	      irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	      irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir) ))
	      fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
 
	      acxm = (1.-fx)*acx(irp-1,1) + fx *acx(irp  ,1)
	      aczm = (1.-fx)*acz(irp-1,1) + fx *acz(irp  ,1)
 
	      vxp = vx + dt1 * acxm * xk / rk
	      vyp = vy + dt1 * acxm * yk / rk
	      vzp = vz + dt1 * aczm
 
	      rk = r
	      r = sqrt(xp*xp + yp*yp)
	      if (r.eq.0.) then
		r  = rt_mc(2) * 1.e-3
		xp = r * sign(1., .5 - ranf_mc(2))
	      endif
 
c	      ir = min(nr_mc+1, int( r * delta_ri_mc ) + 1 )
	      ir = find_ir(r, r-rk, ir)
	      c_th = xp / r
	      s_th = yp / r
 
	  elseif (simf_p) then
	      ir0 = ir
	      iz0 = iz
 
	      fz=min(1.,max(0.,(z-zt_mc(iz))*dzi_mc(iz)))
	      fx=min(1.,max(0.,(r-rt_mc(ir))*dri_mc(ir)))
 
	      aczm = ((1.-fx)*acz(ir-1,iz) +
     ^		          fx *acz(ir  ,iz))*fz
     ^	        +    ((1.-fx)*acz(ir-1,iz-1) +
     ^		          fx *acz(ir  ,iz-1))*(1.-fz)
	      acxm = ((1.-fx)*acx(ir-1,iz) +
     ^	    	          fx *acx(ir  ,iz))*fz
     ^	        +    ((1.-fx)*acx(ir-1,iz-1) +
     ^		          fx *acx(ir  ,iz-1))*(1.-fz)
 
c	      aczm = 3.8975123e15
c	      acxm = -3.0534813e16 * r
 
	      zp = z + dt1 * ( vz + 0.5 * dt1 * aczm )
	      xp = x + dt1 * ( vx + 0.5 * dt1 * acxm * c_th)
	      yp = y + dt1 * ( vy + 0.5 * dt1 * acxm * s_th)
 
	      zk = z + dt1 * vz * 0.5
c	      izp = min(Nz_mc, max(1,
c     ^              iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	      izp = min(nz_mc, max(1, find_iz( zk, zk-z, iz) ))
	      fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
 
	      xk = x + dt1 * vx * 0.5
	      yk = y + dt1 * vy * 0.5
	      rk = sqrt(xk*xk + yk*yk)
	      if (rk.eq.0.) then
		rk = rt_mc(2) * 1.e-3
		xk = rk * sign(1., .5 - ranf_mc(2))
	      endif
c	      irp = min(Nr_mc, max(1,
c     ^            iabs( int(  rk * delta_ri_mc ) + 1 )))
	      irp = min(nr_mc, max(1, find_ir(rk, rk-r, ir) ))
	      fx=min(1.,max(0.,(rk-rt_mc(irp))*dri_mc(irp)))
 
	      aczm = ((1.-fx)*acz(irp-1,izp) +
     ^		          fx *acz(irp  ,izp))*fz
     ^	        +    ((1.-fx)*acz(irp-1,izp-1) +
     ^		          fx *acz(irp  ,izp-1))*(1.-fz)
	      acxm = ((1.-fx)*acx(irp-1,izp) +
     ^	    	          fx *acx(irp  ,izp))*fz
     ^	        +    ((1.-fx)*acx(irp-1,izp-1) +
     ^		          fx *acx(irp  ,izp-1))*(1.-fz)
 
c	      aczm = 3.8975123e15
c	      acxm = -3.0534813e16 * rk
 
	      vzp = vz + dt1 * aczm
	      vxp = vx + dt1 * acxm * xk / rk
	      vyp = vy + dt1 * acxm * yk / rk
 
	      rk = r
	      r = sqrt(xp*xp + yp*yp)
	      if (r.eq.0.) then
		r  = rt_mc(2) * 1.e-3
		xp = r * sign(1., .5 - ranf_mc(2))
	      endif
 
c              iz = min(nz_mc+1,
c     ^  	abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 ))
c	      if (zp.lt.z_min_mc) iz=0
	      iz = find_iz( zp, zp-z, iz)
 
c	      ir = min(nr_mc+1, int( r * delta_ri_mc ) + 1 )
	      ir = find_ir(r, r-rk, ir)
	      c_th = xp / r
	      s_th = yp / r
 
	  elseif ( harmonic_p ) then
	    call accel_cyl_h(x,y,z,vx,vy,vz,xp,yp,zp,vxp,vyp,vzp,
     ^		t1,dt1,ir0,iz0,ir,iz,acx,acz,r,c_th,s_th,acxm,aczm,
     ^		abx,aby,abz,nreg)
	  else
	    call accel_cyl(x,y,z,vx,vy,vz,xp,yp,zp,vxp,vyp,vzp,
     ^		t1,dt1,ir0,iz0,ir,iz,acx,acz,r,c_th,s_th,acxm,aczm,
     ^		abx,aby,abz)
	  endif
 
c update time..
	  t1=t1+dt1
	  gptime=gptime+dt1
 
c distance travelled
c	  delz = delz + (zp-z)
	  gpmfp = sqrt( (xp-x)*(xp-x) + (yp-y)*(yp-y) +
     ^			(zp-z)*(zp-z))
	  if (d1_p) zp = z
 
c out of bounds?
   	  if (mask_mc(ir,iz).ne.0) then
	   call boundaries_mc (xp, yp, zp, vxp, vyp, vzp, hit_bdry,
     1                 end_traj,good_traj,r,ir,iz,ir0,iz0,
     ^                     x,y,z,vx,vy,vz)
	   if (end_traj) then
	    if (see_p_mc) call see_mc(xp,yp,zp,vxp,vyp,vzp,ir ,iz ,
     ^                      x ,y ,z ,vx ,vy ,vz ,ir0,iz0,nreg)
	    z  = zp
	    vz = vzp
	    x  = xp
	    vx = vxp
	    y  = yp
	    vy = vyp
            if (good_traj) then
	      no_loss(1,nreg) = no_loss(1,nreg) + 1
	      n_e_lost = n_e_lost + 1
	      n_loss(ir,iz) = n_loss(ir,iz)+weight_reg(nreg)
	      n_gain(ir,iz) = n_gain(ir,iz)-weight_reg(nreg)
	      energy_ev0 = (vxp*vxp+vyp*vyp+vzp*vzp)*gpf
	      pow_x(ir,iz,nreg) = pow_x(ir,iz,nreg) - energy_ev0
	      frac_e = max(delta_e,min(del_em,
     ^	        energy_ev0+del_e_2))
	      i1 = indx_e(ir,iz,nreg)
	      i  = max(1,min(n_e,int( frac_e * delta_ei )))
	      i0 = min(n_e,i+1)
	      frac_e = 1. - mod(frac_e,delta_e)*delta_ei
	      if (.not. full_eedf_p) then
c	        tN_of_e_rz(1,i,ir,iz,Nreg) =
c     ^	  	  tN_of_e_rz(1,i,ir,iz,Nreg) + frac_e
c	        tN_of_e_rz(1,i0,ir,iz,Nreg) =
c     ^		  tN_of_e_rz(1,i0,ir,iz,Nreg) + (1.0 - frac_e)
	        tn_of_e_rz(1,i,i1) =
     ^	  	  tn_of_e_rz(1,i,i1) + frac_e
	        tn_of_e_rz(1,i0,i1) =
     ^		  tn_of_e_rz(1,i0,i1) + (1.0 - frac_e)
	      elseif (i1.ne.0) then
	        frac_e = 1./sqrt(energy_ev0 / gpf)
	        j1 = max(1, min(nmu_mc,
     ^			int(1+ (1. + vzp * frac_e  )*dimu_mc )))
   	        j0 = max(1, min(nphi_mc,
     ^			int(1+ (pi + atan2(vyp,vxp))*diphi_mc)))
	        i1 = j0 + nphi_mc*((j1-1) + nmu_mc*(i1-1))
	        tn_of_e_rz(1,i,i1) =
     ^		  tn_of_e_rz(1,i,i1) + dt1 * frac_e
	        tn_of_e_rz(1,i0,i1) =
     ^		  tn_of_e_rz(1,i0,i1) + dt1 * (1.0 - frac_e)
	      endif
 
	    elseif (nreg.gt.0) then
	      ntd_mc(7,ir00) = ntd_mc(7,ir00) + 1
	    elseif (nreg.eq.0) then
	      ele_nl_mc(ir00,iz00) = ele_nl_mc(ir00,iz00) + 1
	    else
	      ele_trapped(ir00,iz00) = ele_trapped(ir00,iz00) + 1
	    endif
            return           ! sheath potential; terminate the trajectory
	   else
	     r    = sqrt(xp*xp + yp*yp)
	     if (r.eq.0.) then
	       r  = rt_mc(2) * 1.e-3
	       xp = r * sign(1., .5 - ranf_mc(2))
	     endif
	     c_th = xp / r
	     s_th = yp / r
c	     iz = max(1, min(nz_mc,
c     ^        abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 )))
c	     ir = max(1, min(nr_mc, int( r * delta_ri_mc ) + 1 ))
	     iz = max(1, min(nz_mc, find_iz(zp, zp-zt_mc(1), 1) ))
	     ir = max(1,min(nr_mc,find_ir(r,r-rt_mc(nr_mc+1),nr_mc)))
	   endif
          endif
 
c go at least t_min_region(...) seconds before we record data...
	  if (good_traj.or.t1.gt.t_min_region(nreg)) then
	    itgp              = itgp+1
	    t                 = t + dt1
            total_time        = total_time + dt1! delta_t
            total_time_steps  = total_time_steps + 1
            mfp_per_step      = mfp_per_step + gpmfp
 
	    energy_ev0 = (vxp*vxp+vyp*vyp+vzp*vzp)*gpf
	    if( .not. good_traj ) then
	      pow_x(ir,iz,nreg) = pow_x(ir,iz,nreg) + energy_ev0
	      good_traj         = .true.
	    endif
 
	    pow_t(ir,iz,nreg) = pow_t(ir,iz,nreg) +
     ^		energy_ev0 - energy_evl
c energy, eV
	    energy_ev0 = (energy_ev0+energy_evl)*.5
	    max_ke              = max(max_ke,energy_ev0)
	    energy_ev           = energy_ev0 * dt1
	    e_bar               = e_bar + energy_ev
 
	    e_bar_rz(1,ir,iz,nreg) =
     ^		e_bar_rz(1,ir,iz,nreg) + energy_ev
 
	    frac_e = max(delta_e,min(del_em,
     ^	      energy_ev0+del_e_2))
	    i1 = indx_e(ir,iz,nreg)
	    i  = max(1,min(n_e,int( frac_e * delta_ei )))
	    i0 = min(n_e,i+1)
	    frac_e = 1. - mod(frac_e,delta_e)*delta_ei
	    if (.not. full_eedf_p) then
c	      tN_of_e_rz(1,i,ir,iz,Nreg) =
c     ^		tN_of_e_rz(1,i,ir,iz,Nreg) + dt1 * frac_e
c	      tN_of_e_rz(1,i0,ir,iz,Nreg) =
c     ^		tN_of_e_rz(1,i0,ir,iz,Nreg) + dt1 * (1.0 - frac_e)
	      tn_of_e_rz(1,i,i1) =
     ^		tn_of_e_rz(1,i,i1) + dt1 * frac_e
	      tn_of_e_rz(1,i0,i1) =
     ^		tn_of_e_rz(1,i0,i1) + dt1 * (1.0 - frac_e)
	    elseif (i1.ne.0) then
	      frac_e = 1./sqrt(energy_ev0 / gpf)
	      j1 = max(1, min(nmu_mc,
     ^			int(1+ (1. + vzp * frac_e  )*dimu_mc )))
   	      j0 = max(1, min(nphi_mc,
     ^			int(1+ (pi + atan2(vyp,vxp))*diphi_mc)))
	      i1 = j0 + nphi_mc*((j1-1) + nmu_mc*(i1-1))
	      tn_of_e_rz(1,i,i1) =
     ^		tn_of_e_rz(1,i,i1) + dt1 * frac_e
	      tn_of_e_rz(1,i0,i1) =
     ^		tn_of_e_rz(1,i0,i1) + dt1 * (1.0 - frac_e)
	    endif
            nsteps_rz(1,ir,iz,nreg) =
     ^		nsteps_rz(1,ir,iz,nreg) + dt1
 
	    v_ave(1,ir,iz,nreg) = v_ave(1,ir,iz,nreg) + dt1*
     ^		( vxp*c_th + vyp*s_th)
	    v_ave(2,ir,iz,nreg) = v_ave(2,ir,iz,nreg) + dt1*
     ^		(-vxp*s_th + vyp*c_th)
	    v_ave(3,ir,iz,nreg) = v_ave(3,ir,iz,nreg) + dt1*vzp
 
	    if (nreg.ne.-1.and.
     ^		energy_ev0-phi_pl_mc(ir,iz,1).lt.min_energy) then
	      n_loss(ir,iz) = n_loss(ir,iz) + weight_reg(nreg) !1
	      energy_ev0 = gpf*(vxp*vxp+vyp*vyp+vzp*vzp)
	      ne_loss(ir,iz) = ne_loss(ir,iz) +
     ^                  energy_ev0 * weight_reg(nreg)
	      end_traj = .true.
	      n_cold(nreg) = n_cold(nreg) + 1
	      if( good_traj )
     ^	        pow_x(ir,iz,nreg) = pow_x(ir,iz,nreg) - energy_ev0
	      return
	    endif
          endif
 
c time to do a collision (real or null)?
          if (nextcoll.le.0.) then
c            call COLLISIONS_mc (vxp, vyp, vzp, dt1, coll_prob,
c     x           kcflag, ir, iz, coll_flag,good_traj,
c     ^           1,1,1.,0.,zp,r,t1,Nreg)
 
	    rngp = max_coll_freq * ranf_mc(11)
	    v    = vxp*vxp + vyp*vyp + vzp*vzp
	    energy_ev0 = gpf * v
	    v    = sqrt( v )
 
	    tot_coll_freq = 0.
            i = -1
            nc = 0
            do while (nc.lt.nq.and.i.lt.0)
              nc=nc+1
	      call q_interp_mc( energy_ev0, qx, nc)
	      tot_coll_freq = tot_coll_freq + v * qx *
     ^	         n_density_mc(1,ir,iz,ic0(nc))
	      if ( tot_coll_freq .ge. rngp .and. qx.gt.0.) i=nc
	    enddo
	    coll_prob_max = max(coll_prob_max,tot_coll_freq*dt)
	    coll_flag = .false.
	    if ( i .gt. 0 )
     ^        call collisions_mc (vxp, vyp, vzp, dt1, coll_prob,
     x           kcflag, ir, iz, coll_flag,good_traj,
     ^           1,1,1.,0.,zp,r,t1,nreg,energy_ev0,v,i)
            nextcoll=delta_t*abs(log(ranf_mc(1)))!time interval to NEXT collision
            if (kcflag .eq. -1) return     ! attachment occurs, end trajectory
	  endif
 
c new is now old...
          z   = zp
          vz  = vzp
          x   = xp
          vx  = vxp
	  y   = yp
	  vy  = vyp
 
c coulomb collision of trapped guys?
	  if (fe_last_ee.ne.0.0.and.nreg.eq.-1) then
	    vzp = (vz*vz + vy*vy + vx*vx)*gpf
	    vzp = sqrt((vzp + fe_last_ee*dt1)/vzp)
	    vz  = vz * vzp
	    vy  = vy * vzp
	    vx  = vx * vzp
          endif
 
c new time step...
	  vp  = sqrt(vx*vx + vy*vy)
	  dt1 = (abs(vp)+sqrt(vp*vp+
     ^	        (2.0 * del_xt * dr_mc(ir))*acxmx(ir,iz)))
	  dt1 = (2.0 * del_xt * dr_mc(ir)) / (dt1+1.e-20)
	  if (.not.d1_p) then
	    dt  = (abs(vz)+sqrt(vz*vz+
     ^	        (2.0 * del_xt * dz_mc(iz))*aczmx(ir,iz)))
	    dt  = (2.0 * del_xt * dz_mc(iz)) / (dt+1.e-20)
	    dt1 = min(dt,dt1)
	  endif
	  dt1 = min( dt1, del_bf * bfrq(ir,iz) )
	
c or do we have a collision first?
	  if (nextcoll.gt.dt1) then
	    nextcoll = nextcoll - dt1     !shrink time to next collision....
	  else
	    dt1      = nextcoll
	    nextcoll = -1.
	  endif
 
        enddo

	if( good_traj ) then
                no_loss(2,nreg) = no_loss(2,nreg) + 1
      	        pow_x(ir,iz,nreg) = pow_x(ir,iz,nreg) -
     ^			(vx*vx+vy*vy+vz*vz)*gpf
        endif
 
        return
        end
 
 
c---------------------------------------------------------------------72
 
 
 
        subroutine trajectory_2d_i_cart_mc
     ^	 (x,y,z,vx,vy,vz,t,itgp,end_traj,good_traj,max_ke,nreg,firstp)
 
c this subroutine pushes electrons in a 2-d electrostatic
c potential.
c
c  the 'r' variables are really just 'x'.
c
c NOTE: assumes there is a plane of symmetry at x (r) = 0.
c   that is, if electron passes through x = 0, then x --> |x|
c   and Vx -->  |Vx|
 
c called by experiment
c       integrates each electron trajectory and calls collision
c       routine during course of trajectory
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
        logical coll_flag, end_traj, hit_bdry,good_traj
        real*4 gptime
        real*4 c_th,s_th
c       real*4 Ecar(3,2),Bcar(3,2)
        real*4 nextcoll
	real*4 max_ke
        integer itgp,nreg
c       real*4 rpp(2),zpp(2),c_thpp(2),s_thpp(2),t0pp(2),t1pp(2)
c       integer irpp(2),izpp(2),it0pp(2),it1pp(2)
        parameter(electron_mass = 9.10953179447e-28)
        parameter(e_chg_cgs     =-1.6021892e-12)
        parameter(e_by_m        = 2.84283897135e-16) !0.5e0 * e_chg_cgs / electron_mass)
c        parameter(gpf = 2.84283897135e-16) !0.5 * 9.109e-28 / 1.602e-12)
	real*8 t,t1
	real energy_evl
 
	integer find_ir,find_iz
 
	integer instack,nstack
	parameter (instack=1000)
	real*4  v_stk(instack),z_stk(instack),t_stk(instack)
	real*4  r_stk(instack)
	logical stk_err
	common /stk_com/ v_stk,z_stk,t_stk,nstack,r_stk,stk_err
 
	real*4 acz(0:nrmax_mc,0:nzmax_mc)
	real*4 aczmx(0:nrmax_mc,nzmax_mc)
	real*4 acx(0:nrmax_mc,0:nzmax_mc)
	real*4 acxmx(nrmax_mc,0:nzmax_mc)
	logical firstp
 
c	data firstp /.true./
 
c	save firstp,acz,aczmx,acx,acxmx
	save acz,aczmx,acx,acxmx
 
	gpe = 1.6021892e-19 * spec_tq
	gpf = 0.5 * 1.660565e-24 * spec_tm / 1.6021892e-12   !/ abs(spec_tq)
c------------------------------------------------
 
c       Integration Algorithm:
 
c    2nd order Runge-Kutta:
c
c      x1 = dt * v(n)
c      v1 = dt * (q/m) * (E(n) + v(n) X B(n))
c
c      x2 = dt * ( v(n) + 0.5 * v1)
c      v2 = dt * (q/m) * (E(n+1/2) + [v(n) + 0.5 *v1] X B(n+1/2))
c
c      x --> x + x2
c      v --> v + v2
c
c  where E/B(n) is at the initial position (x) and time (t),
c  and E/B(n+1/2) is at x+0.5*x1 and t+0.5*dt
c
c------------------------------------------------
 
	if (firstp) then
	  firstp = .false.
c compute -eE/m
	  do j = 0, nz_mc
	    do i = 0, nr_mc
	      acx(i,j) = epl_r_mc(i,j,1) / (2.*gpf) * spec_tq
c     ^		* 964845820549.d0 / spec_tm * spec_tq
	      acz(i,j) = epl_z_mc(i,j,1) / (2.*gpf) * spec_tq
c     ^		* 964845820549.d0 / spec_tm * spec_tq
	    enddo
	    if (mask_mc(0,j).eq.0) acx(0,j) = 0.0
	  enddo
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      aczmx(i,j) = max( abs(acz(i-1,j-1)),
     ^		                abs(acz(i  ,j-1)),
     ^                          abs(acz(i-1,j  )),
     ^		                abs(acz(i  ,j  )) )
	      acxmx(i,j) = max( abs(acx(i-1,j-1)),
     ^		                abs(acx(i  ,j-1)),
     ^                          abs(acx(i-1,j  )),
     ^		                abs(acx(i  ,j  )) )
	    enddo
	  enddo
 
	  if (it1_max_mc.ne.1) then
	    write(*,*) '   ERROR- 1d: it1_max_mc .ne. 1',it1_max_mc
c	    pause
	    stop
	  endif
	endif
 
 
c       misc stuff used in this routine:
 
	dt        = 1.0
 
        end_traj  = .false.
        kcflag    = 0
        coll_flag = .false.
        hit_bdry  = .false.
        gptime    = 0.
 
        nextcoll = delta_t*abs(log(ranf_mc(1)))!time interval to next collision
 
c get initial position and velocity...
        i = 1
	if (nstack.ne.0) then
	  x = r_stk(nstack)
	  y = 0.
	  z = z_stk(nstack)
	  ir0 = find_ir(x,x-rt_mc(1),1)
	  iz0 = find_iz(z,z-zt_mc(1),1)
	  vx = v_stk(nstack)
	  vy = 2.0 * ranf_mc(18) - 1.0
	  vz = vx * vy
	  vy = vx * sqrt(1.0 - vy*vy)
	  t1 = 2. * pi * ranf_mc(17)
	  vx = vy * cos(t1)
	  vy = vy * sin(t1)
	  t1 = t_stk(nstack)
c	  t1 = 0.  !t_stk(nstack)
c	  if (nreg.lt.0) t1 = t_stk(nstack)
	  nstack = nstack - 1
	  good_traj = .true.
	else
	  y = 0.
 577	  z = (z_max_mc - z_min_mc) * ranf_mc(4) + z_min_mc
	  x = r_max_mc * ranf_mc(3)
	  ir0 = find_ir(x,x-rt_mc(1),1)
	  iz0 = find_iz(z,z-zt_mc(1),1)
	  if (mask_mc(ir0,iz0).ne.0) go to 577
 
	  vx = nmax(spec_t) * ranf_mc(3)
	  if (vx .gt. n_density_mc(1,ir0,iz0,spec_t)) go to 577
 
	  z = zcm_mc(iz0)
	  x = rcm_mc(ir0)
 
          call v_neutral_mc (300., spec_tm, vx, vy, vz)
 
	  t1 = 0.d0
	  good_traj = (t1.ge.t_min_region(nreg))
 
	endif
	r = x
 
c assume acceleration and vz in same direction, then
c  dt1 is the time to cross 0.5 of a cell width...
	dt1 = (abs(vz)+sqrt(vz*vz+
     ^	      (2.0 * del_xt * dz_mc(iz0))*aczmx(ir0,iz0)))
	dt1 = (2.0 * del_xt * dz_mc(iz0)) / (dt1+1.e-20)
	dt  = (abs(vx)+sqrt(vx*vx+
     ^	      (2.0 * del_xt * dr_mc(ir0))*acxmx(ir0,iz0)))
	dt  = (2.0 * del_xt * dr_mc(ir0)) / (dt+1.e-20)
	dt1 = min(dt,dt1)
c or do we have a collision first...
        if (nextcoll.gt.dt1) then
	  nextcoll = nextcoll - dt1
        else
	  dt1      = nextcoll
	  nextcoll = -1.0
        endif
 
c set some variable that will not change in this 1d calc...
	ir=ir0
	iz=iz0
	c_th = 1.0  !x / r
	s_th = 0.0  !y / r
 
	yp  = y
	vyp = vy
 
 
c------- Beginning of Time Loop -----------------------\
 
        t    = 0.d0     !time of recorded trajectory
	t    = t1
        it   = 0      !total number of time steps
        itgp = 0      !number of recorded time steps
 
	del_e_2 = delta_e * 0.5
	del_em  = n_e * delta_e
 
c do a length of time
        do while ( .true. )  !t.lt.t_max_region(Nreg))
  	  it=it+1
 
	  energy_evl = gpf*(vx*vx+vy*vy+vz*vz)
	
c do a simplified pushed in 1-d...
 
c    do a bi-linear interpolation to get the accelerations...
	  fz=min(1.,max(0.,(z-zt_mc(iz))*dzi_mc(iz)))
	  fx=min(1.,max(0.,(x-rt_mc(ir))*dri_mc(ir)))
 
	  aczm = ((1.-fx)*acz(ir-1,iz) +
     ^		      fx *acz(ir  ,iz))*fz
     ^	    +    ((1.-fx)*acz(ir-1,iz-1) +
     ^		      fx *acz(ir  ,iz-1))*(1.-fz)
	  acxm = ((1.-fx)*acx(ir-1,iz) +
     ^	    	      fx *acx(ir  ,iz))*fz
     ^	    +    ((1.-fx)*acx(ir-1,iz-1) +
     ^		      fx *acx(ir  ,iz-1))*(1.-fz)
 
	  zp = z + dt1 * ( vz + 0.5 * dt1 * aczm )
	  xp = x + dt1 * ( vx + 0.5 * dt1 * acxm )
 
	  zk = z + dt1 * vz * 0.5
c	  izp = min(Nz_mc, max(1,
c     ^          iabs( int( (zk-z_min_mc) * delta_zi_mc ) + 1 )))
	  izp = min(nz_mc, max(1, find_iz(zk, zk-z, iz) ))
	  fz=min(1.,max(0.,(zk-zt_mc(izp))*dzi_mc(izp)))
 
	  xk = x + dt1 * vx * 0.5
	  if (xk.lt.0.0.and.mask_mc(0,izp).eq.0) xk=abs(xk)
c	  ixp = min(Nr_mc, max(1,
c     ^          iabs( int(  xk * delta_ri_mc ) + 1 )))
	  ixp = min(nr_mc, max(1, find_ir(xk, xk-x, ir) ))
	  fx=min(1.,max(0.,(xk-rt_mc(ixp))*dri_mc(ixp)))
 
	  aczm = ((1.-fx)*acz(ixp-1,izp) +
     ^		      fx *acz(ixp  ,izp))*fz
     ^	    +    ((1.-fx)*acz(ixp-1,izp-1) +
     ^		      fx *acz(ixp  ,izp-1))*(1.-fz)
	  acxm =(((1.-fx)*acx(ixp-1,izp) +
     ^	    	      fx *acx(ixp  ,izp))*fz
     ^	    +    ((1.-fx)*acx(ixp-1,izp-1) +
     ^		      fx *acx(ixp  ,izp-1))*(1.-fz))
     ^		* sign(1.,xk)
 
	  vzp = vz + dt1 * aczm
	  vxp = vx + dt1 * acxm
 
c update time..
	  t1=t1+dt1
	  gptime=gptime+dt1
 
c distance travelled
	  gpmfp = sqrt( (xp-x)*(xp-x) + vy*dt1*vy*dt1 +
     ^			(zp-z)*(zp-z))
 
	  iz0=iz
	  ir0=ir
 
c          iz = min(nz_mc+1,
c     ^  	abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 ))
c	  if (zp.lt.z_min_mc) iz=0
	  iz = find_iz(zp, zp-z, iz)
 
	  if (xp.lt.0.0.and.mask_mc(0,iz).eq.0) then
	    xp = abs(xp)
	    vxp = abs(vxp)
	  endif
 
c	  ir = min(nr_mc+1, int( xp * delta_ri_mc ) + 1 )
c	  if (xp.lt.0.) ir = 0
	  ir = find_ir(xp, xp-x, ir)
 
 
c out of bounds?
   	  if (mask_mc(ir,iz).ne.0) then
	    z  = zp
	    vz = vzp
	    x  = xp
	    vx = vxp
            if (good_traj) then
	      n_e_lost = n_e_lost + 1
	      n_loss(ir,iz) = n_loss(ir,iz)+1
	      n_gain(ir,iz) = n_gain(ir,iz)-weight_reg(nreg)
 
	      energy_ev0 = (vxp*vxp+vy*vy+vzp*vzp)*gpf
	      frac_e = max(delta_e,min(del_em,
     ^	        energy_ev0+del_e_2))
	      i1 = indx_e(ir,iz,nreg)
	      i      = max(1,min(n_e,int( frac_e * delta_ei )))
	      i0     = min(n_e,i+1)
	      frac_e = 1. - mod(frac_e,delta_e)*delta_ei
	      if (.not.full_eedf_p) then
c	        tN_of_e_rz(1,i,ir,iz,Nreg) =
c     ^		  tN_of_e_rz(1,i,ir,iz,Nreg) + frac_e
c	        tN_of_e_rz(1,i0,ir,iz,Nreg) =
c     ^		  tN_of_e_rz(1,i0,ir,iz,Nreg) + 1.0 - frac_e
	        tn_of_e_rz(1,i,i1) =
     ^		  tn_of_e_rz(1,i,i1) + frac_e
	        tn_of_e_rz(1,i0,i1) =
     ^		  tn_of_e_rz(1,i0,i1) + 1.0 - frac_e
	      elseif (i1.ne.0) then
	        frac_e = 1./sqrt(energy_ev0 / gpf)
	        j1 = max(1, min(nmu_mc,
     ^			int(1+ (1. + vzp * frac_e  )*dimu_mc )))
   	        j0 = max(1, min(nphi_mc,
     ^			int(1+ (pi + atan2(vyp,vxp))*diphi_mc)))
	        i1 = j0 + nphi_mc*((j1-1) + nmu_mc*(i1-1))
	        tn_of_e_rz(1,i,i1) =
     ^		  tn_of_e_rz(1,i,i1) + dt1 * frac_e
	        tn_of_e_rz(1,i0,i1) =
     ^		  tn_of_e_rz(1,i0,i1) + dt1 * (1.0 - frac_e)
	      endif
 
	    endif
            return           ! sheath potential; terminate the trajectory
          endif
 
c go at least t_min_region(...) seconds before we record data...
	  if (good_traj.or.t1.gt.t_min_region(nreg)) then
	    itgp              = itgp+1
	    t                 = t + dt1
            total_time        = total_time + dt1! delta_t
            total_time_steps  = total_time_steps + 1
            mfp_per_step      = mfp_per_step + gpmfp
	    good_traj         = .true.
 
c energy, eV
	    energy_ev0 = ((vxp*vxp+vy*vy+vzp*vzp)*gpf+energy_evl)*.5
	    max_ke              = max(max_ke,energy_ev0)
	    energy_ev           = energy_ev0 * dt1
	    e_bar               = e_bar + energy_ev
 
	    e_bar_rz(1,ir,iz,nreg) =
     ^		e_bar_rz(1,ir,iz,nreg) + energy_ev
 
	    frac_e = max(delta_e,min(del_em,
     ^	      energy_ev0+del_e_2))
	    i1 = indx_e(ir,iz,nreg)
	    i = max(1,min(n_e,int( frac_e * delta_ei )))
	    frac_e = 1. - mod(frac_e,delta_e)*delta_ei
	    i0 = min(n_e,i+1)
	    if (.not.full_eedf_p) then
c	      tN_of_e_rz(1,i,ir,iz,Nreg) =
c     ^		tN_of_e_rz(1,i,ir,iz,Nreg) + dt1 * frac_e
c	      tN_of_e_rz(1,i0,ir,iz,Nreg) =
c     ^		tN_of_e_rz(1,i0,ir,iz,Nreg) + dt1 * (1.0 - frac_e)
	      tn_of_e_rz(1,i,i1) =
     ^		tn_of_e_rz(1,i,i1) + dt1 * frac_e
	      tn_of_e_rz(1,i0,i1) =
     ^		tn_of_e_rz(1,i0,i1) + dt1 * (1.0 - frac_e)
	    elseif (i1.ne.0) then
	      frac_e = 1./sqrt(energy_ev0 / gpf)
	      j1 = max(1, min(nmu_mc,
     ^			int(1+ (1. + vzp * frac_e  )*dimu_mc )))
   	      j0 = max(1, min(nphi_mc,
     ^			int(1+ (pi + atan2(vyp,vxp))*diphi_mc)))
	      i1 = j0 + nphi_mc*((j1-1) + nmu_mc*(i1-1))
	      tn_of_e_rz(1,i,i1) =
     ^		tn_of_e_rz(1,i,i1) + dt1 * frac_e
	      tn_of_e_rz(1,i0,i1) =
     ^		tn_of_e_rz(1,i0,i1) + dt1 * (1.0 - frac_e)
	    endif
 
            nsteps_rz(1,ir,iz,nreg) =
     ^		nsteps_rz(1,ir,iz,nreg) + dt1
 
	    v_ave(1,ir,iz,nreg) = v_ave(1,ir,iz,nreg) + dt1*vxp
	    v_ave(2,ir,iz,nreg) = v_ave(2,ir,iz,nreg) + dt1*vyp
	    v_ave(3,ir,iz,nreg) = v_ave(3,ir,iz,nreg) + dt1*vzp
 
c	    if (Nreg.ne.-1.and.
c     ^		energy_ev0-phi_pl_mc(ir,iz,1).lt.min_energy) then
c	      n_loss(1,iz) = n_loss(1,iz) + 1
c	      end_traj = .true.
c	      n_cold(NREG) = n_cold(NREG) + 1
c	      return
c	    endif
          endif
 
c time to do a collision (real or null)?
          if (nextcoll.le.0.) then
c            call COLLISIONS_mc (vxp, vyp, vzp, dt1, coll_prob,
c     x           kcflag, ir, iz, coll_flag,good_traj,
c     ^           1,1,1.,0.,zp,xp,t1,Nreg)
 
	    rngp = max_coll_freq * ranf_mc(11)
	    v    = vxp*vxp + vyp*vyp + vzp*vzp
	    energy_ev0 = gpf * v
	    v    = sqrt( v )
 
	    tot_coll_freq = 0.
            i = -1
            nc = 0
            do while (nc.lt.nq.and.i.lt.0)
              nc=nc+1
	      call q_interp_mc( energy_ev0, qx, nc)
	      tot_coll_freq = tot_coll_freq + v * qx *
     ^	         n_density_mc(1,ir,iz,ic0(nc))
	      if ( tot_coll_freq .ge. rngp .and. qx.gt.0.) i=nc
	    enddo
	    coll_prob_max = max(coll_prob_max,tot_coll_freq*dt)
	    coll_flag = .false.
	    if ( i .gt. 0 )
     ^        call collisions_mc (vxp, vyp, vzp, dt1, coll_prob,
     x           kcflag, ir, iz, coll_flag,good_traj,
     ^           1,1,1.,0.,zp,xp,t1,nreg,energy_ev0,v,i)
            nextcoll=delta_t*abs(log(ranf_mc(1)))!time interval to NEXT collision
            if (kcflag .eq. -1) return     ! attachment occurs, end trajectory
	  endif
 
c new is now old...
          z   = zp
          vz  = vzp
          x   = xp
          vx  = vxp
 
c coulomb collision of trapped guys?
	  if (fe_last_ee.ne.0.0.and.nreg.eq.-1) then
	    vzp = (vz*vz + vy*vy + vx*vx)*gpf
	    vzp = sqrt((vzp + fe_last_ee*dt1)/vzp)
	    vz  = vz * vzp
	    vy  = vy * vzp
	    vx  = vx * vzp
          endif
 
c new time step...
	  dt1 = (abs(vz)+sqrt(vz*vz+
     ^	        (2.0 * del_xt * dz_mc(iz))*aczmx(ir,iz)))
	  dt1 = (2.0 * del_xt * dz_mc(iz)) / (dt1+1.e-20)
	  dt  = (abs(vx)+sqrt(vx*vx+
     ^	        (2.0 * del_xt * dr_mc(ir))*acxmx(ir,iz)))
	  dt  = (2.0 * del_xt * dr_mc(ir)) / (dt+1.e-20)
	  dt1 = min(dt,dt1)
c or do we have a collision first?
	  if (nextcoll.gt.dt1) then
	    nextcoll = nextcoll - dt1     !shrink time to next collision....
	  else
	    dt1      = nextcoll
	    nextcoll = -1.
	  endif
 
        enddo
 
        return
        end
 
 
c---------------------------------------------------------------------72
 
 
     	subroutine fe_launch_mc(vx,vy,vz,ir,iz)
 
 
        implicit real*4 (a-h,o-z)
 
        include 'mc1.inc'
	include 'mc2.inc'
 
	real vx, vy, vz, fe_mx, e_mx, fe_tst, gpf, r
	integer ir, iz, ir0, iz0
 
c        parameter(gpf = 1.602e-12 / (0.5 * 9.109e-28))
        parameter(gpf = 1.0/2.84283897135e-16)
 
	data ir0/-1/
	data iz0/-10/
 
	save ir0,iz0,e_mx,k0,fe_mx
 
 
	if (ir0.ne.ir .or. iz0.ne.iz) then
	  ir0 = ir
	  iz0 = iz
	  k0 = 0
	  fe_mx = -1.
	  do k = n_e, 1, -1
	    fe_mx = max( fe_mx, fe_last(k,ir,iz))
	    if (k0.eq.0.and.fe_last(k,ir,iz).ne.0.0) k0 = k
	  enddo
 
c	  e_mx = max(0., min(delta_e*(k0+0.5),
c     ^		min_energy + phi_pl_mc(ir,iz,1)))*delta_ei
	  e_mx = k0  !max(0., delta_e*(k0+0.5))*delta_ei
	endif
 
 57	k = min(k0, max(1, 1+int(ranf_mc(18)*e_mx)))
	fe_tst = fe_mx * ranf_mc(5)
	if (fe_tst .gt. fe_last(k,ir,iz)) go to 57
 
	vx   = sqrt( max(0., min(delta_e*(k-ranf_mc(18)),
     ^		min_energy + phi_pl_mc(ir,iz,1))) * gpf)
	vy   = 2.0 * ranf_mc(18) - 1.0
	vz   = vx * vy
	vx   = vx * sqrt(1.0 - vy*vy)
	r    = 2.0 * pi * ranf_mc(17)
	vy   = vx * sin(r)
	vx   = vx * cos(r)
 
	return
	end
c---------------------------------------------------------------------72
 
        subroutine boundaries_mc (x, y, z, vx, vy, vz, hit_bdry,
     1		 end_traj,good_traj,r,ir,iz,ir0,iz0,
     ^                     x0,y0,z0,vx0,vy0,vz0)
 
c called by trajectory
c       deals with electrons when they reach the boundaries; the cylindrical
c       symmetry is imposed in this routine
 
        implicit real*4 (a-h,o-z)
 
c	real*8 x0,y0,z0,vx0,vy0,vz0,x,y,z,vx,vy,vz
 
	include 'mc1.inc'
	include 'mc2.inc'
 
        logical end_traj, hit_bdry,good_traj
        logical maskrp,maskzp
 
c	integer find_ir, find_iz
 
        data electron_mass / 9.10953179447e-28 /
        data erg_ev        / 1.6021892e-12 /
 
        end_traj = .false.
        hit_bdry = .false.
 
	if (sheath_potential_mc(ir,iz).le.0.0 .and.
     ^       mask_mc(ir,iz).ne.98) then
	  end_traj = .true.
	  return
	endif
 
c the DIRECTED energy must be greater than the sheath potential...
 
 57	gpv2=2.*sheath_potential_mc(ir,iz)*erg_ev/electron_mass
c 57	gpv2=2.*erg_ev/electron_mass*
c     ^	  (sheath_potential_mc(ir,iz)+8.6805555*
c     ^		(1.-(r0/r_max_mc)**2))
 
	r0         = sqrt( x0*x0 + y0*y0 + 1.e-22)
        c_th      = x0 / r0
        s_th      = y0 / r0
 
        vtheta    = -  vx0 * s_th + vy0 * c_th       !old Vtheta
	vr        =   (vx0 * c_th + vy0 * s_th)      !old Vr
 
	rf = max( rt_mc(ir), rt_mc(ir0) )
	vr2 = vr*vr + (1.- (r0/rf)**2) * vtheta*vtheta
 
c integrate Er to find change in potential...
	e0 = min(1.,max(0.,(r0-rt_mc(ir0))*dri_mc(ir0)))
	e0 = (1.-e0)*epl_r_mc(max(0, ir0-1),iz0,1)+
     ^		 e0 *epl_r_mc(       ir0   ,iz0,1)
	ef = min(1.,max(0.,(rf-rt_mc(ir))*dri_mc(ir)))
	ef = (1.-ef)*epl_r_mc(max(0, ir-1),iz0,1)+
     ^		 ef *epl_r_mc(       ir   ,iz0,1)
	rf = rf - r0
	e0 = (e0 + ef) * rf * erg_ev/electron_mass
	vr2 = vr2 - e0
	
c integrate Ez to find change in potential...
	e0 = min(1.,max(0.,(z0-zt_mc(iz0))*dzi_mc(iz0)))
	e0 = (1.-e0)*epl_z_mc(ir0,max(0, iz0-1),1)+
     ^		 e0 *epl_z_mc(ir0,       iz0   ,1)
	rf = max( zt_mc(iz), zt_mc(iz0) )
	ef = min(1.,max(0.,(rf-zt_mc(iz))*dzi_mc(iz)))
	ef = (1.-ef)*epl_z_mc(ir0,max(0, iz-1),1)+
     ^		 ef *epl_z_mc(ir0,       iz   ,1)
	rf = rf - z0
	e0 = (e0 + ef) * rf * erg_ev/electron_mass
        vz2 = vz0*vz0 - e0
 
c maskr(z)p = true if r (z) changed index and went through
c vertical (horizontal) boundary....
 
c the logic here...  whant maskz(r)p to be true if electron went
c through a z(r)-plane boundary from where it started to where it
c is now.
	maskzp=(abs(iz0-iz).ne.0.and.mask_mc(ir ,iz0).eq.0)
	maskrp=(abs(ir0-ir).ne.0.and.mask_mc(ir0,iz ).eq.0)
c so, maskz(r)p is true if [i] change z(r) cell and [ii] cell at new r(z)
c     and old z(r) is plasma region.
c this test works as long as we are not in a corner and we end up
c  in the `kitty-corner', say ir0+1,iz0+1....  if we do, then both will
c  be false and we test for that...
	if ((.not.maskrp).and.(.not.maskzp)) then
	  maskzp=(abs(iz0-iz).ne.0.and.mask_mc(ir0,iz ).ne.0)
	  maskrp=(abs(ir0-ir).ne.0.and.mask_mc(ir ,iz0).ne.0)
	endif
 
        end_traj=(((maskrp.and.maskzp.and.vr2+vz2.ge.gpv2)
     ^	    .or.(maskrp.and.vr2.ge.gpv2)
     ^	    .or.(maskzp.and.vz2.ge.gpv2)).and.
     ^      mask_mc(ir,iz).ne.98)
 
        if (end_traj) then
	  return
        endif
 
 
c       use reflecting boundary conditions:
 
        if (maskrp) then      ! reverse radial velocity component
 
c	  r         = sqrt( x0*x0 + y0*y0 + 1.e-22)
c	  c_th      = x0 / r
c	  s_th      = y0 / r
 
c	  vtheta    = -  vx0 * s_th + vy0 * c_th       !old Vtheta
c	  vr        = - (vx0 * c_th + vy0 * s_th)  !reverse Vr
 
	  vr        = - vr  !reverse Vr
 
          x         = x0
          y         = y0
          vx        = vr * c_th -  vtheta * s_th  !new Vx and Vy
          vy        = vr * s_th +  vtheta * c_th
          if (good_traj) n_coll_r = n_coll_r + 1
	  ir        = ir0
        endif
 
        if (maskzp.and.iz0.gt.iz) then      ! reverse z velocity component
	  z         = z0  !zt_mc(iz0)
          vz        = -vz0
          if (good_traj) n_coll_zmin = n_coll_zmin + 1
	  iz        = iz0
        elseif (maskzp.and.iz0.lt.iz) then      ! reverse z velocity component
	  z         = z0 !zt_mc(iz0+1)
          vz        = -vz0
          if (good_traj) n_coll_zmax = n_coll_zmax + 1
	  iz        = iz0
        endif
 
        return
        end
 
c---------------------------------------------------------------------72
 
	function escape_mc(xp,yp,zp,vxp,vyp,vzp,ir ,iz ,
     ^                         x ,y ,z ,vx ,vy ,vz ,ir0,iz0)
 
 
c called by trajectory
c       deals with electrons when they reach the boundaries;
c       this is for cartesian geometry. cylindrical is handled in boundaries_mc
 
	implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
	logical escape_mc
c	logical end_traj, hit_bdry, good_traj
	logical maskxp, maskzp
 
	integer find_ir, find_iz
 
	real gpf, vv  !, v
c	parameter(gpf = 0.5 * 9.109e-28 / 1.602e-12)
        parameter(gpf = 2.84283897135e-16)
 
 
	data electron_mass / 9.10953179447e-28 /
	data erg_ev        / 1.6021892e-12 /
 
 57	if (sheath_potential_mc(ir,iz) .le. 0.0 
     ^		.and. mask_mc(ir,iz).ne.98) then
	  escape_mc = .true.
	  return
	endif
 
	if (.not.lcyl_mc) then    !cartesian
	  vv = 0.
	  maskxp = (ir0.ne.ir.and.mask_mc(ir,iz0).ne.0)
	  if (maskxp) vv = (0.5 *(vxp + vx))**2
	  maskzp = (iz0.ne.iz.and.mask_mc(ir0,iz).ne.0)
	  if (maskzp) vv = vv + (0.5 * (vzp + vz))**2
	  if (gpf * vv .ge. sheath_potential_mc(ir,iz) 
     ^         .and. mask_mc(ir,iz).ne.98) then
	    escape_mc = .true.
	  else
	    escape_mc = .false.
	    if (maskzp) then
	      zp  = zt_mc(max(iz,iz0)) - (zp - zt_mc(max(iz,iz0)))
	      vzp = sign(vzp, real(iz0-iz))
c	      iz  = min(nz_mc+1,
c     ^          abs( int( (zp - z_min_mc) * delta_zi_mc ) + 1 ))
c	      if (zp.lt.z_min_mc) iz=0
	      iz  = max(iz, iz0)
	      iz  = find_iz(zp, zp-zt_mc(iz), iz)
	    endif
	    if (maskxp) then
	      xp  = rt_mc(max(ir,ir0)) - (xp - rt_mc(max(ir,ir0)))
	      vxp = sign(vxp, real(ir0-ir))
c	      ir  = min(nr_mc+1, int( xp * delta_ri_mc ) + 1 )
c	      if (xp.lt.0.) ir = 0
	      ir  = max(ir, ir0)
	      ir  = find_ir(xp, xp - rt_mc(ir), ir)
	    endif
	    if (mask_mc(ir,iz).ne.0) go to 57
	  endif
	else
	  write(*,*) '    error- escape_mc called and not cartesian!'
c	  pause
	  stop
	endif
 
	return
	end
 
 
c---------------------------------------------------------------------72
 
	subroutine see_mc(xp,yp,zp,vxp,vyp,vzp,ir ,iz ,
     ^                x ,y ,z ,vx ,vy ,vz ,ir0,iz0,nreg)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
	real xp,yp,zp,vxp,vyp,vzp,x,y,z,vx,vy,vz
	integer ir,iz,ir0,iz0
 
	integer i,j,k,n,nn,irf,izf
	real xf,zf,mu0,mz,mx
 
	real gpf
c	parameter(gpf = 0.5 * 9.109e-28 / 1.602e-12)
        parameter(gpf = 2.84283897135e-16)
 
c	integer find_ir,find_iz
 
	integer instack,nstack
	parameter (instack=1000)
	real*4  v_stk(instack),z_stk(instack),t_stk(instack)
	real*4  r_stk(instack)
	logical stk_err
	common /stk_com/ v_stk,z_stk,t_stk,nstack,r_stk,stk_err
 
 
	do n = 1, see_nmask_mc
	  if (mask_mc(ir,iz) .eq. see_mask_mc(n)) exit
	enddo
	if (n .gt. see_nmask_mc) return
 
	if (lcyl_mc) then
	  write(*,*) '    SEE for cylindrical not implemented'
c	  pause
	  stop
	endif
 
	j = iz - iz0
	i = ir - ir0
	if (i.eq.0.and.j.eq.0) then
	  do i = nr_mc+1,1,-1
	    if (x.eq.rt_mc(i)) exit
	  enddo
	  do j = nz_mc+1,1,-1
	    if (z.eq.zt_mc(j)) exit
	  enddo
	  if (i.ne.0) then
	    if (i.eq.ir) then
	      i = 1
	      ir0 = ir0 - 1
	    else
	      i = -1
	      ir0 = ir0 + 1
	    endif
	  endif
	  if (j.ne.0) then
	    if (j.eq.iz) then
	      j = 1
	      iz0 = iz0 - 1
	    else
	      j = -1
	      iz0 = iz0 + 1
	    endif
	  endif
	endif
	irf = ir
	izf = iz
 
	if (i.eq.0.and.j.eq.0)
     ^	  write(*,*) '    warning- see_mc, idx=idz=0',i,j
 
 
	if (max( abs(i), abs(j) ) .gt. 1)
     ^	  write(*,*) '    warning- see_mc, dindex > 1!',i,j
 
	mz = 0.
	if (zp-z .ne. 0.) mz = (xp - x) / (zp - z)
	mx = 0.
	if (xp-x .ne. 0.) mx = (zp - z) / (xp - x)
 
	if (abs(i)*abs(j).gt.0) then
 
	  xf = mz * (zt_mc(max(iz0,iz))-z) + x
	  zf = mx * (rt_mc(max(ir0,ir))-x) + z
 
	  mz = (xf-x)**2 + (zt_mc(max(iz0,iz))-z)**2
	  mx = (zf-z)**2 + (rt_mc(max(ir0,ir))-x)**2
 
	  if (mx .lt. mz) then
	    if (mask_mc(ir,iz0).eq.0) then
	      i = 0
	      zf = zt_mc(max(iz0,iz))
c	      irf = ir0
	    else
	      j = 0
	      xf = rt_mc(max(ir0,ir))
	      izf = iz0
	    endif
	  else
	    if (mask_mc(ir0,iz).eq.0) then
	      j = 0
	      xf = rt_mc(max(ir0,ir))
c	      izf = iz0
	    else
	      i = 0
	      zf = zt_mc(max(iz0,iz))
	      irf = ir0
	    endif
	  endif
 
	else
 
	  if (j .lt. 0) then
	    do k = iz0, iz, -1
	      zf = zt_mc(k)
	      if (mask_mc(ir0,k-1).ne.0) exit
	    enddo
	    izf = max( k-1, iz )
	    xf  = mz * (zf-z) + x
	  elseif (j .gt. 0) then
	    do k = iz0+1, iz
	      zf = zt_mc(k)
	      if (mask_mc(ir0,k).ne.0) exit
	    enddo
	    izf = min( k, iz )
	    xf  = mz * (zf-z) + x
	  endif
 
	  if (i .lt. 0) then
	    do k = ir0, ir, -1
	      xf = rt_mc(k)
	      if (mask_mc(k-1,iz0).ne.0) exit
	    enddo
	    irf = max( k-1, ir )
	    zf = mx * (xf-x) + z
	  elseif (i .gt. 0) then
	    do k = ir0+1, ir
	      xf = rt_mc(k)
	      if (mask_mc(k,iz0).ne.0) exit
	    enddo
	    irf = min( k, ir )
	    zf = mx * (xf-x) + z
	  endif
 
	endif
 
	if (j.eq.0) then
	  mu0 = (z-zp)**2 + (y-yp)**2
c	  mu0 = abs(xp-x)/(sqrt((xp-x)**2+(zp-z)**2)+1.e-22)
	  mu0 = abs(xp-x)/(sqrt((xp-x)**2+ mu0     )+1.e-22)
	else
	  mu0 = (x-xp)**2 + (y-yp)**2
c	  mu0 = abs(zp-z)/(sqrt((xp-x)**2+(zp-z)**2)+1.e-22)
	  mu0 = abs(zp-z)/(sqrt( mu0     +(zp-z)**2)+1.e-22)
	endif
	mu0 = min( 1., max( abs(mu0), see_min_mu_mc ) )
 
	call see_n_mc(nn,gpf*(vx*vx+vy*vy+vz*vz+
     ^		vxp*vxp+vyp*vyp+vzp*vzp)*0.5,mu0,
     ^		see_e0m_mc(n),see_dm_mc(n))
 
	see_surfq_mc(irf,izf) = see_surfq_mc(irf,izf) +
     ^	        (nn - 1.) * see_wgt_mc(nreg)
 
	do k = 1, nn
	  if (nstack.lt.instack) then
	    nstack        = nstack + 1
	    z_stk(nstack) = zf
	    r_stk(nstack) = xf
	    t_stk(nstack) = - see_wf_mc(n)
	    if (j.eq.0) then
	      v_stk(nstack) = -sign(2,i)
	    else
	      v_stk(nstack) = -sign(1,j)
	    endif
	  else
	    stk_err = .true.
	    write(*,"('    WARNING- SEE stack overflow ',i4)") instack
	  endif
	enddo
 
 
	return
	end
 
c---------------------------------------------------------------------72
 
 
        subroutine r_initial_mc (x, y, z, ir0, iz0, it0)
 
c called by trajectory
c       starting position of electron in chamber
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
	integer find_iz, find_ir
 
 
 
 
        do while (.true.)
 
	  if (lcyl_mc) then
            r = r_max_mc * sqrt(ranf_mc(3))
	  else
            r = r_max_mc * ranf_mc(3)
	  endif
          z = (z_max_mc - z_min_mc) * ranf_mc(4) + z_min_mc
	  iz0 = find_iz(z, z-zt_mc(1),1)
	  ir0 = find_ir(r, r-rt_mc(1),1)
          edens = ne_max(it0) * ranf_mc(5)
          if (edens .le. e_density_mc(it0,ir0,iz0)
     ^	    .and.mask_mc(ir0,iz0).eq.0) exit
 
        enddo
 
 
        theta = 2. * pi * ranf_mc(6)    !some azimuthal angle...
        x     = r * cos( theta )
        y     = r * sin( theta )
 
        return
        end
 
c---------------------------------------------------------------------72
 
        subroutine v_initial_mc (vx, vy, vz, ir0, iz0, it0)
 
c called by trajectory
c       initial Maxwellian velocity distribution at temperature Te (eV)
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
        data electron_mass / 9.109e-28 /
        data erg_ev        / 1.602e-12 /
        data jran          / 0 /
 
        save jran,vrx2
 
c get electron temperature...
        te = t_electron_mc(it0,ir0,iz0)
 
        bol = erg_ev * te
        exv = sqrt( 2. * bol / electron_mass )
 
c     C-28 of LASL Sampler 1
 
        r1   = ranf_mc(7)
        r2   = ranf_mc(8)
        t    = 2. * pi * r1
        rx   = exv * sqrt( -log(r2) )
        vx   = rx * cos(t)
        vy   = rx * sin(t)
        if (jran .ne. 0) then
          vz   = vrx2  * exv
          jran = 0
        else
          r3   = ranf_mc(9)
          r4   = ranf_mc(10)
          t    = 2. * pi * r3
          rx2  = exv *sqrt( -log(r4) )
          vz   = rx2 * cos(t)
          vrx2 = rx2 * sin(t) / exv
          jran = 1
        endif
 
        return
        end
 
c---------------------------------------------------------------------72
 
        subroutine x_v_initial_mc (vx, vy, vz, r, z, ir, iz, te)
 
c called by trajectory
c       initial Maxwellian velocity distribution at temperature Te (eV)
c	and uniform in ir0, iz0 cell...
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
	real vx, vy, vz, te, r, z
	integer ir,iz
 
	real electron_mass, erg_ev
	real bol, exv, r1,r2,t,rx,r3,r4  !,vrx2
	integer jran
 
        data electron_mass / 9.109e-28 /
        data erg_ev        / 1.602e-12 /
        data jran          / 0 /
 
c        save jran,vrx2
 
c first do 'uniform' in space...
c z:
	z   = min(zt_mc(iz+1),max(zt_mc(iz),
     ^                zt_mc(iz) + dz_mc(iz) * ranf_mc(4)))
c r:
	r1  = rt_mc(ir)
	r2  = rt_mc(ir+1)
	if (lcyl_mc) then
	  r   = min(r2, max(r1,
     ^          sqrt( ranf_mc(3)*(r2*r2 - r1*r1) + r1*r1)))
	else
	  r   = min(r2, max(r1, r1 + dr_mc(ir) * ranf_mc(3)))
	endif
 
        bol = erg_ev * te
        exv = sqrt( 2. * bol / electron_mass )
 
c     C-28 of LASL Sampler 1
 
        r1   = ranf_mc(7)
        r2   = ranf_mc(8)
        t    = 2. * pi * r1
        rx   = exv * sqrt( -log(r2) )
        vx   = rx * cos(t)
        vy   = rx * sin(t)
c        if (jran .ne. 0) then
c          vz   = vrx2  * exv
c          jran = 0
c        else
          r3   = ranf_mc(9)
          r4   = ranf_mc(10)
          t    = 2. * pi * r3
          rx2  = exv *sqrt( -log(r4) )
          vz   = rx2 * cos(t)
c          vrx2 = rx2 * sin(t) / exv
c          jran = 1
c        endif
 
        return
        end
 
c---------------------------------------------------------------------72
 
        subroutine collisions_mc (vx, vy, vz, dt, coll_prob,
     x             kcflag, ir, iz, coll_flag,good_traj,
     ^             it0,it1,frac_t,cfrac_t,zp,rp,tt,nreg,
     ^		   energy,v,i)
c called by trajectory
c       determines if electron has collision using null collision procedure
c       and, if so, which process the electron participates in and what its
c       final velocity is
 
        implicit real*4 (a-h,o-z)
 
c	real*8 vx,vy,vz,zp,v
 
	include 'mc1.inc'
	include 'mc2.inc'
 
        real*4 rngp,zp,rp,vp
	real*8 tt
 
        logical coll_flag,good_traj
 
	integer instack,nstack
	parameter (instack=1000)
	real*4  v_stk(instack),z_stk(instack),t_stk(instack)
	real*4  r_stk(instack)
	logical stk_err
	common /stk_com/ v_stk,z_stk,t_stk,nstack,r_stk,stk_err
 
	character*40 ele_name
	parameter(pi2 = 6.28318530718)
 
        data electron_mass / 9.109e-28 /
        data erg_ev        / 1.602e-12 /
 
c	gpf = 0.5 * 1.660565e-24 / 1.6021892e-12 * spec_tm
	gpf = 5.1821751139e-13 * spec_tm
c	gpf = 2.84283897135e-16
	gpfi = 1.0 / gpf
 
c some collision frequency from 0 to the maxium ever
c1	rngp = max_coll_freq * RANF_mc(11)
 
c1      v      = vx**2 + vy**2 + vz**2
c1	energy = gpf * v
c1	v      = sqrt(v)
 
c1	tot_coll_freq = 0.
c1      do i = 1, nq
c1	  nc = i
c1	  call Q_INTERP_mc (energy, qx, nc)
c1	  tot_coll_freq = tot_coll_freq + v * qx *
c1c     ^		n_density_mc(1,ir,iz,ic0(i))
c1     ^		(frac_t*n_density_mc(it0,ir,iz,ic0(i)) +
c1     ^          cfrac_t*n_density_mc(it1,ir,iz,ic0(i)))
c1	  if (tot_coll_freq .ge. rngp) exit
c1	enddo
 
c keep track of the max collision probability...
c1	coll_prob_max=max(coll_prob_max,tot_coll_freq*dt)
 
c test for null collision, if so get OUT!
c1	if (i.gt.nq) then
c1	  coll_flag = .false.
c1	  return
c1	endif
 
c we have a REAL collision...
 
        coll_flag = .true.               ! a collision occurs
 
c if this electron been around long enough, record collision...
        if (good_traj) then
          n_collisions(i)    = n_collisions(i) + 1
c         N_collisions(i)    = N_collisions(i) + weight_reg(nreg)
          n_coll_rz(i,ir,iz) = n_coll_rz(i,ir,iz) + 1
	  n_collisions_region(i,nreg) =
     ^	      n_collisions_region(i,nreg) + 1
        endif
 
	vx0 = vx
	vy0 = vy
	vz0 = vz
c	vxn = 0.
c	vyn = 0.
c	vzn = 0.
        call v_neutral_mc (300., a_mass(i), vxn, vyn, vzn)
 
        if (iq_flag(i) .eq. 0) then      ! elastic collision
          kcflag    = 0
 
	  if (.not.dcs_flag(i)) then
c ISOTROPIC scattering... (default setting...)
	    cth = max(-1.,min(1.,2. * ranf_mc(17) -1.))  !cos(theta) = Vz/v
	    sth = sqrt(1.-cth*cth)                       !sin(theta) = Vp/v
	    vz  = v * cth                               !new z component
	    vp  = v * sth				      !new perpendicular speed...
	    phi = pi2 * ranf_mc(18)                     !azimuthal angle
	    vx  = vp  * cos(phi)                        !new x component
c	    vy  = vp  * sin(phi)                        !new y component
	    vy  = sqrt( max( 0., v*v-vx*vx-vz*vz ) )
     ^		* sign(1., pi-phi)
	  else
            call d_cross_sections_mc (v, vx0, vy0, vz0, vx, vy, vz,
     ^		cth, sth, i)
	  endif
 
c         the following takes care of the recoil energy due to the finite
c         temperature of the gas molecules:
cgp          call V_NEUTRAL_mc (T_gas, A_mass(i), vxn, vyn, vzn)
 
cgp          vx = vx - vxn
cgp          vy = vy - vyn
cgp          vz = vz - vzn
cgp          v  = sqrt(vx**2 + vy**2 + vz**2)
 
cgp
cgp          call D_CROSS_SECTIONS_mc (v, theta, phi, i)
cgp          call NEW_VELOCITY_VECTOR_mc (vx0, vy0, vz0, vx, vy, vz,
cgp     x                            theta, phi)
cgp          cos_theta = cos(theta)
 
cgp          Rmass     = 2. / (1. + 1822.35 * A_mass(i))
cgp          f_change  = Rmass * ( 1. - cos_theta )
cgp          v_ratio   = sqrt( 1. - f_change )
 
cgp instead....
c	  call FOUR_PI_mc (vx, vy, vz)      ! do isotropic scattering....
c elastic cooling of dead cold neutral...
c	  v_ratio = sqrt( 1. -  0.00109716052 / A_mass(i))
          rmass     = 2. / (1. + 1822.35 * a_mass(i))
          f_change  = rmass * ( 1. - cth )
          v_ratio   = sqrt( 1. - f_change )
c	  v_ratio   = 1.
          if (good_traj) then
	    n_elastic = n_elastic + 1
	    pow_c(ir,iz,nreg) = pow_c(ir,iz,nreg) -
     ^		energy*(1.-v_ratio*v_ratio)
	  endif
 
	elseif (iq_flag(i) .eq. 3) then   !heavy particle collision...
	  kcflag = 0
 
          call v_neutral_mc (300., a_mass(i), vxn, vyn, vzn)
 
	  vx0 = vx0 - vxn
	  vy0 = vy0 - vyn
	  vz0 = vz0 - vzn
	  v   = sqrt(vx0*vx0 + vy0*vy0 + vz0*vz0)
 
	  if (.not.dcs_flag(i)) then
c ISOTROPIC scattering... (default setting...)
	    cth = max(-1.,min(1.,2. * ranf_mc(17) -1.))  !cos(theta) = Vz/v
	    sth = sqrt(1.-cth*cth)                       !sin(theta) = Vp/v
	    vz  = v * cth                               !new z component
	    vp  = v * sth				      !new perpendicular speed...
	    phi = pi2 * ranf_mc(18)                     !azimuthal angle
	    vx  = vp  * cos(phi)                        !new x component
	    vy  = vp  * sin(phi)                        !new y component
	  else
            call d_cross_sections_mc (v, vx0, vy0, vz0, vx, vy, vz,
     ^		cth, sth, i)
	  endif
	
	  v_ratio = sqrt( 1.0 - 2.0*spec_tm*a_mass(i) *
     ^		 (1.0 - cth) /
     ^		((a_mass(i)+spec_tm)*(a_mass(i)+spec_tm)) )
          if (good_traj) then
	    n_elastic = n_elastic + 1
	    pow_c(ir,iz,nreg) = pow_c(ir,iz,nreg) -
     ^		energy*v_ratio*v_ratio
	  endif
 
        elseif (iq_flag(i) .eq. 1) then      ! excitation collision
          kcflag      = 1
          if (good_traj) then
	    n_inelastic = n_inelastic + 1
	    pow_c(ir,iz,nreg) = pow_c(ir,iz,nreg) - energy_loss(i)
	  endif
	  if (.not.dcs_flag(i)) then
c ISOTROPIC scattering... (default setting...)
	    cth = max(-1.,min(1.,2. * ranf_mc(17) -1.))  !cos(theta) = Vz/v
	    sth = sqrt(1.-cth*cth)                       !sin(theta) = Vp/v
	    vz  = v * cth                               !new z component
	    vp  = v * sth				      !new perpendicular speed...
	    phi = pi2 * ranf_mc(18)                     !azimuthal angle
	    vx  = vp  * cos(phi)                        !new x component
	    vy  = vp  * sin(phi)                        !new y component
	  else
            call d_cross_sections_mc (v, vx0, vy0, vz0, vx, vy, vz,
     ^		cth, sth, i)
	  endif
c         v_ratio     = sqrt( max(0.,v*v - energy_loss(i) * gpfi)) / 
c    .              (v+1.e-20)
          v_ratio     = sqrt( v*v - energy_loss(i) * gpfi) / v
        elseif (iq_flag(i) .eq. -1) then   ! attachment collision
          kcflag   = -1
          if (good_traj) then
	    n_attach = n_attach + 1
	    n_gain(ir,iz) = n_gain(ir,iz)-weight_reg(nreg)
	    pow_c(ir,iz,nreg) = pow_c(ir,iz,nreg) - energy
            no_loss(2,nreg) = no_loss(2,nreg) + 1
	  endif
          return
        elseif (iq_flag(i) .eq. 2) then      ! ionization collision
          kcflag = 2
          if (good_traj) then
	    n_ion  = n_ion + 1
	    n_gain(ir,iz) = n_gain(ir,iz)+weight_reg(nreg)
	    pow_c(ir,iz,nreg) = pow_c(ir,iz,nreg) - energy_loss(i)
	  endif
 
c         v_ratio = max(0.,energy - energy_loss(i))   !maxium final energy...
	  v_ratio = energy - energy_loss(i)   !maxium final energy...
 
c do diff. ionization cross section if we have it...
 
	  ele_name = coll_p_name(i)
	  if (ion_equal_e_p) then
	    rngp = v_ratio * 0.5
c Neon!
	  elseif (ele_name.eq.'NE ') then !neon ionization...
	    t0 = -6.49 - 100.0/(energy+43.2)
	    t1 = 24.3 * energy / (energy + 21.6)
	    tn0 = atan((max(0.,energy-21.6)-t0)/t1)
	    tn1 = atan(-t0/t1)
	    rngp = min(v_ratio,max(0.,
     ^		t0 + t1 * tan((tn0-tn1)*ranf_mc(12)+tn1)))
c Xeon!
	  elseif (ele_name.eq.'XE ') then !xenon ionization...
	    t0 = 3.81 - 100.0/(energy+24.2)
	    t1 = 7.93 * energy / (energy - 11.5)
	    tn0 = atan((max(0.,energy-12.03)-t0)/t1)
	    tn1 = atan(-t0/t1)
	    rngp = min(v_ratio,max(0.,
     ^		t0 + t1 * tan((tn0-tn1)*ranf_mc(12)+tn1)))
c Helium!
	  elseif (ele_name.eq.'HE '.or.ele_name.eq.'HE(1S1S1S)') then
	    t0 = -2.25 - 100.0/(energy+49.0)
	    t1 = 15.5 * energy / (energy + 24.5)
	    tn0 = atan((max(0.,energy-24.5)-t0)/t1)
	    tn1 = atan(-t0/t1)
	    rngp = min(v_ratio,max(0.,
     ^		t0 + t1 * tan((tn0-tn1)*ranf_mc(12)+tn1)))
c Argon!
	  elseif (ele_name.eq.'AR ') then !argon ionization...
	    t0 = 6.87 - 100.0/(energy+31.4)
	    t1 = 6.92 * energy / (energy - 7.85)
	    tn0 = atan((max(0.,energy-15.7)-t0)/t1)
	    tn1 = atan(-t0/t1)
	    rngp = min(v_ratio,max(0.,
     ^		t0 + t1 * tan((tn0-tn1)*ranf_mc(12)+tn1)))
c everything else...
	  else
c fine just do uniformly distributed then...
	    rngp = v_ratio * ranf_mc(12)
	  endif
 
c	  t0 = max(0.,v_ratio - rngp)	  !energy of #2 = available energy - energy of #1
	  t0 = v_ratio - rngp	  !energy of #2 = available energy - energy of #1
	  v_ratio = min(t0,rngp)   !store lower  energy electron
	  rngp    = max(t0,rngp)   !run   higher energy electron
 
	  if ( secondary_p .and. good_traj ) then 	!store one?
	    nstack        = nstack + 1
	    if ( nstack .ge. instack ) then
	      nstack  = instack
	      stk_err = .true.
	      write(*,"('WARNING- ionization stack overflow ',i4)") instack
	    endif
	    z_stk(nstack) = zp
	    r_stk(nstack) = rp
	    v_stk(nstack) = sqrt(max(0.,v_ratio)*gpfi)
	    t_stk(nstack) = real(tt)
 	    if (nreg.gt.-1.and.max(nr_mc,nz_mc).ne.1
     ^	       .and.(.not.accum_t_p)) t_stk(nstack)=0.
	  elseif ( ranf_mc(12) .lt. 0.5 ) then	!if only doing one, pick it at random...
	    rngp = v_ratio
	  endif
	  v_ratio = sqrt(rngp / energy)
	  if (.not.dcs_flag(i)) then
c ISOTROPIC scattering... (default setting...)
	    cth = max(-1.,min(1.,2. * ranf_mc(17) -1.))  !cos(theta) = Vz/v
	    sth = sqrt(1.-cth*cth)                       !sin(theta) = Vp/v
	    vz  = v * cth                               !new z component
	    vp  = v * sth				      !new perpendicular speed...
	    phi = pi2 * ranf_mc(18)                     !azimuthal angle
	    vx  = vp  * cos(phi)                        !new x component
	    vy  = vp  * sin(phi)                        !new y component
	  else
            call d_cross_sections_mc (v, vx0, vy0, vz0, vx, vy, vz,
     ^		cth, sth, i)
	  endif
        endif
 
        vx = v_ratio * vx + vxn      ! new velocity components
        vy = v_ratio * vy + vyn
        vz = v_ratio * vz + vzn
 
c           if (vx.le.-1e20.or.vx.gt.1.e20.or.
c    .          vy.le.-1e20.or.vy.gt.1.e20.or.
c    .          vz.le.-1e20.or.vz.gt.1.e20) then
c            write(*,*) 'coll:',v_ratio,vx,vy,vz,vxn,vyn,vzn,
c    .         iq_flag(i),i,dcs_flag(i),vx0,vy0,vz0,cth,sth,
c    .         energy,energy_loss(i),rngp,t0
c            stop
c           endif

        return
        end
 
c---------------------------------------------------------------------72
 
        subroutine new_velocity_vector_mc (v, vx, vy, vz, vxp, vyp, vzp,
     x           cts, sts, cps, sps)
 
c called by d_cross_sections
c       performs coordinate rotations to get new velocity vector after
c       electron has been scattered
 
        implicit real*4 (a-h,o-z)
 
c       vx, vy, vz     = original electron velocity components (input)
c       vxp, vyp, vzp  = final velocitiy components (output)
c       cts (sts) = cos (sin) of polar scattering angle
c       cps (sps) = cos (sin) of azimuthal scattering angle
 
 
c     spherical components of original velocity vector:
 
c     trig functions of original angles:
 
        b1  =  (v * sts) * cps
        b2  =  (v * sts) * sps
        b3  =   v * cts
 
	vi = 1.0 / (v + 1.e-30)       !make sure it's not a hard zero
 
	ct  = vz * vi                 !cos(original theta)
	vp  = sqrt( vx*vx + vy*vy )
	st  = vp * vi                 !sin(original theta)
 
	vp = 1.0 / (vp + 1.e-30)
	cp = (vx + 1.e-30) * vp
	sp = vy            * vp
 
c     new velocity components:
        vxp =  cp * (ct * b1) - sp * b2 + cp * (st * b3)
        vyp =  sp * (ct * b1) + cp * b2 + sp * (st * b3)
        vzp = -st       * b1            + ct       * b3
 
 
        return
        end
 
c---------------------------------------------------------------------72
 
        subroutine t_cross_sections_mc (v)
 
 
c called by mc_input
c       Total collision cross sections (cm^2)
 
        implicit real*4 (a-h,o-z)
 
        include 'mc1.inc'
	include 'mc2.inc'
 
 
        data electron_mass / 9.109e-28 /
        data erg_ev        / 1.602e-12 /
 
c        energy = 0.5 * electron_mass * v**2 / erg_eV  ! electron energy in eV
	energy = v
 
	nc = 1
	do while (nc.le.nq)
 
	  call q_interp_mc (energy, qx, nc)
	  sigma(nc) = qx
	  nc = nc + 1
 
        enddo
 
        return
        end
 
c---------------------------------------------------------------------72
 
 
       subroutine d_cross_sections_mc (v, vx0, vy0, vz0, vx, vy, vz,
     ^		cth, sth, i)
 
c called by collisions
c       use differential cross section, if available, to determine
c       scattering angles or else use isotropic scattering
 
 
       implicit real*4 (a-h,o-z)
c
	include 'mc1.inc'
	include 'mc2.inc'
 
	real q_max,q_tmp(inadcs)!,vp
c
        parameter(electron_mass=9.10953179447e-28)
        parameter(erg_ev=1.6021892e-12)
        parameter(pi2=6.28318530718)
 
c
 
	beta   = dcs_beta(i)             !cos of the scattering angle...
	abeta  = 3.0 * abs(beta)
 
c	if (.not.dcs_flag(i)) then
c ISOTROPIC scattering... (default setting...)
 
c	  cth = max(-1.,min(1.,2. * RANF_mc(17) -1.))  !cos(theta) = Vz/v
c	  sth = sqrt(1.-cth*cth)                       !sin(theta) = Vp/v
c	  vz  = v * cth                               !new z component
c	  vp  = v * sth				      !new perpendicular speed...
 
c	  phi = pi2 * RANF_mc(18)                     !azimuthal angle
c	  vx  = vp  * cos(phi)                        !new x component
c	  vy  = vp  * sin(phi)                        !new y component
 
c	  return
 
c	elseif (abeta.lt.1.0) then
	if (abeta.lt.1.0) then
c ANISOTROPIC scattering, f(mu) = 0.5 * ( 1 + 3 beta mu), find some mu...
 
	  sbta = sign(1.0,beta)
	  phi  = ranf_mc(17)                          !some random # to find cos(of angle scattered)
	  cth  = sbta * max(-1.,min(1.,
     ^	    (4.0 * phi + abeta - 2.0) /
     ^	    (1.0 + sqrt( 4.0 * phi * abeta +
     ^	    (abeta - 1.0) * (abeta - 1.0)))))
	  sth  = sqrt(1.0 - cth * cth)                ! and the corresponding sin(...)
	  phi  = pi2 * ranf_mc(18)		      ! and the azimuthal angle is uniformly distributed
 
	elseif (abeta.le.3.1) then
c ANISOTROPIC scattering, f(mu) =  2 (mu - mu_l) / (mu_l - 1)^2, find some mu; mu_l =< mu =< 1
c                                 mu_l = 3 beta - 2
	  abeta = min(abeta,3.0)
	  sbta  = sign(1.0,beta)
	  phi   = pi2 * ranf_mc(18)
	  cth   = sbta * max(-1.,min(1.,
     ^		  abeta - 2.0 -
     ^		  sqrt(ranf_mc(17)) * (abeta - 3.0)))
	  sth   = sqrt(1.0 - cth * cth)
 
	else
c ANISOTROPIC scattering- use table....
          energy = v * v * 0.5 * electron_mass / erg_ev ! electron energy in eV
	  k0 = nedcs(i)
	  if (energy.le.edcs(1,i)) then
	    k   = 0
	    fk  = 0.0
	    fk1 = 1.0
	  elseif (energy.ge.edcs(k0,i)) then
	    k   = k0 - 1
	    fk  = 0.0
	    fk1 = 1.0
	  else
	    n0  = 0
	    n1  = k0
	    k   = (n1 + n0) / 2
	    do while (n1 - n0 .gt. 1)
	      k = (n1 + n0) / 2
	      if (edcs(k,i).gt.energy) then
	        n1 = k
	      else
	        n0 = k
	      endif
	    enddo
	    k   = n0
	    fk1 = (energy - edcs(k,i))/(edcs(k+1,i) - edcs(k,i))
	    fk  = 1.0 - fk1
	  endif
c ok, the energy is between k and k+1, find interpolated dq/dsigma
          k0 = nadcs(i)
          q_max = 0.0
          do j = 1, k0
	    q_tmp(j) = qdcs(j,k,i)*fk + qdcs(j,k+1,i)*fk1
	    q_max    = max(q_max, q_tmp(j))
	  enddo
c ok, now generate angles until we find one that is valid....
	  do while (.true.)
	    cth = ranf_mc(17) * 2.0 - 1.0  ! * pi
	    if (cth.ge.adcs(1,i)) then   !cos(theta)....
	      fk = q_tmp(1)
	    elseif (cth.le.adcs(k0,i)) then
	      fk = q_tmp(k0)
	    else
	      n0  = 1
	      n1  = k0
	      k   = (n1 + n0) / 2
	      do while (n1 - n0 .gt. 1)
	        k = (n1 + n0) / 2
	        if (adcs(k,i).lt.cth) then
	          n1 = k
	        else
	          n0 = k
	        endif
	      enddo
	      fk = (q_tmp(n1) - q_tmp(n0))*(cth - adcs(n0,i))
     ^		      /(adcs(n1,i) - adcs(n0,i)) + q_tmp(n0)
	    endif
	    if (ranf_mc(17).le.fk/q_max) exit
	  enddo
 
	  phi = pi2 * ranf_mc(18)
	  sth   = sqrt(1.0 - cth * cth)
 
	endif
 
        call new_velocity_vector_mc (v, vx0, vy0, vz0, vx, vy, vz,
     ^      cth,sth,cos(phi),sin(phi))
 
	return
	end
 
c---------------------------------------------------------------------72
 
        subroutine v_neutral_mc (tg, amass, vx, vy, vz)
 
c called by collisions
c       Maxwellian velocity distribution at temperature Tg (K)
c       for an atom or molecule of mass Amass (amu)
 
c WARNING: currently not used...
        implicit real*4 (a-h,o-z)
 
        data pi        / 3.141592654 /
        data amu       / 1.660e-24 /
        data erg_ev    / 1.602e-12 /
        data jran      / 0 /
 
        save jran,vrx2
 
        bol = erg_ev * tg / 11605.
        exg = sqrt( 2. * bol / (amass * amu) )
 
c     C-28 of LASL Sampler 1
 
        r1   = ranf_mc(13)
        r2   = ranf_mc(14)
        t    = 2. * pi * r1
        rx   = exg * sqrt( -log(r2) )
        vx   = rx * cos(t)
        vy   = rx * sin(t)
        if (jran .ne. 0) then
          vz   = vrx2
          jran = 0
        else
          r3   = ranf_mc(15)
          r4   = ranf_mc(16)
          t    = 2. * pi * r3
          rx2  = exg * sqrt( -log(r4) )
          vz   = rx2 * cos(t)
          vrx2 = rx2 * sin(t)
          jran = 1
        endif
 
        return
        end
 
c---------------------------------------------------------------------72
 
        subroutine four_pi_mc (gx, gy, gz)
 
c called by collisions
c Randomization of a vector into 4 Pi
c that is, isotropic distribution...
 
        implicit real*4 (a-h,o-z)
 
        data pi2/6.28318530718/
 
        vm = sqrt(gx*gx + gy*gy + gz*gz)   !magnitude of vector...
 
c
c first find cos(theta), for isotropic, cos(theta) is
c uniform
	rt = max(-1.,min(1.,2. * ranf_mc(17) -1.))
 
c now find phi, this is uniformly distributed...
 
	r = pi2 * ranf_mc(18)
 
	r1 = sin(r)
	r2 = cos(r)
 
	gz = vm * rt            !new z component
 
	rt = sqrt(1.-rt*rt)
 
	gx = vm * rt * r2       !new x component
	gy = vm * rt * r1       !new y component
 
        return
        end
 
c---------------------------------------------------------------------72

        function ranf_mc(ind)

c should be inlined...
c called by trajectory, collisions, r_intial, v_initial, D_CROSS_SECTIONS,
c v_neutral and four_pi
        implicit real*4 (a-h,o-z)

c       Uniform random number generator from
c       Press, et al. "Numerical Recipes", 2nd edition,
c          called 'ran1'.
c       notice, idum is NOT passed in, it's saved in this
c       routine...  also, this is not just ONE generator,
c       but 20 of them!  hopefully we will NOT exhaust
c       the period of the generator this way, and no
c       correlations between random numbers....

        integer idum(20),ia,im,iq,ir,ntab,ndiv
        real*4 ranf_mc,am,eps,rnmx
c       real seconds
c       parameter(ia=16807,im=2147483647,am=1./im,
c    ^    iq=127773,ir=2836,
c    ^    ntab=32,ndiv=1+(im-1)/ntab,eps=1.2e-7,rnmx=1.-eps)
        parameter(ia=16807,im=2147483647,am=4.65661287525e-10,
     ^    iq=127773,ir=2836,
     ^    ntab=32,ndiv=67108864,eps=1.2e-7,rnmx=1.-eps)

        integer j,k,iv(ntab,20),iy(20)

c       data iv /ntab*0/, iy /0/
c initial values.....
        data iv /640*0/, iy /20*0/
        data idum /20*0/

c do ANSI standard and save the variables...
        save iv,iy,idum

        if (ind.gt.0) then      !generate a random number
          k=idum(ind)/iq
          idum(ind)=ia*(idum(ind)-k*iq)-ir*k
          if (idum(ind).lt.0) idum(ind)=idum(ind)+im
          j=1+iy(ind)/ndiv
          iy(ind)=iv(j,ind)
          iv(j,ind)=idum(ind)
          ranf_mc=min(am*iy(ind),rnmx)
        else                    !initialize
          ind1=abs(ind)
          do ind=1,20           !start each generator
            idum(ind) = ind+ind1        !initial seed
            do j=ntab+8,1,-1
              k=idum(ind)/iq
              idum(ind)=ia*(idum(ind)-k*iq)-ir*k
              if (idum(ind).lt.0) idum(ind)=idum(ind)+im
              if (j.le.ntab) iv(j,ind)=idum(ind)
            enddo
            iy(ind)=iv(1,ind)
          enddo
          write(*,*) '(initialized random number generators)'
          ranf_mc =0.
        endif

c original (before random_seed)
c        if (idum(ind).le.0.or.iy(ind).eq.0) then             !initialize
c          write(*,*) 'initialized random number generator #',ind
c
c SYSTEM DEPENDENT SEED....  want idum(ind) to be a large integer...
cc        call idate(i1,i2,i3)
cc        idum(ind) = max(1,int(secnds(0.0)+1.0)*i1*i2*ind)  !`random' seed...
c fine, use a simple (and reproducable) seed...
c         idum(ind) = ind
c         do j=ntab+8,1,-1
c           k=idum(ind)/iq
c           idum(ind)=ia*(idum(ind)-k*iq)-ir*k
c           if (idum(ind).lt.0) idum(ind)=idum(ind)+im
c           if (j.le.ntab) iv(j,ind)=idum(ind)
c         enddo
c         iy(ind)=iv(1,ind)
c        endif
c
c        k=idum(ind)/iq
c        idum(ind)=ia*(idum(ind)-k*iq)-ir*k
c        if (idum(ind).lt.0) idum(ind)=idum(ind)+im
c        j=1+iy(ind)/ndiv
c        iy(ind)=iv(j,ind)
c        iv(j,ind)=idum(ind)
c        ranf_mc=min(am*iy(ind),rnmx)
        return
        end

c*--------------------------------------------------------------------72

        subroutine in_params_mc (err_flag)
 
c called by mc_input
c   this subroutine reads in the names of the species, their fractional
c   concentrations, and the elastic differential cross sections
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
c        common /Qmap/ e_min, e_max
 
        character nam*10
	character*40 nam40
	character*80 noblankupper_mc
        logical err_flag
	integer i0
 
c---------------------------------------------------------------------
 
	  icount = 0
	  do j = 1, nqi_mc
	    nam40 = noblankupper_mc(specn_mc(ic_mc(j)),80)
	    nam   = nam40
	    i0 = icount+1
	    do i = icount, 1, -1
	      if (nam40.eq.name_cs(i)) i0 = i
	    enddo
	    if (i0.gt.icount) then
	      icount           = icount + 1
	      name_cs(icount)  = nam40
	    endif
	  enddo
	  ncomp  = icount
	  if (icount.gt.nq_max_mc) then
            print *, ' ERROR: Number of species > nq_max_mc dimension'
            write(7,*) ' ERROR: Number of species > nq_max_mc dimension'
            err_flag = .true.
            return
	  endif
 
 
        return
        end
 
c*--------------------------------------------------------------------72
 
        subroutine in_dcs_mc
c called by mc_input
 
c   this subroutine reads in the dcs cross sections
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
 
	character*80 nam80
	character*80 noblankupper_mc
 
	integer i,j,k
	real x
 
 
  210   read( 15,9001,end=215 ) nam80
	if (nam80(1:4).ne.'>>>>') go to 210
 
	read( 15,9001,end=215 ) nam80
	nam80 = noblankupper_mc(nam80,80)
        do i = 1, nq
          if (nam80.eq.noblankupper_mc(raten_in(i),80)) exit
        enddo
	if (i.gt.nq) go to 210
 
	read( 15,9001,end=215) nam80
	nam80 = noblankupper_mc(nam80,80)
c ok, find a cross section name corresponding to a total cross section...
c
	if (nam80(1:5).eq.'TABLE') then
	  dcs_flag(i) = .true.
	  dcs_beta(i) = 2.0
	  read(15,*) nedcs(i),nadcs(i)
 
	  read(15,*) (adcs(min(j,inadcs),i),j=1,nadcs(i))
	  do k = 1, nedcs(i)
	    read(15,*) edcs(min(k,inedcs),i),
     ^	     (qdcs(min(j,inadcs),min(k,inedcs),i), j=1,nadcs(i))
	  enddo
	  if (nedcs(i) .gt.inedcs) then
	    write(7,*) '   WARNING- too many energy points in dcs'
	    write(7,*) '       cutting off table'
	    write(7,*) '       (or increase inedcs in mc2.inc)'
	    write(7,*)
	    write(*,*) '       WARNING- too many energy points in dcs'
	    write(*,*) '           cutting off table'
	    write(*,*) '           (or increase inedcs in mc2.inc)'
	    write(*,*)
	    nedcs(i) = inedcs
	  endif
	  if (nadcs(i) .gt.inadcs) then
	    write(7,*) '   WARNING- too many angular points in dcs'
	    write(7,*) '       cutting off table'
	    write(7,*) '       (or increase inadcs in mc2.inc)'
	    write(7,*)
	    write(*,*) '       WARNING- too many angular points in dcs'
	    write(*,*) '           cutting off table'
	    write(*,*) '           (or increase inadcs in mc2.inc)'
	    write(*,*)
	    nadcs(i) = inadcs
	  endif
 
	  x = 0.0
	  do k = 1, nadcs(i)
	    x = max(x,adcs(k,i))
	  enddo
	  if (x.gt.2.1*pi) then
	    do k = 1, nadcs(i)
	      adcs(k,i) = adcs(k,i) * pi / 180.0
	    enddo
	  endif
	  do k = 1, nadcs(i)
	    adcs(k,i) = cos(adcs(k,i)) ! store cos(theta), not theta...
	  enddo
	  edcs(0,i) = 0.0
 
	elseif (nam80(1:4).eq.'BETA') then
	  dcs_flag(i) = .true.
	  read(15,*) dcs_beta(i)
	  dcs_beta(i) = max(-1.,min(1.0,dcs_beta(i)))
	  if (dcs_beta(i).eq.0.0) dcs_flag(i)=.false.
	endif
 
	go to 210
	
	
 215    continue
        return
 
 9001   format(a)
 
	end
 
 
 
        subroutine in_cross_mc (err_flag)
c called by mc_input
 
c   this subroutine reads in the total cross sections
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
 
        character iblnk*10, nam*10
	character*40 nam40
	character*80 noblankupper_mc,nam80
        logical err_flag
 
	integer nq_nc,i,j,igp
 
 
        data
     +   iblnk/'          '/
 
c---------------------------------------------------------------------
 
c   Initialize some arrays
 
	nq = 0
 
        do nc = 1, ncomp
c
c       species name
c
  210     read( 15,9001,end=215 ) nam40
	  nam40 = noblankupper_mc(nam40,40)
	  if ( nam40(1:7) .ne. 'SPECIES') goto 210
 
c  read in species name!
          read( 15,9001,end=215 ) nam40
	  nam40 = noblankupper_mc(nam40,40)
	  nam   = nam40
          do ic = 1, ncomp
              if (nam40 .eq. name_cs(ic)) goto 220
          enddo
          goto 210
 
c           error return:
  215     continue
 
	  if (spec_t.eq.0) then
	    write(*,*) '    ERROR: Species has not been found ',
     x                'in the cross section data file'
            write(7,*) 'ERROR: Species has not been found ',
     x                'in the cross section data file'
            err_flag = .true.
	  endif
          return
 
  220     continue
 
 
 
c	number of collisions w/ this species...
          read( 15,* )
          read( 15,* )
	  read( 15,* ) i, j, nq_nc
	  nq_nc = nq_nc + i + j + 1
 
 
c       molecular weight
          read( 15,* )
          read( 15,* )
	  igp = 0
	  if (spec_t.eq.0) then
            read( 15,* ) amolwt
	  else
	    read( 15,* ) amolwt,x
	    igp = x
	  endif
 
          read( 15,* )
          read( 15,9001,end=215 ) nam80
	  nam80 = noblankupper_mc(nam80,80)
	  if (nam80(1:4).eq.'CAR:') then
            read( 15,* )
            read( 15,* )
            read( 15,9001,end=215 ) nam80
	    nam80 = noblankupper_mc(nam80,80)
	  endif
 
c ok, start reading in the cross sections....
 
	  do i = 1, nq_nc
 
	    nq = nq + 1
	    if (nq.gt.nq_max_mc) then
              print *, ' ERROR: more cross sections than ',nq_max_mc
              print *, '   increase nq_max_mc in mc1.inc'
              err_flag = .true.
              return
	    endif
	    a_mass(nq) = amolwt
	    if (spec_t.ne.0) then
	      ic0(nq)    = igp
	      ic_mc(nq)  = igp
	    elseif (raten_fl_mc(1)
     ^		      .eq. 'mc-standalone gjp') then
	      ic_mc(nq)  = ic
	    endif
	    coll_p_name(nq) = name_cs(ic)
 
c save name for cross section..
	    raten_in(nq) = nam80
	    read( 15,* )
 
	    if ( index(nam80,'_MOMXFR') .gt. 0 ) then		!elastic
	      iq_flag(nq) = 0
	    elseif ( index(nam80,'_ION') .gt. 0 ) then		!ionization
	      iq_flag(nq) = 2
	    elseif ( index(nam80,'_ATT') .gt. 0 .or.		!attatchment
     ^               index(nam80,'_DISATT') .gt. 0 ) then
	      iq_flag(nq) = -1
	    else
	      iq_flag(nq) = 1
	    endif
	
c get delta E, number of points, type and scaling factor
	    if ( iq_flag(nq) .ne. 0 ) then
	      read( 15,* ) energy_loss(nq),x,npts_cs(nq),qscale
	    else
	      read( 15,*)  npts_cs(nq), qscale
	      energy_loss(nq) = 0.
	    endif
	
	    read( 15,* )
 
c do some checks....
            if (spec_t.ne.0) then
	      iq_flag(nq) = 3
	    elseif (iq_flag(nq).lt.-1.or.iq_flag(nq).gt.2) then
	      if (energy_loss(nq).eq.0.0) then
                print *, ' WARNING: Species ',ic,' Collision ',i,
     x            ' undefined type; using 0: momentum transfer'
		iq_flag(nq) = 0
	      elseif (energy_loss(nq).gt.0.0) then
                print *, ' WARNING: Species ',ic,' Collision ',i,
     x            ' undefined type; using 1: inelastic loss'
		iq_flag(nq) = 1
	      else
                print *, ' WARNING: Species ',ic,' Collision ',i,
     x            ' undefined type; using 1: superelastic!'
		iq_flag(nq) = 1
	      endif
	    endif
	    if (qscale.le.0.0) then
              print *, ' WARNING: Species ',ic,' Collision ',i,
     x                 ' Q scale factor =< 0; reset to 1'
              qscale = 1.
	    endif
	    if (npts_cs(nq).gt.npts) then
              print *, ' WARNING: Species ',ic,' Collision ',i,
     x                 ' too many points; using first ',npts
              print *, '  (otherwise increase npts in mc2.inc)'
	    endif
 
c get cross section....
            read( 15,* ) (e_cs(min(j,npts),nq),
     ^		  q_cs(min(j,npts),nq), j = 1, npts_cs(nq))
	
	    npts_cs(nq) = min( npts_cs(nq), npts)
 
c scale cross section to cm^2
            do j=1,npts_cs(nq)
                q_cs(j,nq) = abs(qscale *
     ^		       q_cs(j,nq) * 1.e-16)
            enddo
 
	    if (i .ne. nq_nc) then
	      read( 15,* )
              read( 15,9001,end=215 ) nam80
	      nam80 = noblankupper_mc(nam80,80)
	    endif
 
	  enddo     !next cross section for this species...
 
        enddo
 
        return
 
 9001   format(a)
 
        end
 
c*--------------------------------------------------------------------72
 
 
        subroutine q_remap_mc
 
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
 
        do nc=1, nq
 
c	  if (iq_flag(nc).eq.0) then !momentum transfer cross section
c             New Couloumb log relations from Zollweg & Liebermann (1987):
 
*           elambda = sqrt( (743. * sqrt(tel / edens))**2
*    1                + 1. / (4.*Pi * edens/3.)**(0.66667) )
*    2                / (4.8e-8 / tel)
*           dlogelam = log( sqrt( 1. + 1.4 * elambda**2) )
*
c             electron - ion momentum transfer cross section:
 
*           qion = fion(nc) * 4.*651.4 * dlogelam / (x + 1.e-6) ** 2
*           thion = thion + qion
c	  endif
 
	  if (iq_flag(nc).eq.2 .or. (iq_flag(nc) .eq. 1
     ^	   	.and. energy_loss(nc).gt.0.0)) then
	    do j = 1, npts_cs(nc)
	      if (e_cs(j,nc).le.energy_loss(nc)) q_cs(j,nc) = 0.
	    enddo
	    do j = 1, npts_cs(nc)
	      if (q_cs(j,nc).ne.0.0) exit
	    enddo
	    if (e_cs(j,nc).gt.energy_loss(nc)) then
	      j = j - 1
	      e_cs(j,nc) = energy_loss(nc)
	      q_cs(j,nc) = 0.d0
	    endif
	    do i = j, npts_cs(nc)
	      e_cs(i-j,nc) = e_cs(i,nc)
	      q_cs(i-j,nc) = q_cs(i,nc)
	    enddo
	    npts_cs(nc) = npts_cs(nc) - j
	  else
	    do j = 1, npts_cs(nc)
		e_cs(j-1,nc) = e_cs(j,nc)
		q_cs(j-1,nc) = q_cs(j,nc)
	    enddo
	    npts_cs(nc) = npts_cs(nc) - 1
	  endif
 
	  qe_cs(npts_cs(nc),nc) = 0.
	  do j = 0, npts_cs(nc) - 1
	    qe_cs(j,nc) = (q_cs(j+1,nc) - q_cs(j,nc)) /
     ^			  (e_cs(j+1,nc) - e_cs(j,nc))
	  enddo
 
	enddo
 
	return
	end
 
 
c*--------------------------------------------------------------------72
 
        subroutine q_interp_mc (x, qx, nc)
 
c should be inlined if possible...
 
        implicit real*4 (a-h,o-z)
 
	include 'mc1.inc'
	include 'mc2.inc'
 
	if (.false.) then
c	if (.true. ) then
	  qx = 0.
	  xx = x / 11.55
	  if (nc.eq.1) then
	    if (xx.lt.1.0) then
	      qx = 1.59e-15 * xx
	    else
	      qx = 1.59e-15 / sqrt(xx)
	    endif
	  elseif (nc.eq.2) then
	    if (xx.gt.1.0)
     ^		qx = 1.56e-16 * log(xx) / xx
	  else
	    xx = x / 15.9
	    if (xx.gt.1.0)
     ^		qx = 3.18e-16 * log(xx) / xx
	  endif
	  return
	endif
 
	np = npts_cs(nc)
 
	if (x .le. e_cs(0,nc)) then
	
	  qx = q_cs(0,nc)
	  return
 
	elseif (x .ge. e_cs(np,nc)) then
 
	  qx = q_cs(np,nc)
	  return
 
	else
 
c bisection search....
	  n0 = 0
	  n1 = np
	  n  = (n1 + n0) / 2
	  do while (n1 - n0 .gt. 1)
	    n = (n1 + n0) / 2
	    if (e_cs(n,nc).gt.x) then
	      n1 = n
	    else
	      n0 = n
	    endif
	  enddo
 
	  qx = qe_cs(n0,nc) * (x - e_cs(n0,nc)) + q_cs(n0,nc)
 
	  return
 
	endif
 
        end
 
c*--------------------------------------------------------------------72
 
        subroutine see_n_mc (num, e0, mu0, e0m, dm)
 
c routine to compute number of secondary electrons emitted via electron impact
 
	integer num
	real e0, mu0, e0m, dm
	real electron_mass, erg_ev  !, min_mu
	real ranf_mc
 
	real z, d
 
        parameter(electron_mass=9.10953179447e-28)
        parameter(erg_ev=1.6021892e-12)
c	parameter(min_mu=0.173648177672)  !cos(80 degrees)
 
c for the yield of secondaries, use the expression given by E.A. Burke,
c IEEE Trans. Nucl. Sci., NS-27, p 1760, (1980). this expression is
c for normal incidence. use approximate correction for angle impact
c given in H. Seiler, JAP, 54, pR1, (1983): 1 / cos(theta)
 
	z = 1.284 * e0 / e0m
	d = dm * 1.526 * (1. - exp( - z ** (1.725) )) / z ** (0.725) /
     ^			                  mu0
c     ^			max( min_mu, abs( mu0 ) )
	num = int(d)
	if (num .ne. d) then
	  d = d - int(d)
	  if (ranf_mc(19) .le. d) num = num + 1
	endif
 
	return
	end
 
 
 
c*--------------------------------------------------------------------72
 
        subroutine see_v_mc (vx, vy, vz, wf)
 
c routine to compute secondary electron emission from electron impact...
 
	real vx, vy, vz, wf
	real electron_mass, erg_ev
	real ranf_mc
 
	real r1, th1, x, pi, th2
 
        parameter(electron_mass=9.10953179447e-28)
        parameter(erg_ev=1.6021892e-12)
 
c first, let's get the speed which the secondary is coming off...
c  use expression given in H. Seiler, JAP, 54, pR1, (1983): ie
c     dN/dE = k (E - Ef - WF) / (E - Ef)^4, Ef is Fermi energy,
c  WF is work function of material (expression valid really only
c  for metals, but...). range of E is Ef + WorkFunction < E < infinity.
c  integrating the above from Ef+WorkFunction to infinity, setting equal to 1,
c  give the normalization. then setting it equal to a random number 1-r
c  we find:
c     E'^3 + E'^2  3 WF + E' 3 WF^2 (1 - 1/r) + WF^3 (1 - 1/r) = 0,
c  E' = E - Ef - WF, ie Energy measured from continuum...
c  this is a cubic equation for E', solutions are:
c     theta = arccos( sqrt(r) ),
c     x1 = - WK * (2 / sqrt(r) * cos(   theta        / 3 ) + 1)
c     x2 = - WK * (2 / sqrt(r) * cos( (theta + 2 pi) / 3 ) + 1)
c     x3 = - WK * (2 / sqrt(r) * cos( (theta - 2 pi) / 3 ) + 1)
c only one is positive (at least it better be!) and that's the energy of
c the electron. so...
 
	pi = acos(-1.)
 
	r1 = sqrt( ranf_mc(20) )
	do while (r1 .eq. 0.)
	  r1 = sqrt( ranf_mc(20) )
	enddo
 
c	th1 = acos( r1 ) / 3.
c	r1  = 2. / r1
c	x = r1 * cos( th1 ) + 1.
c	if (x .ge. 0.) then
c	  x = r1 * cos( th1 + 2. * pi / 3. ) + 1.
c	  if (x. ge. 0.) then
c	    x = min( 0., r1 * cos( th1 - 2. * pi / 3. ) + 1.)
c	  endif
c	endif
c and finally, the speed...
c	x = sqrt( - 2. * wf * x * erg_ev / electron_mass)
 
	th1 = acos( -r1 ) / 3.
	th2 = sqrt( 3. ) * sin( th1 )
	th1 = cos( th1 )
	r1  = 1. / r1
	x = 2.*r1*th1 - 1.
	if (x .lt. 0.) x = 1.e22
	r1  = r1 * (th2 - th1) - 1.
	if (r1 .lt. 0.) r1 = 1.e22
	x = min( x, r1 )
	if (x.gt.1e10) x = 1.
c
c and finally, the speed...
	x = sqrt( 2. * wf * x * erg_ev / electron_mass)
 
c ok, and now find the velocity vector. just assume we are emitting to positive
c z. the angular distribution is a cosine (ie 2 mu du) phi is uniform...
	r1 = 2. * pi * ranf_mc(17)   !phi- reference line is arbitary....
	th1 = sqrt( ranf_mc(18) )    !cos(theta_normal)
 
	vz = x * th1
	th1 = sqrt( 1. - th1 * th1)  !sin(theta_normal)
	vx = x * th1 * sin(r1)
	vy = x * th1 * cos(r1)
 
	return
	end
 
 
 
c*--------------------------------------------------------------------72
 
        subroutine chrlen_mc (string, nchar, lchar)
 
c   chrlen accepts a string of nchar characters and returns lchar,
c   the length in characters of the string up to the last nonblank.
 
        character string(nchar), null*1
 
        null=char(0)
        do i = 1, nchar
          lchar = nchar + 1 - i
          if (string(lchar) .ne. ' ' .and. string(lchar) .ne. null)
     +      return
        enddo
        lchar=0
        return
        end
 
 
 
c---------------------------------------------------------------------72
 
 
	function find_ir(rf,idir,ir0)
 
        include 'mc1.inc'
 
	integer find_ir,ir0,i
	real rf,idir
 
	if (idir.lt.0) then
	  do i = ir0, 1, -1
	    if (rf.ge.rt_mc(i)) exit
	  enddo
	else
	  do i = ir0, nr_mc
	    if (rf.le.rt_mc(i+1)) exit
	  enddo
	endif
	find_ir = i
 
	return
	end
 
 
c---------------------------------------------------------------------72
 
	function find_iz(zf,idir,iz0)
 
        include 'mc1.inc'
 
	integer find_iz,iz0,i
	real zf,idir
 
	if (idir.lt.0) then
	  do i = iz0, 1, -1
	    if (zf.ge.zt_mc(i)) exit
	  enddo
	else
	  do i = iz0, nz_mc
	    if (zf.le.zt_mc(i+1)) exit
	  enddo
	endif
	find_iz = i
 
	return
	end
 
 
c---------------------------------------------------------------------72
 
      	subroutine mc_input_mc
 
c called by BACCARAT
c       reads input parameter file and does miscellaneous setup stuff
c       calls routines to read cross sections
c       a bunch of busy work... (initialization)
 
      	implicit real*4 (a-h,o-z)
 
      	include 'mc1.inc'
      	include 'mc2.inc'
 
	logical err_flag
	logical donep
	character*80 noblankupper_mc
	character*80 fl_n(nq_max_mc),in_n
	character*100 str1
	integer j0
c	integer ic0(nq_max_mc)
	real*4 maxsign,maxsignt
	real*8 nudt(it1_mc)
	real*8 navec(inspec_mc),xc,xd,xe,xb
	real*4 phil,phih,phib(nzmax_mc+1),slp
	real*4 ee_max_old, fe_ltmp(n_e)
	integer find_ir, find_iz, random_seed
 
	logical min_energy_eq_ion_p
 
	logical start_p
 
 
	namelist /mcpar/ run_label, q_tot_data, q_dcs_data, ee_max,
     ^          e_min,random_seed,
     ^          sheath_potential0,
     ^	        t_max,t_min,harmonic_p,push_order,
     ^          b_fieldsp,m_ave_time,dumpfile_p,
     ^		n_traj,min_energy,ext_max_p,
     ^          nfe,zfe,rfe,t_min_region,full_eedf_p,nmu_mc,nphi_mc,
     ^		spec_t,spec_tm,spec_tq,  !min_energy_eq_ion_p,
     ^		fe_last_p, trapped_p,secondary_p,del_xt,ion_equal_e_p,
     ^		see_p_mc,see_mask_mc,see_nmask_mc,see_e0m_mc,see_dm_mc,
     ^		see_wf_mc,see_min_mu_mc,te0,accum_t_p,del_bf,df_mb_mc
 
	data start_p /.true./
 
	save start_p, ee_max_old
 
 
 
	df_mb_mc  = .false.
	accum_t_p = .false.
	run_label = 'mc_out'
	q_tot_data = 'none'
	q_dcs_data = 'none'
	ee_max      = 50.0
	e_min      = 0.1
	sheath_potential0 = 0.
	t_max      = 200.0 / rf_freq_mc
	t_min      = 5.0 / rf_freq_mc
	b_fieldsp  = .false.
	harmonic_p = .false.
	m_ave_time = 1500.0
	dumpfile_p = .false.
	min_energy = 0.0
	n_traj     = 1000
	fe_last_p  = .true.
	push_order = 2
	trapped_p  = .true.
	secondary_p = .true.
	do i = -1, ntmax_mc
	  t_max_region(i) = 0.
	  t_min_region(i) = 0.
	enddo
	del_xt     = 0.5
	del_bf     = 0.1
	ext_max_p  = .false.
	ion_equal_e_p = .false.
 
	nmu_mc     = 20
	nphi_mc    = 40
	full_eedf_p = .false.
 
	nfe        = 0
	zfe(1)     = 0.5 * (z_max_mc + z_min_mc)
	rfe(1)     = 0.5 * r_max_mc
 
	spec_t     = 0
	spec_tq    = -1.
	spec_tm    = 5.4858026e-4   !mass of electron in amu...
 
	min_energy_eq_ion_p = .false.
 
	te0	   = 0.0000001   !energy of electrons emitted off surface...
 
	see_p_mc   = .false.
	see_nmask_mc = 0
	see_min_mu_mc = 0.173648177672  !cos(80 degrees)
 
	do i = 1, see_i_mc
	  see_mask_mc(i) = -1
	  see_e0m_mc( i) = 1.
	  see_dm_mc(  i) = 0.
	  see_wf_mc(  i) = 1.
	enddo
	do j = 0, nzmax_mc+1
	  do i = 0, nrmax_mc+1
	    see_surfq_mc(i,j) = 0.
	  enddo
	enddo

        random_seed = 0
 
      	open (unit=5, file='mc.in',status='old')
	read(5,mcpar)
	close(5)
 
c initialize random number generators
        if (start_p)
     ^    random_seed = int(ranf_mc(-abs(random_seed)))

	if (see_p_mc) then
	  see_nmask_mc = min( see_nmask_mc, see_i_mc)
	  if (see_nmask_mc.lt.1) see_p_mc = .false.
	  see_min_mu_mc = min(1.,max(0.173648177672,see_min_mu_mc))
	endif
 
c	if (run_label.eq.'') run_label = 'mc_out'
	i = index(run_label,' ')
	if (i.eq.1) run_label = 'mc.out'
        call chrlen_mc (run_label, 8, l_label)   ! length of Run_label
 
	if (push_order.ne.4.and.push_order.ne.2) push_order = 2
 
	del_xt = min(1., max(0.001, del_xt))
 
	if (start_p) then
	  start_p = .false.
	  ee_max_old = -1.
	  min_energy_eq_ion_p = .false.
 
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      do k = 1, n_e
		fe_last(k,i,j) =0.
	      enddo
	      fe_lastt(i,j) = 0.
	    enddo
	  enddo
	  if (fe_last_p) then
	    fe_last(1,1,1) = -1.
	  else
	    fe_last(1,1,1) = 0.
	  endif
	  fe_lastmx = -1.e22
	  fe_last_ee = 0.0
 
 
	  inquire(file='fe_last.dat',exist=donep)
	  if (fe_last_p.and.donep) then
	    open(67,file='fe_last.dat',form='unformatted',
     ^					status='old')
	    read(67) i, j, k, fe_last_ee, ee_max_old
	    if (i.eq.nr_mc.and.j.eq.nz_mc.and.k.eq.n_e) then
	      write(*,"('      [reading fe_last.dat file]')")
	      read(67) (((fe_last(k,i,j),k=1,n_e),
     ^				i=1,nr_mc),j=1,nz_mc)
	      do j = 1, nz_mc
	        do i = 1, nr_mc
		  if (mask_mc(i,j).ne.0) then
		    do k = 1, n_e
		      fe_last(k,i,j) = 0.
		    enddo
		    fe_lastt(i,j) = 0.
		  else
		    xc = 0.d0
		    do k = n_e, 1, -1
		      xc = xc + fe_last(k,i,j)
		    enddo
		    fe_lastt(i,j) = xc
		    fe_lastmx = max( fe_lastmx, fe_lastt(i,j))
		  endif
	        enddo
	      enddo
	      if (fe_lastmx.le.0.0) then
		fe_last(1,1,1) = -1.
		fe_lastmx = -1.e22
		fe_last_ee = 0.
	      endif
	    else
	      fe_last_ee =  0.
	      ee_max_old = -1.
	    endif
	    close(67)
	  endif
	endif
 
	if (nfe.lt.0) then
	  nfe = 0
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      if (mask_mc(i,j).eq.0) then
		nfe = nfe + 1
		zfe(nfe)  = zcm_mc(j)
		rfe(nfe)  = rcm_mc(i)
		izfe(nfe) = j
		irfe(nfe) = i
	      endif
	    enddo
	  enddo
	endif
c get rid of any invalid spatial positions for EEDF
	k = nfe
	do i = nfe, 1, -1
	  irfe(i) = min(nr_mc, max(1,
     ^		find_ir(rfe(i),rfe(i)-rt_mc(1),1)))
	  izfe(i) = min(nz_mc, max(1,
     ^		find_iz(zfe(i),zfe(i)-zt_mc(1),1)))
	  if (zfe(i).lt.z_min_mc.or.zfe(i).gt.z_max_mc.or.
     ^	      rfe(i).lt.0.0     .or.rfe(i).gt.r_max_mc.or.
     ^	      mask_mc(irfe(i),izfe(i)).ne.0) then
	    k = k - 1
	    do j = i+1, nfe
	      zfe( j-1) =  zfe(j)
	      rfe( j-1) =  rfe(j)
	      izfe(j-1) = izfe(j)
	      irfe(j-1) = irfe(j)
	    enddo
	  endif
	enddo
	nfe = k
c and get rid of any repeat indices...
	do i = nfe, 1, -1
	  do j = 1, i-1
	    if (izfe(j).eq.izfe(i).and.irfe(i).eq.irfe(j)) exit
	  enddo
	  if (j.ne.i) then
	    k = k - 1
	    do j = i+1, nfe
	      zfe( j-1) =  zfe(j)
	      rfe( j-1) =  rfe(j)
	      izfe(j-1) = izfe(j)
	      irfe(j-1) = irfe(j)
	    enddo
	  endif
	enddo
	nfe = k
 
c let's save the old *.7 file...
	inquire(file=run_label(1:l_label)//'.7',exist=donep)
	if ( donep ) then
	  open(7,file=run_label(1:l_label)//'.7',status='old')
	  open(8,file=run_label(1:l_label)//'.7.old',status='unknown')
	  do while (.true.)
	    read(7,"(a)",err=56,end=56) str1
	    do k = len(str1), 2, -1
	      if (str1(k:k) .ne. ' ') exit
	    enddo
	    write(8,"(a)") str1(1:k)
	  enddo
 56	  close(7)
	  close(8)
	endif

      	open (unit=7, file=run_label(1:l_label)//'.7',status='unknown')  ! main output file
 
c if this is a standalone program, we must set the following
c variables:
c  nr_mc: maxium number of radial cells (int)
c  nz_mc: maxium number of axial cells (int)
c  nspec_mc: number of non-electron species (int)
c  n_density_mc(it,ir,iz,im): density of non-electron species (real)
c  e_density_mc(it,ir,iz): electron density (real)
c  t_electron_mc(it,ir,iz): electron temperature (real)
c  rt_mc(1...nr_mc+1): location of radial cell boundary (real)
c  rcm_mc(1...nr_mc): location of radial cell center (real)
c  zt_mc(1...nz_mc+1): location of axial cell boundary (real)
c  zcm_mc(1...nz_mc): location of axial cell center (real)
c  delta_r: radial mesh size (real)
c  delta_z: axial mesh size (real)
c  rf_freq_mc: rf driving frequency for inductive fields (real)
c  r_max_mc: maxium radius (real)
c  z_min_mc: minium axial height (always been 0) (real)
c  z_max_mc: maxium axial height (real)
c  sheath_potential(0...nr_mc+1,0...nz_mc+1): always been 0 (real)
c  mask_mc(1...nr_mc+1,0...nz_mc+1): mask array, if zero then plasma (int)
c  e_theta_mc(ir,iz): inductive E field (complex)
c  b_r_mc(ir,iz): inductive Br field (complex)
c  b_z_mc(ir,iz): inductive Bz field (complex)
c  epl_r_mc(ir,iz,it): 'static' Er field (real)
c  epl_z_mc(ir,iz,it): 'static' Ez field (real)
c  bpl_r_mc(ir,iz,it):  static  Br field (real)
c  bpl_z_mc(ir,iz,it):  static  Bz field (real)
c  raten_fl_mc(nc): name of collision (string*80)
c  specn_mc(nspec): name of species (string*80)
c  ic_mc(nc): species index of colliding partner (int)
c  nqi_mc: number of cross sections
c  it1_max_mc:  number of `time slices' (int)
c  ph1_t_max_mc: period of time slices (real)
c  ph1_t_min_mc: minium period of time between slices (real)
c  ph1_v_mc(0...it1_max_md): time for slice (real)
 
	delta_zi_mc = 1.0 / delta_z_mc
	delta_ri_mc = 1.0 / delta_r_mc
 
	if (trapped_p) then
	if (nr_mc.eq.1) then
 
	  phil = 0.
	  phih = 0.
	  phib(1) = 0.
	  do k = 1, nz_mc
	    v = (epl_z_mc(1,k,1) - epl_z_mc(1,k-1,1)) /
     ^        (     zt_mc(k+1) -      zt_mc(k)    )
	    phi_pl_mc(1,k,1) = phih - (
     ^        v * 0.5 * (zcm_mc(k) - zt_mc(k))**2 +
     ^        epl_z_mc(1,k-1,1) * (zcm_mc(k) - zt_mc(k)))
	    phih = phih - (
     ^        v * 0.5 * (zt_mc(k+1) - zt_mc(k))**2 +
     ^        epl_z_mc(1,k-1,1) * (zt_mc(k+1) - zt_mc(k)))
	    phib(k+1) = phih
	  enddo
 
	  if (abs(phil-phih).lt.5.0) then
	    phil=min(phil,phih)
	    phih=phil
	  endif
 
	  v = -1e22
	  v  = max(phil,phih)
	  v0 = min(phil,phih)
	  do k = 1, nz_mc
	    v  = max(v ,phi_pl_mc(1,k,1))
	    v0 = min(v0,phi_pl_mc(1,k,1))
	  enddo
	  do k = 1, nz_mc
	    phi_pl_mc(1,k,1) = phi_pl_mc(1,k,1) - v
	    phib(k)        = phib(k) - v
	  enddo
	  phib(nz_mc+1)    = phib(nz_mc+1) - v
	  phil = phil - v
	  phih = phih - v
 
	  ee_max = max( 25., min(ee_max, 1.1 * abs(v-v0) ) )
	  v0 = abs(max(phil,phih))
 
	else
 
	  phil =  1.e22
	  phih = -1.e22
 
	  v  = -1.e22
	  v0 = 0.0
	  do itm = 1, it1_max_mc
	    x  = -1.e22
	    x0 = -1.e22
	    slp = 1.e22
	    do j = 0, nz_mc+1
	      j0 = max(0,j-1)
	      j1 = min(nz_mc+1, j+1)
	      do i = 1, nr_mc+1
		i0 = max(1,i-1)
		i1 = min(nr_mc+1,i+1)
	        if (mask_mc(i,j).eq.0) then
		  x  = max(x , phi_pl_mc(i,j,itm))
	        else
		  if ( mask_mc(i1,j ) .eq. 0  .or.
     ^		       mask_mc(i ,j1) .eq. 0  .or.
     ^		       mask_mc(i ,j0) .eq. 0  .or.
     ^		       mask_mc(i0,j ) .eq. 0)
     ^              x0 = max(x0, phi_pl_mc(i,j,itm))
		    slp = min(slp, phi_pl_mc(i,j,itm))
	        endif
	      enddo
	    enddo
	    v0 = max(v0, x - x0)
	    v  = max(v , x)
	    phil = min( phil, slp)
	    phih = max( phih, x)
	  enddo
 
	  do itm = 1, it1_max_mc
	    do j = 0, nz_mc+1
	      do i = 0, nr_mc+1
		phi_pl_mc(i,j,itm) = phi_pl_mc(i,j,itm) - v
	      enddo
	    enddo
	  enddo
	  ee_max = max( 25., min(ee_max, 1.1*(phih-phil)))
c	  if ( v0 .lt. min(.1, .05 * (phih-phil)) ) v0 = 0.
	  if ( v0 .lt. .15 ) v0 = 0.
	endif
	else
	  v0 = 0.
	  phil = 0.
	  phih = 0.
	endif
 
 
	e_min     = max(0.1d0,e_min)
	delta_e   = ee_max / n_e
	delta_ei  = 1.0 / delta_e
 
	if (nqi_mc.gt.nq_max_mc) then
	   write(*,*) '    nqi_mc .gt. nq_max_mc!'
	   write(*,*) '          MC aborting...'
	   write(7,*) 'nqi_mc .gt. nq_max_mc!'
	   write(7,*) '      MC aborting...'
	endif
 
        omega0_mc    = 2. * pi * rf_freq_mc
	rf_period_mc = 1./rf_freq_mc
 
c test for uniform grid...
c	do i=1,nr_mc
c	  if (abs(rt_mc(i+1)-rt_mc(i)-delta_r_mc)/
c     ^		      delta_r_mc.gt.5.e-5) then
c	     write(*,*) 'WARNING: non-uniform grid in radius'
c	     write(*,*) '   MC may need recoding in routines:'
c	     write(*,*) '       CELL_INDEX(_in) and BOUNDARIES'
c	     write(7,*) 'WARNING: non-uniform grid in radius'
c	     write(7,*) '   MC may need recoding in routines:'
c	     write(7,*) '       CELL_INDEX(_in) and BOUNDARIES'
c	  endif
c	enddo
 
c	do i=1,nz_mc
c	  if (abs(zt_mc(i+1)-zt_mc(i)-delta_z_mc)/
c     ^		      delta_z_mc.gt.5.e-5) then
c	     write(*,*) 'WARNING: non-uniform grid in z'
c	     write(*,*) '   MC may need recoding in routines:'
c	     write(*,*) '       CELL_INDEX(_in) and BOUNDARIES'
c	     write(7,*) 'WARNING: non-uniform grid in radius'
c	     write(7,*) '   MC may need recoding in routines:'
c	     write(7,*) '       CELL_INDEX(_in) and BOUNDARIES'
c	  endif
c	enddo
 
c------------------------------------------------
 
c     MISCELLANEOUS SETUP STUFF:
 
c   cell volumes:
        vol_total = 0.
        do j=1, nz_mc
          do i=1, nr_mc
            if (lcyl_mc) then
     	      vol_cell(i,j) = pi * (rt_mc(i+1)**2 - rt_mc(i)**2)
     x                       * (zt_mc(j+1) - zt_mc(j))
	    else
	      vol_cell(i,j) = (rt_mc(i+1) - rt_mc(i)) *
     ^			      (zt_mc(j+1) - zt_mc(j))
	    endif
            vol_total     = vol_total + vol_cell(i,j)
          enddo
        enddo
 
c   areas of cell faces:
        do j=1, nr_mc
          if (lcyl_mc) then
	    a_z(j) = pi * (rt_mc(j+1)**2 - rt_mc(j)**2)
	  else
	    a_z(j) = rt_mc(j+1) - rt_mc(j)
	  endif
        enddo
 
        do k=1, nz_mc
          dz = zt_mc(k+1) - zt_mc(k)
          do j=1, nr_mc+1
	    if (lcyl_mc) then
              a_r(j,k) = 2. * pi * rt_mc(j) * dz     ! faces along r-direction
	    else
	      a_r(j,k) = dz
	    endif
          enddo                                    ! j = mesh pt, k = CELL #
        enddo
 
	err_flag = .false.
c0	open (unit=15, file=Q_dcs_data)
	call in_params_mc (err_flag)       ! reads names of atoms or molecules
c0	close(15)                       ! and differential elastic cross
                                        ! sections, if any.
c instead, since differential cross section not implement, don't bother
c w/ file...
 
        if (err_flag) then
c	  write(*,*) 'ERROR in ',q_dcs_data
c	  write(*,*) '      MC aborting...'
c	  write(7,*) 'ERROR in ',q_dcs_data
c	  write(7,*) '      MC aborting...'
c	  pause
	  stop
	endif
 
      	open (unit=15, file=q_tot_data,status='old')
      	call in_cross_mc (err_flag)          ! reads total cross sections
      	close (15)
 
        if (err_flag) then
	  write(*,*) '    ERROR in ',q_tot_dat
	  write(*,*) '          MC aborting...'
	  write(7,*) 'ERROR in ',q_tot_dat
	  write(7,*) '      MC aborting...'
c	  pause
	  stop
	endif
 
	if (raten_fl_mc(1) .eq. 'mc-standalone gjp') nqi_mc = nq
 
      	call q_remap_mc
 
c parameter consistencies...
	do k=1,nq
	  if (ic_mc(k).lt.1.or.ic_mc(k).gt.nspec_mc) then
	    write(*,"(4x,'k=',i2,', ic_mc(k)=',i2,', nspec_mc=',i2)")
     ^	      k,ic_mc(k),nspec_mc
	    write(*,*) '          MC aborting...'
	    write(7,"('k=',i2,', ic_mc(k)=',i2,', nspec_mc=',i2)")
     ^	      k,ic_mc(k),nspec_mc
	    write(7,*) '      MC aborting...'
c	    pause
	    stop
	  endif
	enddo
 
c construct mask...
	do i=1,nr_mc+1
	  n_loss(i,   0) = 0.
	  n_loss(i,nz_mc+1) = 0.
	  n_gain(i,   0) = 0.
	  n_gain(i,nz_mc+1) = 0.
	enddo
	do i=1,nz_mc
	  n_gain(nr_mc+1,i) = 0.
	  n_gain(      0,i) = 0.
	  n_loss(nr_mc+1,i) = 0.
	  n_loss(      0,i) = 0.
	enddo
 
c initialization...
	e_bar=0.
	n_elastic = 0
	n_inelastic = 0
	n_ion = 0
	coll_prob_max = 0.
	mfp_per_step = 0.
	n_coll_r = 0
	n_coll_zmin = 0
	n_coll_zmax = 0
	n_e_lost = 0
	do j=1,nz_mc
	  do i=1,nr_mc
	    n_loss( i,j)           = 0.
	    n_gain( i,j)           = 0.
	    ne_loss(i,j)	   = 0.
	  enddo
	enddo
	do nreg = -1, nt_mc
	  no_loss(1,nreg) = 0
          no_loss(2,nreg) = 0
	  do j=1,nz_mc
	    do i=1,nr_mc
	      do itm = 1, it1_mc
	        nsteps_rz(itm,i,j,nreg) = 0.
	        e_bar_rz( itm,i,j,nreg) = 0.
	      enddo
	    enddo
	  enddo
	enddo
	do nreg = -1, ntmax_mc
	  do j=0,nz_mc+1
	    do i=0,nr_mc+1
	      i0 = i+1 + (nr_mc+2)*(j+(nz_mc+2)*(nreg+1))
	      do k=1,n_e
	        do itm = 1, it1_mc
c		  tN_of_e_rz(itm,k,i,j,Nreg) = 0.
		  tn_of_e_rz(itm,k,i0) = 0.
	        enddo
	      enddo
	    enddo
	  enddo
	  do k = 1, n_e
	    do itm = 1, it1_mc
	      tn_of_e_rz(itm,k,0) = 0.
	    enddo
	  enddo
	  do j = 0, nz_mc+1
	    do i = 0, nr_mc+1
	      do k = 1, 3
		v_ave(k,i,j,nreg) = 0.
	      enddo
	      pow_t(i,j,nreg) = 0.
	      pow_c(i,j,nreg) = 0.
	      pow_i(i,j,nreg) = 0.
	    enddo
	  enddo
	enddo
 
	do j = 0, nz_mc + 1
	  do i = 0, nr_mc + 1
	    if (mask_mc(i,j).eq.0) sheath_potential_mc(i,j) = 0.
	  enddo
	enddo
c sheath potential?
        sheath_potential0 = max(0., sheath_potential0)
	if (sheath_potential0.ne.0.) then
	  sheath_potential0=max(0.,sheath_potential0)
	  do j = 0, nz_mc+1
	    j0 = max(0,       j-1)
	    j1 = min(nz_mc+1, j+1)
	    do i = 0, nr_mc+1
	      i0 = max(0,       i-1)
	      i1 = min(nr_mc+1, i+1)
	      if (mask_mc(i,j)  .ne. 0 .and.
     ^           (mask_mc(i0,j0).eq.0.or.mask_mc(i1,j0).eq.0 .or.
     ^            mask_mc(i0,j1).eq.0.or.mask_mc(i1,j1).eq.0 .or.
     ^            mask_mc(i0,j ).eq.0.or.mask_mc(i1,j ).eq.0 .or.
     ^            mask_mc(i ,j0).eq.0.or.mask_mc(i ,j1).eq.0)) then
		if (sheath_potential_mc(i,j) .ne. 0.0) then
		  sheath_potential_mc(i,j) = max(sheath_potential0 *
     ^                        sheath_potential_mc(i,j), 0.)
		else
		  sheath_potential_mc(i,j) = sheath_potential0
		endif
	      endif
	    enddo
	  enddo
	else
	  do j = 0, nz_mc + 1
	    do i = 0, nr_mc + 1
	      sheath_potential_mc(i,j) = 0.
	    enddo
	  enddo
	endif
 
c more initialiation...
        if (.not.b_fieldsp) then
          do k=0, nz_mc
            do i=0, nr_mc
              b_r_mc   (i,k) = (0.,0.)
	      b_z_mc   (i,k) = (0.,0.)
	      b_th_mc  (i,k) = (0.,0.)
	      bpl_r_mc (i,k) = 0.
	      bpl_th_mc(i,k) = 0.
	      bpl_z_mc (i,k) = 0.
            enddo
          enddo
	endif
	if ( harmonic_p ) then
	  harmonic_p = .false.
          do k=0, nz_mc
            do i=0, nr_mc
	      harmonic_p = (harmonic_p .or.
     ^	        ( max( abs(b_r_mc(i,k)), abs(b_th_mc(i,k)),
     ^		  abs(b_z_mc(i,k)), abs(e_theta_mc(i,k)),
     ^		  abs(e_z_mc(i,k)), abs(e_r_mc(i,k))) .gt. 0. ) )
            enddo
          enddo
	  if ( .not. harmonic_p )
     ^	    write(7,"('all harmonic fields are zero, ',
     ^	      'harmonic_p = .false.')")
	endif
 
c	write(*,*) 'sh (z=0): ',
c     ^	  (sheath_potential_mc(i,0),i=1,nr_mc+1)
c        write(*,*) 'sh (z=Z):',
c     ^	  (sheath_potential_mc(i,nz_mc+1),i=1,nr_mc+1)
c        write(*,*) 'sh (r=R):',
c     ^	  (sheath_potential_mc(nr_mc+1,i),i=0,nz_mc+1)
 
c compute electron # and find max electron denstiy...
	do itm = 1, it1_max_mc
          ne_max(itm) = 0.
          do k=1, nz_mc
            do i=1, nr_mc
              ne(itm,i,k) = e_density_mc(itm,i,k) * vol_cell(i,k)
	      ne_max(itm) = max( e_density_mc(itm,i,k), ne_max(itm))
            enddo
          enddo
	enddo
 
	do itm=1,it1_max_mc-1
	  nudt(itm) = 0.5d0 * (ph1_v_mc(itm+1) - ph1_v_mc(itm-1))
	enddo
	nudt(it1_max_mc) =  0.5d0 * (ph1_v_mc(1) +
     ^		    ph1_v_mc(it1_max_mc) - ph1_v_mc(it1_max_mc-1))
 
 
c       max values:
        d1max = 0.
        d2max = 0.
        d3max = 0.
        f2max = 0.
        f1max = 0.
        s1max = 0.
        f3max = 0.
        s3max = 0.
        s2max = 0.
	s4max = 0.
	do k = 1, nspec_mc
	  nmax( k) = 0.
	  navec(k) = 0.
	enddo
	do itm = 1, it1_max_mc
          xc=0.d0
          do j=1, nz_mc
            do i=1, nr_mc
              if (e_density_mc(itm,i,j) .gt. d2max)
     ^		     d2max = e_density_mc(itm,i,j)
	      if (mask_mc(i,j).eq.0) then
	        xc = xc + vol_cell(i,j)
	        do k=1,nspec_mc
	          navec(k)=navec(k)+n_density_mc(itm,i,j,k)
     ^		    * vol_cell(i,j) * nudt(itm) / ph1_t_max_mc
                  if (n_density_mc(itm,i,j,k).gt.nmax(k))
     ^	  	     nmax(k) = n_density_mc(itm,i,j,k)
	        enddo
	      endif
	    enddo
	  enddo
	enddo
c spatially averaged densities...
        x = xc
        do k=1,nspec_mc
	  nave(k)=navec(k)/xc
        enddo
	do itm = 1, it1_max_mc
          do j=0, nz_mc
            do i=0, nr_mc
	      if (itm.eq.1) then
                if (abs(e_theta_mc(i,j)).gt.f2max)
     ^		  f2max=abs(e_theta_mc(i,j))
                if (abs(b_r_mc(    i,j)).gt.f1max)
     ^		  f1max=abs(b_r_mc(    i,j))
                if (abs(b_th_mc(   i,j)).gt.f3max)
     ^		  s4max=abs(b_th_mc(   i,j))
                if (abs(b_z_mc(    i,j)).gt.f3max)
     ^		  f3max=abs(b_z_mc(    i,j))
	        if (abs(bpl_r_mc(  i,j)).gt.d1max)
     ^		  d1max=abs(bpl_r_mc(  i,j))
	        if (abs(bpl_z_mc(  i,j)).gt.d3max)
     ^		  d3max=abs(bpl_z_mc(  i,j))
	        if (abs(bpl_th_mc( i,j)).gt.s2max)
     ^		  s2max=abs(bpl_z_mc(  i,j))
	      endif
	      if (abs(epl_r_mc(i,j,itm)).gt.s1max)
     ^		s1max=abs(epl_r_mc(i,j,itm))
	      if (abs(epl_z_mc(i,j,itm)).gt.s3max)
     ^		s3max=abs(epl_z_mc(i,j,itm))
            enddo
          enddo
	enddo
 
c write some stuff out...
        write(7,*)
	write(7,"('Nr =',i3,', Nz =',i3)") nr_mc,nz_mc
	write(7,"('Nt =',i3,', Nq =',i3)") it1_max_mc,nqi_mc
        write(7,"('max Ne      = ',
     ^		1pe14.3,' (1/cc)')") d2max
        do k=1,nspec_mc
          write(7,"('#',i2,' max Nn  = ',
     ^		1pe14.3,' (1/cc)')") k,nmax(k)
        enddo
        do k=1,nspec_mc
          write(7,"('#',i2,' ave Nn  = ',
     ^		1pe14.3,' (1/cc)')") k,nave(k)
        enddo
        if (lcyl_mc) then
          write(7,"('max |E_r pl|  = ', 1pe14.3,
     ^		' (V/cm)')") s1max
          write(7,"('max |E_theta| = ', 1pe14.3,
     ^		' (V/cm)')") f2max
          write(7,"('max |E_z pl|  = ', 1pe14.3,
     ^		' (V/cm)')") s3max
          write(7,"('max |B_r|     = ', 1pe14.3,
     ^		' (gauss)')") f1max*1.d8
          write(7,"('max |B_th|    = ', 1pe14.3,
     ^		' (gauss)')") s4max*1.d8
          write(7,"('max |B_z|     = ', 1pe14.3,
     ^		' (gauss)')") f3max*1.d8
          write(7,"('max |B_r pl|  = ', 1pe14.3,
     ^		' (gauss)')") d1max*1.d8
          write(7,"('max |B_th pl| = ', 1pe14.3,
     ^		' (gauss)')") s2max*1.d8
          write(7,"('max |B_z pl|  = ', 1pe14.3,
     ^		' (gauss)')") d3max*1.d8
          write(7,*)
	else
          write(7,"('max |E_x pl|  = ', 1pe14.3,
     ^		' (V/cm)')") s1max
          write(7,"('max |E_y pll  = ', 1pe14.3,
     ^		' (V/cm)')") f2max
          write(7,"('max |E_z pl|  = ', 1pe14.3,
     ^		' (V/cm)')") s3max
          write(7,"('max |B_x|     = ', 1pe14.3,
     ^		' (gauss)')") f1max*1.d8
          write(7,"('max |B_y|     = ', 1pe14.3,
     ^		' (gauss)')") s4max*1.d8
          write(7,"('max |B_z|     = ', 1pe14.3,
     ^		' (gauss)')") f3max*1.d8
          write(7,"('max |B_x pl|  = ', 1pe14.3,
     ^		' (gauss)')") d1max*1.d8
          write(7,"('max |B_y pl|  = ', 1pe14.3,
     ^		' (gauss)')") s2max*1.d8
          write(7,"('max |B_z pl|  = ', 1pe14.3,
     ^		' (gauss)')") d3max*1.d8
          write(7,*)
	endif
	if (spec_t .eq. 0) then
c consistency check for cross sections...
	if (nqi_mc.ne.nq.and.spec_t.eq.0) then
	   write(*,*) '    nqi_mc .ne. nq!'
	   write(7,*) 'nqi_mc .ne. nq!'
	   write(*,*) '          MC aborting...'
	   write(7,*) '      MC aborting...'
c	   pause
	   stop
	endif
 
c make sure we have the same set of cross sections for the
c same set of rates
 
	if (raten_fl_mc(1) .eq. 'mc-standalone gjp') then
	  do i = 1, nq
	    raten_fl_mc(i) = raten_in(i)
	  enddo
	endif
 
	err_flag = .false.
	do i = 1, nq
	  fl_n(i) = noblankupper_mc(raten_fl_mc(i),80)
	enddo
 
	do i = 1, nq
 
	  in_n = noblankupper_mc(raten_in(i),80)
 
	  j = nq+1
	  do j0 = nq, 1, -1
	    if (fl_n(j0).eq.in_n) j = j0
	  enddo
 
	  if (j.le.nq) then
	    cross_mc(i) = j
	    ic0(i)      = ic_mc(j)
	  else
	    write(7,*) 'Error: no reaction in fluid- ',i
	    write(*,*) '    Error: no reaction in fluid- ',i
	    err_flag    = .true.
	    cross_mc(i) = 0
	    ic0(i)      = 0
	  endif
 
	enddo
 
        if (.not.err_flag) then
	  do i=1,nq
	    do j=i+1,nq
	      if (cross_mc(i).eq.cross_mc(j)) then
		err_flag = .true.
		write(7,*) 'Error: two cross sections point ',
     ^			   'to same reaction- ',i,j
		write(*,*) '    Error: two cross sections point ',
     ^			   'to same reaction- ',i,j
	      endif
	    enddo
	  enddo
c	  ic_mc(i) = ic0(i)
	endif
 
	if (err_flag) then
	  write(*,*) '    List of reactions: '
	  write(7,*) 'List of reactions: '
	  do i=1,nq
	    write(*,*) '    ',i,raten_fl_mc(i)
	    write(7,*) i,raten_fl_mc(i)
	  enddo
	  write(*,*) '    List of cross sections: '
	  write(7,*) 'List of cross sections: '
	  do i=1,nq
	    write(*,*) '    ',i,raten_in(i)
	    write(7,*) i,raten_in(i)
	  enddo
	  write(*,*) '          MC aborting...'
	  write(7,*) '      MC aborting...'
c	  pause
	  stop
	endif
	endif
	
c read if the differential cross sections if there are some...
c  assume there are none, and so it's all isotropic scattering...
	do i = 1, nq
	  dcs_flag(i) = .false.
	  dcs_beta(i) = 0.
	enddo
 
	inquire(file=q_dcs_data,exist=donep)
	if (donep.and.q_dcs_data.ne.'none') then
      	  open (unit=15, file=q_dcs_data,status='old')
      	  call in_dcs_mc
      	  close (15)
	endif
 
	max_coll_freq = 0.
	xc            = 0.d0
c now, let's go find the maxium collision frequency...
	do ei0=0.,ee_max,0.2
 
	  v = ei0
	  call t_cross_sections_mc(v)
 
 	  v=1.e2*sqrt(2.*ei0*1.75880521211e11)
	  if (spec_t.ne.0) v = 1.e2 *
     ^		sqrt( 2. * ei0 * 96484582.0549 / spec_tm)
 
	  maxsign = 0.
	  do itm = 1, it1_max_mc
	    do j = 1, nz_mc
	      do i = 1, nr_mc
	        if (mask_mc(i,j).eq.0) then
		  maxsignt = 0.
		  do k = 1, nq
		    maxsignt = maxsignt +
     ^			 sigma(k) * n_density_mc(itm,i,j,ic0(k))
	  	  enddo
		  maxsign = max(maxsign,maxsignt)
	        endif
	      enddo
	    enddo
	  enddo
	  xc = max( xc, dble(maxsign) )
 
          tot_coll_freq = maxsign * v
 
	  max_coll_freq = max(max_coll_freq,tot_coll_freq)
 
	enddo
 
	v = 1.e22
	do k = 1, nq
	  n_collisions(k) = 0
	  if (iq_flag(k).eq.2) v = min(v,energy_loss(k))
	enddo
	if (min_energy_eq_ion_p) then
	  min_energy = 0.99 * v
	else
	  min_energy = abs(min_energy)
	endif
c	if (min_energy.ge.0.0) then
c	  v = min(v, min_energy)
c	else
c	  v = abs(min_energy) / 0.99
c	endif
c	min_energy = 0.99 * v
 
	do j = 1, nz_mc
	  do i = 1, nr_mc
	    do k = 1, nq
	      n_coll_rz(k,i,j) = 0
	    enddo
	  enddo
	enddo
	do nreg = -1, nt_mc
	  do k = 1, nq
	    n_collisions_region(k,nreg) = 0
	  enddo
	enddo
 
	if (nr_mc.eq.1) then
	  if (phil.eq.phih) then
	    v0 = min_energy
	  else		!not all 'trapped'
	    if (phil.lt.phih) then
	      slp = 1.0
	      d2max = 0.0
	      do k = 1, nz_mc-1
c look for change in slope of potential...  looking for traps...
	        if (slp * (phi_pl_mc(1,k+1,1)-
     ^			phi_pl_mc(1,k,1)) .lt. 0.0) then
		  if (slp.lt.0.0)
     ^		    d2max = max(d2max,abs(phi_pl_mc(1,k,1)))
		  slp = - slp
	        endif
	      enddo
	      v0 = max(d2max,v0)
	    else
	      slp = 1.0
	      d2max = 0.0
	      do k = nz_mc, 2, -1
c look for change in slope of potential...  looking for traps...
	        if (slp * (phi_pl_mc(1,k-1,1)-
     ^			phi_pl_mc(1,k,1)) .lt. 0.0) then
		  if (slp.lt.0.0)
     ^		    d2max = max(d2max,abs(phi_pl_mc(1,k,1)))
		  slp = - slp
	        endif
	      enddo
	      v0 = max(d2max,v0)
	    endif
	  endif
 
	endif
 
c	if (min_energy_eq_ion_p) v0 = max(v0, min_energy)
	if (min_energy_eq_ion_p) then
	  if (v0 .lt. min_energy) then
	    v0 = min_energy
	    if (fe_last_p) then
	      fe_lastmx = -1.e22
	      fe_last(1,1,1) = -1.
	      fe_last_p = .false.
	      fe_last_ee = 0.
	    endif
	  endif
	endif
 
	if (.not.trapped_p) v0 = 0.
 
	xb = xc
	xc = 0.d0
	xd = 0.d0
	if (v0 .ne.0.) then
	  do xe = 0.d0, v0 , v0 * 1.d-4
	    x0 = real(xe)
 	    v  = 1.e2*sqrt(2.*xe*1.75880521211e11)
	    do i = 1, nq
	      if (iq_flag(i).eq.0) then
	        j = i
	        call q_interp_mc( x0, d2max, j)
		xc = xc + v * d2max *
     ^			nave(ic0(i)) / a_mass(i)
		xd = xd + 1.d0
	      endif
	    enddo
	  enddo
	endif
	if (xc .ne. 0.d0) then
	  x0 = xd / (xc * 0.00109716) * 5.0
c	  t_max = min(t_max, max(2.*t_min, x0))
	  t_max_region(-1) = min(t_max, max(2.*t_min, x0))
	endif
 
 
c store minium collision time
	delta_t = 1./max_coll_freq
 
	if (v0.ne.0.0) then
c	  write(*,"(4x,'electrons followed until total energy is ',
c     ^        'less than ',0pf6.3,' (eV)')") v0 !min_energy
c	  write(7,"('electrons followed until total energy is ',
c     ^        'less than ',0pf6.3,' (eV)')") v0 !min_energy
	  write(*,"(4x,'electrons followed until total energy is ',
     ^        'less than ',1p,e12.5,' (eV)')") v0 !min_energy
	  write(7,"('electrons followed until total energy is ',
     ^        'less than ',1p,e12.5,' (eV)')") v0 !min_energy
	endif
	write(*,"(4x,'maximum collision frequency:',1pe12.5,
     ^	      ' (1/s)')") max_coll_freq
	write(*,"(4x,'minimum collision time:     ',1pe12.5,
     ^	      ' (s)')") delta_t
	write(*,"(4x,'minimum mean free path:     ',1pe12.5,
     ^	      ' (cm)')") 1.0/xb
	write(7,"('maximum collision frequency:',1pe12.5,
     ^	      ' (1/s)')") max_coll_freq
	write(7,"('minimum collision time:     ',1pe12.5,
     ^	      ' (s)')") delta_t
	write(7,"('minimum mean free path:     ',1pe12.5,
     ^	      ' (cm)')") 1.0/xb
	write(7,*)
	write(*,*)
		
	close(7)
 
	if (t_max.lt.0.0 .and. m_ave_time.ne.0) then
c	  t_min = min(0.2, abs(t_min/(t_max-1.e-32)))
	  t_min = abs(t_min)/(abs(t_max)+1.e-32)
	  t_max = delta_t * m_ave_time
	  t_min = t_min * t_max
	  t_max_region(-1) = min(t_max, max(2.*t_min, x0))
	endif
	t_max = abs(t_max)
 
	x0 =    max(t_max, t_min)
	t_min = min(t_max, t_min)
	t_max = x0
 
	t_min_region(-1) = t_min
	do i = 0, nt_mc
	  if (t_min_region(i).lt.0.0) t_min_region(i) = t_min
	  n_cold(      i) = 0
	enddo
 
	do i = 0, nt_mc
	  t_max_region(i) = t_min_region(i) + t_max
	  weight_reg(i) = 1.0
	enddo
	weight_reg(-1) = 1.0
 
	if (inductp_mc) then
	  x0 = (1.0 / 6.0) * n_traj  !ionization comes from 'heating' fields
	else			     !ionization comes from cathode surfaces
	  x0 = n_traj
	  if (v0.ne.0.0) x0 = (5.0 / 6.0) * n_traj
	endif
c x0 = number of electrons ejected off and via heavy particle collisions

c old-
c find total #/s being (i) created or (ii) ejected
c	xc = 0.d0
c	do i = 0, nt_mc
c	  xc = xc + nt_f_mc(i)
c	enddo
c	xd = max(0.d0, xc - nt_f_mc(0))
c	if (xc .ne. 0.) xc = 1./xc
 
c ok, let's say that for each electron off the surface, we get 8 ions
c	xe = nt_f_mc(0) / (8.d0*xd + nt_f_mc(0) + 1d-57)
c	j  = x0 * xe
c	n_traj_region(0) = j
c	x0 = x0 - j
	
c get total #/s being ejected...
c	xc = 0.d0
c	do i = 1, nt_mc
c	  xc = xc + nt_f_mc(i)
c	enddo
c	if (xc.ne.0.d0) xc = 1.d0 / xc
c	do i = nt_mc, 1, -1
c	  n_traj_region(i) = x0 * nt_f_mc(i) * xc
c	  j                = j + n_traj_region(i)
c	enddo
c	if (v0.eq.0.0) n_traj = j
c	n_traj_region(-1) = n_traj - j
c new- number of electrons is directly proportional to the source rates
c   but each source gets at least 10% of the initially requested electrons
	xd = 0.d0
	do i = 0, nt_mc
	  xd = xd + nt_f_mc(i)
	enddo
        xc = xd
	if ( xd .ne. 0. ) xc = 1. / xd

	n_traj_region(0) = 0
	if ( nt_f_mc(0) .gt. 0.0 ) n_traj_region(0) = 
     ^	                 int(x0 * max(0.1d0, nt_f_mc(0) * xc))
c	if ( nt_f_mc(0) .gt. 0.0 ) n_traj_region(0) = max(
c    ^	  nr_mc * nz_mc, int(x0 * max(0.1d0, nt_f_mc(0) * xc)) )
	j = n_traj_region(0)
	do i = 1, nt_mc
	  if ( nt_f_mc(i) .gt. 0.0 ) then
	    n_traj_region(i) = x0 * max(0.1d0, nt_f_mc(i) * xc)
	    j                = j + n_traj_region(i)
	  else
	    n_traj_region(i) = 0
	  endif
	enddo
	if ( v0 .ne. 0.0 ) then
	  n_traj_region(-1) = n_traj - x0
	  n_traj = j + n_traj_region(-1)
	else
	  n_traj = j
	  n_traj_region(-1) = 0
	endif

c set structures...
	if (ntn_mc.ne.0) then  !number of elements
	  i = ntd_mc(6,1)	!find start/end index for each structure
	  ntsn_mc(1,i) = 1
	  do j = 2, ntn_mc
	    if (ntd_mc(6,j).ne.i) then
	      ntsn_mc(2,i) = j-1
	      i            = ntd_mc(6,j)
	      ntsn_mc(1,i) = j
	    endif
	  enddo
	  ntsn_mc(2,i) = ntn_mc
	endif
 
	do i = 1, nt_mc
	  x0 = 0.
	  do j = ntsn_mc(1,i), ntsn_mc(2,i)
	    x0 = max( x0, ntn_f_mc(j) )
	  enddo
          xc = x0
	  if (x0.ne.0.) xc = 1. / x0
	  do j = ntsn_mc(1,i), ntsn_mc(2,i)
	    ntn_f_mc(j) = ntn_f_mc(j) * xc
	  enddo
	enddo
 
c new... try to launch the electrons via how the flux IS..
	do i = 1, nt_mc
	  if (n_traj_region(i).ne.0) then
	    x0 = 0.
	    do j = ntsn_mc(1,i), ntsn_mc(2,i)
	      x0 = x0 + ntn_f_mc(j)
	    enddo
 
	    k  = 0
	    x0 = x0 / n_traj_region(i)
	    do j = ntsn_mc(1,i), ntsn_mc(2,i)
	      k0 = ntn_f_mc(j) / x0 + 1
	      ntd_mc(7,j) = k0
	      k  = k + k0
	    enddo
 
c some left over? randomly distribute the remainder...
	    if (k .lt. n_traj_region(i)) then
	      k = n_traj_region(i) - k
	      do k0 = 1, k
 477		j = int( (ntsn_mc(2,i) - ntsn_mc(1,i) + 1) *
     ^			ranf_mc(4) + ntsn_mc(1,i))
		v = ranf_mc(3)
		if (v .gt. ntn_f_mc(j)-x0*ntd_mc(7,j)) go to 477
		ntd_mc(7,j) = ntd_mc(7,j) + 1
	      enddo
	    elseif (k.gt.n_traj_region(i)) then
	      n_traj = n_traj - n_traj_region(i) + k
	      n_traj_region(i) = k
c	      write(*,*) 'error- too many e',i,k,n_traj_region(i)
c	      stop
	    endif
	  endif
	enddo
 
 
 
	if (n_traj_region(0) .ne. 0) then
	  xc = 0.d0
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      xc = xc + ele_s_mc(i,j)
	    enddo
	  enddo
 
	  k  = 0
	  x0 = xc / n_traj_region(0)
	  do j = 1, nz_mc
	    do i = 1, nr_mc
c	      k0 = ele_s_mc(i,j) / x0
c	      if (ele_s_mc(i,j).ne.0) k0 = k0 + 1
	      k0 = ele_s_mc(i,j) / x0
	      if (ele_s_mc(i,j).ne.0) k0 = 0.5 + ele_s_mc(i,j) / x0
	      ele_nl_mc(i,j) = k0
	      k  = k + k0
	    enddo
	  enddo
 
	  if (k .lt. n_traj_region(0)) then
	    k = n_traj_region(0) - k
	    do k0 = 1, k
 577	      i = max(1, min(nr_mc, int( nr_mc * ranf_mc(3) + 1)))
 	      j = max(1, min(nz_mc, int( nz_mc * ranf_mc(4) + 1)))
	      v = ele_s_max_mc * ranf_mc(4)
	      if (v .gt. ele_s_mc(i,j) - x0 * ele_nl_mc(i,j)) go to 577
	      ele_nl_mc(i,j) = ele_nl_mc(i,j) + 1
	    enddo
	  elseif (k .gt. n_traj_region(0)) then
	    n_traj = n_traj - n_traj_region(0) + k
	    n_traj_region(0) = k
c	    write(*,*) 'error- too many e',0,k,n_traj_region(0)
c	    stop
	  endif
	endif
 
	if (spec_t.ne.0) then
	  v0 = 0.
	  do i = -1, nt_mc
	    t_min_region(i) = 0.
	    t_max_region(i) = 1.0
	    n_cold(      i) = 0
	    n_traj_region(i) = 0
	    weight_reg(i)   = 1.0
	  enddo
	  weight_mx       = 1.0
	  n_traj_region(-1) = n_traj
	endif
 
	min_energy = v0
 
c if we are using the 'old' distribution, let's tighten it
c up and make sure it's valid for this confining potential...
	if (fe_last_p .and. min_energy.gt.0.0
     ^		.and. n_traj_region(-1).ne.0) then
	  if (ee_max_old.le.0.0) ee_max_old = ee_max
	  if (ee_max_old .ne. ee_max) then
	    v = ee_max_old / n_e
	    do j = 1, nz_mc
	      do i = 1, nr_mc
	        if (fe_lastt(i,j).gt.0.0) then
		  do k0 = 1, n_e
		    fe_ltmp(k0) = fe_last(k0,i,j)
		    fe_last(k0,i,j) = 0.
		  enddo
		  do k0 = n_e, 1, -1
	            x0 = max(delta_e, min(n_e*delta_e,
     ^		      v*(k0-0.5) + 0.5*delta_e))
	            k = max(1, min(n_e, int(x0*delta_ei)))
	            x0 = 1. - mod(x0, delta_e)*delta_ei
		    fe_last(k,i,j) = fe_last(k,i,j) +
     ^			    x0 *fe_ltmp(k0)
		    k = min(n_e, k+1)
		    fe_last(k,i,j) = fe_last(k,i,j) +
     ^			(1.-x0)*fe_ltmp(k0)
		  enddo
	        endif
	      enddo
	    enddo
	  endif
	  fe_lastmx = -1.e22
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      if (mask_mc(i,j).ne.0
     ^		.or.-phi_pl_mc(i,j,1).ge.min_energy
     ^						   ) then
		do k = 1, n_e
		  fe_last(k,i,j) = 0.
		enddo
		fe_lastt(i,j) = 0.
	      else
		xc = 0.d0
		do k = n_e, 1, -1
c		  if (delta_e*(k-0.5)-phi_pl_mc(i,j,1)
c     ^		    .ge.min_energy) fe_last(k,i,j) = 0.
		  xc = xc + fe_last(k,i,j)
		enddo
		fe_lastt(i,j) = xc
		fe_lastmx = max(fe_lastmx, fe_lastt(i,j))
	      endif
	    enddo
	  enddo
	  if (fe_lastmx .le. 0.0) then
	    fe_lastmx = -1e22
	    fe_last(1,1,1) = -1.
	    fe_last_p = .false.
	    fe_last_ee = 0.
	  endif
	  if (spec_t.eq.0 .and. t_run_mc.ne.0.0) then
	    v = t_max_region(-1) / t_run_mc
	    v = max(v, delta_t * 10. * 1000. / t_run_mc)
	    x = max(0., max(1.,min(1000.,v)) - 1.)
	    i = n_traj_region(-1) * x
	    n_traj = n_traj + i
	    n_traj_region(-1) = i + n_traj_region(-1)
	    t_min_region(-1) = 0.
	    t_max_region(-1) = t_run_mc
	  endif
	endif
	
	if (n_traj_region(-1).ne.0.and.spec_t.eq.0) then
	  xc = 0.d0
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      if (mask_mc(i,j).eq.0
     ^			.and.-phi_pl_mc(i,j,1).le.v0) then
		xc = xc + vol_cell(i,j)
	      endif
	    enddo
	  enddo
	  if (xc.eq.0.d0) then
	    write(*,*) '    error- no cells for ',
     ^			  minimum energy electrons!'
c	    pause
	    stop
	  endif
 
	  k  = 0
	  x0 = n_traj_region(-1) / xc
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      ele_trapped(i,j) = 0
	      if (mask_mc(i,j).eq.0
     ^			.and.-phi_pl_mc(i,j,1).le.v0) then
		k0 = vol_cell(i,j) * x0
		k = k + k0
		ele_trapped(i,j) = k0
	      endif
	    enddo
	  enddo
 
	  if (k .ge. 200) then
	    n_traj = n_traj - n_traj_region(-1) + k
	    n_traj_region(-1) = k
	  endif
 
	  if (k .lt. n_traj_region(-1)) then
	    k = n_traj_region(-1) - k
	    do k0 = 1, k
	      x0 = xc * ranf_mc(3)
	      xd = 0.d0
	      do j = 1, nz_mc
		do i = 1, nr_mc
	          if (mask_mc(i,j).eq.0
     ^			.and.-phi_pl_mc(i,j,1).le.v0) then
		    xd = xd + vol_cell(i,j)
		    if (xd.ge.x0) exit
		  endif
		enddo
		if (xd.ge.x0) exit
	      enddo
	      if (i.gt.nr_mc.or.j.gt.nz_mc) then
		write(*,*) '    error- didn''t find volume for trapped'
c		pause
		stop
	      endif
	      ele_trapped(i,j) = ele_trapped(i,j) + 1
	    enddo
	  elseif (k .gt. n_traj_region(-1)) then
	    write(*,*) 'error- too many trapped e',
     ^				-1,k,n_traj_region(-1)
c	    pause
	    stop
	  endif
	endif
 
c now, set weights so that they reflect the actual #/s being
c injected...
	weight_mx = 0.
	do i = 0, nt_mc
	  weight_reg(i) = nt_f_mc(i) /
     ^		real(max(1, n_traj_region(i)))
     	  weight_mx = max(weight_reg(i), weight_mx)
	enddo
	if (weight_mx .gt. 0.0) then
	  do i = 0, nt_mc
	    weight_reg(i) = weight_reg(i) / weight_mx
	  enddo
	else
	  weight_mx = 1.
	endif
 
	i0 = -1
	x  = 0.
	do i = 0, nt_mc
	  see_wgt_mc(i) = 0.
	  if (x.lt.nt_f_mc(i)) then
	    i0 = i
	    x  = nt_f_mc(i)
	  endif
	enddo
	if (see_p_mc) then
	  if (x.ne.0.) x = n_traj_region(i0) / x
	  do i = 0, nt_mc
	    if (n_traj_region(i).ne.0)
     ^	      see_wgt_mc(i) = nt_f_mc(i) * x / real(n_traj_region(i))
	  enddo
	endif
 
c set up energy index for eedf...
	do nreg = -1, ntmax_mc
	  do j = 0, nzmax_mc+1
	    do i = 0, nrmax_mc+1
	      indx_e(i,j,nreg) = 0
	    enddo
	  enddo
	enddo
 
	if (full_eedf_p .and. nfe.lt.1) then
	  write(*,"('     WARNING- no specified spatial cells,')")
	  write(*,"('       full_eedf_p is reassigned to false')")
	  write(*,*)
	  full_eedf_p = .false.
	endif
 
	i0 = 0
	i1 = 0
	if (full_eedf_p) then
	  do nreg = -1, nt_mc
	    if (n_traj_region(nreg).ne.0) then
	      do k = 1, nfe
		i0 = i0 + 1
		indx_e(irfe(k),izfe(k),nreg) = i0
	      enddo
	    endif
	  enddo
	  if (i0*nmu_mc*nphi_mc.gt.indx_e_max) then
	    x = real(indx_e_max) / real(i0)
	    x0 = real(nmu_mc) / real(nphi_mc)
	    nmu_mc = max(1, int(sqrt(x * x0)))
	    nphi_mc = max(1, indx_e_max / (i0*nmu_mc))
	    if (i0*nmu_mc*nphi_mc.gt.indx_e_max) then
	      write(*,"(5x,'WARNING- nmu_mc and nphi_mc changed: [',
     ^		i6, ',',i6,']')") nmu_mc, nphi_mc
              write(*,*)
	    endif
	  endif
	  i1       = i0 * nmu_mc * nphi_mc
	  dmu_mc   = 2.0 / nmu_mc
	  dphi_mc  = 2.0 * acos(-1.) / nphi_mc
	  dimu_mc  = 1.0 / dmu_mc
	  diphi_mc = 1.0 / dphi_mc
	else
	  do nreg = -1, nt_mc
	    if (n_traj_region(nreg).ne.0) then
	      do j = 0, nz_mc+1
	        do i = 0, nr_mc+1
		  indx_e(i,j,nreg) =
     ^		    (i+1)+(nr_mc+2)*(j+(nz_mc+2)*i0)
		  i1 = max(i1, indx_e(i,j,nreg))
	        enddo
	      enddo
	      i0 = i0 + 1
	    endif
	  enddo
	endif
	if (i1.gt.indx_e_max) then
	  write(*,"(' ERROR- indx_e_max < max(indx_e(...)) [',
     ^		i6, '<',i6,']')") indx_e_max, i1
	  write(*,"(' run aborting...')")
c	  pause
	  stop
	endif
	n_cold(-1) = n_traj_region(-1)
 
	ee_max_old = ee_max
        return
        end
 
 
 
 
	subroutine trapped_dist_mc
      	implicit real*4 (a-h,o-z)
 
      	include 'mc1.inc'
      	include 'mc2.inc'
 
	real*8 xc,x0
	real max_loss
	logical fe_p
 
 
	fe_p = .false.
 
	xc = 0.d0
	max_loss = 0
	do j = 1, nz_mc
	  do i = 1, nr_mc
	    if (mask_mc(i,j).eq.0) then
	      x            =  n_loss(i,j)
	      ne_loss(i,j) = ne_loss(i,j) / (x + 1.e-22)
	      xc           = xc + x
	      max_loss     = max(max_loss, x)
	    endif
	  enddo
  	enddo
	if (fe_last_p .and. fe_lastmx.gt.0.0) then
	  fe_p = .true.
	  xc = 0.d0
	  max_loss = 0
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      if (mask_mc(i,j).eq.0) then
	        x            = fe_lastt(i,j)
	        xc           = xc + x
	        max_loss     = max(max_loss, x)
	      endif
	    enddo
  	  enddo
	endif
	if (xc.ne.0.d0) then
	  k  = 0
	  x0 = n_traj_region(-1) / xc
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      ele_trapped(i,j) = 0
	      if (mask_mc(i,j).eq.0) then
		if (fe_p) then
		  k0 = fe_lastt(i,j) * x0
		else
		  k0 =  n_loss(i,j) * x0
		endif
		k = k + k0
		ele_trapped(i,j) = k0
	      endif
	    enddo
	  enddo
 
	  x0 = 1.0 / x0
 
	  if (k .gt. 200) then
	    n_traj = n_traj - n_traj_region(-1) + k
	    n_traj_region(-1) = k
	  endif
 
	  if (k .lt. n_traj_region(-1)) then
	    k = n_traj_region(-1) - k
	    do k0 = 1, k
 577	      i = max(1, min(nr_mc, int( nr_mc * ranf_mc(3) + 1)))
 	      j = max(1, min(nz_mc, int( nz_mc * ranf_mc(4) + 1)))
	      v = max_loss * ranf_mc(4)
	      if ((.not.fe_p) .and.
     ^		(v .gt.  n_loss( i,j) - ele_trapped(i,j)*x0 .or.
     ^		   mask_mc(i,j).ne.0)) go to 577
	      if ((fe_p) .and.
     ^		(v .gt. fe_lastt(i,j) - ele_trapped(i,j)*x0 .or.
     ^		   mask_mc(i,j).ne.0)) go to 577
c	      if (v .gt. n_loss( i,j) - ele_trapped(i,j)*x0 .or.
c     ^		   mask_mc(i,j).ne.0) go to 577
	      ele_trapped(i,j) = ele_trapped(i,j) + 1
	    enddo
	  elseif (k .gt. n_traj_region(-1)) then
	    write(*,*) 'error- too many trapped e',
     ^				-1,k,n_traj_region(-1)
c	    pause
	    stop
	  endif
	else
	  do j = 1, nz_mc
	    do i = 1, nr_mc
	      ne_loss(i,j) = max(0., phi_pl_mc(i,j,1) + min_energy)
	      n_loss(i,j)  = sign(1.,phi_pl_mc(i,j,1) + min_energy)+1.
	    enddo
	  enddo
	endif
 
c find average energy...
	xc = 0.d0
	x0 = 0.d0
	do j = 1, nz_mc
	  do i = 1, nr_mc
	    if (mask_mc(i,j).eq.0) then
	      x0 = x0 + ne_loss(i,j) * n_loss(i,j)
	      xc = xc + n_loss(i,j)
	    endif
	  enddo
	enddo
	ne_loss_av = x0 / (xc + 1.d-302)
	ne_loss_tot = x0
 
	return
	end
 
 
 
	function noblankupper_mc(str1,n)
 
	implicit none
	character*80 noblankupper_mc,str1,str2
	integer n,i,j,iua,ila,ilz,ichr
	character*1 chr
 
	iua = ichar('A')
	ila = ichar('a')
	ilz = ichar('z')
 
	i = 1
	do j = 1, n
	  chr  = str1(j:j)
	  ichr = ichar(chr)
	  if (ichr.ge.ila.and.ichr.le.ilz)
     ^		 ichr = ichr + iua - ila
	  if (chr .ne. ' ') then
	    str2(i:i) = char(ichr)
	    i         = i + 1
	  endif
	enddo
 
	do j = i, 80
	  str2(j:j) = ' '
	enddo
 
	noblankupper_mc = str2
	return
	end
c EOF
